@@ -23,6 +23,17 @@ using namespace MSO;
 
 namespace {
 
+    enum TextTypeEnum {
+        Tx_TYPE_TITLE = 0,
+        Tx_TYPE_BODY,
+        Tx_TYPE_NOTES,
+        Tx_TYPE_OTHER = 4,
+        Tx_TYPE_CENTERBODY,
+        Tx_TYPE_CENTERTITLE,
+        Tx_TYPE_HALFBODY,
+        Tx_TYPE_QUARTERBODY
+    };
+
 const TextMasterStyleAtom*
 getTextMasterStyleAtom(const MasterOrSlideContainer* m, quint16 texttype)
 {
@@ -112,13 +123,6 @@ const TextPFRun* getPFRun(const TextContainer* tc, quint32 start)
     }
     return 0;
 }
-const TextCFRun* getCFRun(const TextContainer* tc, quint32 start)
-{
-    if (tc && tc->style) {
-        return getRun<TextCFRun>(tc->style->rgTextCFRun, start);
-    }
-    return 0;
-}
 const TextPFException* getLevelPF(const MasterOrSlideContainer* m,
                                   quint32 textType, quint16 level)
 {
@@ -140,18 +144,31 @@ const TextCFException* getLevelCF(const MasterOrSlideContainer* m,
     return (ml) ?&ml->cf :0;
 }
 const TextMasterStyleLevel* getBaseLevel(const MasterOrSlideContainer* m,
-                                  quint32 textType, quint16 level)
+                                         quint32 textType, quint16 level)
 {
     const TextMasterStyleAtom* ms = 0;
-    if (textType == 6) {
-        // inherit from Tx_TYPE_TITLE
-        ms = getTextMasterStyleAtom(m, 0);
-    } else if (textType == 4 || textType == 5 || textType == 7
-               || textType == 8) {
-        // inherit from Tx_TYPE_BODY
-        ms = getTextMasterStyleAtom(m, 1);
+    const TextMasterStyleLevel* ml = 0;
+
+    if (textType == Tx_TYPE_CENTERTITLE) {
+        ms = getTextMasterStyleAtom(m, Tx_TYPE_TITLE);
+        ml = getTextMasterStyleLevel(ms, level); 
     }
-    return getTextMasterStyleLevel(ms, level);
+    //those are placeholder shapes so let's inherit from Tx_TYPE_BODY
+    //NOTE: testing required!
+//     else if (textType == 5 || textType == 7 || textType == 8) {
+//         ms = getTextMasterStyleAtom(m, 1);
+//     }
+
+    else if (textType == Tx_TYPE_OTHER) {
+        //MS Office 2007 specific at the moment 
+	if (level) {
+            ms = getTextMasterStyleAtom(m, Tx_TYPE_OTHER);
+            ml = getTextMasterStyleLevel(ms, 0);
+        }
+    }
+    //NOTE: Tx_TYPE_NOTES do not inherit at the moment
+
+    return ml;
 }
 const TextMasterStyleLevel* getBaseLevel(const MasterOrSlideContainer* m,
                                   const TextContainer* tc, quint16 level)
@@ -339,7 +356,16 @@ void addStyle(const Style** list, const Style* style)
         *list = 0;
     }
 }
+} //namespace
+
+const TextCFRun* getCFRun(const TextContainer* tc, const quint32 start)
+{
+    if (tc && tc->style) {
+        return getRun<TextCFRun>(tc->style->rgTextCFRun, start);
+    }
+    return 0;
 }
+
 const TextCFException*
 getTextCFException(const MSO::TextContainer* tc, const int start)
 {
@@ -393,9 +419,12 @@ PptTextPFRun::PptTextPFRun(const DocumentContainer* d,
 
     *pfs = 0;
     addStyle(pfs, (pfrun) ?&pfrun->pf :0);
+    //check the main master/title master slide's TextMasterStyleAtom
     addStyle(pfs, getLevelPF(m, tc, level));
     addStyle(pfs, getBaseLevelPF(m, tc, level));
+    //check DocumentContainer/DocumentTextInfoContainer/textMasterStyleAtom
     addStyle(pfs, getDefaultLevelPF(d, level));
+    //check DocumentContainer/DocumentTextInfoContainer/textPFDefaultsAtom
     addStyle(pfs, getDefaultPF(d));
 
     *pf9s = 0;
@@ -406,22 +435,52 @@ PptTextPFRun::PptTextPFRun(const DocumentContainer* d,
 
     // the level reported by PptPFRun is 0 when not bullets, i.e. no list is
     // active, 1 is lowest list level, 5 is the highest list level
-    level_ = (level || fHasBullet()) ?level + 1 :0;
-}
-PptTextCFRun::PptTextCFRun(const MSO::DocumentContainer* d,
-                           const MasterOrSlideContainer* m,
-                           const TextContainer& tc,
-                           quint16 level,
-                           quint32 start)
-{
-    const TextCFRun* cfrun = getCFRun(&tc, start);
+//     level_ = (level || fHasBullet()) ?level + 1 :0;
 
-    *cfs = 0;
-    addStyle(cfs, (cfrun) ?&cfrun->cf :0);
-    addStyle(cfs, getLevelCF(m, tc, level));
-    addStyle(cfs, getBaseLevelCF(m, &tc, level));
-    addStyle(cfs, getDefaultLevelCF(d, level));
-    addStyle(cfs, getDefaultCF(d));
+    //NOTE: The previous assumption is not true, there are cases when
+    //(indentLevel == 1 && fHasBullet == false).  We should interpret this text
+    //as content of a paragraph with corresponding indentLevel, thus avoid
+    //placing it into a text:list element.
+    level_ = level;
+}
+
+PptTextCFRun::PptTextCFRun(const MSO::DocumentContainer* d,
+                           QList<const MSO::MasterOrSlideContainer*> &mh,
+                           const MSO::TextContainer& tc,
+                           quint16 level)
+{
+    //check the main master/title master slide's TextMasterStyleAtom
+    for (int i = 0; i < mh.size(); i++) {
+        cfs.append(getLevelCF(mh[i], tc, level));
+        cfs.append(getBaseLevelCF(mh[i], &tc, level));
+    }
+
+    //check DocumentContainer/DocumentTextInfoContainer/textMasterStyleAtom
+    cfs.append(getDefaultLevelCF(d, level));
+    //check DocumentContainer/DocumentTextInfoContainer/textCFDefaultsAtom
+    cfs.append(getDefaultCF(d));
+
+    level_ = level;
+    cfrun_rm = false;
+}
+
+int PptTextCFRun::addCurrentCFRun(const MSO::TextContainer& tc, quint32 start)
+{
+    int n = 0;
+
+    //remove pointer to TextCFException structure of the previous run of text
+    if (cfrun_rm) {
+        cfs.removeFirst();
+        cfrun_rm = false;
+    }
+
+    const TextCFRun* cfrun = getCFRun(&tc, start);
+    if (cfrun) {
+        cfs.prepend(&cfrun->cf);
+        n = cfrun->count;
+        cfrun_rm = true;
+    }
+    return n;
 }
 
 #define GETTER(TYPE, PARENT, PRE, NAME, TEST, DEFAULT) \
@@ -436,16 +495,12 @@ TYPE PptTextPFRun::NAME() const \
     } \
     return DEFAULT; \
 }
-
 //     TYPE      PARENT       PRE NAME             TEST            DEFAULT
 GETTER(bool,     bulletFlags->,,  fHasBullet,      hasBullet,      false)
 GETTER(bool,     bulletFlags->,,  fBulletHasFont,  bulletHasFont,  false)
 GETTER(bool,     bulletFlags->,,  fBulletHasColor, bulletHasColor, false)
 GETTER(bool,     bulletFlags->,,  fBulletHasSize,  bulletHasSize,  false)
 GETTER(qint16,   ,             ,  bulletChar,      bulletChar,     0)
-GETTER(quint16,  ,             ,  bulletFontRef,   bulletFont,     0)
-GETTER(qint16,   ,             ,  bulletSize,      bulletSize,     0)
-GETTER(ColorIndexStruct,,      *, bulletColor,   bulletColor,ColorIndexStruct())
 GETTER(quint16,  ,             ,  textAlignment,   align,          0)
 GETTER(qint16,   ,             ,  lineSpacing,     lineSpacing,    0)
 GETTER(qint16,   ,             ,  spaceBefore,     spaceBefore,    0)
@@ -461,6 +516,26 @@ GETTER(bool,     wrapFlags->,  ,  overflow,        overflow,       false)
 GETTER(quint16,  ,             ,  textDirection,   textDirection,  0)
 #undef GETTER
 
+#define GETTER(TYPE, PRE, NAME, TEST, VALID, DEFAULT) \
+TYPE PptTextPFRun::NAME() const \
+{ \
+    const MSO::TextPFException* const * p = pfs; \
+    while (*p) { \
+        if ((*p)->masks.TEST) { \
+ 	    if (VALID()) { \
+                return PRE (*p)->NAME; \
+            } \
+        } \
+        ++p; \
+    } \
+    return DEFAULT; \
+}
+//     TYPE             PRE NAME             TEST         VALID            DEFAULT
+GETTER(quint16,          ,  bulletFontRef,   bulletFont,  fBulletHasFont,  0)
+GETTER(qint16,           ,  bulletSize,      bulletSize,  fBulletHasSize,  0)
+GETTER(ColorIndexStruct, *, bulletColor,     bulletColor, fBulletHasColor, ColorIndexStruct())
+#undef GETTER
+
 qint32 PptTextPFRun::bulletBlipRef() const {
     const MSO::TextPFException9* const * p = pf9s;
     while (*p) {
@@ -481,6 +556,8 @@ qint16 PptTextPFRun::fBulletHasAutoNumber() const {
     }
     return 0;
 }
+
+#define ANM_Default 3
 quint16 PptTextPFRun::scheme() const {
     const MSO::TextPFException9* const * p = pf9s;
     while (*p) {
@@ -489,7 +566,8 @@ quint16 PptTextPFRun::scheme() const {
         }
         ++p;
     }
-    return 0;
+    
+    return ANM_Default;
 }
 qint16 PptTextPFRun::startNum() const {
     const MSO::TextPFException9* const * p = pf9s;
@@ -502,15 +580,29 @@ qint16 PptTextPFRun::startNum() const {
     return 1;
 }
 
+//NOTE: Logic exploited from test documents saved in MS PowerPoint 2003/2007
+bool PptTextPFRun::isList() const {
+    bool ret = false;
+    if (fHasBullet() ||
+        fBulletHasAutoNumber() ||
+        (bulletBlipRef() != 65535)
+//         (bulletChar() > 0) ||
+//         fBulletHasFont() ||
+       ) {
+        ret = true;
+    }
+    return ret;
+}
+
 #define GETTER(TYPE, PARENT, PRE, NAME, TEST, DEFAULT) \
 TYPE PptTextCFRun::NAME() const \
 { \
-    const MSO::TextCFException* const * c = cfs; \
-    while (*c) { \
-        if ((*c)->TEST) { \
-            return PRE (*c)->PARENT NAME; \
+    for (int i = 0; i < cfs.size(); i++) { \
+        if (cfs[i]) { \
+            if ((cfs[i])->TEST) { \
+                return PRE (cfs[i])->PARENT NAME; \
+            } \
         } \
-        ++c; \
     } \
     return DEFAULT; \
 }
@@ -531,4 +623,4 @@ GETTER(quint16,         ,           ,  symbolFontRef, masks.symbolTypeface,  0)
 GETTER(quint16,         ,           ,  fontSize,      masks.size,            0)
 GETTER(ColorIndexStruct,,           *, color,   masks.color, ColorIndexStruct())
 GETTER(qint16,          ,           ,  position,      masks.position,        0)
-
+#undef GETTER
