@@ -20,8 +20,8 @@
 #ifndef _IMPORT_HELPERS_H
 #define _IMPORT_HELPERS_H
 
-#include <qstring.h>
-#include <qmap.h>
+#include <QString>
+#include <QMap>
 
 /**
  * \file ImportHelpers.h
@@ -34,20 +34,22 @@
 class AbiProps
 {
 public:
-    AbiProps() {};
-    AbiProps(QString newValue) : m_value(newValue) {};
-    virtual ~AbiProps() {};
+    AbiProps() {}
+    AbiProps(QString newValue) : m_value(newValue) {}
+    virtual ~AbiProps() {}
 public:
-    inline QString getValue(void) const { return m_value; }
+    inline QString getValue(void) const {
+        return m_value;
+    }
 private:
     QString m_value;
 };
 
-class AbiPropsMap : public QMap<QString,AbiProps>
+class AbiPropsMap : public QMap<QString, AbiProps>
 {
 public:
-    AbiPropsMap() {};
-    virtual ~AbiPropsMap() {};
+    AbiPropsMap() {}
+    virtual ~AbiPropsMap() {}
 public:
     bool setProperty(const QString& newName, const QString& newValue);
     void splitAndAddAbiProps(const QString& strProps);
@@ -80,6 +82,6 @@ inline double PicaToPoints(const double d)
  * @param atleast was there a + character after the unit to denote an "at-least" property
  * @return the value as double (in points)
  */
-double ValueWithLengthUnit( const QString& _str, bool* atleast = NULL );
+double ValueWithLengthUnit(const QString& _str, bool* atleast = NULL);
 
 #endif // _IMPORT_HELPERS_H
