@@ -20,22 +20,18 @@
 #ifndef DOCBOOKEXPORT_H
 #define DOCBOOKEXPORT_H
 
-#include <QString>
-#include <QFile>
-#include <QObject>
-
 #include <KoFilter.h>
-#include <KoStore.h>
+#include <QVariantList>
 
-
-class DocBookExport : public KoFilter {
+class DocBookExport : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    DocBookExport (QObject *parent, const QStringList & );
+    DocBookExport(QObject *parent, const QVariantList &);
     virtual ~DocBookExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QByteArray& from, const QByteArray& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 #endif // DOCBOOKEXPORT_H
