@@ -29,7 +29,7 @@
 #include "kword13document.h"
 #include "kword13postparsing.h"
 
-KWord13PostParsing::KWord13PostParsing(void) : m_kwordDocument( 0 )
+KWord13PostParsing::KWord13PostParsing(void) : m_kwordDocument(0)
 {
 }
 
@@ -37,16 +37,14 @@ KWord13PostParsing::~KWord13PostParsing(void)
 {
 }
 
-bool KWord13PostParsing::postParsePictures( KoStore* store )
+bool KWord13PostParsing::postParsePictures(KoStore* store)
 {
-    if ( ! m_kwordDocument )
+    if (! m_kwordDocument)
         return false;
-        
-    for ( Q3DictIterator<KWord13Picture> it( m_kwordDocument->m_pictureDict ) ; it.current(); ++it )
-    {
-        kDebug(30520) <<"Loading..." << it.currentKey();
-        if ( ! it.current()->loadPicture( store ) )
-        {
+
+    for (Q3DictIterator<KWord13Picture> it(m_kwordDocument->m_pictureDict) ; it.current(); ++it) {
+        kDebug(30520) << "Loading..." << it.currentKey();
+        if (! it.current()->loadPicture(store)) {
             kWarning(30520) << "Could not load picture!";
             return false;
         }
@@ -55,14 +53,14 @@ bool KWord13PostParsing::postParsePictures( KoStore* store )
 }
 
 
-bool KWord13PostParsing::postParse( KoStore* store, KWord13Document& doc )
+bool KWord13PostParsing::postParse(KoStore* store, KWord13Document& doc)
 {
     // ### TODO: check if it is the same document if called twice
     m_kwordDocument = &doc;
-    
+
     // ### TODO: return value
-    postParsePictures( store );
+    postParsePictures(store);
     // ### TODO
-    
+
     return 1;
 }
