@@ -23,17 +23,17 @@
 #define __STARWRITERIMPORT_H
 
 #include <KoFilter.h>
-#include <qstring.h>
-#include <qcstring.h>
+#include <QString>
+#include <QVariantList>
 
 class StarWriterImport: public KoFilter
 {
     Q_OBJECT
 
 public:
-    StarWriterImport(KoFilter *parent, const char *name, const QStringList&);
+    StarWriterImport(QObject* parent, const QVariantList&);
     virtual ~StarWriterImport();
-    KoFilter::ConversionStatus convert(const QCString& from, const QCString& to);
+    KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 
 private:
     // most important OLE streams
@@ -41,7 +41,7 @@ private:
     QByteArray StarWriterDocument;
 
     // supplementary variables
-    Q_UINT8 tablesNumber;
+    quint8 tablesNumber;
     QString bodyStuff, tablesStuff, picturesStuff;
 
     // needed for ATTRIBUTES
@@ -58,13 +58,13 @@ private:
     bool addHeaders();
     bool addFooters();
     bool addBody();
-    QString convertToKWordString(QByteArray s);
+    QString convertToKWordString(const QByteArray& s);
 
     // Node routines
-    bool parseNodes(QByteArray n);
-    bool parseText(QByteArray n);
-    bool parseTable(QByteArray n);
-    bool parseGraphics(QByteArray n);
+    bool parseNodes(const QByteArray& n);
+    bool parseText(const QByteArray& n);
+    bool parseTable(const QByteArray& n);
+    bool parseGraphics(const QByteArray& n);
 
     // finished KWord document
     QString maindoc;
