@@ -25,6 +25,8 @@
 #include "MsooXmlSchemas.h"
 #include "MsooXmlUtils.h"
 #include "KoXmlWriter.h"
+#include <KoOdfGraphicStyles.h>
+#include <MsooXmlRelationships.h>
 
 #define MSOOXML_CURRENT_NS "a"
 #define MSOOXML_CURRENT_CLASS MsooXmlThemesReader
@@ -37,6 +39,72 @@
 
 using namespace MSOOXML;
 
+DrawingMLGradientFill::DrawingMLGradientFill(QVector<qreal> shadeModifier, QVector<qreal> tintModifier, QVector<qreal> satModifier, QVector<int> alphaModifier) 
+    : m_shadeModifier(shadeModifier),m_tintModifier(tintModifier), m_satModifier(satModifier), m_alphaModifier(alphaModifier)
+{
+}
+
+void DrawingMLGradientFill::writeStyles(KoGenStyles& styles, KoGenStyle *graphicStyle, QColor color)
+{
+    KoGenStyle gradientStyle = KoGenStyle(KoGenStyle::GradientStyle);
+
+    // This is over-simplified gradient style
+    gradientStyle.addAttribute("draw:style", "linear");
+    if (m_alphaModifier.at(0) > 0) {
+        gradientStyle.addAttribute("draw:start-intensity", QString("%1%").arg(m_alphaModifier.at(0)));
+    }
+    else {
+        gradientStyle.addAttribute("draw:start-intensity", "100%");
+    }
+    if (m_alphaModifier.at(m_alphaModifier.size()-1)) {
+        gradientStyle.addAttribute("draw:end-intensity", QString("%1%").arg(m_alphaModifier.at(m_alphaModifier.size() - 1)));
+    }
+    else {
+        gradientStyle.addAttribute("draw:end-intensity", "100%");
+    }
+
+    QColor startColor = color;
+    QColor endColor = color;
+
+    MSOOXML::Utils::modifyColor(startColor, m_tintModifier.at(0), m_shadeModifier.at(0), m_satModifier.at(0));
+    MSOOXML::Utils::modifyColor(endColor, m_tintModifier.at(m_tintModifier.size() - 1), m_shadeModifier.at(m_shadeModifier.size() - 1),
+        m_satModifier.at(m_satModifier.size() - 1));
+
+    gradientStyle.addAttribute("draw:start-color", startColor.name());
+    gradientStyle.addAttribute("draw:end-color", endColor.name());
+
+    graphicStyle->addProperty("draw:fill", "gradient");
+    const QString gradName = styles.insert(gradientStyle);
+    graphicStyle->addProperty("draw:fill-gradient-name", gradName);
+}
+
+DrawingMLBlipFill::DrawingMLBlipFill(QString filePath) : m_filePath(filePath)
+{
+}
+
+void DrawingMLBlipFill::writeStyles(KoGenStyles& styles, KoGenStyle *graphicStyle, QColor color)
+{
+    Q_UNUSED(color)
+
+    KoGenStyle fillImageStyle(KoGenStyle::FillImageStyle);
+    fillImageStyle.addAttribute("xlink:href", m_filePath);
+    fillImageStyle.addAttribute("xlink:type", "simple");
+    fillImageStyle.addAttribute("xlink:show", "embed");
+    fillImageStyle.addAttribute("xlink:actuate", "onLoad");
+    QString fillImageName = styles.insert(fillImageStyle);
+
+    graphicStyle->addProperty("draw:fill", "bitmap");
+    graphicStyle->addProperty("draw:fill-image-name", fillImageName);
+}
+
+void DrawingMLSolidFill::writeStyles(KoGenStyles& styles, KoGenStyle *graphicStyle, QColor color)
+{
+    if (color.isValid()) {
+        QBrush brush(color, Qt::SolidPattern);
+        KoOdfGraphicStyles::saveOdfFillStyle(*graphicStyle, styles, brush);
+    }
+}
+
 DrawingMLColorSchemeItem::DrawingMLColorSchemeItem()
 {
 }
@@ -47,20 +115,27 @@ DrawingMLColorSchemeSystemItem::DrawingMLColorSchemeSystemItem()
 
 QColor DrawingMLColorSchemeSystemItem::value() const
 {
-    kDebug() << systemColor;
+    
+    if (lastColor.isValid()) {
+        return lastColor;
+    }
     //! 20.1.10.58 ST_SystemColorVal (System Color Value)
     if (   systemColor == QLatin1String("windowText")
         || systemColor == QLatin1String("menuText"))
     {
-//        return QPalette().color(QPalette::Active, QPalette::WindowText);
-        return QPalette().color(QPalette::Active, QPalette::Window);
+//         if (!spreadsheetMode) {
+            return QPalette().color(QPalette::Active, QPalette::WindowText);
+//         }
+//         return QPalette().color(QPalette::Active, QPalette::Window);
     }
     else if (    systemColor == QLatin1String("window")
               || systemColor == QLatin1String("menu")
               || systemColor == QLatin1String("menuBar"))
     {
-//        return QPalette().color(QPalette::Active, QPalette::Window);
-        return QPalette().color(QPalette::Active, QPalette::WindowText);
+//         if (!spreadsheetMode) {
+            return QPalette().color(QPalette::Active, QPalette::Window);
+//         }
+//         return QPalette().color(QPalette::Active, QPalette::WindowText);
     }
     else if (systemColor == QLatin1String("highlightText")) {
         return QPalette().color(QPalette::Active, QPalette::HighlightedText);
@@ -84,7 +159,7 @@ QColor DrawingMLColorSchemeSystemItem::value() const
         return QPalette().color(QPalette::Active, QPalette::Dark);
     }
 //! @todo Use more of systemColor
-    return lastColor;
+    return QColor(Qt::black); // TODO; better default?
 }
 
 DrawingMLColorSchemeItemBase::DrawingMLColorSchemeItemBase()
@@ -115,6 +190,26 @@ DrawingMLColorScheme::~DrawingMLColorScheme()
     qDeleteAll(set);
 }
 
+DrawingMLColorScheme& DrawingMLColorScheme::operator=(const DrawingMLColorScheme& scheme)
+{
+    QHashIterator<QString, DrawingMLColorSchemeItemBase*> i(scheme);
+    while (i.hasNext()) {
+        i.next();
+        insert(i.key(), i.value()->clone());
+    }
+
+    return *this;
+}
+
+DrawingMLColorScheme::DrawingMLColorScheme(const DrawingMLColorScheme& scheme) : QHash<QString, DrawingMLColorSchemeItemBase*>()
+{
+    QHashIterator<QString, DrawingMLColorSchemeItemBase*> i(scheme);
+    while (i.hasNext()) {
+        i.next();
+        insert(i.key(), i.value()->clone());
+    }
+}
+
 DrawingMLColorSchemeItemBase* DrawingMLColorScheme::value(int index) const
 {
     return DrawingMLColorSchemeItemHash::value( QString::number(index) );
@@ -128,40 +223,36 @@ DrawingMLFontSet::DrawingMLFontSet()
 {
 }
 
-DrawingMLFillStyleBase::DrawingMLFillStyleBase()
+DrawingMLFillBase::~DrawingMLFillBase()
 {
 }
 
-DrawingMLFillStyleBase::~DrawingMLFillStyleBase()
-{
-}
-
-DrawingMLSolidFillStyle* DrawingMLFillStyleBase::toSolidFillStyle()
-{
-    return dynamic_cast<DrawingMLSolidFillStyle*>(this);
-}
-
-DrawingMLPatternFillStyle* DrawingMLFillStyleBase::toPatternFillStyle()
-{
-    return dynamic_cast<DrawingMLPatternFillStyle*>(this);
-}
-
-DrawingMLNoFillStyle* DrawingMLFillStyleBase::toNoFillStyle()
-{
-    return dynamic_cast<DrawingMLNoFillStyle*>(this);
-}
-
-DrawingMLNoFillStyle::DrawingMLNoFillStyle()
+DrawingMLFormatScheme::~DrawingMLFormatScheme()
 {
+    qDeleteAll(fillStyles);
 }
 
 DrawingMLFormatScheme::DrawingMLFormatScheme()
 {
 }
 
-DrawingMLPatternFillStyle::DrawingMLPatternFillStyle()
-        : backgroundColor(0), foregroundColor(0)
+DrawingMLFormatScheme::DrawingMLFormatScheme(const DrawingMLFormatScheme& format)
 {
+    QMapIterator<int, DrawingMLFillBase*> i(format.fillStyles);
+    while (i.hasNext()) {
+        i.next();
+        fillStyles.insert(i.key(), i.value()->clone());
+    }
+}
+
+DrawingMLFormatScheme& DrawingMLFormatScheme::operator=(const DrawingMLFormatScheme& format)
+{
+    QMapIterator<int, DrawingMLFillBase*> i(format.fillStyles);
+    while (i.hasNext()) {
+        i.next();
+        fillStyles.insert(i.key(), i.value()->clone());
+    }
+    return *this;
 }
 
 DrawingMLTheme::DrawingMLTheme()
@@ -170,16 +261,17 @@ DrawingMLTheme::DrawingMLTheme()
 
 // ---------------------------------------------------
 
-MsooXmlThemesReaderContext::MsooXmlThemesReaderContext(DrawingMLTheme& t)
+MsooXmlThemesReaderContext::MsooXmlThemesReaderContext(DrawingMLTheme& t, MSOOXML::MsooXmlRelationships* rel,
+        MSOOXML::MsooXmlImport* imp, QString pathName, QString fileName)
         : MsooXmlReaderContext()
-        , theme(&t)
+        , theme(&t), relationships(rel), import(imp), path(pathName), file(fileName)
 {
 }
 
 // ---------------------------------------------------
 
 MsooXmlThemesReader::MsooXmlThemesReader(KoOdfWriters *writers)
-        : MsooXmlReader(writers)
+        : MsooXmlCommonReader(writers)
         , m_currentColor(0)
         , m_context(0)
 {
@@ -200,6 +292,10 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read(MsooXmlReaderContext* conte
 {
     m_context = dynamic_cast<MsooXmlThemesReaderContext*>(context);
     Q_ASSERT(m_context);
+    m_import = m_context->import;
+    m_path = m_context->path;
+    m_file = m_context->file;
+    m_relationships = m_context->relationships;
     *m_context->theme = DrawingMLTheme(); //clear
     const KoFilter::ConversionStatus result = readInternal();
     m_context = 0;
@@ -211,21 +307,21 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read(MsooXmlReaderContext* conte
 
 KoFilter::ConversionStatus MsooXmlThemesReader::readInternal()
 {
-    kDebug() << "=============================";
+    //kDebug() << "=============================";
     readNext();
     if (!isStartDocument()) {
         return KoFilter::WrongFormat;
     }
 
     readNext();
-    kDebug() << *this;
+    //kDebug() << *this;
     if (isStartElement()) {
         TRY_READ_IF(theme)
         ELSE_WRONG_FORMAT
     } else {
         return KoFilter::WrongFormat;
     }
-    kDebug() << "===========finished============";
+    //kDebug() << "===========finished============";
     return KoFilter::OK;
 }
 
@@ -245,7 +341,7 @@ KoFilter::ConversionStatus MsooXmlThemesReader::readInternal()
 KoFilter::ConversionStatus MsooXmlThemesReader::read_theme()
 {
     READ_PROLOGUE
-    kDebug() << namespaceUri();
+    //kDebug() << namespaceUri();
 
     if (!expectNS(Schemas::drawingml::main)) {
         return KoFilter::WrongFormat;
@@ -253,12 +349,12 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_theme()
 
     const QXmlStreamAttributes attrs(attributes());
     READ_ATTR_WITHOUT_NS_INTO(name, m_context->theme->name)
-    kDebug() << "name:" << m_context->theme->name;
+    //kDebug() << "name:" << m_context->theme->name;
 
     QXmlStreamNamespaceDeclarations namespaces(namespaceDeclarations());
-    for (int i = 0; i < namespaces.count(); i++) {
-        kDebug() << "NS prefix:" << namespaces[i].prefix() << "uri:" << namespaces[i].namespaceUri();
-    }
+    //for (int i = 0; i < namespaces.count(); i++) {
+        //kDebug() << "NS prefix:" << namespaces[i].prefix() << "uri:" << namespaces[i].namespaceUri();
+    //}
 //! @todo find out whether the namespace returned by namespaceUri()
 //!       is exactly the same ref as the element of namespaceDeclarations()
     if (!namespaces.contains(QXmlStreamNamespaceDeclaration(MSOOXML_CURRENT_NS, Schemas::drawingml::main))) {
@@ -269,7 +365,8 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_theme()
 
     while (!atEnd()) {
         readNext();
-        kDebug() << *this;
+        //kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(themeElements)
             ELSE_TRY_READ_IF(custClrLst)
@@ -279,7 +376,6 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_theme()
             ELSE_TRY_READ_IF(custClrLst)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -302,7 +398,8 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_themeElements()
     READ_PROLOGUE
     while (!atEnd()) {
         readNext();
-        kDebug() << *this;
+        //kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(clrScheme)
             ELSE_TRY_READ_IF(extLst)
@@ -310,7 +407,6 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_themeElements()
             ELSE_TRY_READ_IF(fontScheme)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -363,6 +459,18 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_extLst()
     SKIP_EVERYTHING_AND_RETURN
 }
 
+#undef CURRENT_EL
+#define CURRENT_EL clrMap
+KoFilter::ConversionStatus MsooXmlThemesReader::read_clrMap()
+{
+    READ_PROLOGUE
+    const QXmlStreamAttributes attrs(attributes());
+
+    SKIP_EVERYTHING
+
+    READ_EPILOGUE
+}
+
 //! @todo !!!!!!!!!!!!!MERGE with Document Reader!!!!!!!!!!!!!
 #undef CURRENT_EL
 #define CURRENT_EL extraClrSchemeLst
@@ -373,7 +481,38 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_extLst()
 */
 KoFilter::ConversionStatus MsooXmlThemesReader::read_extraClrSchemeLst()
 {
-    SKIP_EVERYTHING_AND_RETURN
+   READ_PROLOGUE
+
+   while (!atEnd()) {
+        readNext();
+        //kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            TRY_READ_IF(extraClrScheme)
+        }
+    }
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL extraClrScheme
+//! extraClrScheme (Extra Color Scheme)
+/*!
+
+ @todo implement
+*/
+KoFilter::ConversionStatus MsooXmlThemesReader::read_extraClrScheme()
+{
+   READ_PROLOGUE
+
+   while (!atEnd()) {
+        readNext();
+        //kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+        }
+    }
+    READ_EPILOGUE
 }
 
 #define INIT_COLOR_ITEM(name, index) \
@@ -429,7 +568,8 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_clrScheme()
 
     while (!atEnd()) {
         readNext();
-        kDebug() << *this;
+        //kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             ReadMethod readMethod = m_readMethods.value(this->name().toString());
             if (readMethod) {
@@ -440,17 +580,17 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_clrScheme()
                     return KoFilter::InternalError;
                 }
                 const QString colorName( this->name().toString() );
-                kDebug() << "inserting color for" << colorName;
+                //kDebug() << "inserting color for" << colorName;
                 m_context->theme->colorScheme.insert(colorName, m_currentColor);
                 const QString colorIndex(m_colorSchemeIndices.value(colorName));
                 if (!colorIndex.isEmpty()) {
                     m_context->theme->colorScheme.insert(colorIndex, m_currentColor);
                 }
+
                 m_currentColor = 0;
             }
             ELSE_WRONG_FORMAT_DEBUG("!readMethod")
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -491,7 +631,8 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_color()
 
     while (!atEnd()) {
         readNext();
-        kDebug() << *this;
+        //kDebug() << *this;
+        BREAK_IF_END_OF_QSTRING(qn)
         if (isStartElement()) {
             ReadMethod readMethod = m_readMethods.value(this->name().toString());
             if (readMethod) {
@@ -499,7 +640,6 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_color()
             }
             ELSE_WRONG_FORMAT_DEBUG("!readMethod")
         }
-        BREAK_IF_END_OF_QSTRING(qn)
     }
 
     if (!expectElEnd(qn)) {
@@ -573,11 +713,12 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_srgbClr()
 
     READ_ATTR_WITHOUT_NS(val)
     color.get()->color = Utils::ST_HexColorRGB_to_QColor(val);
-    kDebug() << color.get()->color;
+    //kDebug() << color.get()->color;
 
     readNext();
     READ_EPILOGUE_WITHOUT_RETURN
     m_currentColor = color.release();
+
     return KoFilter::OK;
 }
 
@@ -643,16 +784,17 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_sysClr()
 
     READ_ATTR_WITHOUT_NS(lastClr)
     color.get()->lastColor = Utils::ST_HexColorRGB_to_QColor(lastClr);
-    kDebug() << "lastClr:" << color.get()->lastColor;
+    //kDebug() << "lastClr:" << color.get()->lastColor;
 
     // System color value. This color is based upon the value that this color currently has
     // within the system on which the document is being viewed.
     READ_ATTR_WITHOUT_NS_INTO(val, color.get()->systemColor)
-    kDebug() << "val:" << color.get()->systemColor;
+    //kDebug() << "val:" << color.get()->systemColor;
 
     readNext();
     READ_EPILOGUE_WITHOUT_RETURN
     m_currentColor = color.release();
+
     return KoFilter::OK;
 }
 
@@ -665,13 +807,127 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_sysClr()
  - [done] themeElements (§20.1.6.10)
  - themeOverride (§20.1.6.12)
  Child elements:
- - bgFillStyleLst (Background Fill Style List) §20.1.4.1.7
+ - [done] bgFillStyleLst (Background Fill Style List) §20.1.4.1.7
  - effectStyleLst (Effect Style List) §20.1.4.1.12
  - fillStyleLst (Fill Style List) §20.1.4.1.13
  - lnStyleLst (Line Style List) §20.1.4.1.21*/
 KoFilter::ConversionStatus MsooXmlThemesReader::read_fmtScheme()
 {
-    SKIP_EVERYTHING_AND_RETURN
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        //kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            TRY_READ_IF(bgFillStyleLst)
+        }
+    }
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL bgFillStyleLst
+//! bgFillStyleLst (background fill style list)
+KoFilter::ConversionStatus MsooXmlThemesReader::read_bgFillStyleLst()
+{
+    READ_PROLOGUE
+
+    int index = 1001; // The first style goes to 1001
+    QString element;
+
+    QXmlStreamAttributes attrs;
+
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            element = qualifiedName().toString();
+            // Commented out for now, until there's a nice implementation for duotone effect
+            /*if (element == "a:blipFill") {
+                readNext(); // Going to a:blip
+                attrs = attributes();
+                TRY_READ_ATTR_WITH_NS(r, embed)
+                if (!r_embed.isEmpty()) {
+                    const QString sourceName(m_relationships->target(m_path, m_file, r_embed));
+                    QString destinationName = "Pictures/" + sourceName.mid(sourceName.lastIndexOf('/') + 1);
+                    RETURN_IF_ERROR( m_import->copyFile(sourceName, destinationName, false) )
+                    addManifestEntryForFile(destinationName);
+                    addManifestEntryForPicturesDir();
+                    m_context->theme->formatScheme.fillStyles[index] = new DrawingMLBlipFill(destinationName);
+                }
+                while (true) {
+                    readNext();
+                    if (isEndElement() && qualifiedName() == element) {
+                        break;
+                    }
+                }
+            }*/
+            if (element == "a:gradFill") {
+                QVector<qreal> shadeModifiers;
+                QVector<qreal> tintModifiers;
+                QVector<qreal> satModifiers;
+                QVector<int> alphaModifiers;
+                readNext(); // a:gsLst
+                while (true) {
+                    readNext();
+                    if (isStartElement() && qualifiedName() == "a:gs") {
+                        readNext();
+                        if (isStartElement() && qualifiedName() == "a:schemeClr") {
+                           qreal shadeModifier = 0;
+                           qreal tintModifier = 0;
+                           qreal satModifier = 0;
+                           int alphaModifier = 0;
+                           while (true) {
+                               readNext();
+                               if (isEndElement() && qualifiedName() == "a:schemeClr") {
+                                   break;
+                               }
+                               if (isStartElement()) {
+                                   attrs = attributes();
+                                   TRY_READ_ATTR_WITHOUT_NS(val)
+                                   if (qualifiedName() == "a:tint") {
+                                       tintModifier = val.toInt()/100000.0;
+                                   }
+                                   else if (qualifiedName() == "a:shade") {
+                                       shadeModifier = val.toInt()/100000.0;
+                                   }
+                                   else if (qualifiedName() == "a:satMod") {
+                                       satModifier = val.toDouble()/100000.0;
+                                   }
+                                   else if (qualifiedName() == "a:alpha") {
+                                      alphaModifier = val.toInt()/1000;
+                                   }
+                               }
+                           }
+                           tintModifiers.push_back(tintModifier);
+                           shadeModifiers.push_back(shadeModifier);
+                           satModifiers.push_back(satModifier),
+                           alphaModifiers.push_back(alphaModifier);
+                        }
+                        else if (isEndElement() && qualifiedName() == "a:gs") {
+                            break;
+                        }
+                    }
+                    else if (isEndElement() && qualifiedName() == "a:gsLst") {
+                        break;
+                    }
+                }
+                m_context->theme->formatScheme.fillStyles[index] = new DrawingMLGradientFill(shadeModifiers, tintModifiers, satModifiers, alphaModifiers);
+                while (true) {
+                    readNext();
+                    if (isEndElement() && qualifiedName() == element) {
+                        break;
+                    }
+                }
+            }
+            else { //if (element == "a:solidFill") { //For now, let's have this as default
+                m_context->theme->formatScheme.fillStyles[index] = new DrawingMLSolidFill;
+                skipCurrentElement();
+            } // todo, handle rest
+            ++index;
+        }
+    }
+    READ_EPILOGUE
 }
 
 //! @todo !!!!!!!!!!!!!MERGE with Document Reader!!!!!!!!!!!!!
@@ -684,14 +940,84 @@ KoFilter::ConversionStatus MsooXmlThemesReader::read_fmtScheme()
  - themeOverride (§20.1.6.12)
  Child elements:
  - extLst (Extension List) §20.1.2.2.15
- - majorFont (Major Font) §20.1.4.1.24
- - minorFont (Minor fonts) §20.1.4.1.25*/
+ - [done] majorFont (Major Font) §20.1.4.1.24
+ - [done] minorFont (Minor fonts) §20.1.4.1.25*/
 KoFilter::ConversionStatus MsooXmlThemesReader::read_fontScheme()
 {
-    SKIP_EVERYTHING_AND_RETURN
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            TRY_READ_IF(majorFont)
+            ELSE_TRY_READ_IF(minorFont)
+        }
+    }
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL majorFont
+//! majorFont (Major Font)
+KoFilter::ConversionStatus MsooXmlThemesReader::read_majorFont()
+{
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            if (qualifiedName() == "a:latin") {
+                const QXmlStreamAttributes attrs(attributes());
+                TRY_READ_ATTR_WITHOUT_NS(typeface)
+                m_context->theme->fontScheme.majorFonts.latinTypeface = typeface;
+            }
+            else if (qualifiedName() == "a:ea") {
+                const QXmlStreamAttributes attrs(attributes());
+                TRY_READ_ATTR_WITHOUT_NS(typeface)
+                m_context->theme->fontScheme.minorFonts.eaTypeface = typeface;
+            }
+            else if (qualifiedName() == "a:cs") {
+                const QXmlStreamAttributes attrs(attributes());
+                TRY_READ_ATTR_WITHOUT_NS(typeface)
+                m_context->theme->fontScheme.minorFonts.csTypeface = typeface;
+            }
+        }
+    }
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL minorFont
+//! minorFont (Minor Font)
+KoFilter::ConversionStatus MsooXmlThemesReader::read_minorFont()
+{
+    READ_PROLOGUE
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            if (qualifiedName() == "a:latin") {
+                const QXmlStreamAttributes attrs(attributes());
+                TRY_READ_ATTR_WITHOUT_NS(typeface)
+                m_context->theme->fontScheme.minorFonts.latinTypeface = typeface;
+            }
+            else if (qualifiedName() == "a:ea") {
+                const QXmlStreamAttributes attrs(attributes());
+                TRY_READ_ATTR_WITHOUT_NS(typeface)
+                m_context->theme->fontScheme.minorFonts.eaTypeface = typeface;
+            }
+            else if (qualifiedName() == "a:cs") {
+                const QXmlStreamAttributes attrs(attributes());
+                TRY_READ_ATTR_WITHOUT_NS(typeface)
+                m_context->theme->fontScheme.minorFonts.csTypeface = typeface;
+            }
+        }
+    }
+    READ_EPILOGUE
 }
 
 KoFilter::ConversionStatus MsooXmlThemesReader::read_SKIP()
 {
     SKIP_EVERYTHING_AND_RETURN
 }
+
