@@ -46,6 +46,8 @@
 #include "MsooXmlSchemas.h"
 #include "MsooXmlReader.h"
 
+#include "pole.h"
+
 #include <KoOdfReadStore.h>
 #include <styles/KoCharacterStyle.h>
 #include <KoXmlReader.h>
@@ -56,6 +58,7 @@
 #include <QColor>
 #include <QBrush>
 #include <QImage>
+#include <QImageReader>
 #include <QPalette>
 #include <QRegExp>
 #include <QtXml>
@@ -82,6 +85,7 @@ const char* const MSOOXML::ContentTypes::wordEndnotes =         "application/vnd
 const char* const MSOOXML::ContentTypes::wordFontTable =        "application/vnd.openxmlformats-officedocument.wordprocessingml.fontTable+xml";
 const char* const MSOOXML::ContentTypes::wordWebSettings =      "application/vnd.openxmlformats-officedocument.wordprocessingml.webSettings+xml";
 const char* const MSOOXML::ContentTypes::wordTemplate =         "application/vnd.openxmlformats-officedocument.wordprocessingml.template.main+xml";
+const char* const MSOOXML::ContentTypes::wordComments =         "application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml";
 
 // presentationml-specific content types
 const char* const MSOOXML::ContentTypes::presentationDocument =      "application/vnd.openxmlformats-officedocument.presentationml.presentation.main+xml";
@@ -93,15 +97,18 @@ const char* const MSOOXML::ContentTypes::presentationNotes =         "applicatio
 const char* const MSOOXML::ContentTypes::presentationTableStyles =   "application/vnd.openxmlformats-officedocument.presentationml.tableStyles+xml";
 const char* const MSOOXML::ContentTypes::presentationProps =         "application/vnd.openxmlformats-officedocument.presentationml.presProps+xml";
 const char* const MSOOXML::ContentTypes::presentationViewProps =     "application/vnd.openxmlformats-officedocument.presentationml.viewProps+xml";
+const char* const MSOOXML::ContentTypes::presentationComments =      "application/vnd.openxmlformats-officedocument.presentationml.comments+xml";
 
 // spreadsheetml-specific content types
 const char* const MSOOXML::ContentTypes::spreadsheetDocument =        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml";
+const char* const MSOOXML::ContentTypes::spreadsheetMacroDocument =   "application/vnd.ms-excel.sheet.macroEnabled.main+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetPrinterSettings = "application/vnd.openxmlformats-officedocument.spreadsheetml.printerSettings";
 const char* const MSOOXML::ContentTypes::spreadsheetStyles =          "application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetWorksheet =       "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetCalcChain =       "application/vnd.openxmlformats-officedocument.spreadsheetml.calcChain+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetSharedStrings =   "application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetTemplate =        "application/vnd.openxmlformats-officedocument.spreadsheetml.template.main+xml";
+const char* const MSOOXML::ContentTypes::spreadsheetComments =        "application/vnd.openxmlformats-officedocument.spreadsheetml.comments+xml";
 
 //generic namespaces
 const char* const MSOOXML::Schemas::dublin_core =                                "http://purl.org/dc/elements/1.1/";
@@ -150,6 +157,9 @@ const char* const MSOOXML::Schemas::officeDocument::sharedTypes =
 // A.7 Custom XML Schema References
 const char* const MSOOXML::Schemas::schemaLibrary =                              "http://schemas.openxmlformats.org/schemaLibrary/2006/main";
 
+// Marks that the value has not been modified;
+const char* UNUSED = "UNUSED";
+
 using namespace MSOOXML;
 
 //-----------------------------------------
@@ -159,18 +169,9 @@ KoFilter::ConversionStatus Utils::loadAndParse(QIODevice* io, KoXmlDocument& doc
 {
     errorMessage.clear();
 
-    // We need to be able to see the space in <text:span> </text:span>, this is why
-    // we activate the "report-whitespace-only-CharData" feature.
-    // Unfortunately this leads to lots of whitespace text nodes in between real
-    // elements in the rest of the document, watch out for that.
-    QXmlInputSource source(io);
-    // Copied from QDomDocumentPrivate::setContent, to change the whitespace thing
-    QXmlSimpleReader reader;
-    KoOdfReadStore::setupXmlReader(reader, true /*namespaceProcessing*/);
-
     QString errorMsg;
     int errorLine, errorColumn;
-    bool ok = doc.setContent(&source, &reader, &errorMsg, &errorLine, &errorColumn);
+    bool ok = doc.setContent(io, true, &errorMsg, &errorLine, &errorColumn);
     if (!ok) {
         kError() << "Parsing error in " << fileName << ", aborting!" << endl
         << " In line: " << errorLine << ", column: " << errorColumn << endl
@@ -239,27 +240,125 @@ QIODevice* Utils::openDeviceForFile(const KZip* zip, QString& errorMessage, cons
     const KZipFileEntry* f = static_cast<const KZipFileEntry *>(entry);
     kDebug() << "Entry" << fileName << "has size" << f->size();
     status = KoFilter::OK;
-    return f->createDevice();
+    // There seem to be some problems with kde/zlib when trying to read
+    // multiple streams, this functionality is needed in the filter
+    // Until there's another solution for this, this avoids the problem
+    //return f->createDevice();
+    QBuffer *device = new QBuffer();
+    device->setData(f->data());
+    device->open(QIODevice::ReadOnly);
+    return device;
 }
 
 #define BLOCK_SIZE 4096
-KoFilter::ConversionStatus Utils::copyFile(const KZip* zip, QString& errorMessage,
-        const QString& sourceName,
-        KoStore *outputStore,
-        const QString& destinationName)
+static KoFilter::ConversionStatus copyOle(QString& errorMessage,
+                                    const QString sourceName, KoStore *outputStore,
+                                    const QString& destinationName, const KZip* zip)
 {
+    KoFilter::ConversionStatus status = KoFilter::OK;
+
+    QIODevice* inputDevice = Utils::openDeviceForFile(zip, errorMessage, sourceName, status);
+    inputDevice->open(QIODevice::ReadOnly);
+
+    POLE::Storage storage(inputDevice);
+    if (!storage.open()) {
+        kDebug(30513) << "Cannot open " << sourceName;
+        return KoFilter::WrongFormat;
+    }
+
+    std::list<std::string> lista = storage.entries();
+    std::string oleType = "Contents";
+
+    for (std::list<std::string>::iterator it = lista.begin(); it != lista.end(); ++it)  {
+        //qDebug() << "ENTRY " << (*it).c_str();
+        if (QString((*it).c_str()).contains("Ole10Native")) {
+            oleType = "Ole10Native";
+        }
+        else if (QString((*it).c_str()).contains("CONTENTS")) {
+            oleType = "CONTENTS";
+        }
+    }
+
+    POLE::Stream stream(&storage, oleType);
+    QByteArray array;
+    array.resize(stream.size());
+
+    unsigned long r = stream.read((unsigned char*)array.data(), stream.size());
+    if (r != stream.size()) {
+        kError(30513) << "Error while reading from stream";
+        return KoFilter::WrongFormat;
+    }
+
+    if (oleType == "Contents" || oleType == "Ole10Native") {
+     // Removing first 4 bytes which are the size
+        array = array.right(array.length() - 4);
+    }
+
+    // Uncomment to write any ole file for testing
+    //POLE::Stream streamTemp(&storage, "Ole");
+    //QByteArray arrayTemp;
+    //arrayTemp.resize(streamTemp.size());
+    //streamTemp.read((unsigned char*)arrayTemp.data(), streamTemp.size());
+    //QFile file("olething.ole");
+    //file.open(QIODevice::WriteOnly);
+    //QDataStream out(&file);
+    //out.writeRawData(arrayTemp.data(), arrayTemp.length());
+
+    kDebug() << "mode:" << outputStore->mode();
+    if (!outputStore->open(destinationName)) {
+        errorMessage = i18n("Could not open entry \"%1\" for writing.", destinationName);
+        return KoFilter::CreationError;
+    }
+
+    QByteArray array2;
+    while (true) {
+        array2 = array.left(BLOCK_SIZE);
+        array = array.right(array.size() - array2.size());
+        const qint64 in = array2.size();
+        if (in <= 0) {
+            break;
+        }
+        char *block = array2.data();
+        if (in != outputStore->write(block, in)) {
+            errorMessage = i18n("Could not write block");
+            status = KoFilter::CreationError;
+            break;
+        }
+    }
+    outputStore->close();
+    delete inputDevice;
+    inputDevice = 0;
+    return status;
+}
+#undef BLOCK_SIZE
+
+#define BLOCK_SIZE 4096
+KoFilter::ConversionStatus Utils::copyFile(const KZip* zip, QString& errorMessage,
+                                           const QString& sourceName, KoStore *outputStore,
+                                           const QString& destinationName, bool oleType)
+{
+    if (outputStore->hasFile(destinationName)) {
+        return KoFilter::OK;
+    }
+
     KoFilter::ConversionStatus status;
-    std::auto_ptr<QIODevice> inputDevice(Utils::openDeviceForFile(zip, errorMessage, sourceName, status));
+    if (oleType) {
+        status = copyOle(errorMessage, sourceName, outputStore, destinationName, zip);
+        return status;
+    }
+
+    std::auto_ptr<QIODevice> inputDevice = std::auto_ptr<QIODevice>(Utils::openDeviceForFile(zip, errorMessage, sourceName, status));
+
     if (!inputDevice.get()) {
         return status;
     }
+
     kDebug() << "mode:" << outputStore->mode();
     if (!outputStore->open(destinationName)) {
         errorMessage = i18n("Could not open entry \"%1\" for writing.", destinationName);
         return KoFilter::CreationError;
     }
     status = KoFilter::OK;
-//    QIODevice* outputDevice = outputStore->device();
     char block[BLOCK_SIZE];
     while (true) {
         const qint64 in = inputDevice->read(block, BLOCK_SIZE);
@@ -277,6 +376,23 @@ KoFilter::ConversionStatus Utils::copyFile(const KZip* zip, QString& errorMessag
 }
 #undef BLOCK_SIZE
 
+KoFilter::ConversionStatus Utils::imageSize(const KZip* zip, QString& errorMessage, const QString& sourceName,
+                                            QSize* size)
+{
+    Q_ASSERT(size);
+    KoFilter::ConversionStatus status;
+    std::auto_ptr<QIODevice> inputDevice(Utils::openDeviceForFile(zip, errorMessage, sourceName, status));
+    if (!inputDevice.get()) {
+        return status;
+    }
+    QImageReader r(inputDevice.get(), QFileInfo(sourceName).suffix().toLatin1());
+    if (!r.canRead())
+        return KoFilter::WrongFormat;
+    *size = r.size();
+    kDebug() << *size;
+    return KoFilter::OK;
+}
+
 KoFilter::ConversionStatus Utils::loadThumbnail(QImage& thumbnail, KZip* zip)
 {
 //! @todo
@@ -354,6 +470,27 @@ KoFilter::ConversionStatus Utils::loadContentTypes(
     return KoFilter::OK;
 }
 
+KoFilter::ConversionStatus Utils::loadDocumentProperties(const KoXmlDocument& appXML, QMap<QString, QVariant>& properties)
+{
+    KoXmlElement typesEl(appXML.documentElement());
+    KoXmlElement e, elem, element;
+    forEachElement(element, typesEl) {
+        QVariant v;
+        forEachElement(elem, element) {
+            if(elem.tagName() == "vector") {
+                QVariantList list;
+                forEachElement(e, elem)
+                    list.append(e.text());
+                v = list;
+            }
+        }
+        if(!v.isValid())
+            v = element.text();
+        properties[element.tagName()] = v;
+    }
+    return KoFilter::OK;
+}
+
 bool Utils::ST_Lang_to_languageAndCountry(const QString& value, QString& language, QString& country)
 {
     int indexForCountry =  value.indexOf('-');
@@ -704,7 +841,7 @@ QLocale Utils::localeForLangId( int langid )
     return QLocale( s_LangIdToLocaleMapping->value( langid ) );
 }
 
-double Utils::ST_Percentage_to_double(const QString& val, bool& ok)
+qreal Utils::ST_Percentage_to_double(const QString& val, bool& ok)
 {
     if (!val.endsWith('%')) {
         ok = false;
@@ -715,16 +852,16 @@ double Utils::ST_Percentage_to_double(const QString& val, bool& ok)
     return result.toDouble(&ok);
 }
 
-double Utils::ST_Percentage_withMsooxmlFix_to_double(const QString& val, bool& ok)
+qreal Utils::ST_Percentage_withMsooxmlFix_to_double(const QString& val, bool& ok)
 {
-    const double result = ST_Percentage_to_double(val, ok);
+    const qreal result = ST_Percentage_to_double(val, ok);
     if (ok)
         return result;
     // MSOOXML fix: the format is int({ST_Percentage}*1000)
     const int resultInt = val.toInt(&ok);
     if (!ok)
         return 0.0;
-    return double(resultInt) / 1000.0;
+    return qreal(resultInt) / 1000.0;
 }
 
 QColor Utils::colorForLuminance(const QColor& color, const DoubleModifier& modulation, const DoubleModifier& offset)
@@ -753,7 +890,7 @@ class ST_PlaceholderType_to_ODFMapping : public QHash<QByteArray, QByteArray>
 {
 public:
     ST_PlaceholderType_to_ODFMapping() {
-        insert("body", "text");
+        insert("body", "outline");
         insert("chart", "chart");
         insert("clipArt", "graphic");
         insert("ctrTitle", "title");
@@ -775,13 +912,13 @@ public:
     }
 };
 
-QByteArray Utils::ST_PlaceholderType_to_ODF(const QByteArray& ecmaType)
+QString Utils::ST_PlaceholderType_to_ODF(const QString& ecmaType)
 {
     K_GLOBAL_STATIC(ST_PlaceholderType_to_ODFMapping, s_ST_PlaceholderType_to_ODF)
-    QHash<QByteArray, QByteArray>::ConstIterator it(s_ST_PlaceholderType_to_ODF->constFind(ecmaType));
+    QHash<QByteArray, QByteArray>::ConstIterator it(s_ST_PlaceholderType_to_ODF->constFind(ecmaType.toLatin1()));
     if (it == s_ST_PlaceholderType_to_ODF->constEnd())
-        return "outline";
-    return it.value();
+        return QLatin1String("text");
+    return QString(it.value());
 }
 
 //! Mapping for handling u element, used in setupUnderLineStyle()
@@ -820,6 +957,10 @@ public:
                new UnderlineStyle(KoCharacterStyle::SolidLine, KoCharacterStyle::DoubleLine,
                                   KoCharacterStyle::AutoLineWeight)
               );
+        insert("dbl",
+               new UnderlineStyle(KoCharacterStyle::SolidLine, KoCharacterStyle::DoubleLine,
+                                  KoCharacterStyle::AutoLineWeight)
+              );
         insert("words",
                new UnderlineStyle(KoCharacterStyle::SolidLine, KoCharacterStyle::SingleLine,
                                   KoCharacterStyle::AutoLineWeight, KoCharacterStyle::SkipWhiteSpaceLineMode)
@@ -856,6 +997,10 @@ public:
                new UnderlineStyle(KoCharacterStyle::WaveLine, KoCharacterStyle::DoubleLine,
                                   KoCharacterStyle::AutoLineWeight)
               );
+        insert("wavyDbl",
+               new UnderlineStyle(KoCharacterStyle::WaveLine, KoCharacterStyle::DoubleLine,
+                                  KoCharacterStyle::AutoLineWeight)
+              );
         insert("wavyHeavy",
                new UnderlineStyle(KoCharacterStyle::WaveLine, KoCharacterStyle::SingleLine,
                                   KoCharacterStyle::BoldLineWeight)
@@ -896,6 +1041,21 @@ public:
     }
 };
 
+void Utils::rotateString(const qreal rotation, const qreal width, const qreal height, qreal& angle, qreal& xDiff, qreal& yDiff,
+    bool flipH, bool flipV)
+{
+    angle = -(qreal)rotation * ((qreal)(M_PI) / (qreal)180.0)/ (qreal)60000.0;
+    // Angle seems to be negative if one of the flips is active
+    if ((flipH && !flipV) || (!flipH && flipV)) {
+        angle = -angle;
+    }
+    //position change is calculated based on the fact that center point stays in the same location
+    // Width/2 = Xnew + cos(angle)*Width/2 - sin(angle)*Height/2
+    // Height/2 = Ynew + sin(angle)*Width/2 + cos(angle)*Height/2
+    xDiff = width/2 - cos(-angle)*width/2 + sin(-angle)*height/2;
+    yDiff = height/2 - sin(-angle)*width/2 - cos(-angle)*height/2;
+}
+
 void Utils::setupUnderLineStyle(const QString& msooxmlName, KoCharacterStyle* textStyleProperties)
 {
     K_GLOBAL_STATIC(UnderlineStylesHash, s_underLineStyles)
@@ -964,6 +1124,14 @@ QString Utils::columnName(uint column)
     return str;
 }
 
+void Utils::splitPathAndFile(const QString& pathAndFile, QString* path, QString* file)
+{
+    Q_ASSERT(path);
+    Q_ASSERT(file);
+    *path = pathAndFile.left(pathAndFile.lastIndexOf('/'));
+    *file = pathAndFile.mid(pathAndFile.lastIndexOf('/') + 1);
+}
+
 // <units> -------------------
 
 QString Utils::EMU_to_ODF(const QString& twipValue)
@@ -989,8 +1157,7 @@ QString Utils::TWIP_to_ODF(const QString& twipValue)
         return QString();
     if (twip == 0)
         return QLatin1String("0cm");
-    QString res;
-    return res.sprintf("%3.3fcm", TWIP_TO_CM(double(twip)));
+    return cmString(TWIP_TO_CM(qreal(twip)));
 }
 
 QString Utils::ST_EighthPointMeasure_to_ODF(const QString& value)
@@ -1050,7 +1217,7 @@ static bool isUnitAcceptable(const QString& unit)
 }
 
 static QString ST_TwipsMeasure_to_ODF_with_unit(const QString& value,
-                                                double (*convertFromTwips)(double), const char* unit)
+                                                qreal (*convertFromTwips)(qreal), const char* unit)
 {
     if (value.isEmpty())
         return QString();
@@ -1103,13 +1270,334 @@ MSOOXML_EXPORT QString Utils::ST_PositiveUniversalMeasure_to_ODF(const QString&
     return value; // the original is OK
 }
 
+MSOOXML_EXPORT QString Utils::rgbColor(QString color)
+{
+    // It is possible that color is eg #abcdef [adddd], this removes the extra end
+    if (color.indexOf(' ') > 0) {
+        color = color.left(color.indexOf(' '));
+    }
+
+    QString newColor;
+    if (color == "red") {
+        newColor = "#ff0000";
+    }
+    else if (color == "green") {
+        newColor = "#00ff00";
+    }
+    else if (color == "blue") {
+        newColor = "#0000ff";
+    }
+    else if (color == "yellow") {
+        newColor = "#ffff00";
+    }
+    else {
+        newColor = color;
+    }
+
+    return newColor;
+}
+
 MSOOXML_EXPORT QString Utils::ST_PositiveUniversalMeasure_to_cm(const QString& value)
 {
     QString v(ST_PositiveUniversalMeasure_to_ODF(value));
     if (v.isEmpty())
         return QString();
-    QString res;
-    return res.sprintf("%3.3fcm", POINT_TO_CM(KoUnit::parseValue(v)));
+    return cmString(POINT_TO_CM(KoUnit::parseValue(v)));
 }
 
 // </units> -------------------
+
+MSOOXML_EXPORT Utils::ParagraphBulletProperties::ParagraphBulletProperties() :
+    m_type(ParagraphBulletProperties::DefaultType), m_startValue(UNUSED), m_bulletFont(UNUSED),
+    m_bulletChar(UNUSED), m_numFormat(UNUSED), m_suffix(UNUSED), m_align(UNUSED),
+    m_indent(UNUSED), m_picturePath(UNUSED), m_bulletColor(UNUSED), m_bulletRelativeSize(UNUSED)
+{
+}
+
+MSOOXML_EXPORT bool Utils::ParagraphBulletProperties::isEmpty() const
+{
+    if (m_type == ParagraphBulletProperties::DefaultType) {
+        return true;
+    }
+    return false;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::clear()
+{
+    m_startValue = UNUSED;
+    m_type = ParagraphBulletProperties::DefaultType;
+    m_bulletFont = UNUSED;
+    m_bulletChar = UNUSED;
+    m_numFormat = UNUSED;
+    m_suffix = UNUSED;
+    m_align = UNUSED;
+    m_indent = UNUSED;
+    m_picturePath = UNUSED;
+    m_bulletSize = QSize();
+    m_bulletColor = UNUSED;
+    m_bulletRelativeSize = UNUSED;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setAlign(const QString& align)
+{
+    m_align = align;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setBulletRelativeSize(int size)
+{
+    m_bulletRelativeSize = QString("%1").arg(size);
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setBulletChar(const QString& bulletChar)
+{
+    m_bulletChar = bulletChar;
+    m_type = ParagraphBulletProperties::BulletType;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setStartValue(const QString& value)
+{
+    m_startValue = value;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setIndent(const qreal indent)
+{
+    m_indent = QString("%1").arg(indent);
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setSuffix(const QString& suffixChar)
+{
+    m_suffix = suffixChar;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setNumFormat(const QString& numFormat)
+{
+    m_numFormat = numFormat;
+    m_type = ParagraphBulletProperties::NumberType;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setPicturePath(const QString& picturePath)
+{
+    m_picturePath = picturePath;
+    m_type = ParagraphBulletProperties::PictureType;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setBulletSize(const QSize& size)
+{
+    m_bulletSize = size;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setBulletFont(const QString& font)
+{
+    m_bulletFont = font;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setBulletColor(const QString& bulletColor)
+{
+    m_bulletColor = bulletColor;
+}
+
+MSOOXML_EXPORT QString Utils::ParagraphBulletProperties::bulletColor() const
+{
+    return m_bulletColor;
+}
+
+MSOOXML_EXPORT QString Utils::ParagraphBulletProperties::bulletChar() const
+{
+    return m_bulletChar;
+}
+
+MSOOXML_EXPORT QString Utils::ParagraphBulletProperties::bulletFont() const
+{
+    return m_bulletFont;
+}
+
+MSOOXML_EXPORT QString Utils::ParagraphBulletProperties::bulletRelativeSize() const
+{
+    return m_bulletRelativeSize;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::addInheritedValues(const ParagraphBulletProperties& properties)
+{
+    // This function is intented for helping to inherit some values from other properties
+    if (properties.m_type != ParagraphBulletProperties::DefaultType) {
+        m_type = properties.m_type;
+    }
+    if (properties.m_bulletFont != UNUSED) {
+        m_bulletFont = properties.m_bulletFont;
+    }
+    if (properties.m_bulletChar != UNUSED) {
+        m_bulletChar = properties.m_bulletChar;
+    }
+    if (properties.m_numFormat != UNUSED) {
+        m_numFormat = properties.m_numFormat;
+    }
+    if (properties.m_suffix != UNUSED) {
+        m_suffix = properties.m_suffix;
+    }
+    if (properties.m_align != UNUSED) {
+        m_align = properties.m_align;
+    }
+    if (properties.m_indent != UNUSED) {
+        m_indent = properties.m_indent;
+    }
+    if (properties.m_picturePath != UNUSED) {
+        m_picturePath = properties.m_picturePath;
+    }
+    if (properties.m_bulletColor != UNUSED) {
+        m_bulletColor = properties.m_bulletColor;
+    }
+    if (properties.m_bulletRelativeSize != UNUSED) {
+        m_bulletRelativeSize = properties.m_bulletRelativeSize;
+    }
+    if (!properties.m_bulletSize.isEmpty()) {
+        m_bulletSize = properties.m_bulletSize;
+    }
+}
+
+MSOOXML_EXPORT QString Utils::ParagraphBulletProperties::convertToListProperties() const
+{
+    QString returnValue;
+    QString ending;
+    if (m_type == ParagraphBulletProperties::NumberType) {
+        returnValue = QString("<text:list-level-style-number text:level=\"%1\" ").arg(m_level);
+        returnValue += QString("style:num-suffix=\"%1\" style:num-format=\"%2\" ").arg(m_suffix).arg(m_numFormat);
+        if (m_startValue != UNUSED) {
+            returnValue += QString("text:start-value=\"%1\" ").arg(m_startValue);
+        }
+        ending = "</text:list-level-style-number>";
+    }
+    else if (m_type == ParagraphBulletProperties::PictureType) {
+        returnValue = QString("<text:list-level-style-image text:level=\"%1\" ").arg(m_level);
+        returnValue += QString("xlink:href=\"%1\" ").arg(m_picturePath);
+        returnValue += "xlink:type=\"simple\" xlink:show=\"embed\" xlink:actuate=\"onLoad\" ";
+        ending = "</text:list-level-style-image>";
+    }
+    else {
+        returnValue = QString("<text:list-level-style-bullet text:level=\"%1\" ").arg(m_level);
+        if ((m_bulletFont.startsWith("Wingdings") || m_bulletFont.startsWith("Symbol")) && m_bulletChar != "") {
+            // In case of wingdings we replace with 'best guess'
+            returnValue += QString("text:bullet-char=\"%1\" ").arg("-");
+        }
+        else {
+            if (m_bulletChar == UNUSED) {
+                returnValue += QString("text:bullet-char=\"\" ");
+            }
+            else {
+                returnValue += QString("text:bullet-char=\"%1\" ").arg(m_bulletChar);
+            }
+        }
+        ending = "</text:list-level-style-bullet>";
+    }
+    if (m_align != "UNUSED") {
+        returnValue += QString("fo:text-align=\"%1\" ").arg(m_align);
+    }
+    returnValue += ">";
+
+    returnValue += "<style:list-level-properties ";
+
+    if (m_indent != "UNUSED") {
+        returnValue += QString("text:space-before=\"%1pt\" ").arg(m_indent);
+    }
+
+    if (!m_bulletSize.isEmpty()) {
+        returnValue += QString("fo:width=\"%1\" fo:height=\"%2\" ").arg(MSOOXML::Utils::cmString(POINT_TO_CM(m_bulletSize.width()))).
+            arg(MSOOXML::Utils::cmString(POINT_TO_CM(m_bulletSize.height())));
+    }
+
+    returnValue += "/>";
+
+    returnValue += "<style:text-properties ";
+
+    if (m_bulletColor != "UNUSED") {
+    	returnValue += QString("fo:color=\"%1\" ").arg(m_bulletColor);
+    }
+
+    returnValue += "/>";
+
+    returnValue += ending;
+
+    return returnValue;
+}
+
+MSOOXML_EXPORT void Utils::copyPropertiesFromStyle(const KoGenStyle& sourceStyle, KoGenStyle& targetStyle, KoGenStyle::PropertyType type)
+{
+    if (sourceStyle.isEmpty()) {
+        return;
+    }
+
+    QBuffer buffer;
+    buffer.open(QIODevice::WriteOnly);
+    KoXmlWriter tempWriter(&buffer);
+    sourceStyle.writeStyleProperties(&tempWriter, type);
+
+    QString content = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
+
+    // We have to add the properties in a loop
+    // This works as long as text-properties don't have children
+    // Currenty KoGenStyle doesn't support adding this in other ways.
+
+    int separatorLocation = content.indexOf(' ');
+    content = content.right(content.size() - (separatorLocation + 1));
+    separatorLocation = content.indexOf(' ');
+    if (separatorLocation < 0) {
+        separatorLocation = content.indexOf('/');
+    }
+    while (separatorLocation > 0) {
+        int equalSignLocation = content.indexOf('=');
+        if (equalSignLocation < 0) {
+            break;
+        }
+        QString propertyName = content.left(equalSignLocation);
+        // Removing equal and one quota
+        content = content.right(content.size() - (equalSignLocation) - 2);
+        separatorLocation = content.indexOf('\"');
+        if (separatorLocation < 0) {
+            separatorLocation = content.indexOf('/');
+        }
+        QString propertyValue = content.left(separatorLocation);
+        content = content.right(content.size() - (separatorLocation + 2));
+        targetStyle.addProperty(propertyName, propertyValue, type);
+    }
+}
+
+MSOOXML_EXPORT void Utils::modifyColor(QColor& color, qreal tint, qreal shade, qreal satMod)
+{
+    int red = color.red();
+    int green = color.green();
+    int blue = color.blue();
+
+    if (tint > 0) {
+        red = tint * red + (1 - tint) * 255;
+        green = tint * green + (1 - tint) * 255;
+        blue = tint * blue + (1 - tint) * 255;
+    }
+    if (shade > 0) {
+        red = shade * red;
+        green = shade * green;
+        blue = shade * blue;
+    }
+
+    // FIXME: This calculation for sure is incorrect,
+    // According to MS forums, RGB should first be converted to linear RGB
+    // Then to HSL and then multiply saturation value by satMod
+    // SatMod can be for example 3.5 so converting RGB -> HSL is not an option
+    // ADD INFO: MS document does not say that when calculating TINT and SHADE
+    // That whether one should use normal RGB or linear RGB, check it!
+    if (satMod > 0) {
+        red = red * satMod;
+        green = green * satMod;
+        blue = blue * satMod;
+        if (red > 255) {
+            red = 255;
+        }
+        if (green > 255) {
+            green = 255;
+        }
+        if (blue > 255) {
+            blue = 255;
+        }
+    }
+
+    color = QColor(red, green, blue);
+}
+
