@@ -77,9 +77,6 @@ private:
 
     QCheckBox* m_showStatusBar;
 
-    KIntNumInput* m_copyOffset;
-    int m_oldCopyOffset;
-
     KIntNumInput* m_dockerFontSize;
     int m_oldDockerFontSize;
 
