@@ -282,6 +282,12 @@ public:
     void addPropertyCommandGroup(PropertyCommandGroup *commandGroup,
                                  AddCommandOption addOption, uint idOfPropertyCommand = 0);
 
+    /*! @return Command that being executed through addCommand() or 0 if addCommand()
+     is not currently being executed.
+     Currently used by WidgetTreeWidget to check if tab page adding was
+     the original reason for adding widget. */
+    const Command* executingCommand() const;
+
     /*! \return tabstops list. It contains all the widgets that can have focus 
      (i.e. no labels, etc.) in the order of the tabs.*/
     ObjectTreeList* tabStops();
@@ -314,10 +320,19 @@ public:
      child widgets should ordered by parent tab's order. */
     void autoAssignTabStops();
 
+    //! Specifies target of context menu. Used in createContextMenu().
+    enum ContextMenuTarget
+    {
+        FormContextMenuTarget, //!< Context menu should be displayed on the form
+                               //!< with offset specified by menuPos argument.
+        WidgetTreeContextMenuTarget //!< Context menu should be displayed on the widget tree
+                                    //!< area with offset specified by menuPos argument.
+    };
+
     /*! This function creates and displays the context menu corresponding to the widget \a w.
         The menu item are disabled if necessary, and
         the widget specific part is added (menu from the factory and buddy selection). */
-    void createContextMenu(QWidget *w, Container *container, const QPoint& menuPos); //bool popupAtCursor = true);
+    void createContextMenu(QWidget *w, Container *container, const QPoint& menuPos, ContextMenuTarget target);
 
     //! @return true if snapping widgets to grid is enabled.
     bool isSnapWidgetsToGridEnabled() const;
