@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2004 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -23,7 +23,8 @@
 #include <qobject.h>
 #include <qstring.h>
 
-#include <sys/stat.h>
+//#include <sys/stat.h>
+#include <kde_file.h>
 
 #include <kexiutils/tristate.h>
 
@@ -32,28 +33,28 @@ class KProgressDialog;
 
 class SQLite2ToSQLite3Migration : public QObject
 {
-	Q_OBJECT
-	public:
-		SQLite2ToSQLite3Migration(const QString& filePath);
-		~SQLite2ToSQLite3Migration();
+    Q_OBJECT
+public:
+    SQLite2ToSQLite3Migration(const QString& filePath);
+    ~SQLite2ToSQLite3Migration();
 
-		tristate run();
+    tristate run();
 
-	public slots:
-		void processExited(KProcess*);
-		void receivedStderr(KProcess*,char*,int);
-		void cancelClicked();
+public slots:
+    void processExited(KProcess*);
+    void receivedStderr(KProcess*, char*, int);
+    void cancelClicked();
 
-	protected:
-	QString m_filePath;
-	KProcess *m_process;
-	KProgressDialog* m_dlg;
-	
-	struct stat m_st;
-	bool m_restoreStat : 1;
-	bool m_run : 1;
+protected:
+    QString m_filePath;
+    KProcess *m_process;
+    KProgressDialog* m_dlg;
 
-	tristate result;
+    struct stat m_st;
+    bool m_restoreStat;
+    bool m_run;
+
+    tristate result;
 };
 
 #endif
