@@ -296,6 +296,18 @@ void KexiDBImageBox::setScaledContents(bool set)
     repaint();
 }
 
+bool KexiDBImageBox::smoothTransformation() const
+{
+    return m_smoothTransformation;
+}
+
+void KexiDBImageBox::setSmoothTransformation(bool set)
+{
+    m_smoothTransformation = set;
+    m_currentScaledPixmap = QPixmap();
+    repaint();
+}
+
 void KexiDBImageBox::setKeepAspectRatio(bool set)
 {
     m_keepAspectRatio = set;
