@@ -26,7 +26,7 @@ class KexiRelationDesignFactory : public KoShapeFactoryBase {
     public:
         KexiRelationDesignFactory( QObject *parent );
         virtual KoShape *createDefaultShape(KoResourceManager *documentResources = 0) const;
-        virtual bool supports(const KoXmlElement & e) const;
+        virtual bool supports(const KoXmlElement & e, KoShapeLoadingContext &context) const;
 };
 
 #endif // KEXIRELATIONDESIGNFACTORY_H
