@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2005 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2005 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,19 +20,22 @@
 #ifndef KEXIPRJTYPESELECTOR_H
 #define KEXIPRJTYPESELECTOR_H
 
-#include "kexiprjtypeselectorbase.h"
+#include <kexi_export.h>
+#include <QWidget>
+#include "ui_kexiprjtypeselector.h"
 
 //! @short A simple widget with radio buttons with "show file/server-based projects" options
-class KEXIEXTWIDGETS_EXPORT KexiPrjTypeSelector : public KexiPrjTypeSelectorBase
+class KEXIEXTWIDGETS_EXPORT KexiPrjTypeSelector
+            : public QWidget, public Ui_KexiPrjTypeSelector
 {
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-		KexiPrjTypeSelector( QWidget* parent = 0 );
-		~KexiPrjTypeSelector();
+public:
+    KexiPrjTypeSelector(QWidget* parent = 0);
+    ~KexiPrjTypeSelector();
 
-	protected slots:
-		virtual void languageChange() { KexiPrjTypeSelectorBase::languageChange(); } 
+public slots:
+    void slotSelectionChanged(int id);
 };
 
 #endif // KEXIPRJTYPESELECTOR_H
