@@ -351,6 +351,8 @@ KexiTableView::~KexiTableView()
             data->deleteLater();
     }
     delete d;
+    m_horizontalHeader->setStyle(0);
+    delete m_horizontalHeader;
 }
 
 void KexiTableView::clearVariables()
@@ -665,11 +667,13 @@ inline void KexiTableView::paintRow(KexiDB::RecordData *record,
 
 void KexiTableView::drawContents(QPainter *p, int cx, int cy, int cw, int ch)
 {
+#ifdef KEXITABLEVIEW_DEBUG
+    kDebug() << "disable" << d->disableDrawContents << "cx" << cx << "cy" << cy << "cw" << cw << "ch" << ch;
+#endif
+
     if (d->disableDrawContents)
         return;
 
-//kDebug() << "cx" << cx << "cy" << cy << "cw" << cw << "ch" << ch;
-
     bool paintOnlyInsertRow = false;
     bool inserting = isInsertingEnabled();
     bool plus1row = false; //true if we should show 'inserting' row at the end
@@ -679,7 +683,9 @@ void KexiTableView::drawContents(QPainter *p, int cx, int cy, int cw, int ch)
     int rowlast = rowAt(cy + ch - 1);
     if (rowfirst == -1 && (cy / d->rowHeight) == (int)rows()) {
         // make the insert row paint too when requested
+#ifdef KEXITABLEVIEW_DEBUG
         kDebug() << "rowfirst == -1 && (cy / d->rowHeight) == (int)rows()";
+#endif
         rowfirst = m_data->count();
         rowlast = rowfirst;
         paintOnlyInsertRow = true;
@@ -910,6 +916,8 @@ void KexiTableView::paintCell(QPainter* p, KexiDB::RecordData *record, int col,
         //get visible lookup value if available
         getVisibleLookupValue(cellValue, edit, record, tvcol);
 
+/*kDebug() << "edit->setupContents()" << (m_currentItem == record && col == m_curCol)
+        << cellValue << txt << align << x << y_offset << w << h;*/
         edit->setupContents(p, m_currentItem == record && col == m_curCol,
                             cellValue, txt, align, x, y_offset, w, h);
     }
@@ -1016,8 +1024,11 @@ void KexiTableView::paintEmptyArea(QPainter *p, int cx, int cy, int cw, int ch)
 
     // Subtract the table from it
 // reg = reg.subtract( QRect( QPoint( 0, 0 ), ts-QSize(0,m_navPanel->isVisible() ? m_navPanel->height() : 0) ) );
+    const int scrollBarHeight = horizontalScrollBar()->isVisible()
+        ? horizontalScrollBar()->sizeHint().height() : 0;
+    
     reg = reg.subtract(QRect(QPoint(0, 0), ts
-                             - QSize(0, qMax((m_navPanel ? m_navPanel->height() : 0), horizontalScrollBar()->sizeHint().height())
+                             - QSize(0, qMax(((m_navPanel && m_navPanel->isVisible()) ? m_navPanel->height() : 0), scrollBarHeight)
                                      /*- (horizontalScrollBar()->isVisible() ? horizontalScrollBar()->sizeHint().height() / 2 : 0)
                                      + (horizontalScrollBar()->isVisible() ? 0 :
                                         d->internal_bottomMargin
@@ -2013,7 +2024,9 @@ QSize KexiTableView::tableSize() const
         );
 
 //  kDebug() << rows()-1 <<" "<< (isInsertingEnabled()?1:0) <<" "<< (m_rowEditing?1:0) << " " <<  s;
+#ifdef KEXITABLEVIEW_DEBUG
 kDebug() << s;
+#endif
         return s;
 //   +horizontalScrollBar()->sizeHint().height() + margin() );
     }
@@ -2046,7 +2059,9 @@ void KexiTableView::ensureCellVisible(int row, int col/*=-1*/)
     }
 
     QPoint pcenter = r.center();
+#ifdef KEXITABLEVIEW_DEBUG
     kDebug() << pcenter.x() << pcenter.y() << (r.width() / 2)  << (r.height() / 2);
+#endif
     ensureVisible(pcenter.x(), pcenter.y(), r.width() / 2, r.height() / 2);
 //    ensureVisible(r.x(), r.y(), 0, 0);
 //    ensureVisible(r.bottomRight().x(), r.bottomRight().y(), 0, 0);
@@ -2361,8 +2376,10 @@ void KexiTableView::triggerUpdate()
 
 void KexiTableView::setHBarGeometry(QScrollBar & hbar, int x, int y, int w, int h)
 {
+#ifdef KEXITABLEVIEW_DEBUG
     /*todo*/
     kDebug();
+#endif
     if (d->appearance.navigatorEnabled) {
         m_navPanel->setHBarGeometry(hbar, x, y, w, h);
     } else {
