@@ -114,10 +114,12 @@ int Project::type() const { return Node::Type_Project; }
 void Project::generateUniqueNodeIds()
 {
     foreach ( Node *n, nodeIdDict ) {
+        kDebug()<<n->name()<<"old"<<n->id();
         QString uid = uniqueNodeId();
         nodeIdDict.remove( n->id() );
         n->setId( uid );
         nodeIdDict[ uid ] = n;
+        kDebug()<<n->name()<<"new"<<n->id();
     }
 }
 
@@ -158,6 +160,7 @@ void Project::calculate( const DateTime &dt )
         kError() << "No current schedule to calculate";
         return ;
     }
+    stopcalculation = false;
     KLocale *locale = KGlobal::locale();
     DateTime time = dt.isValid() ? dt : DateTime( KDateTime::currentLocalDateTime() );
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
@@ -168,17 +171,17 @@ void Project::calculate( const DateTime &dt )
         initiateCalculation( *cs );
         initiateCalculationLists( *cs ); // must be after initiateCalculation() !!
         propagateEarliestStart( time );
-        // Calculate lateFinish from time. If a task has started, remaingEffort is used.
+        // Calculate lateFinish from time. If a task has started, remainingEffort is used.
         cs->setPhaseName( 1, i18nc( "Schedule project forward", "Forward" ) );
         cs->logInfo( i18n( "Calculate finish" ), 1 );
         cs->lateFinish = calculateForward( estType );
         cs->lateFinish = checkEndConstraints( cs->lateFinish );
         propagateLatestFinish( cs->lateFinish );
-        // Calculate earlyFinish. If a task has started, remaingEffort is used.
+        // Calculate earlyFinish. If a task has started, remainingEffort is used.
         cs->setPhaseName( 2, i18nc( "Schedule project backward","Backward" ) );
         cs->logInfo( i18n( "Calculate start" ), 2 );
-        cs->calculateBackward( estType );
-        // Schedule. If a task has started, remaingEffort is used and appointments are copied from parent
+        calculateBackward( estType );
+        // Schedule. If a task has started, remainingEffort is used and appointments are copied from parent
         cs->setPhaseName( 3, i18n( "Schedule" ) );
         cs->logInfo( i18n( "Schedule tasks forward" ), 3 );
         cs->endTime = scheduleForward( cs->startTime, estType );
@@ -215,12 +218,10 @@ void Project::calculate( ScheduleManager &sm )
         }
     }
     int maxprogress = nodes * 3;
-    //kDebug();
     if ( sm.recalculate() ) {
         emit maxProgress( maxprogress );
+        sm.setMaxProgress( maxprogress );
         incProgress();
-        sm.setCalculateAll( false );
-        sm.createSchedules();
         if ( sm.parentManager() ) {
             sm.expected()->startTime = sm.parentManager()->expected()->startTime;
             sm.expected()->earlyStart = sm.parentManager()->expected()->earlyStart;
@@ -235,7 +236,7 @@ void Project::calculate( ScheduleManager &sm )
             maxprogress += nodes * 3;
         }
         emit maxProgress( maxprogress );
-        sm.createSchedules();
+        sm.setMaxProgress( maxprogress );
         calculate( sm.expected() );
         emit scheduleChanged( sm.expected() );
         if ( sm.optimistic() ) {
@@ -270,6 +271,7 @@ void Project::calculate()
         kError() << "No current schedule to calculate";
         return ;
     }
+    stopcalculation = false;
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
     bool backwards = false;
     if ( cs->manager() ) {
@@ -278,6 +280,7 @@ void Project::calculate()
     KLocale *locale = KGlobal::locale();
     Estimate::Use estType = ( Estimate::Use ) cs->type();
     if ( type() == Type_Project ) {
+        QTime timer; timer.start();
         initiateCalculation( *cs );
         initiateCalculationLists( *cs ); // must be after initiateCalculation() !!
         if ( ! backwards ) {
@@ -290,11 +293,12 @@ void Project::calculate()
             cs->setPhaseName( 1, i18nc( "Schedule project forward", "Forward" ) );
             cs->logInfo( i18n( "Calculate late finish" ), 1 );
             cs->lateFinish = calculateForward( estType );
-            cs->lateFinish = checkEndConstraints( cs->lateFinish );
+//            cs->lateFinish = checkEndConstraints( cs->lateFinish );
+            cs->logInfo( i18n( "Late finish calculated: %1", locale->formatDateTime( cs->lateFinish ) ), 1 );
             propagateLatestFinish( cs->lateFinish );
             cs->setPhaseName( 2, i18nc( "Schedule project backward", "Backward" ) );
             cs->logInfo( i18n( "Calculate early start" ), 2 );
-            cs->calculateBackward( estType );
+            calculateBackward( estType );
             cs->setPhaseName( 3, i18n( "Schedule" ) );
             cs->logInfo( i18n( "Schedule tasks forward" ), 3 );
             cs->endTime = scheduleForward( cs->startTime, estType );
@@ -317,11 +321,13 @@ void Project::calculate()
             cs->setPhaseName( 1, i18nc( "Schedule project backward", "Backward" ) );
             cs->logInfo( i18n( "Calculate early start" ), 1 );
             cs->earlyStart = calculateBackward( estType );
-            cs->earlyStart = checkStartConstraints( cs->earlyStart );
+//            cs->earlyStart = checkStartConstraints( cs->earlyStart );
+            cs->logInfo( i18n( "Early start calculated: %1", locale->formatDateTime( cs->earlyStart ) ), 1 );
             propagateEarliestStart( cs->earlyStart );
             cs->setPhaseName( 2, i18nc( "Schedule project forward", "Forward" ) );
             cs->logInfo( i18n( "Calculate late finish" ), 2 );
-            cs->lateFinish = cs->calculateForward( estType );
+            cs->lateFinish = qMax( m_constraintEndTime, calculateForward( estType ) );
+            cs->logInfo( i18n( "Late finish calculated: %1", locale->formatDateTime( cs->lateFinish ) ), 2 );
             cs->setPhaseName( 3, i18n( "Schedule" ) );
             cs->logInfo( i18n( "Schedule tasks backward" ), 3 );
             cs->startTime = scheduleBackward( cs->lateFinish, estType );
@@ -350,6 +356,7 @@ void Project::calculate()
             }
             calcCriticalPath( true );
         }
+        cs->logInfo( i18n( "Calculation took: %1", locale->formatDuration( timer.elapsed() ) ) );
         //makeAppointments();
         calcResourceOverbooked();
         cs->notScheduled = false;
@@ -546,78 +553,199 @@ DateTime Project::checkEndConstraints( const DateTime &dt ) const
     return t;
 }
 
+void Project::tasksForward()
+{
+    m_hardConstraints.clear();
+    m_softConstraints.clear();
+    m_terminalNodes.clear();
+    foreach ( Task *t, allTasks() ) {
+        switch ( t->constraint() ) {
+            case Node::MustStartOn:
+            case Node::MustFinishOn:
+            case Node::FixedInterval:
+                m_hardConstraints.append( t );
+                break;
+            case Node::StartNotEarlier:
+            case Node::FinishNotLater:
+                m_softConstraints.append( t );
+                break;
+            default:
+                if ( t->isEndNode() ) {
+                    m_terminalNodes.append( t );
+                }
+                break;
+        }
+    }
+}
+
+void Project::tasksBackward()
+{
+    m_hardConstraints.clear();
+    m_softConstraints.clear();
+    m_terminalNodes.clear();
+    foreach ( Task *t, allTasks() ) {
+        switch ( t->constraint() ) {
+            case Node::MustStartOn:
+            case Node::MustFinishOn:
+            case Node::FixedInterval:
+                m_hardConstraints.append( t );
+                break;
+            case Node::StartNotEarlier:
+            case Node::FinishNotLater:
+                m_softConstraints.append( t );
+                break;
+            default:
+                if ( t->isEndNode() ) {
+                    m_terminalNodes.append( t );
+                }
+                break;
+        }
+    }
+}
+
 DateTime Project::calculateForward( int use )
 {
     //kDebug()<<m_name;
+    DateTime finish;
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
     if ( cs == 0 ) {
-        return DateTime();
+        return finish;
     }
     if ( type() == Node::Type_Project ) {
-        // Do constrained first
-        foreach ( Node *n, cs->hardConstraints() ) {
-            n->calculateEarlyFinish( use );
+        m_visitedForward = true;
+        if ( ! m_visitedBackward ) {
+            // setup tasks
+            tasksForward();
+            // Do all hard constrained first
+            foreach ( Node *n, m_hardConstraints ) {
+                cs->logDebug( "Calculate task with hard constraint:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateEarlyFinish( use ); // do not do predeccessors
+                if ( time > finish ) {
+                    finish = time;
+                }
+            }
+            // do the predeccessors
+            foreach ( Node *n, m_hardConstraints ) {
+                cs->logDebug( "Calculate predeccessors to hard constrained task:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateForward( use );
+                if ( time > finish ) {
+                    finish = time;
+                }
+            }
+            // now try to schedule soft constrained *with* predeccessors
+            foreach ( Node *n, m_softConstraints ) {
+                cs->logDebug( "Calculate task with soft constraint:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateForward( use );
+                if ( time > finish ) {
+                    finish = time;
+                }
+            }
+            // and then the rest using the end nodes to calculate everything (remaining)
+            foreach ( Task *n, m_terminalNodes ) {
+                cs->logDebug( "Calculate using end task:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateForward( use );
+                if ( time > finish ) {
+                    finish = time;
+                }
+            }
+        } else {
+            // tasks have been calculated backwards in this order
+            foreach ( Node *n, cs->backwardNodes() ) {
+                DateTime time = n->calculateForward( use );
+                if ( time > finish ) {
+                    finish = time;
+                }
+            }
         }
-        // Follow *parent* relations back and
-        // calculate forwards following the child relations
-        DateTime finish;
-        DateTime time;
-        foreach ( Node *n, cs->endNodes() ) {
-            time = n->calculateForward( use );
-            if ( !finish.isValid() || time > finish )
-                finish = time;
-        }
-        //kDebug()<<m_name<<" finish="<<finish.toString();
-        return finish;
     } else {
         //TODO: subproject
     }
-    return DateTime();
+    return finish;
 }
 
 DateTime Project::calculateBackward( int use )
 {
     //kDebug()<<m_name;
+    DateTime start;
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
     if ( cs == 0 ) {
-        return DateTime();
+        return start;
     }
     if ( type() == Node::Type_Project ) {
-        // Do constrained first
-        foreach ( Node *n, cs->hardConstraints() ) {
-            n->calculateLateStart( use );
+        m_visitedBackward = true;
+        if ( ! m_visitedForward ) {
+            // setup tasks
+            tasksBackward();
+            // Do all hard constrained first
+            foreach ( Task *n, m_hardConstraints ) {
+                cs->logDebug( "Calculate task with hard constraint:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateLateStart( use ); // do not do predeccessors
+                if ( ! start.isValid() || time < start ) {
+                    start = time;
+                }
+            }
+            // then do the predeccessors
+            foreach ( Task *n, m_hardConstraints ) {
+                cs->logDebug( "Calculate predeccessors to hard constrained task:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateBackward( use );
+                if ( ! start.isValid() || time < start ) {
+                    start = time;
+                }
+            }
+            // now try to schedule soft constrained *with* predeccessors
+            foreach ( Task *n, m_softConstraints ) {
+                cs->logDebug( "Calculate task with soft constraint:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateBackward( use );
+                if ( ! start.isValid() || time < start ) {
+                    start = time;
+                }
+            }
+            // and then the rest using the start nodes to calculate everything (remaining)
+            foreach ( Task *n, m_terminalNodes ) {
+                cs->logDebug( "Calculate using start task:" + n->name() + " : " + n->constraintToString() );
+                DateTime time = n->calculateBackward( use );
+                if ( ! start.isValid() || time < start ) {
+                    start = time;
+                }
+            }
+        } else {
+            // tasks have been calculated forwards in this order
+            foreach ( Node *n, cs->forwardNodes() ) {
+                DateTime time = n->calculateBackward( use );
+                if ( ! start.isValid() || time < start ) {
+                    start = time;
+                }
+            }
         }
-        // Follow *child* relations back and
-        // calculate backwards following parent relation
-        DateTime start;
-        DateTime time;
-        foreach ( Node *n, cs->startNodes() ) {
-            time = n->calculateBackward( use );
-            if ( !start.isValid() || time < start )
-                start = time;
-        }
-        //kDebug()<<m_name<<" start="<<start.toString();
-        return start;
     } else {
         //TODO: subproject
     }
-    return DateTime();
+    return start;
 }
 
 DateTime Project::scheduleForward( const DateTime &earliest, int use )
 {
+    DateTime end;
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
-    if ( cs == 0 ) {
+    if ( cs == 0 || stopcalculation ) {
         return DateTime();
     }
     resetVisited();
-    DateTime end = earliest;
-    DateTime time;
-    QListIterator<Node*> it( cs->endNodes() );
-    while ( it.hasNext() ) {
-        time = it.next() ->scheduleForward( earliest, use );
-        if ( time > end )
+    // Schedule in the same order as calculated forward
+    // Do all hard constrained first
+    foreach ( Node *n, m_hardConstraints ) {
+        cs->logDebug( "Schedule task with hard constraint:" + n->name() + " : " + n->constraintToString() );
+        DateTime time = n->scheduleFromStartTime( use ); // do not do predeccessors
+        if ( time > end ) {
             end = time;
+        }
+    }
+    foreach ( Node *n, cs->forwardNodes() ) {
+        cs->logDebug( "Schedule task:" + n->name() + " : " + n->constraintToString() );
+        DateTime time = n->scheduleForward( earliest, use );
+        if ( time > end ) {
+            end = time;
+        }
     }
     // Fix summarytasks
     adjustSummarytask();
@@ -626,18 +754,27 @@ DateTime Project::scheduleForward( const DateTime &earliest, int use )
 
 DateTime Project::scheduleBackward( const DateTime &latest, int use )
 {
+    DateTime start;
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
-    if ( cs == 0 ) {
-        return DateTime();
+    if ( cs == 0 || stopcalculation ) {
+        return start;
     }
     resetVisited();
-    DateTime start = latest;
-    DateTime time;
-    QListIterator<Node*> it( cs->startNodes() );
-    while ( it.hasNext() ) {
-        time = it.next() ->scheduleBackward( latest, use );
-        if ( time < start )
+    // Schedule in the same order as calculated backward
+    // Do all hard constrained first
+    foreach ( Node *n, m_hardConstraints ) {
+        cs->logDebug( "Schedule task with hard constraint:" + n->name() + " : " + n->constraintToString() );
+        DateTime time = n->scheduleFromEndTime( use ); // do not do predeccessors
+        if ( ! start.isValid() || time < start ) {
             start = time;
+        }
+    }
+    foreach ( Node *n, cs->backwardNodes() ) {
+        cs->logDebug( "Schedule task:" + n->name() + " : " + n->constraintToString() );
+        DateTime time = n->scheduleBackward( latest, use );
+        if ( ! start.isValid() || time < start ) {
+            start = time;
+        }
     }
     // Fix summarytasks
     adjustSummarytask();
@@ -647,7 +784,7 @@ DateTime Project::scheduleBackward( const DateTime &latest, int use )
 void Project::adjustSummarytask()
 {
     MainSchedule *cs = static_cast<MainSchedule*>( m_currentSchedule );
-    if ( cs == 0 ) {
+    if ( cs == 0 || stopcalculation ) {
         return;
     }
     QListIterator<Node*> it( cs->summaryTasks() );
@@ -717,7 +854,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
     QList<Calendar*> cals;
     QString s;
     bool ok = false;
-    m_name = element.attribute( "name" );
+    setName( element.attribute( "name" ) );
     removeId( m_id );
     m_id = element.attribute( "id" );
     registerNodeId( this );
@@ -867,6 +1004,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
     // These go last
     n = element.firstChild();
     for ( ; ! n.isNull(); n = n.nextSibling() ) {
+        kDebug()<<n.isElement();
         if ( ! n.isElement() ) {
             continue;
         }
@@ -892,18 +1030,18 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
         } else if ( e.tagName() == "schedules" ) {
             //kDebug()<<"Project schedules & task appointments--->";
             // References tasks and resources
-            n = e.firstChild();
-            for ( ; ! n.isNull(); n = n.nextSibling() ) {
-                if ( ! n.isElement() ) {
+            KoXmlNode sn = e.firstChild();
+            for ( ; ! sn.isNull(); sn = sn.nextSibling() ) {
+                if ( ! sn.isElement() ) {
                     continue;
                 }
-                KoXmlElement el = n.toElement();
+                KoXmlElement el = sn.toElement();
                 //kDebug()<<el.tagName()<<" Version="<<status.version();
                 ScheduleManager *sm = 0;
                 bool add = false;
                 if ( status.version() <= "0.5" ) {
                     if ( el.tagName() == "schedule" ) {
-                        sm = findScheduleManager( el.attribute( "name" ) );
+                        sm = findScheduleManagerByName( el.attribute( "name" ) );
                         if ( sm == 0 ) {
                             sm = new ScheduleManager( *this, el.attribute( "name" ) );
                             add = true;
@@ -926,8 +1064,46 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
                 }
             }
             //kDebug()<<"Node schedules<---";
+        } else if ( e.tagName() == "resource-teams" ) {
+            //kDebug()<<"Resource teams--->";
+            // References other resources
+            KoXmlNode tn = e.firstChild();
+            for ( ; ! tn.isNull(); tn = tn.nextSibling() ) {
+                if ( ! tn.isElement() ) {
+                    continue;
+                }
+                KoXmlElement el = tn.toElement();
+                if ( el.tagName() == "team" ) {
+                    Resource *r = findResource( el.attribute( "team-id" ) );
+                    Resource *tm = findResource( el.attribute( "member-id" ) );
+                    if ( r == 0 || tm == 0 ) {
+                        kError()<<"resource-teams: cannot find resources";
+                        continue;
+                    }
+                    if ( r == tm ) {
+                        kError()<<"resource-teams: a team cannot be a member of itself";
+                        continue;
+                    }
+                    r->addTeamMember( tm );
+                } else {
+                    kError()<<"resource-teams: unhandled tag"<<el.tagName();
+                }
+            }
+            //kDebug()<<"Resource teams<---";
         } else if ( e.tagName() == "wbs-definition" ) {
             m_wbsDefinition.loadXML( e, status );
+        } else if ( e.tagName() == "locale" ) {
+            // handled earlier
+        } else if ( e.tagName() == "resource-group" ) {
+            // handled earlier
+        } else if ( e.tagName() == "calendar" ) {
+            // handled earlier
+        } else if ( e.tagName() == "standard-worktime" ) {
+            // handled earlier
+        } else if ( e.tagName() == "project" ) {
+            // handled earlier
+        } else if ( e.tagName() == "task" ) {
+            // handled earlier
         } else {
             kWarning()<<"Unhandled tag:"<<e.tagName();
         }
@@ -1003,6 +1179,27 @@ void Project::save( QDomElement &element ) const
             sm->saveXML( el );
         }
     }
+    // save resource teams
+    QMap<QString, QString> rmap;
+    foreach ( Resource *r, resourceIdDict ) {
+        if ( r->type() == Resource::Type_Team ) {
+            foreach ( Resource *tm, r->teamMembers() ) {
+                rmap.insertMulti( r->id(), tm->id() );
+            }
+        }
+    }
+    if ( ! rmap.isEmpty() ) {
+        QDomElement el = me.ownerDocument().createElement( "resource-teams" );
+        me.appendChild( el );
+        QMap<QString, QString>::const_iterator i = rmap.constBegin();
+        while ( i != rmap.constEnd() ) {
+            QDomElement e = el.ownerDocument().createElement( "team" );
+            el.appendChild( e );
+            e.setAttribute( "team-id", i.key() );
+            e.setAttribute( "member-id", i.value() );
+            ++i;
+        }
+    }
 }
 
 void Project::saveWorkPackageXML( QDomElement &element, const Node *node, long id ) const
@@ -1430,10 +1627,11 @@ bool Project::nodeIdentExists( const QString &id ) const
 QString Project::uniqueNodeId( int seed ) const
 {
     Q_UNUSED(seed);
-    QString ident = KRandom::randomString( 10 );
+    QString s = QDateTime::currentDateTime().toString( Qt::ISODate ) + ' ';
+    QString ident = s + KRandom::randomString( 10 );
 //    int i = seed;
     while ( nodeIdentExists( ident ) ) {
-        ident = KRandom::randomString( 10 );
+        ident = s + KRandom::randomString( 10 );
     }
     return ident;
 }
@@ -1484,16 +1682,29 @@ bool Project::registerNodeId( Node *node )
     return true;
 }
 
-QList<Node*> Project::allNodes()
+QList<Node*> Project::allNodes() const
 {
     QList<Node*> lst = nodeIdDict.values();
-    int me = lst.indexOf( this );
+    int me = lst.indexOf( const_cast<Project*>( this ) );
     if ( me != -1 ) {
         lst.removeAt( me );
     }
     return lst;
 }
-    
+
+QList<Task*> Project::allTasks( const Node *parent ) const
+{
+    QList<Task*> lst;
+    const Node *p = parent ? parent : this;
+    foreach ( Node *n, p->childNodeIterator() ) {
+        if ( n->type() == Node::Type_Task || n->type() == Type_Milestone ) {
+            lst << static_cast<Task*>( n );
+        }
+        lst += allTasks( n );
+    }
+    return lst;
+}
+
 bool Project::setResourceGroupId( ResourceGroup *group )
 {
     if ( group == 0 ) {
@@ -1518,9 +1729,10 @@ bool Project::setResourceGroupId( ResourceGroup *group )
 }
 
 QString Project::uniqueResourceGroupId() const {
-    QString id = KRandom::randomString( 10 );
+    QString s = QDateTime::currentDateTime().toString( Qt::ISODate ) + ' ';
+    QString id = s + KRandom::randomString( 10 );
     while ( resourceGroupIdDict.contains( id ) ) {
-        id = KRandom::randomString( 10 );
+        id = s + KRandom::randomString( 10 );
     }
     return id;
 }
@@ -1576,9 +1788,10 @@ bool Project::setResourceId( Resource *resource )
 }
 
 QString Project::uniqueResourceId() const {
-    QString id = KRandom::randomString( 10 );
+    QString s = QDateTime::currentDateTime().toString( Qt::ISODate ) + ' ';
+    QString id = s + KRandom::randomString( 10 );
     while ( resourceIdDict.contains( id ) ) {
-        id = KRandom::randomString( 10 );
+        id = s + KRandom::randomString( 10 );
     }
     return id;
 }
@@ -1609,82 +1822,82 @@ QStringList Project::resourceNameList() const
     return lst;
 }
 
-EffortCostMap Project::plannedEffortCostPrDay( const QDate & start, const QDate &end, long id ) const
+EffortCostMap Project::plannedEffortCostPrDay( const QDate & start, const QDate &end, long id, EffortCostCalculationType typ ) const
 {
     //kDebug()<<start<<end<<id;
     EffortCostMap ec;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        ec += it.next() ->plannedEffortCostPrDay( start, end, id );
+        ec += it.next() ->plannedEffortCostPrDay( start, end, id, typ );
     }
     return ec;
 }
 
-EffortCostMap Project::plannedEffortCostPrDay( const Resource *resource, const QDate & start, const QDate &end, long id ) const
+EffortCostMap Project::plannedEffortCostPrDay( const Resource *resource, const QDate & start, const QDate &end, long id, EffortCostCalculationType typ ) const
 {
     //kDebug()<<start<<end<<id;
     EffortCostMap ec;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        ec += it.next() ->plannedEffortCostPrDay( resource, start, end, id );
+        ec += it.next() ->plannedEffortCostPrDay( resource, start, end, id, typ );
     }
     return ec;
 }
 
-EffortCostMap Project::actualEffortCostPrDay( const QDate & start, const QDate &end, long id ) const
+EffortCostMap Project::actualEffortCostPrDay( const QDate & start, const QDate &end, long id, EffortCostCalculationType typ ) const
 {
     //kDebug()<<start<<end<<id;
     EffortCostMap ec;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        ec += it.next() ->actualEffortCostPrDay( start, end, id );
+        ec += it.next() ->actualEffortCostPrDay( start, end, id, typ );
     }
     return ec;
 }
 
-EffortCostMap Project::actualEffortCostPrDay( const Resource *resource, const QDate & start, const QDate &end, long id ) const
+EffortCostMap Project::actualEffortCostPrDay( const Resource *resource, const QDate & start, const QDate &end, long id,  EffortCostCalculationType typ ) const
 {
     //kDebug()<<start<<end<<id;
     EffortCostMap ec;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        ec += it.next() ->actualEffortCostPrDay( resource, start, end, id );
+        ec += it.next() ->actualEffortCostPrDay( resource, start, end, id, typ );
     }
     return ec;
 }
 
 // Returns the total planned effort for this project (or subproject)
-Duration Project::plannedEffort( long id ) const
+Duration Project::plannedEffort( long id, EffortCostCalculationType typ ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        eff += it.next() ->plannedEffort(id);
+        eff += it.next() ->plannedEffort( id, typ );
     }
     return eff;
 }
 
 // Returns the total planned effort for this project (or subproject) on date
-Duration Project::plannedEffort( const QDate &date, long id ) const
+Duration Project::plannedEffort( const QDate &date, long id, EffortCostCalculationType typ ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        eff += it.next() ->plannedEffort( date, id );
+        eff += it.next() ->plannedEffort( date, id, typ );
     }
     return eff;
 }
 
 // Returns the total planned effort for this project (or subproject) upto and including date
-Duration Project::plannedEffortTo( const QDate &date, long id ) const
+Duration Project::plannedEffortTo( const QDate &date, long id, EffortCostCalculationType typ ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        eff += it.next() ->plannedEffortTo( date, id );
+        eff += it.next() ->plannedEffortTo( date, id, typ );
     }
     return eff;
 }
@@ -1726,40 +1939,40 @@ Duration Project::actualEffortTo( const QDate &date ) const
     return eff;
 }
 
-EffortCost Project::plannedCost( long id ) const
+EffortCost Project::plannedCost( long id, EffortCostCalculationType typ ) const
 {
     //kDebug();
     EffortCost c;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        c += it.next() ->plannedCost(id);
+        c += it.next() ->plannedCost( id, typ );
     }
     return c;
 }
 
 // Returns the total planned effort for this project (or subproject) on date
-double Project::plannedCost( const QDate &date, long id ) const
+double Project::plannedCost( const QDate &date, long id, EffortCostCalculationType typ ) const
 {
     //kDebug();
     double c = 0;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        c += it.next() ->plannedCost( date, id );
+        c += it.next() ->plannedCost( date, id, typ );
     }
     return c;
 }
 
 // Returns the total planned effort for this project (or subproject) upto and including date
-double Project::plannedCostTo( const QDate &date, long id ) const
+double Project::plannedCostTo( const QDate &date, long id, EffortCostCalculationType typ ) const
 {
     //kDebug();
     double c = 0;
     QListIterator
     <Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
-        c += it.next() ->plannedCostTo( date, id );
+        c += it.next() ->plannedCostTo( date, id, typ );
     }
     return c;
 }
@@ -1776,7 +1989,7 @@ double Project::actualCost() const
     return c;
 }
 
-// Returns the total planned effort for this project (or subproject) on date
+// Returns the total actual cost for this project (or subproject) on date
 double Project::actualCost( const QDate &date ) const
 {
     //kDebug();
@@ -1789,7 +2002,7 @@ double Project::actualCost( const QDate &date ) const
     return c;
 }
 
-// Returns the total planned effort for this project (or subproject) upto and including date
+// Returns the total actual cost for this project (or subproject) upto and including date
 EffortCost Project::actualCostTo( const QDate &date ) const
 {
     //kDebug();
@@ -1852,7 +2065,7 @@ double Project::schedulePerformanceIndex( const QDate &date, long id ) const
 double Project::bcws( const QDate &date, long id ) const
 {
     //kDebug();
-    double c = plannedCostTo( date, id );
+    double c = plannedCostTo( date, id, ECCT_EffortWork );
     kDebug()<<c;
     return c;
 }
@@ -1868,7 +2081,7 @@ double Project::bcwp( const QDate &date, long id ) const
     kDebug()<<date<<id;
     QDate start = startTime( id ).date();
     QDate end = endTime( id ).date();
-    EffortCostMap plan = plannedEffortCostPrDay( start, end, id );
+    EffortCostMap plan = plannedEffortCostPrDay( start, end, id, ECCT_EffortWork );
     EffortCostMap actual = actualEffortCostPrDay( start, (end > date ? end : date), id );
 
     double budgetAtCompletion;
@@ -1895,18 +2108,21 @@ double Project::bcwp( const QDate &date, long id ) const
     return c;
 }
 
-void Project::addCalendar( Calendar *calendar, Calendar *parent )
+void Project::addCalendar( Calendar *calendar, Calendar *parent, int index )
 {
     Q_ASSERT( calendar != 0 );
     //kDebug()<<calendar->name()<<","<<(parent?parent->name():"No parent");
     int row = parent == 0 ? m_calendars.count() : parent->calendars().count();
+    if ( index >= 0 && index < row ) {
+        row = index;
+    }
     emit calendarToBeAdded( parent, row );
     calendar->setProject( this );
     if ( parent == 0 ) {
         calendar->setParentCal( 0 ); // in case
-        m_calendars.append( calendar );
+        m_calendars.insert( row, calendar );
     } else {
-        calendar->setParentCal( parent );
+        calendar->setParentCal( parent, row );
     }
     if ( calendar->isDefault() ) {
         setDefaultCalendar( calendar );
@@ -1999,9 +2215,10 @@ bool Project::setCalendarId( Calendar *calendar )
 }
 
 QString Project::uniqueCalendarId() const {
-    QString id = KRandom::randomString( 10 );
+    QString s = QDateTime::currentDateTime().toString( Qt::ISODate ) + ' ';
+    QString id = s + KRandom::randomString( 10 );
     while ( calendarIdDict.contains( id ) ) {
-        id = KRandom::randomString( 10 );
+        id = s + KRandom::randomString( 10 );
     }
     return id;
 }
@@ -2154,14 +2371,19 @@ void Project::setCurrentSchedule( long id )
 ScheduleManager *Project::scheduleManager( long id ) const
 {
     foreach ( ScheduleManager *sm, m_managers ) {
-        if ( sm->id() == id ) {
+        if ( sm->scheduleId() == id ) {
             return sm;
         }
     }
     return 0;
 }
 
-ScheduleManager *Project::findScheduleManager( const QString &name ) const
+ScheduleManager *Project::scheduleManager( const QString &id ) const
+{
+    return m_managerIdMap.value( id );
+}
+
+ScheduleManager *Project::findScheduleManagerByName( const QString &name ) const
 {
     //kDebug();
     ScheduleManager *m = 0;
@@ -2187,14 +2409,14 @@ QList<ScheduleManager*> Project::allScheduleManagers() const
 QString Project::uniqueScheduleName() const {
     //kDebug();
     QString n = i18n( "Plan" );
-    bool unique = findScheduleManager( n ) == 0;
+    bool unique = findScheduleManagerByName( n ) == 0;
     if ( unique ) {
         return n;
     }
     n += " %1";
     int i = 1;
     for ( ; true; ++i ) {
-        unique = findScheduleManager( n.arg( i ) ) == 0;;
+        unique = findScheduleManagerByName( n.arg( i ) ) == 0;;
         if ( unique ) {
             break;
         }
@@ -2202,15 +2424,25 @@ QString Project::uniqueScheduleName() const {
     return n.arg( i );
 }
 
-void Project::addScheduleManager( ScheduleManager *sm, ScheduleManager *parent )
+void Project::addScheduleManager( ScheduleManager *sm, ScheduleManager *parent, int index )
 {
-    if ( parent == 0 ) {
-        emit scheduleManagerToBeAdded( parent, m_managers.count() );
-        m_managers.append( sm );
-    } else {
-        emit scheduleManagerToBeAdded( parent, parent->children().count() );
-        sm->setParentManager( parent );
+    int row = parent == 0 ? m_managers.count() : parent->childCount();
+    if ( index >= 0 && index < row ) {
+        row = index;
     }
+    if ( parent == 0 ) {
+        emit scheduleManagerToBeAdded( parent, row );
+        m_managers.insert( row, sm );
+    } else {
+        emit scheduleManagerToBeAdded( parent, row );
+        sm->setParentManager( parent, row );
+    }
+    if ( sm->managerId().isEmpty() ) {
+        sm->setManagerId( uniqueScheduleManagerId() );
+    }
+    Q_ASSERT( ! m_managerIdMap.contains( sm->managerId() ) );
+    m_managerIdMap.insert( sm->managerId(), sm );
+
     emit scheduleManagerAdded( sm );
     emit changed();
     //kDebug()<<"Added:"<<sm->name()<<", now"<<m_managers.count();
@@ -2230,6 +2462,7 @@ int Project::takeScheduleManager( ScheduleManager *sm )
         if ( index >= 0 ) {
             emit scheduleManagerToBeRemoved( sm );
             sm->setParentManager( 0 );
+            m_managerIdMap.remove( sm->managerId() );
             emit scheduleManagerRemoved( sm );
             emit changed();
         }
@@ -2238,6 +2471,7 @@ int Project::takeScheduleManager( ScheduleManager *sm )
         if ( index >= 0 ) {
             emit scheduleManagerToBeRemoved( sm );
             m_managers.removeAt( indexOf( sm ) );
+            m_managerIdMap.remove( sm->managerId() );
             emit scheduleManagerRemoved( sm );
             emit changed();
         }
@@ -2272,6 +2506,15 @@ ScheduleManager *Project::createScheduleManager()
     return createScheduleManager( uniqueScheduleName() );
 }
 
+QString Project::uniqueScheduleManagerId() const
+{
+    QString ident = KRandom::randomString( 10 );
+    while ( m_managerIdMap.contains( ident ) ) {
+        ident = KRandom::randomString( 10 );
+    }
+    return ident;
+}
+
 bool Project::isBaselined( long id ) const
 {
     if ( id == ANYSCHEDULED ) {
