@@ -84,6 +84,7 @@ GanttChartDisplayOptionsPanel::GanttChartDisplayOptionsPanel( GanttItemDelegate
     connect( ui_showCriticalPath, SIGNAL(  stateChanged ( int ) ), SIGNAL( changed() ) );
     connect( ui_showCriticalTasks, SIGNAL(  stateChanged ( int ) ), SIGNAL( changed() ) );
     connect( ui_showCompletion, SIGNAL(  stateChanged ( int ) ), SIGNAL( changed() ) );
+    connect( ui_showSchedulingError, SIGNAL(  stateChanged ( int ) ), SIGNAL( changed() ) );
 }
 
 void GanttChartDisplayOptionsPanel::slotOk()
@@ -95,6 +96,7 @@ void GanttChartDisplayOptionsPanel::slotOk()
     m_delegate->showCriticalPath = ui_showCriticalPath->checkState() == Qt::Checked;
     m_delegate->showCriticalTasks = ui_showCriticalTasks->checkState() == Qt::Checked;
     m_delegate->showProgress = ui_showCompletion->checkState() == Qt::Checked;
+    m_delegate->showSchedulingError = ui_showSchedulingError->checkState() == Qt::Checked;
 }
 
 void GanttChartDisplayOptionsPanel::setValues( const GanttItemDelegate &del )
@@ -106,6 +108,7 @@ void GanttChartDisplayOptionsPanel::setValues( const GanttItemDelegate &del )
     ui_showCriticalPath->setCheckState( del.showCriticalPath ? Qt::Checked : Qt::Unchecked );
     ui_showCriticalTasks->setCheckState( del.showCriticalTasks ? Qt::Checked : Qt::Unchecked );
     ui_showCompletion->setCheckState( del.showProgress ? Qt::Checked : Qt::Unchecked );
+    ui_showSchedulingError->setCheckState( del.showSchedulingError ? Qt::Checked : Qt::Unchecked );
 }
 
 void GanttChartDisplayOptionsPanel::setDefault()
@@ -119,8 +122,8 @@ GanttViewSettingsDialog::GanttViewSettingsDialog( TreeViewBase *view, GanttItemD
     : ItemViewSettupDialog( view, true, parent )
 {
     GanttChartDisplayOptionsPanel *panel = new GanttChartDisplayOptionsPanel( delegate );
-    KPageWidgetItem *page = insertWidget( 1, panel, i18n( "Chart" ), i18n( "Gantt Chart Settings" ) );
-    
+    /*KPageWidgetItem *page = */insertWidget( 1, panel, i18n( "Chart" ), i18n( "Gantt Chart Settings" ) );
+
     connect( this, SIGNAL( okClicked() ), panel, SLOT( slotOk() ) );
     connect( this, SIGNAL( defaultClicked() ), panel, SLOT( setDefault() ) );
 }
@@ -175,9 +178,9 @@ QList<QWidget*> GanttPrintingDialog::createOptionWidgets() const
     GanttPrintingOptions *w = new GanttPrintingOptions();
     //w->setPrintRowLabels( m_printRowLabels );
     w->setSinglePage( m_singlePage );
-    
+
     const_cast<GanttPrintingDialog*>( this )->m_options = w;
-    
+
     return QList<QWidget*>() << createPageLayoutWidget() << m_options;
 }
 
@@ -221,9 +224,9 @@ GanttTreeView::GanttTreeView( QWidget* parent )
 {
     disconnect( header() );
     setHeader( new HeaderView );
-    
+
     setSelectionMode( QAbstractItemView::ExtendedSelection );
-    
+
     header()->setContextMenuPolicy( Qt::CustomContextMenu );
     connect( header(), SIGNAL( customContextMenuRequested( const QPoint& ) ), this, SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
 }
@@ -244,7 +247,7 @@ GanttViewBase::GanttViewBase( QWidget *parent )
     setLeftView( tv );
     setRowController( new KDGantt::TreeViewRowController( tv, ganttProxyModel() ) );
     tv->header()->setStretchLastSection( true );
-    
+
     NodeSortFilterProxyModel *m = new NodeSortFilterProxyModel( &m_defaultModel, this );
     KDGantt::View::setModel( m );
 }
@@ -289,6 +292,7 @@ bool GanttViewBase::loadContext( const KoXmlElement &settings )
         m_ganttdelegate->showCriticalPath = (bool)( e.attribute( "show-criticalpath", "0" ).toInt() );
         m_ganttdelegate->showCriticalTasks = (bool)( e.attribute( "show-criticaltasks", "0" ).toInt() );
         m_ganttdelegate->showPositiveFloat = (bool)( e.attribute( "show-positivefloat", "0" ).toInt() );
+        m_ganttdelegate->showSchedulingError = (bool)( e.attribute( "show-schedulingerror", "0" ).toInt() );
     }
     return true;
 }
@@ -297,7 +301,7 @@ void GanttViewBase::saveContext( QDomElement &settings ) const
 {
     kDebug();
     treeView()->saveContext( model()->columnMap(), settings );
-    
+
     QDomElement e = settings.ownerDocument().createElement( "ganttchart" );
     settings.appendChild( e );
     e.setAttribute( "show-dependencies", m_ganttdelegate->showTaskLinks );
@@ -307,6 +311,7 @@ void GanttViewBase::saveContext( QDomElement &settings ) const
     e.setAttribute( "show-criticalpath", m_ganttdelegate->showCriticalPath  );
     e.setAttribute( "show-criticaltasks", m_ganttdelegate->showCriticalTasks );
     e.setAttribute( "show-positivefloat", m_ganttdelegate->showPositiveFloat );
+    e.setAttribute( "show-schedulingerror", m_ganttdelegate->showSchedulingError );
 }
 
 //-------------------------------------------
@@ -336,15 +341,15 @@ MyKDGanttView::MyKDGanttView( QWidget *parent )
     m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole ); // To provide correct format
     m->setRole( KDGantt::StartTimeRole, Qt::EditRole ); // To provide correct format
     m->setRole( KDGantt::EndTimeRole, Qt::EditRole ); // To provide correct format
-    
+
     m->setColumn( KDGantt::ItemTypeRole, NodeModel::NodeType );
     m->setColumn( KDGantt::StartTimeRole, NodeModel::NodeStartTime );
     m->setColumn( KDGantt::EndTimeRole, NodeModel::NodeEndTime );
     m->setColumn( KDGantt::TaskCompletionRole, NodeModel::NodeCompleted );
-    
+
     static_cast<KDGantt::DateTimeGrid*>( grid() )->setDayWidth( 30 );
     //static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
-    
+
     connect( model(), SIGNAL( nodeInserted( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
 }
 
@@ -385,7 +390,7 @@ void MyKDGanttView::setScheduleManager( ScheduleManager *sm )
     model()->setScheduleManager( sm );
     m_manager = sm;
     if ( sm && project() ) {
-        QDateTime start = project()->startTime( sm->id() ).dateTime().addDays( -1 );
+        QDateTime start = project()->startTime( sm->scheduleId() ).dateTime().addDays( -1 );
         KDGantt::DateTimeGrid *g = static_cast<KDGantt::DateTimeGrid*>( grid() );
         if ( g->startDateTime() !=  start ) {
             g->setStartDateTime( start );
@@ -408,9 +413,9 @@ void MyKDGanttView::addDependency( Relation *rel )
 {
     QModelIndex par = sfModel()->mapFromSource( model()->index( rel->parent() ) );
     QModelIndex ch = sfModel()->mapFromSource( model()->index( rel->child() ) );
-    kDebug()<<"addDependency() "<<model()<<par.model();
+//    kDebug()<<"addDependency() "<<model()<<par.model();
     if ( par.isValid() && ch.isValid() ) {
-        KDGantt::Constraint con( par, ch, KDGantt::Constraint::TypeSoft, 
+        KDGantt::Constraint con( par, ch, KDGantt::Constraint::TypeSoft,
                                  static_cast<KDGantt::Constraint::RelationType>( rel->type() )/*NOTE!!*/
                                );
         if ( ! constraintModel()->hasConstraint( con ) ) {
@@ -423,7 +428,7 @@ void MyKDGanttView::removeDependency( Relation *rel )
 {
     QModelIndex par = sfModel()->mapFromSource( model()->index( rel->parent() ) );
     QModelIndex ch = sfModel()->mapFromSource( model()->index( rel->child() ) );
-    KDGantt::Constraint con( par, ch, KDGantt::Constraint::TypeSoft, 
+    KDGantt::Constraint con( par, ch, KDGantt::Constraint::TypeSoft,
                              static_cast<KDGantt::Constraint::RelationType>( rel->type() )/*NOTE!!*/
                            );
     constraintModel()->removeConstraint( con );
@@ -464,11 +469,11 @@ GanttView::GanttView( KoDocument *part, QWidget *parent, bool readWrite )
     m_gantt = new MyKDGanttView( m_splitter );
 
     setupGui();
-    
+
     updateReadWrite( readWrite );
     //connect( m_gantt->constraintModel(), SIGNAL( constraintAdded( const Constraint& )), this, SLOT( update() ) );
     kDebug() <<m_gantt->constraintModel();
-    
+
     connect( m_gantt->treeView(), SIGNAL( contextMenuRequested( QModelIndex, const QPoint& ) ), SLOT( slotContextMenuRequested( QModelIndex, const QPoint& ) ) );
 
     connect( m_gantt->treeView(), SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
@@ -652,13 +657,13 @@ MilestoneKDGanttView::MilestoneKDGanttView( QWidget *parent )
     m_manager( 0 )
 {
     kDebug()<<"------------------- create MilestoneKDGanttView -----------------------";
-    
+
     setItemModel( new MilestoneItemModel( this ) );
-    
+
     sfModel()->setFilterRole ( Qt::EditRole );
     sfModel()->setFilterFixedString( QString::number( Node::Type_Milestone ) );
     sfModel()->setFilterKeyColumn( NodeModel::NodeType );
-    
+
     QList<int> show;
     show << NodeModel::NodeName
             << NodeModel::NodeStartTime;
@@ -670,13 +675,13 @@ MilestoneKDGanttView::MilestoneKDGanttView( QWidget *parent )
         }
     }
     treeView()->setRootIsDecorated ( false );
-    
+
     KDGantt::ProxyModel *m = static_cast<KDGantt::ProxyModel*>( ganttProxyModel() );
-    
+
     m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole ); // To provide correct format
     m->setRole( KDGantt::StartTimeRole, Qt::EditRole ); // To provide correct format
     m->setRole( KDGantt::EndTimeRole, Qt::EditRole ); // To provide correct format
-    
+
     m->setColumn( KDGantt::ItemTypeRole, NodeModel::NodeType );
     m->setColumn( KDGantt::StartTimeRole, NodeModel::NodeStartTime );
     m->setColumn( KDGantt::EndTimeRole, NodeModel::NodeEndTime );
@@ -717,7 +722,7 @@ void MilestoneKDGanttView::setScheduleManager( ScheduleManager *sm )
     if ( sm && m_project ) {
         QDateTime start;
         foreach ( const Node *n, model()->mileStones() ) {
-            QDateTime nt = n->startTime( sm->id() ).dateTime();
+            QDateTime nt = n->startTime( sm->scheduleId() ).dateTime();
             if ( ! nt.isValid() ) {
                 continue;
             }
@@ -727,7 +732,7 @@ void MilestoneKDGanttView::setScheduleManager( ScheduleManager *sm )
             }
         }
         if ( ! start.isValid() ) {
-            start = project()->startTime( sm->id() ).dateTime();
+            start = project()->startTime( sm->scheduleId() ).dateTime();
         }
         KDGantt::DateTimeGrid *g = static_cast<KDGantt::DateTimeGrid*>( grid() );
         start = start.addDays( -1 );
@@ -897,7 +902,7 @@ ResourceAppointmentsGanttView::ResourceAppointmentsGanttView( KoDocument *part,
     m_gantt->setLeftView( tv );
     m_gantt->setRowController( new KDGantt::TreeViewRowController( tv, m_gantt->ganttProxyModel() ) );
     tv->header()->setStretchLastSection( true );
-    
+
 
     KDGantt::ProxyModel *m = static_cast<KDGantt::ProxyModel*>( m_gantt->ganttProxyModel() );
     m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole );
