@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2010 Benjamin Port <port.benjamin@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,11 +19,13 @@
  */
 
 #include "KPrAnimationDirector.h"
-
+#include <QList>
 #include <QPainter>
 #include <QPaintEvent>
 #include <QWidget>
+#include <QVariant>
 #include <kapplication.h>
+#include <kdebug.h>
 #include <KoPageLayout.h>
 #include <KoShapeManager.h>
 #include <KoShapeManagerPaintingStrategy.h>
@@ -39,14 +42,15 @@
 #include "KPrPageApplicationData.h"
 #include "KPrShapeManagerAnimationStrategy.h"
 #include "KPrShapeManagerDisplayMasterStrategy.h"
+#include "KPrPageSelectStrategyActive.h"
 #include "pageeffects/KPrPageEffectRunner.h"
 #include "pageeffects/KPrPageEffect.h"
-#include "shapeanimations/KPrAnimationData.h"
-#include "shapeanimations/KPrShapeAnimation.h"
 
 #include <KoShapeLayer.h>
 #include <KoPAMasterPage.h>
 #include <KoSelection.h>
+#include "KPrShapeAnimations.h"
+#include "animations/KPrAnimationCache.h"
 
 KPrAnimationDirector::KPrAnimationDirector( KoPAView * view, KoPACanvas * canvas, const QList<KoPAPageBase*> & pages, KoPAPageBase* currentPage )
 : m_view( view )
@@ -56,9 +60,10 @@ KPrAnimationDirector::KPrAnimationDirector( KoPAView * view, KoPACanvas * canvas
 , m_stepIndex( 0 )
 , m_maxShapeDuration( 0 )
 , m_hasAnimation( false )
+, m_animationCache( 0 )
 {
     Q_ASSERT( !m_pages.empty() );
-
+    m_animationCache = new KPrAnimationCache();
     if( !currentPage || !pages.contains(currentPage))
         updateActivePage( m_pages[0] );
     else
@@ -66,18 +71,21 @@ KPrAnimationDirector::KPrAnimationDirector( KoPAView * view, KoPACanvas * canvas
 
     m_pageIndex = m_pages.indexOf( m_view->activePage() );
 
-    updateAnimations();
+    // updatePageAnimation was called from updateZoom() [updateActivePage()]
+
     connect( &m_timeLine, SIGNAL( valueChanged( qreal ) ), this, SLOT( animate() ) );
     // this is needed as after a call to m_canvas->showFullScreen the canvas is not made fullscreen right away
     connect( m_canvas, SIGNAL( sizeChanged( const QSize & ) ), this, SLOT( updateZoom( const QSize & ) ) );
     m_timeLine.setCurveShape( QTimeLine::LinearCurve );
     m_timeLine.setUpdateInterval( 20 );
     // set the animation strategy in the KoShapeManagers
-    m_canvas->shapeManager()->setPaintingStrategy( new KPrShapeManagerAnimationStrategy( m_canvas->shapeManager(), this ) );
-    m_canvas->masterShapeManager()->setPaintingStrategy( new KPrShapeManagerAnimationStrategy( m_canvas->masterShapeManager(), this ) );
+    m_canvas->shapeManager()->setPaintingStrategy( new KPrShapeManagerAnimationStrategy( m_canvas->shapeManager(), m_animationCache,
+                                                       new KPrPageSelectStrategyActive( m_view->kopaCanvas() ) ) );
+    m_canvas->masterShapeManager()->setPaintingStrategy( new KPrShapeManagerAnimationStrategy( m_canvas->masterShapeManager(), m_animationCache,
+                                                             new KPrPageSelectStrategyActive( m_view->kopaCanvas() ) ) );
 
     if ( hasAnimation() ) {
-        startTimeLine( m_maxShapeDuration );
+        startTimeLine( m_animations.at(m_stepIndex)->totalDuration() );
     }
 }
 
@@ -85,16 +93,17 @@ KPrAnimationDirector::~KPrAnimationDirector()
 {
     // free used resources
     delete m_pageEffectRunner;
-    clearAnimations();
+    delete m_animationCache;
     //set the KoShapeManagerPaintingStrategy in the KoShapeManagers
     m_canvas->shapeManager()->setPaintingStrategy( new KoShapeManagerPaintingStrategy( m_canvas->shapeManager() ) );
-    m_canvas->masterShapeManager()->setPaintingStrategy( new KPrShapeManagerDisplayMasterStrategy( m_canvas->masterShapeManager() ) );
+    m_canvas->masterShapeManager()->setPaintingStrategy( new KPrShapeManagerDisplayMasterStrategy( m_canvas->masterShapeManager(),
+                                                             new KPrPageSelectStrategyActive( m_view->kopaCanvas() ) ) );
 }
 
-void KPrAnimationDirector::paintEvent( QPaintEvent* event )
-{
-    QPainter painter( m_canvas );
 
+
+void KPrAnimationDirector::paint(QPainter& painter, const QRectF &paintRect)
+{
     if ( m_pageEffectRunner )
     {
         bool finished = m_pageEffectRunner->isFinished();
@@ -106,34 +115,39 @@ void KPrAnimationDirector::paintEvent( QPaintEvent* event )
             // check if there where a animation to start
             if ( hasAnimation() ) {
                 if ( finished ) {
-                    QRect clipRect = m_pageRect.intersected( event->rect() );
+                    QRect clipRect = m_pageRect.intersected( paintRect.toRect() );
                     painter.setClipRect( clipRect );
                     painter.setRenderHint( QPainter::Antialiasing );
                     paintStep( painter );
                 }
                 else {
                     // start the animations
-                    startTimeLine( m_maxShapeDuration );
+                    startTimeLine( m_animations.at(m_stepIndex)->totalDuration() );
                 }
             }
         }
     }
-    else
-    {
-        QRect clipRect = m_pageRect.intersected( event->rect() );
+    else {
+        QRect clipRect = m_pageRect.intersected( paintRect.toRect() );
         painter.setClipRect( clipRect );
         painter.setRenderHint( QPainter::Antialiasing );
         paintStep( painter );
     }
     // This is needed as otherwise on some ATI graphic cards it leads to
-    // 100% CPU load of the x server and no more key events are received 
+    // 100% CPU load of the x server and no more key events are received
     // until the page effect is finished. With it is made sure that key
-    // events still get through so that it is possible to cancel the 
+    // events still get through so that it is possible to cancel the
     // events. It looks like this is not a problem with nvidia graphic
     // cards.
     KApplication::kApplication()->syncX();
 }
 
+void KPrAnimationDirector::paintEvent( QPaintEvent* event )
+{
+    QPainter painter( m_canvas );
+    paint(painter, event->rect());
+}
+
 KoViewConverter * KPrAnimationDirector::viewConverter()
 {
     return &m_zoomHandler;
@@ -151,7 +165,7 @@ int KPrAnimationDirector::currentPage() const
 
 int KPrAnimationDirector::numStepsInPage() const
 {
-    return m_steps.size();
+    return m_animations.size();
 }
 
 int KPrAnimationDirector::currentStep() const
@@ -170,7 +184,6 @@ bool KPrAnimationDirector::navigate( Navigation navigation )
         finished = true;
     }
     else if ( m_timeLine.state() == QTimeLine::Running ) { // there are still shape animations running
-        Q_ASSERT( !m_animations.isEmpty() );
         finishAnimations();
         m_timeLine.stop();
         finished = true;
@@ -210,7 +223,6 @@ void KPrAnimationDirector::navigateToPage( int index )
         m_timeLine.stop();
     }
     else if ( m_timeLine.state() == QTimeLine::Running ) { // there are still shape animations running
-        Q_ASSERT( !m_animations.isEmpty() );
         finishAnimations();
         m_timeLine.stop();
     }
@@ -221,36 +233,25 @@ void KPrAnimationDirector::navigateToPage( int index )
     m_stepIndex = 0;
 
     updateActivePage( page );
-    updateAnimations();
-
+    updatePageAnimation();
+    updateStepAnimation();
     // trigger a repaint
     m_canvas->update();
 
     if ( hasAnimation() ) {
-        startTimeLine( m_maxShapeDuration );
+        startTimeLine( m_animations.at(m_stepIndex)->totalDuration() );
     }
 }
 
-bool KPrAnimationDirector::shapeShown( KoShape * shape )
-{
-    return !m_animations.contains( shape ) || m_animations[shape].first != 0;
-}
-
-QPair<KPrShapeAnimation *, KPrAnimationData *> KPrAnimationDirector::shapeAnimation( KoShape * shape )
-{
-    QMap<KoShape *, QPair<KPrShapeAnimation *, KPrAnimationData *> >::iterator it( m_animations.find( shape ) );
-
-    return ( it != m_animations.end() ) ? it.value() : QPair<KPrShapeAnimation *, KPrAnimationData *>( 0, 0 );
-}
-
-
 void KPrAnimationDirector::updateActivePage( KoPAPageBase * page )
 {
+    deactivate();
+
     if ( m_canvas == m_view->kopaCanvas() ) {
         m_view->viewMode()->updateActivePage( page );
     }
     else {
-        QList<KoShape*> shapes = page->childShapes();
+        QList<KoShape*> shapes = page->shapes();
         m_canvas->shapeManager()->setShapes(shapes, KoShapeManager::AddWithoutRepaint);
         //Make the top most layer active
         if ( !shapes.isEmpty() ) {
@@ -263,7 +264,7 @@ void KPrAnimationDirector::updateActivePage( KoPAPageBase * page )
 
         Q_ASSERT( paPage );
         KoPAMasterPage * masterPage = paPage->masterPage();
-        QList<KoShape*> masterShapes = masterPage->childShapes();
+        QList<KoShape*> masterShapes = masterPage->shapes();
         m_canvas->masterShapeManager()->setShapes(masterShapes, KoShapeManager::AddWithoutRepaint);
         // Make the top most layer active
         if ( !masterShapes.isEmpty() ) {
@@ -272,19 +273,37 @@ void KPrAnimationDirector::updateActivePage( KoPAPageBase * page )
         }
     }
 
+    KPrPage * kprPage = dynamic_cast<KPrPage *>( page );
+    Q_ASSERT( kprPage );
+    m_pageIndex = m_pages.indexOf(page);
+    m_animations = kprPage->animations().steps();
+
     // it can be that the pages have different sizes. So we need to recalulate
     // the zoom when we change the page
     updateZoom( m_canvas->size() );
-
-    KPrPage * kprPage = dynamic_cast<KPrPage *>( page );
-    Q_ASSERT( kprPage );
-    KPrPageData * pageData = dynamic_cast<KPrPageData*>( kprPage->masterPage() );
-    Q_ASSERT( pageData );
-    QSet<int> steps = kprPage->animations().steps().toSet();
-    steps.unite( pageData->animations().steps().toSet() );
-    m_steps = steps.toList();
 }
 
+void KPrAnimationDirector::updatePageAnimation()
+{
+    m_animationCache->clear();
+
+    m_animationCache->setPageSize(m_pages[m_pageIndex]->size());
+    qreal zoom;
+    m_zoomHandler.zoom(&zoom, &zoom);
+    m_animationCache->setZoom(zoom);
+    int i = 0;
+    foreach (KPrAnimationStep *step, m_animations) {
+        step->init(m_animationCache, i);
+        i++;
+    }
+}
+
+void KPrAnimationDirector::updateStepAnimation()
+{
+    m_animationCache->startStep(m_stepIndex);
+}
+
+
 bool KPrAnimationDirector::changePage( Navigation navigation )
 {
     switch ( navigation )
@@ -319,14 +338,14 @@ bool KPrAnimationDirector::changePage( Navigation navigation )
     m_stepIndex = 0;
 
     updateActivePage( m_pages[m_pageIndex] );
-    updateAnimations();
+    updatePageAnimation();
+    updateStepAnimation();
 
     // trigger a repaint
     m_canvas->update();
 
-    //TODO start animations on step 0
     if ( hasAnimation() ) {
-        startTimeLine( m_maxShapeDuration );
+        startTimeLine( m_animations.at(m_stepIndex)->totalDuration() );
     }
 
     return false;
@@ -338,6 +357,10 @@ void KPrAnimationDirector::updateZoom( const QSize & size )
     KoPAUtil::setZoom( pageLayout, size, m_zoomHandler );
     m_pageRect = KoPAUtil::pageRect( pageLayout, size, m_zoomHandler );
     m_canvas->setDocumentOffset( -m_pageRect.topLeft() );
+
+    // reinit page animation, because somi init method contain zoom
+    updatePageAnimation();
+    updateStepAnimation();
 }
 
 void KPrAnimationDirector::paintStep( QPainter & painter )
@@ -350,21 +373,21 @@ void KPrAnimationDirector::paintStep( QPainter & painter )
             shape->waitUntilReady( m_zoomHandler, false );
         }
 
-        m_canvas->masterShapeManager()->paint( painter, m_zoomHandler, false );
+        m_canvas->masterShapeManager()->paint( painter, m_zoomHandler, true );
     }
     foreach ( KoShape *shape, m_canvas->shapeManager()->shapes() ) {
         shape->waitUntilReady( m_zoomHandler, false );
     }
-    m_canvas->shapeManager()->paint( painter, m_zoomHandler, false );
+    m_canvas->shapeManager()->paint( painter, m_zoomHandler, true );
 }
 
 bool KPrAnimationDirector::nextStep()
 {
-    if ( ! m_steps.isEmpty() && m_stepIndex < m_steps.size() - 1 ) {
+    if ( m_stepIndex < numStepsInPage() - 1 ) {
         // if there are sub steps go to the next substep
         ++m_stepIndex;
-        updateAnimations();
-        startTimeLine( m_maxShapeDuration );
+        updateStepAnimation();
+        startTimeLine(m_animations.at(m_stepIndex)->totalDuration());
     }
     else {
         // if there are no more sub steps go to the next page
@@ -386,8 +409,8 @@ bool KPrAnimationDirector::nextStep()
             m_canvas->render( &oldPage );
 
             updateActivePage( m_pages[m_pageIndex] );
-            updateAnimations();
-
+            updatePageAnimation();
+            updateStepAnimation();
             QPixmap newPage( m_canvas->size() );
             newPage.fill( Qt::white ); // TODO
             QPainter newPainter( &newPage );
@@ -400,11 +423,11 @@ bool KPrAnimationDirector::nextStep()
         }
         else {
             updateActivePage( m_pages[m_pageIndex] );
-            updateAnimations();
-
+            updatePageAnimation();
+            updateStepAnimation();
             m_canvas->update();
             if ( hasAnimation() ) {
-                startTimeLine( m_maxShapeDuration );
+                startTimeLine( m_animations.at(m_stepIndex)->totalDuration() );
             }
         }
     }
@@ -415,17 +438,18 @@ void KPrAnimationDirector::previousStep()
 {
     if ( m_stepIndex > 0 ) {
         --m_stepIndex;
-        // trigger a repaint of the running animations
-        finishAnimations();
-        updateAnimations();
     }
     else {
         if ( m_pageIndex > 0 ) {
             --m_pageIndex;
             updateActivePage( m_pages[m_pageIndex] );
-            Q_ASSERT( m_steps.size() > 0 );
-            m_stepIndex = m_steps.size() - 1;
-            updateAnimations();
+            if(hasAnimation()) {
+                m_stepIndex = m_animations.size() - 1;
+            }
+            else {
+                m_stepIndex = m_animations.size();
+            }
+            updatePageAnimation();
             // trigger repaint
             m_canvas->update();
             // cancel a running page effect
@@ -437,57 +461,10 @@ void KPrAnimationDirector::previousStep()
     finishAnimations();
 }
 
-void KPrAnimationDirector::updateAnimations()
-{
-    clearAnimations();
-    if ( m_steps.size() > m_stepIndex ) {
-        m_maxShapeDuration = 0;
-
-        KPrPage * page = dynamic_cast<KPrPage *>( m_pages[m_pageIndex] );
-        KPrPageData * pageData = dynamic_cast<KPrPageData*>( page->masterPage() );
-        Q_ASSERT( page );
-        Q_ASSERT( pageData );
-
-        m_hasAnimation = false;
-        insertAnimations( page, m_canvas->shapeManager() );
-        insertAnimations( pageData, m_canvas->masterShapeManager() );
-    }
-}
-
-void KPrAnimationDirector::insertAnimations( KPrPageData * controller, KoShapeManager * shapeManager )
-{
-    KoPageLayout pageLayout = m_view->activePage()->pageLayout();
-    QRectF pageRect( 0, 0, pageLayout.width, pageLayout.height );
-
-    QMap<KoShape *, KPrShapeAnimation *> animations = controller->animations().animations( m_steps[m_stepIndex] );
-    QMap<KoShape *, KPrShapeAnimation *>::iterator it( animations.begin() );
-    for ( ; it != animations.end(); ++it ) {
-        KPrShapeAnimation * animation = it.value();
-        if ( animation ) {
-            m_animations.insert( it.key(), qMakePair( it.value(), animation->animationData( m_canvas, shapeManager, pageRect ) ) );
-            m_hasAnimation = true;
-            if ( animation->duration() > m_maxShapeDuration ) {
-                m_maxShapeDuration = animation->duration();
-            }
-        }
-        else {
-            m_animations.insert( it.key(), QPair<KPrShapeAnimation *, KPrAnimationData *>( 0, 0 ) );
-        }
-    }
-}
-
-void KPrAnimationDirector::clearAnimations()
-{
-    QMap<KoShape *, QPair<KPrShapeAnimation *, KPrAnimationData *> >::iterator it( m_animations.begin() );
-    for ( ; it != m_animations.end(); ++it ) {
-        delete it.value().second;
-    }
-    m_animations.clear();
-}
 
 bool KPrAnimationDirector::hasAnimation()
 {
-    return m_hasAnimation;
+    return m_animations.size() > 0;
 }
 
 void KPrAnimationDirector::animate()
@@ -495,41 +472,37 @@ void KPrAnimationDirector::animate()
     if ( m_pageEffectRunner ) {
         m_pageEffectRunner->next( m_timeLine.currentTime() );
     }
-    else if ( ! m_animations.empty() ) { //if there are animnations
-        animateShapes( m_timeLine.currentTime() );
+    else if ( hasAnimation() ) { //if there are animnations
+        // set current time, to the current step
+        m_animationCache->next();
+        m_animations.at(m_stepIndex)->setCurrentTime(m_timeLine.currentTime());
+        m_canvas->update();
     }
 }
 
 void KPrAnimationDirector::finishAnimations()
 {
-    QMap<KoShape *, QPair<KPrShapeAnimation *, KPrAnimationData *> >::iterator it( m_animations.begin() );
-    for ( ; it != m_animations.end(); ++it ) {
-        KPrShapeAnimation * animation = it.value().first;
-        if ( animation ) {
-            animation->finish( it.value().second );
-            it.value().second->m_shapeManager->notifyShapeChanged( it.key() );
-        }
-    }
-}
-
-
-void KPrAnimationDirector::animateShapes( int currentTime )
-{
-    QMap<KoShape *, QPair<KPrShapeAnimation *, KPrAnimationData *> >::iterator it( m_animations.begin() );
-    for ( ; it != m_animations.end(); ++it ) {
-        KPrShapeAnimation * animation = it.value().first;
-        if ( animation ) {
-            animation->next( currentTime, it.value().second );
-            it.value().second->m_shapeManager->notifyShapeChanged( it.key() );
-        }
-    }
+    m_animationCache->endStep(m_stepIndex);
+    m_canvas->update();
 }
 
 void KPrAnimationDirector::startTimeLine( int duration )
 {
-    m_timeLine.setDuration( duration );
+    if (duration == 0) {
+        m_timeLine.setDuration( 1 );
+    }
+    else {
+       m_timeLine.setDuration( duration );
+    }
     m_timeLine.setCurrentTime( 0 );
     m_timeLine.start();
 }
 
+void KPrAnimationDirector::deactivate()
+{
+    foreach (KPrAnimationStep *step, m_animations) {
+        step->deactivate();
+    }
+}
+
 #include "KPrAnimationDirector.moc"
