@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2010 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2010 Benjamin Port <port.benjamin@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,12 +26,15 @@
 class KPrAnimSet : public KPrAnimationBase
 {
 public:
-    KPrAnimSet();
+    KPrAnimSet(KPrShapeAnimation *shapeAnimation);
     virtual ~KPrAnimSet();
 
     virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
-    virtual void saveOdf(KoShapeSavingContext &context) const;
-    virtual void init(KPrAnimationCache *animationCache) const;
+    virtual bool saveOdf(KoPASavingContext & paContext) const;
+    virtual bool saveAttribute(KoPASavingContext &paContext) const;
+    virtual void init(KPrAnimationCache *animationCache, int step);
+protected:
+    virtual void next(int currentTime);
 
 private:
     bool m_visible;
