@@ -62,6 +62,7 @@ class KRITAIMAGE_EXPORT KisImage : public QObject, public KisNodeFacade, public
 
 public:
 
+    /// @param colorSpace can be null. in that case it will be initialised to a default color space.
     KisImage(KisUndoAdapter * adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name, bool startProjection = true);
     KisImage(const KisImage& rhs);
     virtual ~KisImage();
@@ -74,6 +75,7 @@ public: // KisNodeGraphListener implementation
     void nodeHasBeenRemoved(KisNode *parent, int index);
     void aboutToMoveNode(KisNode * parent, int oldIndex, int newIndex);
     void nodeHasBeenMoved(KisNode * parent, int oldIndex, int newIndex);
+    void nodeChanged(KisNode * node);
 
 public:
 
@@ -99,6 +101,13 @@ public:
                            qint32 height,
                            const KoColorProfile * profile);
 
+    /**
+     * Render the projection onto a QImage.
+     * (this is an overloaded function)
+     */
+    QImage convertToQImage(QRect imageRect,
+                           const KoColorProfile * profile);
+
 
     /**
      * XXX: docs!
@@ -210,8 +219,13 @@ public:
 
     /**
      * Execute a scale transform on all layers in this image.
+     * @param sx x-axis scale factor
+     * @param sy y-axis scale factor
+     * @param m_progress progress updater
+     * @param filterStrategy filtering strategy be used to scaling pixels 
+     * @param scaleOnlyShapes only scale shapes, but not image and pixels
      */
-    void scale(double sx, double sy, KoUpdater *m_progress, KisFilterStrategy *filterStrategy);
+    void scale(double sx, double sy, KoUpdater *m_progress, KisFilterStrategy *filterStrategy, bool scaleOnlyShapes = false);
 
     /**
      * Execute a rotate transform on all layers in this image.
@@ -226,20 +240,29 @@ public:
     /**
      * Convert the image and all its layers to the dstColorSpace
      */
-    void convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent = KoColorConversionTransformation::IntentPerceptual);
+    void convertImageColorSpace(const KoColorSpace *dstColorSpace, KoColorConversionTransformation::Intent renderingIntent = KoColorConversionTransformation::IntentPerceptual);
+
+    /**
+     * Set the color space of  the projection (and the root layer)
+     * to dstColorSpace. No conversion is done for other layers,
+     * their colorspace can differ.
+     * NOTE: Note conversion is done, only regeneration, so no rendering
+     * intent needed
+     */
+    void convertProjectionColorSpace(const KoColorSpace *dstColorSpace);
 
     // Get the profile associated with this image
     const KoColorProfile *  profile() const;
 
     /**
      * Set the profile of the image to the new profile and do the same for
-     * all layers that have the same colorspace and profile as the image.
+     * all layers that have the same colorspace and profile of the image.
      * It doesn't do any pixel conversion.
      *
      * This is essential if you have loaded an image that didn't
      * have an embedded profile to which you want to attach the right profile.
      */
-    void setProfile(const KoColorProfile * profile);
+    void assignImageProfile(const KoColorProfile *profile);
 
     /**
      * Replace the current undo adapter with the specified undo adapter.
@@ -258,11 +281,6 @@ public:
      */
     KisActionRecorder* actionRecorder() const;
 
-    /**
-     * Returns true if this image wants undo information, false otherwise
-     */
-    bool undo() const;
-
     /**
      * Tell the image it's modified; this emits the sigImageModified
      * signal. This happens when the image needs to be saved
@@ -358,12 +376,20 @@ public:
     }
 
     /**
-     * returns a paintdevice that contains the merged layers of this
-     * image, within the bounds of this image (with the colorspace and
-     * profile of this image) It will cause a synchronous update to the
-     * projection.
+     * Starting form 2.3 mergedImage() is declared deprecated.
+     * If you want to get a projection of the image, please use
+     * something like:
+     *
+     * image->lock();
+     * read_something_from_the_image(image->projection());
+     * image->unlock();
+     *
+     * or if you want to get a full refresh of the image graph
+     * performed beforehand (do you really want it?) (sure?) then
+     * you can add a call to image->refreshGraph() before locking
+     * the image.
      */
-    KisPaintDeviceSP mergedImage();
+    KDE_DEPRECATED KisPaintDeviceSP mergedImage();
 
     /**
      * @return the root node of the image node graph
@@ -426,8 +452,6 @@ public:
     */
     void notifyLayerUpdated(KisLayerSP layer);
 
-    void setColorSpace(const KoColorSpace * colorSpace);
-
     void setRootLayer(KisGroupLayerSP rootLayer);
 
     /**
@@ -483,7 +507,7 @@ signals:
     void sigImageModified();
 
     void sigSizeChanged(qint32 w, qint32 h);
-    void sigProfileChanged(KoColorProfile *  profile);
+    void sigProfileChanged(const KoColorProfile *  profile);
     void sigColorSpaceChanged(const KoColorSpace*  cs);
     void sigResolutionChanged(double xRes, double yRes);
 
@@ -516,6 +540,11 @@ signals:
      * Inform the model we're done moving a layer.
      */
     void sigNodeHasBeenMoved(KisNode *parent, int oldIndex, int newIndex);
+    
+    /**
+     * Inform the model that a node was changed
+     */
+    void sigNodeChanged(KisNode * node);
 
 public slots:
 
@@ -526,13 +555,21 @@ public slots:
      * Triggers synchronous recomposition of the projection
      */
     void refreshGraph(KisNodeSP root = 0);
+    void refreshGraph(KisNodeSP root, const QRect& rc, const QRect &cropRect);
 
 private:
     KisImage& operator=(const KisImage& rhs);
     void init(KisUndoAdapter * adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace);
     void emitSizeChanged();
-    void preparePaintLayerAfterAdding(KisLayerSP layer);
 
+    void refreshHiddenArea(KisNodeSP rootNode, const QRect &preparedArea);
+    static QRect realNodeExtent(KisNodeSP rootNode, QRect currentRect = QRect());
+
+    friend class KisImageResizeCommand;
+    void setSize(const QSize& size);
+
+    friend class KisImageSetProjectionColorSpaceCommand;
+    void setProjectionColorSpace(const KoColorSpace * colorSpace);
 private:
     class KisImagePrivate;
     KisImagePrivate * const m_d;
