@@ -52,7 +52,7 @@ struct KisSelection::Private {
     KisSelectionComponent* shapeSelection;
 };
 
-KisSelection::KisSelection(KisPaintDeviceSP dev, KisDefaultBounds defaultBounds)
+KisSelection::KisSelection(KisPaintDeviceSP dev, KisDefaultBoundsSP defaultBounds)
         : KisPaintDevice(0, KoColorSpaceRegistry::instance()->alpha8(), defaultBounds)
         , m_d(new Private)
 {
@@ -69,7 +69,7 @@ KisSelection::KisSelection(KisPaintDeviceSP dev, KisDefaultBounds defaultBounds)
 }
 
 
-KisSelection::KisSelection(KisPaintDeviceSP parent, KisMaskSP mask, KisDefaultBounds defaultBounds)
+KisSelection::KisSelection(KisPaintDeviceSP parent, KisMaskSP mask, KisDefaultBoundsSP defaultBounds)
         : KisPaintDevice(0, KoColorSpaceRegistry::instance()->alpha8(), defaultBounds)
         , m_d(new Private)
 {
@@ -96,7 +96,7 @@ KisSelection::KisSelection(KisPaintDeviceSP parent, KisMaskSP mask, KisDefaultBo
      * with the pixel selection until we get an additional
      * selection component
      */
-    m_datamanager = m_d->pixelSelection->dataManager();
+    setDataManager(m_d->pixelSelection->dataManager());
 }
 
 KisSelection::KisSelection()
@@ -122,7 +122,7 @@ KisSelection::KisSelection(const KisSelection& rhs)
     m_d->interestedInDirtyness = false;
     if (rhs.m_d->hasPixelSelection) {
         m_d->pixelSelection = new KisPixelSelection(*rhs.m_d->pixelSelection.data());
-        m_d->pixelSelection->dataManager()->setDefaultPixel(rhs.m_d->pixelSelection->dataManager()->defaultPixel());
+        m_d->pixelSelection->setDefaultPixel(rhs.m_d->pixelSelection->defaultPixel());
     }
     m_d->hasPixelSelection = rhs.m_d->hasPixelSelection;
     m_d->isDeselected = rhs.m_d->isDeselected;
@@ -160,8 +160,8 @@ quint8 KisSelection::selected(qint32 x, qint32 y) const
 void KisSelection::clear()
 {
     quint8 defPixel = MIN_SELECTED;
-    dataManager()->setDefaultPixel(&defPixel);
-    dataManager()->clear();
+    setDefaultPixel(&defPixel);
+    KisPaintDevice::clear();
 }
 
 void KisSelection::clear(const QRect& r)
@@ -169,13 +169,13 @@ void KisSelection::clear(const QRect& r)
 
     KisFillPainter painter(KisPaintDeviceSP(this));
     const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
-    painter.fillRect(r, KoColor(Qt::white, cs), *(dataManager()->defaultPixel()));
+    painter.fillRect(r, KoColor(Qt::white, cs), *(defaultPixel()));
 }
 
 
 bool KisSelection::isTotallyUnselected(const QRect & r) const
 {
-    if (*(dataManager()->defaultPixel()) != MIN_SELECTED)
+    if (*defaultPixel() != MIN_SELECTED)
         return false;
     QRect sr = selectedExactRect();
     return ! r.intersects(sr);
@@ -183,7 +183,7 @@ bool KisSelection::isTotallyUnselected(const QRect & r) const
 
 bool KisSelection::isProbablyTotallyUnselected(const QRect & r) const
 {
-    if (*(dataManager()->defaultPixel()) != MIN_SELECTED)
+    if (*defaultPixel() != MIN_SELECTED)
         return false;
     QRect sr = selectedRect();
     return ! r.intersects(sr);
@@ -191,20 +191,12 @@ bool KisSelection::isProbablyTotallyUnselected(const QRect & r) const
 
 QRect KisSelection::selectedRect() const
 {
-    if (*(m_datamanager->defaultPixel()) == MIN_SELECTED) {
-        return extent().intersected(defaultBounds().bounds());
-    } else {
-        return defaultBounds().bounds();
-    }
+    return extent();
 }
 
 QRect KisSelection::selectedExactRect() const
 {
-    if (*(m_datamanager->defaultPixel()) == MIN_SELECTED) {
-        return exactBounds().intersected(defaultBounds().bounds());
-    } else {
-        return defaultBounds().bounds();
-    }
+    return exactBounds();
 }
 
 void KisSelection::setInterestedInDirtyness(bool b)
@@ -269,7 +261,7 @@ KisPixelSelectionSP KisSelection::getOrCreatePixelSelection()
     }
     // Share the datamanager unless there's a shape selection
     if (!m_d->hasShapeSelection) {
-        m_datamanager = m_d->pixelSelection->dataManager();
+        setDataManager(m_d->pixelSelection->dataManager());
     }
     return m_d->pixelSelection;
 }
@@ -280,7 +272,7 @@ void KisSelection::setPixelSelection(KisPixelSelectionSP pixelSelection)
     m_d->hasPixelSelection = true;
     // Share the datamanager unless there's a shape selection
     if (!m_d->hasShapeSelection) {
-        m_datamanager = pixelSelection->dataManager();
+        setDataManager(pixelSelection->dataManager());
     }
 }
 
@@ -290,8 +282,8 @@ void KisSelection::setShapeSelection(KisSelectionComponent* shapeSelection)
     m_d->hasShapeSelection = true;
     Q_ASSERT(m_d->shapeSelection);
     // Unshare the data manager of the pixel selection
-    if (m_d->hasPixelSelection && m_datamanager == m_d->pixelSelection->dataManager()) {
-        m_datamanager = new KisDataManager(1, m_d->pixelSelection->dataManager()->defaultPixel());
+    if (m_d->hasPixelSelection && dataManager() == m_d->pixelSelection->dataManager()) {
+        setDataManager(new KisDataManager(1, m_d->pixelSelection->defaultPixel()));
     }
 }
 
@@ -303,8 +295,8 @@ void KisSelection::updateProjection()
 
     clear();
     if (m_d->hasPixelSelection) {
-        quint8 defPixel = *(m_d->pixelSelection->dataManager()->defaultPixel());
-        dataManager()->setDefaultPixel(&defPixel);
+        quint8 defPixel = *(m_d->pixelSelection->defaultPixel());
+        setDefaultPixel(&defPixel);
         m_d->pixelSelection->renderToProjection(this);
     }
     m_d->shapeSelection->renderToProjection(this);
@@ -320,8 +312,8 @@ void KisSelection::updateProjection(const QRect& r)
 
     clear(r);
     if (m_d->hasPixelSelection) {
-        quint8 defPixel = *(m_d->pixelSelection->dataManager()->defaultPixel());
-        dataManager()->setDefaultPixel(&defPixel);
+        quint8 defPixel = *(m_d->pixelSelection->defaultPixel());
+        setDefaultPixel(&defPixel);
 
         m_d->pixelSelection->renderToProjection(this, r);
     }
@@ -348,7 +340,7 @@ bool KisSelection::isVisible()
     return m_d->isVisible;
 }
 
-void KisSelection::setDefaultBounds(KisDefaultBounds bounds)
+void KisSelection::setDefaultBounds(KisDefaultBoundsSP bounds)
 {
     KisPaintDevice::setDefaultBounds(bounds);
     if(m_d->hasPixelSelection) {
@@ -356,3 +348,28 @@ void KisSelection::setDefaultBounds(KisDefaultBounds bounds)
     }
 }
 
+void KisSelection::setX(qint32 x)
+{
+    qint32 delta = x - this->x();
+    KisPaintDevice::setX(x);
+    if (hasPixelSelection()) {
+        m_d->pixelSelection->setX(x);
+    }
+    if (hasShapeSelection()) {
+        m_d->shapeSelection->moveX(delta);
+    }
+}
+
+void KisSelection::setY(qint32 y)
+{
+    qint32 delta = y - this->y();
+    KisPaintDevice::setY(y);
+    if (hasPixelSelection()) {
+        m_d->pixelSelection->setY(y);
+    }
+    if (hasShapeSelection()) {
+        m_d->shapeSelection->moveY(delta);
+    }
+}
+
+
