@@ -270,8 +270,7 @@ void KisRecordedPaintAction::play(KisNodeSP node, const KisPlayInfo& info) const
     QList<KisNodeSP> nodes = nodeQueryPath().queryNodes(info.image(), info.currentNode());
     foreach(KisNodeSP node, nodes) {
         dbgUI << "Play recorded paint action on node : " << node->name() ;
-        KisTransaction * cmd = 0;
-        if (info.undoAdapter()) cmd = new KisTransaction("", node->paintDevice());
+        KisTransaction transaction("", node->paintDevice());
 
         KisPaintDeviceSP target = 0;
         if (d->paintIncremental) {
@@ -312,7 +311,7 @@ void KisRecordedPaintAction::play(KisNodeSP node, const KisPlayInfo& info) const
             painter2.setCompositeOp(d->compositeOp);
             painter2.setOpacity(d->opacity * 255);
 
-            QRegion r = painter->dirtyRegion();
+            QRegion r = painter->takeDirtyRegion();
             QVector<QRect> dirtyRects = r.rects();
             QVector<QRect>::iterator it = dirtyRects.begin();
             QVector<QRect>::iterator end = dirtyRects.end();
@@ -321,14 +320,16 @@ void KisRecordedPaintAction::play(KisNodeSP node, const KisPlayInfo& info) const
                 ++it;
             }
 
-            node->setDirty(painter2.dirtyRegion());
+            node->setDirty(painter2.takeDirtyRegion());
         } else {
-            node->setDirty(painter->dirtyRegion());
+            node->setDirty(painter->takeDirtyRegion());
         }
         delete painter;
-        if (info.undoAdapter()) info.undoAdapter()->addCommand(cmd);
+
         if (d->paintOpPreset)
             d->paintOpPreset->settings()->setNode(0);
+
+        transaction.commit(info.undoAdapter());
     }
 }
 
