@@ -42,6 +42,8 @@ private slots:
 //    void testThreadedWriteAccess();
 //    void testThreadedReadWriteAccess();
 
+private:
+    bool memoryIsFilled(quint8 c, quint8 *mem, qint32 size);
 };
 
 #endif
