@@ -120,17 +120,17 @@ void KisPaintDeviceTest::testGeometry()
     cs->fromQColor(Qt::white, pixel);
     dev->fill(0, 0, 512, 512, pixel);
 
-    QVERIFY(dev->exactBounds() == QRect(0, 0, 512, 512));
-    QVERIFY(dev->extent() == QRect(0, 0, 512, 512));
+    QCOMPARE(dev->exactBounds(), QRect(0, 0, 512, 512));
+    QCOMPARE(dev->extent(), QRect(0, 0, 512, 512));
 
     dev->move(10, 10);
 
-    QVERIFY(dev->exactBounds() == QRect(10, 10, 512, 512));
-    QVERIFY(dev->extent() == QRect(10, 10, 512, 512));
+    QCOMPARE(dev->exactBounds(), QRect(10, 10, 512, 512));
+    QCOMPARE(dev->extent(), QRect(10, 10, 512, 512));
 
     dev->crop(50, 50, 50, 50);
-    QVERIFY(dev->exactBounds() == QRect(50, 50, 50, 50));
-    QVERIFY(dev->extent() == QRect(50, 50, 50, 50));
+    QCOMPARE(dev->exactBounds(), QRect(50, 50, 50, 50));
+    QCOMPARE(dev->extent(), QRect(10, 10, 128, 128));
 
     QColor c;
 
@@ -147,11 +147,8 @@ void KisPaintDeviceTest::testGeometry()
     dev->pixel(80, 80, &c);
     QVERIFY(c.alpha() == OPACITY_TRANSPARENT_U8);
 
-    // XXX: No idea why we get this extent and bounds after a clear --
-    // but I want to know as soon as possible if this behaviour
-    // changes in any way.
-    QVERIFY(dev->extent() == QRect(74, 74, 64, 64));
-    QVERIFY(dev->exactBounds() == QRect(74, 74, 64, 64));
+    QVERIFY(dev->extent().isEmpty());
+    QVERIFY(dev->exactBounds().isEmpty());
 
 }
 
@@ -276,6 +273,82 @@ void KisPaintDeviceTest::testRoundtripConversion()
     }
 }
 
+void KisPaintDeviceTest::testFastBitBlt()
+{
+    QImage image(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+    KisPaintDeviceSP dstDev = new KisPaintDevice(cs);
+    KisPaintDeviceSP srcDev = new KisPaintDevice(cs);
+    srcDev->convertFromQImage(image, "");
+
+    QRect cloneRect(100,100,200,200);
+    QPoint errpoint;
+
+
+    QVERIFY(dstDev->fastBitBltPossible(srcDev));
+    dstDev->fastBitBlt(srcDev, cloneRect);
+
+    QImage srcImage = srcDev->convertToQImage(0, cloneRect.x(), cloneRect.y(),
+                                               cloneRect.width(), cloneRect.height());
+    QImage dstImage = dstDev->convertToQImage(0, cloneRect.x(), cloneRect.y(),
+                                              cloneRect.width(), cloneRect.height());
+
+    if (!TestUtil::compareQImages(errpoint, srcImage, dstImage)) {
+        QFAIL(QString("Failed to create identical image, first different pixel: %1,%2 \n").arg(errpoint.x()).arg(errpoint.y()).toAscii());
+    }
+
+    // Test Rough version
+    dstDev->clear();
+    dstDev->fastBitBltRough(srcDev, cloneRect);
+
+    srcImage = srcDev->convertToQImage(0, cloneRect.x(), cloneRect.y(),
+                                       cloneRect.width(), cloneRect.height());
+    dstImage = dstDev->convertToQImage(0, cloneRect.x(), cloneRect.y(),
+                                       cloneRect.width(), cloneRect.height());
+
+    if (!TestUtil::compareQImages(errpoint, srcImage, dstImage)) {
+        QFAIL(QString("Failed to create identical image, first different pixel: %1,%2 \n").arg(errpoint.x()).arg(errpoint.y()).toAscii());
+    }
+
+    srcDev->move(10,10);
+    QVERIFY(!dstDev->fastBitBltPossible(srcDev));
+}
+
+void KisPaintDeviceTest::testMakeClone()
+{
+    QImage image(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
+
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+    KisPaintDeviceSP srcDev = new KisPaintDevice(cs);
+    srcDev->convertFromQImage(image, "");
+    srcDev->move(10,10);
+
+    const KoColorSpace * weirdCS = KoColorSpaceRegistry::instance()->lab16();
+    KisPaintDeviceSP dstDev = new KisPaintDevice(weirdCS);
+    dstDev->move(1000,1000);
+
+    QVERIFY(!dstDev->fastBitBltPossible(srcDev));
+
+    QRect cloneRect(100,100,200,200);
+    QPoint errpoint;
+
+    dstDev->makeCloneFrom(srcDev, cloneRect);
+
+    QVERIFY(*dstDev->colorSpace() == *srcDev->colorSpace());
+    QCOMPARE(dstDev->pixelSize(), srcDev->pixelSize());
+    QCOMPARE(dstDev->x(), srcDev->x());
+    QCOMPARE(dstDev->y(), srcDev->y());
+    QCOMPARE(dstDev->exactBounds(), cloneRect);
+
+    QImage srcImage = srcDev->convertToQImage(0, cloneRect.x(), cloneRect.y(),
+                                              cloneRect.width(), cloneRect.height());
+    QImage dstImage = dstDev->convertToQImage(0, cloneRect.x(), cloneRect.y(),
+                                              cloneRect.width(), cloneRect.height());
+    if (!TestUtil::compareQImages(errpoint, dstImage, srcImage)) {
+        QFAIL(QString("Failed to create identical image, first different pixel: %1,%2 \n").arg(errpoint.x()).arg(errpoint.y()).toAscii());
+    }
+}
+
 void KisPaintDeviceTest::testThumbnail()
 {
     QImage image(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
@@ -290,11 +363,72 @@ void KisPaintDeviceTest::testThumbnail()
     }
     {
         QImage thumb = dev->createThumbnail(50, 50);
+        QVERIFY(!thumb.isNull());
         QVERIFY(thumb.width() <= 50);
         QVERIFY(thumb.height() <= 50);
         image.save("kis_paint_device_test_test_thumbnail.png");
     }
+}
 
+void KisPaintDeviceTest::testCaching()
+{
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+    KisPaintDeviceSP dev = new KisPaintDevice(cs);
+
+    quint8* whitePixel = cs->allocPixelBuffer(1);
+    cs->fromQColor(Qt::white, whitePixel);
+
+    quint8* blackPixel = cs->allocPixelBuffer(1);
+    cs->fromQColor(Qt::black, blackPixel);
+
+    dev->fill(0, 0, 512, 512, whitePixel);
+    QImage thumb1 = dev->createThumbnail(50, 50);
+    QRect exactBounds1 = dev->exactBounds();
+
+    dev->fill(0, 0, 768, 768, blackPixel);
+    QImage thumb2 = dev->createThumbnail(50, 50);
+    QRect exactBounds2 = dev->exactBounds();
+
+    dev->move(10, 10);
+    QImage thumb3 = dev->createThumbnail(50, 50);
+    QRect exactBounds3 = dev->exactBounds();
+
+    dev->crop(50, 50, 50, 50);
+    QImage thumb4 = dev->createThumbnail(50, 50);
+    QRect exactBounds4 = dev->exactBounds();
+
+    QVERIFY(thumb1 != thumb2);
+    QVERIFY(thumb2 == thumb3); // Cache miss, but image is the same
+    QVERIFY(thumb3 != thumb4);
+    QVERIFY(thumb4 != thumb1);
+
+    QCOMPARE(exactBounds1, QRect(0,0,512,512));
+    QCOMPARE(exactBounds2, QRect(0,0,768,768));
+    QCOMPARE(exactBounds3, QRect(10,10,768,768));
+    QCOMPARE(exactBounds4, QRect(50,50,50,50));
+}
+
+void KisPaintDeviceTest::testRegion()
+{
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+    KisPaintDeviceSP dev = new KisPaintDevice(cs);
+
+    quint8* whitePixel = cs->allocPixelBuffer(1);
+    cs->fromQColor(Qt::white, whitePixel);
+
+    dev->fill(0, 0, 10, 10, whitePixel);
+    dev->fill(70, 70, 10, 10, whitePixel);
+    dev->fill(129, 0, 10, 10, whitePixel);
+    dev->fill(0, 1030, 10, 10, whitePixel);
+
+    QRegion referenceRegion;
+    referenceRegion += QRect(0,0,64,64);
+    referenceRegion += QRect(64,64,64,64);
+    referenceRegion += QRect(128,0,64,64);
+    referenceRegion += QRect(0,1024,64,64);
+
+    QCOMPARE(dev->exactBounds(), QRect(0,0,139,1040));
+    QCOMPARE(dev->region(), referenceRegion);
 }
 
 void KisPaintDeviceTest::testPixel()
@@ -411,14 +545,15 @@ void KisPaintDeviceTest::testDeviceDuplication()
 //    qDebug()<<"FILLING";
     device->fill(fillRect.left(), fillRect.top(),
                  fillRect.width(), fillRect.height(),fillPixel);
-
     referenceImage = device->convertToQImage(0);
 
 
     KisTransaction transaction1("", device);
 //    qDebug()<<"CLEARING";
     device->clear(clearRect);
-    transaction1.undo();
+
+    transaction1.revert();
+
     resultImage = device->convertToQImage(0);
     QVERIFY(resultImage == referenceImage);
 
@@ -427,7 +562,9 @@ void KisPaintDeviceTest::testDeviceDuplication()
     KisTransaction transaction("", clone);
 //    qDebug()<<"CLEARING";
     clone->clear(clearRect);
-    transaction.undo();
+
+    transaction.revert();
+
     resultImage = clone->convertToQImage(0);
     QVERIFY(resultImage == referenceImage);
 
@@ -450,6 +587,32 @@ void KisPaintDeviceTest::testTranslate()
     QCOMPARE(device->extent(), QRect(-10,10,64,64));
 }
 
+void KisPaintDeviceTest::testOpacity()
+{
+    // blt a semi-transparent image on a white paint device
+
+    QImage image(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa_transparent.png");
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+    KisPaintDeviceSP fdev = new KisPaintDevice(cs);
+    fdev->convertFromQImage(image, "");
+
+    KisPaintDeviceSP dev = new KisPaintDevice(cs);
+    dev->fill(0, 0, 640, 441, KoColor(Qt::white, cs).data());
+    KisPainter gc(dev);
+    gc.bitBlt(QPoint(0, 0), fdev, image.rect());
+
+    QImage result = dev->convertToQImage(0, 0, 0, 640, 441);
+    QImage checkResult(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa_transparent_result.png");
+    QPoint errpoint;
+
+    if (!TestUtil::compareQImages(errpoint, checkResult, result)) {
+        checkResult.save("kis_paint_device_test_test_blt_fixed_opactiy_expected.png");
+        result.save("kis_paint_device_test_test_blt_fixed_opacity_result.png");
+        QFAIL(QString("Failed to create identical image, first different pixel: %1,%2 \n").arg(errpoint.x()).arg(errpoint.y()).toAscii());
+    }
+}
+
+
 QTEST_KDEMAIN(KisPaintDeviceTest, GUI)
 #include "kis_paint_device_test.moc"
 
