@@ -26,6 +26,8 @@
 #include "kis_tiled_data_manager.h"
 #include "kis_tiled_data_manager_p.h"
 #include "kis_memento_manager.h"
+#include "swap/kis_legacy_tile_compressor.h"
+#include "swap/kis_tile_compressor_factory.h"
 
 #include <KoStore.h>
 
@@ -64,7 +66,10 @@ KisTiledDataManager::KisTiledDataManager(const KisTiledDataManager &dm)
         m_lock(QReadWriteLock::NonRecursive)
 {
     /* See comment in destructor for details */
-    m_mementoManager = new KisMementoManager(*dm.m_mementoManager);
+
+    /* We do not clone the history of the device, there is no usecase for it */
+    m_mementoManager = new KisMementoManager();
+    m_mementoManager->setDefaultTileData(dm.m_hashTable->defaultTileData());
     m_hashTable = new KisTileHashTable(*dm.m_hashTable, m_mementoManager);
 
     m_pixelSize = dm.m_pixelSize;
@@ -74,7 +79,6 @@ KisTiledDataManager::KisTiledDataManager(const KisTiledDataManager &dm)
      * has already been made shared in m_hashTable(dm->m_hashTable)
      */
     memcpy(m_defaultPixel, dm.m_defaultPixel, m_pixelSize);
-    m_defaultTile = new KisTile(-1,-1, m_hashTable->defaultTileData(), 0);
 
     m_extentMinX = dm.m_extentMinX;
     m_extentMinY = dm.m_extentMinY;
@@ -102,11 +106,9 @@ void KisTiledDataManager::setDefaultPixel(const quint8 *defaultPixel)
 {
     QWriteLocker locker(&m_lock);
 
-    KisTileData *td = globalTileDataStore.createDefaultTileData(pixelSize(),
-                      defaultPixel);
+    KisTileData *td = KisTileDataStore::instance()->createDefaultTileData(pixelSize(), defaultPixel);
     m_hashTable->setDefaultTileData(td);
     m_mementoManager->setDefaultTileData(td);
-    m_defaultTile = new KisTile(-1,-1, td, 0);
 
     memcpy(m_defaultPixel, defaultPixel, pixelSize());
 }
@@ -117,96 +119,173 @@ bool KisTiledDataManager::write(KoStore *store)
     QReadLocker locker(&m_lock);
     if (!store) return false;
 
-    char str[80];
-
-    sprintf(str, "%d\n", m_hashTable->numTiles());
-    store->write(str, strlen(str));
+    if(CURRENT_VERSION == LEGACY_VERSION) {
+        char str[80];
+        sprintf(str, "%d\n", m_hashTable->numTiles());
+        store->write(str, strlen(str));
+    }
+    else {
+        writeTilesHeader(store, m_hashTable->numTiles());
+    }
 
 
     KisTileHashTableIterator iter(m_hashTable);
     KisTileSP tile;
-    qint32 x, y;
-    qint32 width, height;
 
-    const qint32 tileDataSize = KisTileData::HEIGHT * KisTileData::WIDTH * pixelSize();
+    KisAbstractTileCompressorSP compressor =
+        KisTileCompressorFactory::create(CURRENT_VERSION);
 
     while (tile = iter.tile()) {
-        tile->extent().getRect(&x, &y, &width, &height);
-        sprintf(str, "%d,%d,%d,%d\n", x, y, width, height);
-        store->write(str, strlen(str));
-
-        tile->lockForRead();
-        store->write((char *)tile->data(), tileDataSize);
-        tile->unlock();
-
+        compressor->writeTile(tile, store);
         ++iter;
     }
 
-    printf("*** KisTiledDataManager::write called \n");
     return true;
 }
 bool KisTiledDataManager::read(KoStore *store)
 {
-    QWriteLocker locker(&m_lock);
     if (!store) return false;
+    clear();
 
-    //clear(); - needed?
-
+    QWriteLocker locker(&m_lock);
     KisMementoSP nothing = m_mementoManager->getMemento();
 
-    KisTileSP tile;
-    const qint32 tileDataSize = KisTileData::HEIGHT * KisTileData::WIDTH * pixelSize();
-    qint32 x, y;
-    qint32 width, height;
-    char str[80];
-
     QIODevice *stream = store->device();
     if (!stream) {
-      nothing->setInvalid();
-      return false;
+        m_mementoManager->commit();
+        return false;
     }
 
+    const qint32 maxLineLength = 79; // Legacy magic
+    QByteArray line = stream->readLine(maxLineLength);
+    line = line.trimmed();
+
     quint32 numTiles;
-    stream->readLine(str, 79);
-    sscanf(str, "%u", &numTiles);
+    qint32 tilesVersion = LEGACY_VERSION;
 
-    for (quint32 i = 0; i < numTiles; i++) {
-        stream->readLine(str, 79);
-        sscanf(str, "%d,%d,%d,%d", &x, &y, &width, &height);
+    if (line[0] == 'V') {
+        QList<QByteArray> lineItems = line.split(' ');
 
-        // the following is only correct as long as tile size is not changed
-        // The first time we change tilesize the dimensions just read needs to be respected
-        // but for now we just assume that tiles are the same size as ever.
-        qint32 row = yToRow(y);
-        qint32 col = xToCol(x);
-        bool created;
+        QString keyword = lineItems.takeFirst();
+        Q_ASSERT(keyword == "VERSION");
 
-        tile = m_hashTable->getTileLazy(col, row, created);
-        if (created)
-            updateExtent(col, row);
+        tilesVersion = lineItems.takeFirst().toInt();
 
-        tile->lockForWrite();
-        store->read((char *)tile->data(), tileDataSize);
-        tile->unlock();
+        if(!processTilesHeader(stream, numTiles))
+            return false;
+    }
+    else {
+        numTiles = line.toUInt();
     }
 
-    nothing->setInvalid();
+    KisAbstractTileCompressorSP compressor =
+        KisTileCompressorFactory::create(tilesVersion);
+
+    for (quint32 i = 0; i < numTiles; i++) {
+        compressor->readTile(store, this);
+    }
+
+    m_mementoManager->commit();
     return true;
 }
 
+bool KisTiledDataManager::writeTilesHeader(KoStore *store, quint32 numTiles)
+{
+    QString buffer;
+
+    buffer = QString("VERSION %1\n"
+                     "TILEWIDTH %2\n"
+                     "TILEHEIGHT %3\n"
+                     "PIXELSIZE %4\n"
+                     "DATA %5\n")
+        .arg(CURRENT_VERSION)
+        .arg(KisTileData::WIDTH)
+        .arg(KisTileData::HEIGHT)
+        .arg(pixelSize())
+        .arg(numTiles);
+
+    store->write(buffer.toAscii());
+    return true;
+}
+
+#define takeOneLine(stream, maxLine, keyword, value)            \
+    do {                                                        \
+        QByteArray line = stream->readLine(maxLine);            \
+        line = line.trimmed();                                  \
+        QList<QByteArray> lineItems = line.split(' ');          \
+        keyword = lineItems.takeFirst();                        \
+        value = lineItems.takeFirst().toInt();                  \
+    } while(0)                                                  \
+
+
+bool KisTiledDataManager::processTilesHeader(QIODevice *stream, quint32 &numTiles)
+{
+    /**
+     * We assume that there is only one version of this header
+     * possible. In case we invent something new, it'll be quite easy
+     * to modify the behavior
+     */
+
+    const qint32 maxLineLength = 25;
+    const qint32 totalNumTests = 4;
+    bool foundDataMark = false;
+    qint32 testsPassed = 0;
+
+    QString keyword;
+    qint32 value;
+
+    while(!foundDataMark && stream->canReadLine()) {
+        takeOneLine(stream, maxLineLength, keyword, value);
+
+        if (keyword == "TILEWIDTH") {
+            if(value != KisTileData::WIDTH)
+                goto wrongString;
+        }
+        else if (keyword == "TILEHEIGHT") {
+            if(value != KisTileData::HEIGHT)
+                goto wrongString;
+        }
+        else if (keyword == "PIXELSIZE") {
+            if((quint32)value != pixelSize())
+                goto wrongString;
+        }
+        else if (keyword == "DATA") {
+            numTiles = value;
+            foundDataMark = true;
+        }
+        else {
+            goto wrongString;
+        }
+
+        testsPassed++;
+    }
+
+    if(testsPassed != totalNumTests) {
+        warnTiles << "Not enough fields of tiles header present"
+                  << testsPassed << "of" << totalNumTests;
+    }
+
+    return testsPassed == totalNumTests;
+
+wrongString:
+    warnTiles << "Wrong string in tiles header:" << keyword << value;
+    return false;
+}
+
 void KisTiledDataManager::purge(const QRect& area)
 {
+    QWriteLocker locker(&m_lock);
+
     QList<KisTileSP> tilesToDelete;
     {
-        QWriteLocker locker(&m_lock);
+        const qint32 tileDataSize = KisTileData::HEIGHT * KisTileData::WIDTH * pixelSize();
+        const quint8 *defaultData = m_hashTable->defaultTileData()->data();
 
         KisTileHashTableIterator iter(m_hashTable);
         KisTileSP tile;
 
-        const qint32 tileDataSize = KisTileData::HEIGHT * KisTileData::WIDTH * pixelSize();
-
         while (tile = iter.tile()) {
-            if (tile->extent().intersects(area) && memcmp(m_defaultTile->data(), tile->data(), tileDataSize) == 0) {
+            if (tile->extent().intersects(area) && memcmp(defaultData, tile->data(), tileDataSize) == 0) {
                 tilesToDelete.push_back(tile);
             }
             ++iter;
@@ -275,9 +354,9 @@ void KisTiledDataManager::clear(QRect clearRect, const quint8 *clearPixel)
              */
             td = m_hashTable->defaultTileData();
         else
-            td = globalTileDataStore.createDefaultTileData(pixelSize,
-                    clearPixel);
-        globalTileDataStore.acquireTileData(td);
+            td = KisTileDataStore::instance()->createDefaultTileData(pixelSize, clearPixel);
+
+        td->acquire();
     }
 
     for (qint32 row = firstRow; row <= lastRow; ++row) {
@@ -288,27 +367,11 @@ void KisTiledDataManager::clear(QRect clearRect, const quint8 *clearPixel)
             QRect clearTileRect = clearRect & tileRect;
 
             if (clearTileRect == tileRect) {
-//                 // Clear whole tile
-//                 m_hashTable->deleteTile(column, row);
-//
-//                 KisTileSP clearedTile = new KisTile(column, row, td, m_mementoManager);
-//                 m_hashTable->addTile(clearedTile);
-//                 updateExtent(column, row);
-
-                /**
-                 * Emulate like we've written something to the tile
-                 * to keep the history clean
-                 */
-                // Trigger artificial memory increase when clearing
-//                 KisTileDataWrapper tw(clearedTile, 0,
-//                                       KisTileDataWrapper::WRITE);
-//                const qint32 lineSize = clearTileRect.width() * pixelSize;
-//                qint32 rowsRemaining = clearTileRect.height();
-
-                KisTileDataWrapper tw = pixelPtr(clearTileRect.left(),
-                                                 clearTileRect.top(),
-                                                 KisTileDataWrapper::WRITE);
-                memcpy(tw.data(), td->data(), KisTileData::WIDTH * KisTileData::HEIGHT * td->pixelSize());
+                 // Clear whole tile
+                 m_hashTable->deleteTile(column, row);
+                 KisTileSP clearedTile = new KisTile(column, row, td, m_mementoManager);
+                 m_hashTable->addTile(clearedTile);
+                 updateExtent(column, row);
             } else {
                 const qint32 lineSize = clearTileRect.width() * pixelSize;
                 qint32 rowsRemaining = clearTileRect.height();
@@ -334,7 +397,7 @@ void KisTiledDataManager::clear(QRect clearRect, const quint8 *clearPixel)
             }
         }
     }
-    if (td) globalTileDataStore.releaseTileData(td);
+    if (td) td->release();
     delete[] clearPixelData;
 }
 
@@ -367,6 +430,101 @@ void KisTiledDataManager::clear()
     m_extentMaxY = qint32_MIN;
 }
 
+void KisTiledDataManager::bitBlt(KisTiledDataManager *srcDM, const QRect &rect)
+{
+    QWriteLocker locker(&m_lock);
+
+    if (rect.isEmpty()) return;
+
+    const qint32 pixelSize = this->pixelSize();
+    const quint32 rowStride = KisTileData::WIDTH * pixelSize;
+
+    qint32 firstColumn = xToCol(rect.left());
+    qint32 lastColumn = xToCol(rect.right());
+
+    qint32 firstRow = yToRow(rect.top());
+    qint32 lastRow = yToRow(rect.bottom());
+
+    for (qint32 row = firstRow; row <= lastRow; ++row) {
+        for (qint32 column = firstColumn; column <= lastColumn; ++column) {
+
+            // In most bitBlt code of Krita we do not use oldRawData
+            // so let's just conform this behavior
+            KisTileSP srcTile = srcDM->getTile(column, row, false);
+            QRect tileRect(column*KisTileData::WIDTH, row*KisTileData::HEIGHT,
+                           KisTileData::WIDTH, KisTileData::HEIGHT);
+            QRect cloneTileRect = rect & tileRect;
+
+            if (cloneTileRect == tileRect) {
+                 // Clone whole tile
+                 m_hashTable->deleteTile(column, row);
+
+                 srcTile->lockForRead();
+                 KisTileData *td = srcTile->tileData();
+                 KisTileSP clonedTile = new KisTile(column, row, td, m_mementoManager);
+                 srcTile->unlock();
+
+                 m_hashTable->addTile(clonedTile);
+                 updateExtent(column, row);
+            } else {
+                const qint32 lineSize = cloneTileRect.width() * pixelSize;
+                qint32 rowsRemaining = cloneTileRect.height();
+
+                KisTileDataWrapper tw = pixelPtr(cloneTileRect.left(),
+                                                 cloneTileRect.top(),
+                                                 KisTileDataWrapper::WRITE);
+                srcTile->lockForRead();
+                // We suppose that the shift in both tiles is the same
+                const quint8* srcTileIt = srcTile->data() + tw.offset();
+                quint8* dstTileIt = tw.data();
+
+                while (rowsRemaining > 0) {
+                    memcpy(dstTileIt, srcTileIt, lineSize);
+                    srcTileIt += rowStride;
+                    dstTileIt += rowStride;
+                    rowsRemaining--;
+                }
+
+                srcTile->unlock();
+            }
+        }
+    }
+}
+
+void KisTiledDataManager::bitBltRough(KisTiledDataManager *srcDM, const QRect &rect)
+{
+    QWriteLocker locker(&m_lock);
+
+    if (rect.isEmpty()) return;
+
+    qint32 firstColumn = xToCol(rect.left());
+    qint32 lastColumn = xToCol(rect.right());
+
+    qint32 firstRow = yToRow(rect.top());
+    qint32 lastRow = yToRow(rect.bottom());
+
+    for (qint32 row = firstRow; row <= lastRow; ++row) {
+        for (qint32 column = firstColumn; column <= lastColumn; ++column) {
+
+            /**
+             * We are cloning whole tiles here so let's not be so boring
+             * to check any borders :)
+             */
+
+            KisTileSP srcTile = srcDM->getOldTile(column, row);
+
+            m_hashTable->deleteTile(column, row);
+
+            srcTile->lockForRead();
+            KisTileData *td = srcTile->tileData();
+            KisTileSP clonedTile = new KisTile(column, row, td, m_mementoManager);
+            srcTile->unlock();
+
+            m_hashTable->addTile(clonedTile);
+            updateExtent(column, row);
+        }
+    }
+}
 
 void KisTiledDataManager::setExtent(qint32 x, qint32 y, qint32 w, qint32 h)
 {
@@ -473,7 +631,19 @@ QRect KisTiledDataManager::extent() const
     return QRect(x, y, w, h);
 }
 
+QRegion KisTiledDataManager::region() const
+{
+    QRegion region;
 
+    KisTileHashTableIterator iter(m_hashTable);
+    KisTileSP tile;
+
+    while ((tile = iter.tile())) {
+        region += tile->extent();
+        ++iter;
+    }
+    return region;
+}
 
 KisTileDataWrapper KisTiledDataManager::pixelPtr(qint32 x, qint32 y,
         enum KisTileDataWrapper::accessType type)
