@@ -17,35 +17,22 @@
 #include "ruler_assistant_tool.h"
 #include "kis_ruler_assistant_tool.h"
 
-#include <stdlib.h>
-#include <vector>
+#include <kpluginfactory.h>
 
-#include <qpoint.h>
-
-#include <klocale.h>
-#include <kiconloader.h>
-#include <kcomponentdata.h>
-#include <kmessagebox.h>
-#include <kstandarddirs.h>
-#include <kis_debug.h>
-#include <kgenericfactory.h>
-
-#include <kis_global.h>
-#include <kis_types.h>
 #include <KoToolRegistry.h>
+#include <RulerAssistant.h>
+
+K_PLUGIN_FACTORY(RulerAssistantToolFactory, registerPlugin<RulerAssistantToolPlugin>();)
+K_EXPORT_PLUGIN(RulerAssistantToolFactory("krita"))
 
 
-
-typedef KGenericFactory<RulerAssistantToolPlugin> RulerAssistantToolFactory;
-K_EXPORT_COMPONENT_FACTORY(kritarulerassistanttool, RulerAssistantToolFactory("krita"))
-
-
-RulerAssistantToolPlugin::RulerAssistantToolPlugin(QObject *parent, const QStringList &)
-        : KParts::Plugin(parent)
+RulerAssistantToolPlugin::RulerAssistantToolPlugin(QObject *parent, const QVariantList &)
+        : QObject(parent)
 {
-    setComponentData(RulerAssistantToolFactory::componentData());
     KoToolRegistry * r = KoToolRegistry::instance();
-    r->add(new KisRulerAssistantToolFactory(r, QStringList()));
+    r->add(new KisRulerAssistantToolFactory(r));
+
+    KisPaintingAssistantFactoryRegistry::instance()->add(new RulerAssistantFactory);
 }
 
 RulerAssistantToolPlugin::~RulerAssistantToolPlugin()
