@@ -189,7 +189,6 @@ KisImageBuilder_Result KisJPEGConverter::decode(const KUrl& uri)
         KoColorSpaceRegistry::instance()->colorSpaceId(
       modelId, Integer8BitsColorDepthID.id()))->profileIsCompatible(profile)) {
         warnFile << "The profile " << profile->name() << " is not compatible with the color space model " << modelId;
-        delete profile;
         profile = 0;
     }
 
@@ -225,9 +224,6 @@ KisImageBuilder_Result KisJPEGConverter::decode(const KUrl& uri)
         m_image = new KisImage(m_doc->undoAdapter(),  cinfo.image_width,  cinfo.image_height, cs, "built image");
         Q_CHECK_PTR(m_image);
         m_image->lock();
-        if (profile && !profile->isSuitableForOutput()) {
-            m_image -> addAnnotation(KisAnnotationSP(new KisAnnotation(profile->name(), "", profile_rawdata)));
-        }
     }
 
     // Set resolution
@@ -616,26 +612,10 @@ KisImageBuilder_Result KisJPEGConverter::buildFile(const KUrl& uri, KisPaintLaye
         }
     }
 
-    // Save annotation
-    vKisAnnotationSP_it it = annotationsStart;
-    while (it != annotationsEnd) {
-        if (!(*it) || (*it)->type().isEmpty()) {
-            dbgFile << "Warning: empty annotation";
-            ++it;
-            continue;
-        }
-
-        dbgFile << "Trying to store annotation of type" << (*it) -> type() << " of size" << (*it) -> annotation() . size();
-
-        if ((*it) -> type().startsWith(QLatin1String("krita_attribute:"))) { // Attribute
-            // FIXME
-            dbgFile << "cannot save this annotation :" << (*it) -> type();
-        } else { // Profile
-            write_icc_profile(& cinfo, (uchar*)(*it)->annotation().data(), (*it)->annotation().size());
-        }
-        ++it;
-    }
+    const KoColorProfile* colorProfile = layer->colorSpace()->profile();
+    QByteArray colorProfileData = colorProfile->rawData();
 
+    write_icc_profile(& cinfo, (uchar*) colorProfileData.data(), colorProfileData.size());
 
     // Write data information
 
