@@ -21,6 +21,7 @@
 #include "sensors/kis_dynamic_sensors.h"
 #include "sensors/kis_dynamic_sensor_distance.h"
 #include "sensors/kis_dynamic_sensor_time.h"
+#include "sensors/kis_dynamic_sensor_fade.h"
 
 KisDynamicSensor::KisDynamicSensor(const KoID& id) : m_id(id)
 {
@@ -58,6 +59,8 @@ KisDynamicSensor* KisDynamicSensor::id2Sensor(const KoID& id)
         return new KisDynamicSensorTime();
     } else if (id.id() == FuzzyId.id()) {
         return new KisDynamicSensorFuzzy();
+    } else if (id.id() == FadeId.id()) {
+        return new KisDynamicSensorFade();
     }
 
     dbgPlugins << "Unknown transform parameter :" << id.id();
@@ -86,7 +89,7 @@ KisDynamicSensor* KisDynamicSensor::createFromXML(const QDomElement& e)
 QList<KoID> KisDynamicSensor::sensorsIds()
 {
     QList<KoID> ids;
-    ids << PressureId << XTiltId << YTiltId << SpeedId << DrawingAngleId << RotationId << DistanceId << TimeId << FuzzyId;
+    ids << PressureId << XTiltId << YTiltId << SpeedId << DrawingAngleId << RotationId << DistanceId << TimeId << FuzzyId << FadeId;
     return ids;
 }
 
