@@ -50,12 +50,12 @@
 #include "kis_selection_manager.h"
 #include <commands/kis_image_layer_add_command.h>
 #include <kis_transaction.h>
+#include <commands/kis_deselect_global_selection_command.h>
 
 KisToolMove::KisToolMove(KoCanvasBase * canvas)
         :  KisTool(canvas, KisCursor::moveCursor())
 {
     setObjectName("tool_move");
-    m_dragging = false;
     m_optionsWidget = 0;
 }
 
@@ -82,6 +82,7 @@ KisNodeSP findNode(KisNodeSP node, int x, int y)
         if (cs->opacityU8(color.data()) == OPACITY_TRANSPARENT_U8) {
             return 0;
         }
+        node = node->lastChild();
     }
 
     KisNodeSP foundNode = 0;
@@ -111,16 +112,22 @@ KisNodeSP findNode(KisNodeSP node, int x, int y)
     return 0;
 }
 
-void KisToolMove::mousePressEvent(KoPointerEvent *e)
+void KisToolMove::mousePressEvent(KoPointerEvent *event)
 {
-    if (canvas() && e->button() == Qt::LeftButton) {
-        QPointF pos = convertToPixelCoord(e);
+    if(PRESS_CONDITION_OM(event, KisTool::HOVER_MODE,
+                          Qt::LeftButton,
+                          Qt::ControlModifier | Qt::ShiftModifier)) {
+
+        setMode(KisTool::PAINT_MODE);
 
         KisNodeSP node;
         KisImageWSP image = currentImage();
         if (!image || !image->rootLayer() || image->rootLayer()->childCount() == 0) {
             return;
         }
+
+        QPointF pos = convertToPixelCoord(event);
+
         // shortcut: if the pixel at projection is transparent, it's all transparent
         const KoColorSpace* cs = image->projection()->colorSpace();
         KoColor color(cs);
@@ -128,9 +135,10 @@ void KisToolMove::mousePressEvent(KoPointerEvent *e)
 
         KisSelectionSP selection = currentSelection();
 
-        if (cs->opacityU8(color.data()) == OPACITY_TRANSPARENT_U8
-                || m_optionsWidget->radioSelectedLayer->isChecked()
-                || e->modifiers() == Qt::ControlModifier) {
+        if (cs->opacityU8(color.data()) == OPACITY_TRANSPARENT_U8 ||
+            m_optionsWidget->radioSelectedLayer->isChecked() ||
+            event->modifiers() == Qt::ControlModifier) {
+
             node = currentNode();
         } else {
 
@@ -139,22 +147,19 @@ void KisToolMove::mousePressEvent(KoPointerEvent *e)
             node = findNode(node, pos.x(), pos.y());
 
             // if there is a selection, we cannot move the group
-            if (!selection && (m_optionsWidget->radioGroup->isChecked() || e->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier))) {
+            if (node && !selection && (m_optionsWidget->radioGroup->isChecked() || event->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier))) {
                 node = node->parent();
             }
         }
-
+        Q_ASSERT(node);
         // shouldn't happen
         if (!node) {
             node = currentNode();
         }
 
-        if (currentImage()->undo()) {
-            currentImage()->undoAdapter()->beginMacro(i18n("Move"));
-        }
+        image->undoAdapter()->beginMacro(i18n("Move"));
 
-        if (selection && !selection->isTotallyUnselected(image->bounds())) {
-            //selection->convertToQImage(0).save("selection.png");
+        if (selection && !selection->isTotallyUnselected(image->bounds()) && !selection->isDeselected() && !node->inherits("KisSelectionMask")) {
             // Create a temporary layer with the contents of the selection of the current layer.
             Q_ASSERT(!node->inherits("KisGroupLayer"));
 
@@ -174,12 +179,17 @@ void KisToolMove::mousePressEvent(KoPointerEvent *e)
                 gc.bitBlt(rc.topLeft(), oldLayer->paintDevice(), rc);
                 gc.end();
 
-                // clear the old layer
-                currentImage()->undoAdapter()->addCommand(new KisTransaction("cut", oldLayer->paintDevice()));
-                oldLayer->paintDevice()->clearSelection(selection);
+                {
+                    KisTransaction transaction("cut", oldLayer->paintDevice());
+                    // clear the old layer
+                    oldLayer->paintDevice()->clearSelection(selection);
+                    transaction.commit(image->undoAdapter());
 
-                // deselect away the selection???
-                selection->clear();
+                    if (image->globalSelection()) {
+                        KisDeselectGlobalSelectionCommand* cmd = new KisDeselectGlobalSelectionCommand(image);
+                        image->undoAdapter()->addCommand(cmd);
+                    }
+                }
 
                 KisCanvas2* kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
                 KisView2* view = 0;
@@ -188,11 +198,11 @@ void KisToolMove::mousePressEvent(KoPointerEvent *e)
                 }
 
                 // create the new layer and add it.
-                KisPaintLayerSP layer = new KisPaintLayer(currentImage(),
+                KisPaintLayerSP layer = new KisPaintLayer(image,
                         node->name() + "(moved)",
                         oldLayer->opacity(),
                         dev);
-                currentImage()->undoAdapter()->addCommand(new KisImageLayerAddCommand(currentImage(), layer, node->parent(), node));
+                image->undoAdapter()->addCommand(new KisImageLayerAddCommand(image, layer, node->parent(), node));
                 view->nodeManager()->activateNode(layer);
                 m_targetLayer = node;
                 m_selectedNode = layer;
@@ -203,20 +213,30 @@ void KisToolMove::mousePressEvent(KoPointerEvent *e)
             m_targetLayer = 0;
         }
 
-        m_dragging = true;
-        m_dragStart = QPoint(static_cast<int>(pos.x()), static_cast<int>(pos.y()));
+        /**
+         * FIXME: Hack alert:
+         * Our iterators don't have guarantees on thread-safety
+         * when the offset varies. When it is fixed, remove the locking
+         * see: KisIterator::stressTest()
+         */
+        image->lock();
         m_layerStart.setX(node->x());
         m_layerStart.setY(node->y());
+        image->unlock();
+
         m_layerPosition = m_layerStart;
+        m_dragStart = pos.toPoint();
+    }
+    else {
+        KisTool::mousePressEvent(event);
     }
 }
 
-void KisToolMove::mouseMoveEvent(KoPointerEvent *e)
+void KisToolMove::mouseMoveEvent(KoPointerEvent *event)
 {
-    if (canvas() && m_dragging) {
-        QPointF posf = convertToPixelCoord(e);
-        QPoint pos = QPoint(static_cast<int>(posf.x()), static_cast<int>(posf.y()));
-        if ((e->modifiers() & Qt::AltModifier) || (e->modifiers() & Qt::ControlModifier)) {
+    if(MOVE_CONDITION(event, KisTool::PAINT_MODE)) {
+        QPoint pos = convertToPixelCoord(event).toPoint();
+        if ((event->modifiers() & Qt::AltModifier) || (event->modifiers() & Qt::ControlModifier)) {
             if (fabs(static_cast<double>(pos.x() - m_dragStart.x())) > fabs(static_cast<double>(pos.y() - m_dragStart.y())))
                 pos.setY(m_dragStart.y());
             else
@@ -226,25 +246,30 @@ void KisToolMove::mouseMoveEvent(KoPointerEvent *e)
 
         notifyModified();
     }
+    else {
+        KisTool::mouseMoveEvent(event);
+    }
 }
 
-void KisToolMove::mouseReleaseEvent(KoPointerEvent *e)
+void KisToolMove::mouseReleaseEvent(KoPointerEvent *event)
 {
-    if (m_dragging) {
-        if (canvas() && e->button() == Qt::LeftButton) {
-            QPointF pos = convertToPixelCoord(e);
-            if (m_selectedNode) {
-                drag(QPoint(static_cast<int>(pos.x()), static_cast<int>(pos.y())));
-                m_dragging = false;
+    if(RELEASE_CONDITION(event, KisTool::PAINT_MODE, Qt::LeftButton)) {
+        setMode(KisTool::HOVER_MODE);
 
-                QUndoCommand *cmd = new KisNodeMoveCommand(m_selectedNode, m_layerStart, m_layerPosition);
-                Q_CHECK_PTR(cmd);
+        if (m_selectedNode) {
+            QPoint pos = convertToPixelCoord(event).toPoint();
+            drag(pos);
 
-                canvas()->addCommand(cmd);
-                currentImage()->undoAdapter()->endMacro();
-            }
-            currentImage()->setModified();
+            QUndoCommand *cmd = new KisNodeMoveCommand(m_selectedNode, m_layerStart, m_layerPosition, currentImage());
+            Q_CHECK_PTR(cmd);
+
+            canvas()->addCommand(cmd);
+            currentImage()->undoAdapter()->endMacro();
         }
+        currentImage()->setModified();
+    }
+    else {
+        KisTool::mouseReleaseEvent(event);
     }
 }
 
@@ -257,14 +282,23 @@ void KisToolMove::drag(const QPoint& original)
 
         pos -= m_dragStart; // convert to delta
         rc = m_selectedNode->extent();
+
+        // FIXME: see comment in KisToolMove::mousePressEvent()
+        KisImageWSP image = currentImage();
+        image->lock();
         m_selectedNode->setX(m_selectedNode->x() + pos.x());
         m_selectedNode->setY(m_selectedNode->y() + pos.y());
+        image->unlock();
 
         rc = rc.unite(m_selectedNode->extent());
 
         m_layerPosition = QPoint(m_selectedNode->x(), m_selectedNode->y());
         m_dragStart = original;
 
+        if (m_selectedNode->inherits("KisSelectionMask")) {
+            currentImage()->undoAdapter()->emitSelectionChanged();
+        }
+
         m_selectedNode->setDirty(rc);
     }
 }
