@@ -61,12 +61,11 @@ KisToolSelectContiguous::~KisToolSelectContiguous()
 {
 }
 
-void KisToolSelectContiguous::mousePressEvent(KoPointerEvent * e)
+void KisToolSelectContiguous::mousePressEvent(KoPointerEvent *event)
 {
-    if (e->button() != Qt::LeftButton)
-        return;
+    if(PRESS_CONDITION(event, KisTool::HOVER_MODE,
+                       Qt::LeftButton, Qt::NoModifier)) {
 
-    if (canvas() && currentImage()) {
         QApplication::setOverrideCursor(KisCursor::waitCursor());
 
         if (!currentNode())
@@ -76,7 +75,7 @@ void KisToolSelectContiguous::mousePressEvent(KoPointerEvent * e)
         if (!dev || !currentNode()->visible())
             return;
 
-        QPoint pos = convertToIntPixelCoord(e);
+        QPoint pos = convertToIntPixelCoord(event);
         QRect rc = currentImage()->bounds();
         KisFillPainter fillpainter(dev);
         fillpainter.setHeight(rc.height());
@@ -91,11 +90,13 @@ void KisToolSelectContiguous::mousePressEvent(KoPointerEvent * e)
             return;
 
         KisSelectionToolHelper helper(kisCanvas, currentNode(), i18n("Contiguous Area Selection"));
-        QUndoCommand* cmd = helper.selectPixelSelection(selection->pixelSelection(), m_selectAction);
-        canvas()->addCommand(cmd);
+        helper.selectPixelSelection(selection->pixelSelection(), m_selectAction);
 
         QApplication::restoreOverrideCursor();
     }
+    else {
+        KisTool::mousePressEvent(event);
+    }
 }
 
 void KisToolSelectContiguous::paint(QPainter &painter, const KoViewConverter &converter)
