@@ -24,15 +24,12 @@
 #define __KIS_TOOL_SELECT_ELLIPTICAL_H__
 
 #include <QPoint>
+#include "KoToolFactoryBase.h"
+#include "krita/ui/tool/kis_tool_select_base.h"
+#include "kis_tool_ellipse_base.h"
 
-#include "kis_tool.h"
-#include "kis_selection.h"
-#include "KoToolFactory.h"
-#include "flake/kis_node_shape.h"
 
-class KisSelectionOptions;
-
-class KisToolSelectElliptical : public KisTool
+class KisToolSelectElliptical : public KisToolSelectBase
 {
 
     Q_OBJECT
@@ -42,44 +39,36 @@ public:
     virtual ~KisToolSelectElliptical();
 
     virtual QWidget * createOptionWidget();
-    virtual QWidget* optionWidget();
-//     virtual enumToolType toolType() { return TOOL_SELECT; }
-
-    virtual void paint(QPainter& gc, const KoViewConverter &converter);
-
-    virtual void mousePressEvent(KoPointerEvent *e);
-    virtual void mouseMoveEvent(KoPointerEvent *e);
-    virtual void mouseReleaseEvent(KoPointerEvent *e);
-
-public slots:
-    virtual void slotSetAction(int);
-    virtual void slotSetSelectionMode(int);
-    virtual void activate(bool);
-
 
 private:
-    void clearSelection();
+    class LocalTool : public KisToolEllipseBase {
+    public:
+        LocalTool(KoCanvasBase * canvas, KisToolSelectElliptical* selectingTool)
+            : KisToolEllipseBase(canvas), m_selectingTool(selectingTool) {}
+        void finishEllipse(const QRectF &rect);
+    private:
+        KisToolSelectElliptical* const m_selectingTool;
+    };
+    LocalTool m_localTool;
 
-private:
-    QPointF m_centerPos;
-    QPointF m_startPos;
-    QPointF m_endPos;
-    bool m_selecting;
-    KisSelectionOptions * m_optWidget;
-    selectionAction m_selectAction;
-    selectionMode m_selectionMode;
+    virtual void paint(QPainter& gc, const KoViewConverter &converter) {m_localTool.paint(gc, converter);}
+    virtual void mousePressEvent(KoPointerEvent *e) {m_localTool.mousePressEvent(e);}
+    virtual void mouseMoveEvent(KoPointerEvent *e) {m_localTool.mouseMoveEvent(e);}
+    virtual void mouseReleaseEvent(KoPointerEvent *e) {m_localTool.mouseReleaseEvent(e);}
+    virtual void deactivate() {m_localTool.deactivate();}
 
+    friend class LocalTool;
 };
 
-class KisToolSelectEllipticalFactory : public KoToolFactory
+class KisToolSelectEllipticalFactory : public KoToolFactoryBase
 {
 
 public:
     KisToolSelectEllipticalFactory(QObject *parent, const QStringList&)
-            : KoToolFactory(parent, "KisToolSelectElliptical", i18n("Elliptical Selection")) {
+            : KoToolFactoryBase(parent, "KisToolSelectElliptical") {
         setToolTip(i18n("Select an elliptical area"));
         setToolType(TOOL_TYPE_SELECTED);
-        //setActivationShapeId( KIS_NODE_SHAPE_ID );
+        setActivationShapeId(KRITA_TOOL_ACTIVATION_ID);
         setIcon("tool_elliptical_selection");
         setShortcut(KShortcut(Qt::Key_J));
         setPriority(53);
@@ -87,7 +76,7 @@ public:
 
     virtual ~KisToolSelectEllipticalFactory() {}
 
-    virtual KoTool * createTool(KoCanvasBase *canvas) {
+    virtual KoToolBase * createTool(KoCanvasBase *canvas) {
         return  new KisToolSelectElliptical(canvas);
     }
 
