@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  *
- * Copyright (C) Boudewijn Rempt <boud@valdyas.org>, (C) 2006
+ * Copyright (C) 2006, 2010 Boudewijn Rempt <boud@valdyas.org>
+ * Copyright (C) Lukáš Tvrdý <lukast.dev@gmail.com>, (C) 2010
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,12 +33,13 @@
 #include <KoShapeManager.h>
 #include <KoColorProfile.h>
 #include <KoColorSpaceRegistry.h>
-#include <KoCanvasController.h>
+#include <KoCanvasControllerWidget.h>
 #include <KoDocument.h>
 #include <KoZoomAction.h>
 #include <KoToolProxy.h>
 #include <KoSelection.h>
 
+#include "kis_coordinates_converter.h"
 #include "kis_prescaled_projection.h"
 #include "kis_image.h"
 #include "kis_doc2.h"
@@ -60,7 +62,6 @@
 #ifdef HAVE_OPENGL
 #include <QGLFormat>
 #endif
-#include "kis_projection_cache.h"
 
 //Favorite resource Manager
 #include <ko_favorite_resource_manager.h>
@@ -72,7 +73,8 @@ class KisCanvas2::KisCanvas2Private
 public:
 
     KisCanvas2Private(KoCanvasBase * parent, KoViewConverter * viewConverter, KisView2 * view)
-        : viewConverter(viewConverter)
+        : coordinatesConverter(new KisCoordinatesConverter(viewConverter))
+        , viewConverter(viewConverter)
         , view(view)
         , canvasWidget(0)
         , shapeManager(new KoShapeManager(parent))
@@ -80,14 +82,19 @@ public:
         , currentCanvasIsOpenGL(false)
         , currentCanvasUsesOpenGLShaders(false)
         , toolProxy(new KoToolProxy(parent))
-        , favoriteResourceManager(0) {
+        , favoriteResourceManager(0)
+        , canvasMirroredY(false)
+        , vastScrolling(true) {
     }
 
     ~KisCanvas2Private() {
+        delete coordinatesConverter;
+        delete favoriteResourceManager;
         delete shapeManager;
         delete toolProxy;
     }
 
+    KisCoordinatesConverter *coordinatesConverter;
     KoViewConverter *viewConverter;
     KisView2 *view;
     KisAbstractCanvasWidget *canvasWidget;
@@ -96,23 +103,31 @@ public:
     bool currentCanvasIsOpenGL;
     bool currentCanvasUsesOpenGLShaders;
     KoToolProxy *toolProxy;
-    QPoint documentOffset;
-    KoShapeControllerBase *sc;
     KoFavoriteResourceManager *favoriteResourceManager;
 #ifdef HAVE_OPENGL
     KisOpenGLImageTexturesSP openGLImageTextures;
 #endif
     KisPrescaledProjectionSP prescaledProjection;
+    bool canvasMirroredY;
+    bool vastScrolling;
 };
 
 KisCanvas2::KisCanvas2(KoViewConverter * viewConverter, KisView2 * view, KoShapeControllerBase * sc)
     : KoCanvasBase(sc)
     , m_d(new KisCanvas2Private(this, viewConverter, view))
 {
-    createCanvas();
-    connect(view->canvasController(), SIGNAL(moveDocumentOffset(const QPoint&)), SLOT(documentOffsetMoved(const QPoint&)));
+    // a bit of duplication from slotConfigChanged()
+    KisConfig cfg;
+    m_d->vastScrolling = cfg.vastScrolling();
+    createCanvas(cfg.useOpenGL());
+
+    connect(view->canvasController()->proxyObject, SIGNAL(moveDocumentOffset(const QPoint&)), SLOT(documentOffsetMoved(const QPoint&)));
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(slotConfigChanged()));
     connect(this, SIGNAL(canvasDestroyed(QWidget *)), this, SLOT(slotCanvasDestroyed(QWidget *)));
+
+    KisShapeController *kritaShapeController = dynamic_cast<KisShapeController*>(sc);
+    connect(kritaShapeController, SIGNAL(selectionChanged()),
+            globalShapeManager()->selection(), SIGNAL(selectionChanged()));
 }
 
 KisCanvas2::~KisCanvas2()
@@ -122,6 +137,8 @@ KisCanvas2::~KisCanvas2()
 
 void KisCanvas2::setCanvasWidget(QWidget * widget)
 {
+    connect(widget, SIGNAL(needAdjustOrigin()), this, SLOT(adjustOrigin()), Qt::DirectConnection);
+
     KisAbstractCanvasWidget * tmp = dynamic_cast<KisAbstractCanvasWidget*>(widget);
     Q_ASSERT_X(tmp, "setCanvasWidget", "Cannot cast the widget to a KisAbstractCanvasWidget");
     emit canvasDestroyed(widget);
@@ -134,17 +151,18 @@ void KisCanvas2::setCanvasWidget(QWidget * widget)
     widget->setAttribute(Qt::WA_OpaquePaintEvent);
     widget->setMouseTracking(true);
     widget->setAcceptDrops(true);
-    KoCanvasController *controller = canvasController();
+    KoCanvasControllerWidget *controller = dynamic_cast<KoCanvasControllerWidget*>(canvasController());
     if (controller) {
         Q_ASSERT(controller->canvas() == this);
-        // Avoids jumping and redrawing when changing zoom means the image fits in the area completely
-        controller->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
-        controller->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
-
         controller->changeCanvasWidget(widget);
     }
 }
 
+bool KisCanvas2::canvasIsOpenGL()
+{
+    return m_d->currentCanvasIsOpenGL;
+}
+
 void KisCanvas2::gridSize(qreal *horizontal, qreal *vertical) const
 {
     Q_ASSERT(horizontal);
@@ -158,6 +176,65 @@ bool KisCanvas2::snapToGrid() const
     return m_d->view->document()->gridData().snapToGrid();
 }
 
+void KisCanvas2::pan(QPoint shift)
+{
+    KoCanvasControllerWidget* controller =
+        dynamic_cast<KoCanvasControllerWidget*>(canvasController());
+    controller->pan(shift);
+    updateCanvas();
+}
+
+void KisCanvas2::mirrorCanvas(bool enable)
+{
+    if(enable != m_d->canvasMirroredY) {
+        QPointF oldCenterPoint = m_d->coordinatesConverter->flakeCenterPoint();
+
+        QTransform newTransform = m_d->coordinatesConverter->postprocessingTransform();
+        newTransform *= QTransform::fromScale(-1,1);
+        m_d->coordinatesConverter->setPostprocessingTransform(newTransform);
+        m_d->canvasMirroredY = enable;
+        notifyZoomChanged();
+
+        QPoint shift = m_d->coordinatesConverter->shiftFromFlakeCenterPoint(oldCenterPoint);
+        pan(shift);
+    }
+}
+
+void KisCanvas2::rotateCanvas(qreal angle)
+{
+    QPointF oldCenterPoint = m_d->coordinatesConverter->flakeCenterPoint();
+
+    QTransform newTransform = m_d->coordinatesConverter->postprocessingTransform();
+    QTransform temp; temp.rotate(angle);
+    newTransform *= temp;
+    m_d->coordinatesConverter->setPostprocessingTransform(newTransform);
+    notifyZoomChanged();
+
+    QPoint shift = m_d->coordinatesConverter->shiftFromFlakeCenterPoint(oldCenterPoint);
+    pan(shift);
+}
+
+void KisCanvas2::rotateCanvasRight15()
+{
+    rotateCanvas(15.);
+}
+
+void KisCanvas2::rotateCanvasLeft15()
+{
+    rotateCanvas(-15.);
+}
+
+void KisCanvas2::resetCanvasTransformations()
+{
+    QPointF oldCenterPoint = m_d->coordinatesConverter->flakeCenterPoint();
+
+    m_d->coordinatesConverter->setPostprocessingTransform(QTransform());
+    notifyZoomChanged();
+
+    QPoint shift = m_d->coordinatesConverter->shiftFromFlakeCenterPoint(oldCenterPoint);
+    pan(shift);
+}
+
 void KisCanvas2::addCommand(QUndoCommand *command)
 {
     m_d->view->koDocument()->addCommand(command);
@@ -179,7 +256,7 @@ KoShapeManager* KisCanvas2::shapeManager() const
     if (!m_d->view->layerManager()) return m_d->shapeManager;
 
     KisLayerSP activeLayer = m_d->view->layerManager()->activeLayer();
-    if (activeLayer) {
+    if (activeLayer && activeLayer->isEditable()) {
         KisShapeLayer * shapeLayer = dynamic_cast<KisShapeLayer*>(activeLayer.data());
         if (shapeLayer) {
             return shapeLayer->shapeManager();
@@ -198,21 +275,16 @@ KoShapeManager * KisCanvas2::globalShapeManager() const
     return m_d->shapeManager;
 }
 
-void KisCanvas2::updateCanvas(const QRectF& rc)
-{
-    // updateCanvas is called from tools, never from the projection
-    // updates, so no need to prescale!
-    QRect vRect = viewRectFromDoc(rc);
-    if (!vRect.isEmpty()) {
-        m_d->canvasWidget->widget()->update(vRect);
-    }
-}
-
 void KisCanvas2::updateInputMethodInfo()
 {
     // TODO call (the protected) QWidget::updateMicroFocus() on the proper canvas widget...
 }
 
+const KisCoordinatesConverter* KisCanvas2::coordinatesConverter() const
+{
+    return m_d->coordinatesConverter;
+}
+
 const KoViewConverter* KisCanvas2::viewConverter() const
 {
     return m_d->viewConverter;
@@ -241,58 +313,232 @@ KoToolProxy * KisCanvas2::toolProxy() const
 
 void KisCanvas2::createQPainterCanvas()
 {
-#ifdef HAVE_OPENGL
-    m_d->openGLImageTextures = 0;
-#endif
     m_d->currentCanvasIsOpenGL = false;
 
-    KisQPainterCanvas * canvasWidget = new KisQPainterCanvas(this, m_d->view);
+    KisQPainterCanvas * canvasWidget = new KisQPainterCanvas(this, m_d->coordinatesConverter, m_d->view);
     m_d->prescaledProjection = new KisPrescaledProjection();
-    m_d->prescaledProjection->setViewConverter(m_d->viewConverter);
+    m_d->prescaledProjection->setCoordinatesConverter(m_d->coordinatesConverter);
     m_d->prescaledProjection->setMonitorProfile(monitorProfile());
     canvasWidget->setPrescaledProjection(m_d->prescaledProjection);
-
-    connect(canvasWidget, SIGNAL(documentOriginChanged(const QPoint&)), this, SLOT(updateRulers()));
-
     setCanvasWidget(canvasWidget);
 }
 
 void KisCanvas2::createOpenGLCanvas()
 {
 #ifdef HAVE_OPENGL
-    if (QGLFormat::hasOpenGL()) {
-        // XXX: The image isn't done loading here!
-        m_d->openGLImageTextures = KisOpenGLImageTextures::getImageTextures(m_d->view->image(), m_d->monitorProfile);
-        KisOpenGLCanvas2 * canvasWidget = new KisOpenGLCanvas2(this, m_d->view, m_d->openGLImageTextures);
-        setCanvasWidget(canvasWidget);
-        m_d->currentCanvasIsOpenGL = true;
-        m_d->currentCanvasUsesOpenGLShaders = m_d->openGLImageTextures->usingHDRExposureProgram();
+    m_d->currentCanvasIsOpenGL = true;
 
-        connect(canvasWidget, SIGNAL(documentOriginChanged(const QPoint&)), this, SLOT(updateRulers()));
-
-    } else {
-        warnKrita << "Tried to create OpenGL widget when system doesn't have OpenGL\n";
-        createQPainterCanvas();
-    }
+    // XXX: The image isn't done loading here!
+    m_d->openGLImageTextures = KisOpenGLImageTextures::getImageTextures(m_d->view->image(), m_d->monitorProfile);
+    KisOpenGLCanvas2 * canvasWidget = new KisOpenGLCanvas2(this, m_d->coordinatesConverter, m_d->view, m_d->openGLImageTextures);
+    m_d->currentCanvasUsesOpenGLShaders = m_d->openGLImageTextures->usingHDRExposureProgram();
+    setCanvasWidget(canvasWidget);
+#else
+    qFatal("Bad use of createOpenGLCanvas(). It shouldn't have happened =(");
 #endif
 }
 
-void KisCanvas2::createCanvas()
+void KisCanvas2::createCanvas(bool useOpenGL)
 {
     KisConfig cfg;
     slotSetDisplayProfile(KoColorSpaceRegistry::instance()->profileByName(cfg.monitorProfile()));
 
-    if (cfg.useOpenGL()) {
+    if (useOpenGL) {
 #ifdef HAVE_OPENGL
-        createOpenGLCanvas();
+        if (QGLFormat::hasOpenGL()) {
+            createOpenGLCanvas();
+        } else {
+            warnKrita << "Tried to create OpenGL widget when system doesn't have OpenGL\n";
+            createQPainterCanvas();
+        }
 #else
         warnKrita << "OpenGL requested while its not available, starting qpainter canvas";
         createQPainterCanvas();
 #endif
     } else {
+#ifdef HAVE_OPENGL
+        // Free shared pointer
+        m_d->openGLImageTextures = 0;
+#endif
         createQPainterCanvas();
     }
+}
 
+void KisCanvas2::connectCurrentImage()
+{
+    m_d->coordinatesConverter->setImage(m_d->view->image());
+
+
+    if (m_d->currentCanvasIsOpenGL) {
+#ifdef HAVE_OPENGL
+        Q_ASSERT(m_d->openGLImageTextures);
+
+        connect(m_d->view->image(), SIGNAL(sigSizeChanged(qint32, qint32)),
+                m_d->openGLImageTextures, SLOT(slotImageSizeChanged(qint32, qint32)));
+
+        QRect imageRect = m_d->view->image()->bounds();
+        m_d->openGLImageTextures->slotImageSizeChanged(imageRect.width(), imageRect.height());
+#else
+        qFatal("Bad use of connectCurrentImage(). It shouldn't have happened =(");
+#endif
+    } else {
+        connect(m_d->view->image(), SIGNAL(sigSizeChanged(qint32, qint32)),
+                SLOT(setImageSize(qint32, qint32)));
+
+        Q_ASSERT(m_d->prescaledProjection);
+        m_d->prescaledProjection->setImage(m_d->view->image());
+
+    }
+
+    connect(m_d->view->image(), SIGNAL(sigImageUpdated(const QRect &)),
+            SLOT(startUpdateCanvasProjection(const QRect &)),
+            Qt::DirectConnection);
+    connect(this, SIGNAL(sigCanvasCacheUpdated(KisUpdateInfoSP)),
+            this, SLOT(updateCanvasProjection(KisUpdateInfoSP)));
+
+    emit imageChanged(m_d->view->image());
+}
+
+void KisCanvas2::disconnectCurrentImage()
+{
+    m_d->coordinatesConverter->setImage(0);
+
+    if (m_d->currentCanvasIsOpenGL) {
+#ifdef HAVE_OPENGL
+        Q_ASSERT(m_d->openGLImageTextures);
+        m_d->openGLImageTextures->disconnect(this);
+        m_d->openGLImageTextures->disconnect(m_d->view->image());
+#else
+        qFatal("Bad use of disconnectCurrentImage(). It shouldn't have happened =(");
+#endif
+    }
+
+    disconnect(SIGNAL(sigCanvasCacheUpdated(KisUpdateInfoSP)));
+
+    // for sigSizeChanged()
+    m_d->view->image()->disconnect(this);
+}
+
+void KisCanvas2::resetCanvas(bool useOpenGL)
+{
+#ifdef HAVE_OPENGL
+    KisConfig cfg;
+
+    if (   (useOpenGL != m_d->currentCanvasIsOpenGL)
+        || (   m_d->currentCanvasIsOpenGL
+               && (cfg.useOpenGLShaders() != m_d->currentCanvasUsesOpenGLShaders))) {
+
+        disconnectCurrentImage();
+        createCanvas(useOpenGL);
+        connectCurrentImage();
+        notifyZoomChanged();
+    }
+
+    if (useOpenGL) {
+        Q_ASSERT(m_d->openGLImageTextures);
+        m_d->openGLImageTextures->setMonitorProfile(monitorProfile());
+    } else {
+        if (image()) {
+            startUpdateCanvasProjection(image()->bounds());
+        }
+    }
+
+#endif
+
+    m_d->canvasWidget->widget()->update();
+}
+
+void KisCanvas2::startUpdateCanvasProjection(const QRect & rc)
+{
+    if (m_d->currentCanvasIsOpenGL) {
+#ifdef HAVE_OPENGL
+        Q_ASSERT(m_d->openGLImageTextures);
+        KisUpdateInfoSP info = m_d->openGLImageTextures->updateCache(rc);
+
+        emit sigCanvasCacheUpdated(info);
+#else
+        Q_ASSERT_X(0, "startUpdateCanvasProjection()", "Bad use of startUpdateCanvasProjection(). It shouldn't have happened =(");
+#endif
+    } else {
+        Q_ASSERT(m_d->prescaledProjection);
+        KisUpdateInfoSP info = m_d->prescaledProjection->updateCache(rc);
+
+        emit sigCanvasCacheUpdated(info);
+    }
+
+
+}
+
+void KisCanvas2::updateCanvasProjection(KisUpdateInfoSP info)
+{
+    if (m_d->currentCanvasIsOpenGL) {
+#ifdef HAVE_OPENGL
+        Q_ASSERT(m_d->openGLImageTextures);
+        m_d->openGLImageTextures->recalculateCache(info);
+
+        /**
+         * FIXME: Please not update entire canvas
+         * Implement info->dirtyViewportRect()
+         */
+        m_d->canvasWidget->widget()->update();
+#else
+        Q_ASSERT_X(0, "updateCanvasProjection()", "Bad use of updateCanvasProjection(). It shouldn't have happened =(");
+#endif
+    }
+    else {
+
+        // See comment in startUpdateCanvasProjection()
+        Q_ASSERT(m_d->prescaledProjection);
+
+        m_d->prescaledProjection->recalculateCache(info);
+
+        QRect vRect = m_d->coordinatesConverter->
+            viewportToWidget(info->dirtyViewportRect()).toAlignedRect();
+
+        if (!vRect.isEmpty()) {
+            m_d->canvasWidget->widget()->update(vRect);
+        }
+    }
+}
+
+void KisCanvas2::updateCanvas()
+{
+    m_d->canvasWidget->widget()->update();
+}
+
+void KisCanvas2::updateCanvas(const QRectF& documentRect)
+{
+    // updateCanvas is called from tools, never from the projection
+    // updates, so no need to prescale!
+    QRect widgetRect = m_d->coordinatesConverter->documentToWidget(documentRect).toAlignedRect();
+    if (!widgetRect.isEmpty()) {
+        m_d->canvasWidget->widget()->update(widgetRect);
+    }
+}
+
+void KisCanvas2::notifyZoomChanged()
+{
+    m_d->coordinatesConverter->notifyZoomChanged();
+    adjustOrigin();
+
+    if (!m_d->currentCanvasIsOpenGL) {
+        Q_ASSERT(m_d->prescaledProjection);
+        m_d->prescaledProjection->notifyZoomChanged();
+    }
+    emit scrollAreaSizeChanged();
+}
+
+void KisCanvas2::preScale()
+{
+    if (!m_d->currentCanvasIsOpenGL) {
+        Q_ASSERT(m_d->prescaledProjection);
+        m_d->prescaledProjection->preScale();
+    }
+}
+
+KoColorProfile *  KisCanvas2::monitorProfile()
+{
+    return m_d->monitorProfile;
 }
 
 KisView2* KisCanvas2::view()
@@ -300,48 +546,12 @@ KisView2* KisCanvas2::view()
     return m_d->view;
 }
 
-
-QRect KisCanvas2::viewRectFromDoc(const QRectF & rc)
-{
-    QRect viewRect = m_d->viewConverter->documentToView(rc).toAlignedRect();
-    viewRect = viewRect.translated(-m_d->documentOffset);
-    // comment out this line if zou want to see the preview outside of the canvas
-    // viewRect = viewRect.intersected(QRect(0, 0, m_d->canvasWidget->widget()->width(), m_d->canvasWidget->widget()->height()));
-    viewRect.translate(documentOrigin());
-    return viewRect;
-}
-
-
-void KisCanvas2::updateCanvasProjection(const QRect & rc)
-{
-    if (m_d->prescaledProjection) {
-        QRect vRect = m_d->prescaledProjection->updateCanvasProjection(rc);
-        if (!vRect.isEmpty()) {
-            vRect.translate(m_d->canvasWidget->documentOrigin());
-            m_d->canvasWidget->widget()->update(vRect);
-            //m_d->canvasWidget->widget()->update();
-        }
-    }
-}
-
-
-void KisCanvas2::updateCanvas()
-{
-    m_d->canvasWidget->widget()->update();
-}
-
-
 KisImageWSP KisCanvas2::image()
 {
     return m_d->view->image();
 
 }
 
-KoColorProfile *  KisCanvas2::monitorProfile()
-{
-    return m_d->monitorProfile;
-}
-
 KisImageWSP KisCanvas2::currentImage()
 {
     return m_d->view->image();
@@ -353,82 +563,18 @@ void KisCanvas2::setImageSize(qint32 w, qint32 h)
         m_d->prescaledProjection->setImageSize(w, h);
 }
 
-void KisCanvas2::connectCurrentImage()
-{
-#ifdef HAVE_OPENGL
-    if (m_d->openGLImageTextures) {
-        connect(m_d->openGLImageTextures, SIGNAL(sigImageUpdated(const QRect &)), SLOT(updateCanvas()));
-        connect(m_d->openGLImageTextures, SIGNAL(sigSizeChanged(qint32, qint32)), SLOT(setImageSize(qint32, qint32)));
-    } else {
-#endif
-        connect(m_d->view->image(), SIGNAL(sigImageUpdated(const QRect &)), SLOT(updateCanvasProjection(const QRect &)));
-        connect(m_d->view->image(), SIGNAL(sigSizeChanged(qint32, qint32)), SLOT(setImageSize(qint32, qint32)));
-
-        if (m_d->prescaledProjection) {
-            m_d->prescaledProjection->setImage(m_d->view->image());
-        }
-
-#ifdef HAVE_OPENGL
-    }
-#endif
-    emit imageChanged(m_d->view->image());
-}
-
-void KisCanvas2::disconnectCurrentImage()
-{
-#ifdef HAVE_OPENGL
-    if (m_d->openGLImageTextures) {
-        m_d->openGLImageTextures->disconnect(this);
-    }
-#endif
-    m_d->view->image()->disconnect(this);
-}
-
-void KisCanvas2::resetCanvas(bool useOpenGL)
-{
-    KisConfig cfg;
-#if HAVE_OPENGL
-
-    if (   (useOpenGL != m_d->currentCanvasIsOpenGL)
-        || (   m_d->currentCanvasIsOpenGL
-               && (cfg.useOpenGLShaders() != m_d->currentCanvasUsesOpenGLShaders))) {
-
-        disconnectCurrentImage();
-        slotSetDisplayProfile(KoColorSpaceRegistry::instance()->profileByName(cfg.monitorProfile()));
-        if (useOpenGL) {
-#ifdef HAVE_OPENGL
-            createOpenGLCanvas();
-#else
-            warnKrita << "OpenGL requested while its not available, starting qpainter canvas";
-            createQPainterCanvas();
-#endif
-        } else {
-            createQPainterCanvas();
-        }
-        connectCurrentImage();
-    }
-
-    if (useOpenGL) {
-        m_d->openGLImageTextures->setMonitorProfile(monitorProfile());
-    } else {
-        if (image()) {
-            updateCanvasProjection(image()->bounds());
-        }
-    }
-#endif
-    m_d->canvasWidget->widget()->update();
-}
-
 void KisCanvas2::documentOffsetMoved(const QPoint &documentOffset)
 {
-    m_d->documentOffset = documentOffset;
-    m_d->canvasWidget->documentOffsetMoved(documentOffset);
-}
+    QPointF offsetBefore = m_d->coordinatesConverter->imageRectInViewportPixels().topLeft();
+    m_d->coordinatesConverter->setDocumentOffset(documentOffset);
+    QPointF offsetAfter = m_d->coordinatesConverter->imageRectInViewportPixels().topLeft();
 
-void KisCanvas2::preScale()
-{
-    if (!m_d->currentCanvasIsOpenGL && m_d->prescaledProjection)
-        m_d->prescaledProjection->preScale();
+    QPointF moveOffset = offsetAfter - offsetBefore;
+
+    if (!m_d->currentCanvasIsOpenGL)
+        m_d->prescaledProjection->viewportMoved(moveOffset);
+
+    updateCanvas();
 }
 
 bool KisCanvas2::usingHDRExposureProgram()
@@ -445,8 +591,10 @@ bool KisCanvas2::usingHDRExposureProgram()
 
 void KisCanvas2::slotConfigChanged()
 {
-    // first, assume we're going to crash when switching to opengl
     KisConfig cfg;
+    m_d->vastScrolling = cfg.vastScrolling();
+
+    // first, assume we're going to crash when switching to opengl
     bool useOpenGL = cfg.useOpenGL();
     if (cfg.canvasState() == "TRY_OPENGL" && useOpenGL) {
         cfg.setCanvasState("OPENGL_FAILED");
@@ -475,25 +623,35 @@ KisCanvasDecoration* KisCanvas2::decoration(const QString& id)
 
 QPoint KisCanvas2::documentOrigin() const
 {
-    return m_d->canvasWidget->documentOrigin();
+    return m_d->coordinatesConverter->documentOrigin();
 }
 
 
 void KisCanvas2::adjustOrigin()
 {
-    m_d->canvasWidget->adjustOrigin();
-}
+    QPoint newOrigin;
 
+    QSize documentSize = m_d->coordinatesConverter->imageRectInWidgetPixels().toAlignedRect().size();
+    QSize widgetSize = m_d->canvasWidget->widget()->size();
+
+    if(!m_d->vastScrolling) {
+        int widthDiff = widgetSize.width() - documentSize.width();
+        int heightDiff = widgetSize.height() - documentSize.height();
+
+        if (widthDiff > 0)
+            newOrigin.rx() = qRound(0.5 * widthDiff);
+        if (heightDiff > 0)
+            newOrigin.ry() = qRound(0.5 * heightDiff);
+    }
+
+    m_d->coordinatesConverter->setDocumentOrigin(newOrigin);
 
-void KisCanvas2::updateRulers()
-{
     emit documentOriginChanged();
 }
 
-
 QPoint KisCanvas2::documentOffset() const
 {
-    return m_d->documentOffset;
+    return m_d->coordinatesConverter->documentOffset();
 }
 
 void KisCanvas2::createFavoriteResourceManager(KisPaintopBox* paintopbox)
@@ -520,17 +678,20 @@ KoFavoriteResourceManager* KisCanvas2::favoriteResourceManager()
 }
 
 
-bool KisCanvas2::handlePopupPaletteIsVisible(KoPointerEvent* e)
+bool KisCanvas2::handlePopupPaletteIsVisible()
 {
-    e->accept();
-
-    //TODO: port favoriteResourceManager to KisCanvas2, then enable this function.
-    if (e->button() && (Qt::LeftButton || Qt::MidButton || Qt::RightButton)
+    if (favoriteResourceManager()
         && favoriteResourceManager()->isPopupPaletteVisible()) {
+
         favoriteResourceManager()->slotShowPopupPalette();
         return true;
     }
     return false;
 }
 
+void KisCanvas2::setCursor(const QCursor &cursor)
+{
+    canvasWidget()->setCursor(cursor);
+}
+
 #include "kis_canvas2.moc"
