@@ -49,11 +49,11 @@ KisNodeShape::KisNodeShape(KoShapeContainer * parent, KisNodeSP node)
 
     setShapeId(KIS_NODE_SHAPE_ID);
     KoShape::setParent(parent);
-    parent->addChild(this);
 
-    connect( node, SIGNAL(visibilityChanged( bool )), SLOT( setNodeVisible( bool ) ) );
-    connect( node, SIGNAL(userLockingChanged( bool )), SLOT( editabilityChanged( ) ) );
-    connect( node, SIGNAL(systemLockingChanged( bool )), SLOT( editabilityChanged( ) ) );
+    connect(node, SIGNAL(visibilityChanged(bool)), SLOT(setNodeVisible(bool)));
+    connect(node, SIGNAL(userLockingChanged(bool)), SLOT(editabilityChanged()));
+    connect(node, SIGNAL(systemLockingChanged(bool)), SLOT(editabilityChanged()));
+    editabilityChanged();  // Correctly set the lock at loading
 }
 
 KisNodeShape::~KisNodeShape()
@@ -86,7 +86,7 @@ QSizeF KisNodeShape::size() const
 
     QRect br = m_d->node->extent();
 
-    KisImageSP image = getImage();
+    KisImageWSP image = getImage();
 
     if (!image) return QSizeF(0.0, 0.0);
 
@@ -99,7 +99,7 @@ QRectF KisNodeShape::boundingRect() const
 {
     QRect br = m_d->node->extent();
 
-    KisImageSP image = getImage();
+    KisImageWSP image = getImage();
 
     return QRectF(int(br.left()) / image->xRes(), int(br.top()) / image->yRes(),
                   int(1 + br.right()) / image->xRes(), int(1 + br.bottom()) / image->yRes());
@@ -111,7 +111,7 @@ void KisNodeShape::setPosition(const QPointF & position)
     Q_ASSERT(m_d);
     Q_ASSERT(m_d->node);
 
-    KisImageSP image = getImage();
+    KisImageWSP image = getImage();
 
     if (image) {
         // XXX: Does flake handle undo for us?
@@ -123,16 +123,6 @@ void KisNodeShape::setPosition(const QPointF & position)
 }
 
 
-void KisNodeShape::addChild(KoShape * shape)
-{
-    KisNodeShape* nodeShape = dynamic_cast<KisNodeShape*>( shape );
-    if ( !nodeShape ) return;
-
-    if ( !m_d->node->allowAsChild( nodeShape->node() ) ) return;
-
-    KoShapeContainer::addChild(shape);
-}
-
 void KisNodeShape::saveOdf(KoShapeSavingContext & /*context*/) const
 {
 
@@ -144,34 +134,32 @@ bool KisNodeShape::loadOdf(const KoXmlElement & /*element*/, KoShapeLoadingConte
     return false; // TODO
 }
 
-void KisNodeShape::setNodeVisible( bool v)
+void KisNodeShape::setNodeVisible(bool v)
 {
     // Necessary because shapes are not QObjects
-    setVisible( v );
+    setVisible(v);
 }
 
 // Defined in KisNodeContainerShape... FIXME (2.1) find a better way to share code between those two classes, or even better merge them
-bool recursiveFindActiveLayerInChildren( KoSelection* _selection, KoShapeLayer* _currentLayer );
+bool recursiveFindActiveLayerInChildren(KoSelection* _selection, KoShapeLayer* _currentLayer);
 
-void KisNodeShape::editabilityChanged( )
+void KisNodeShape::editabilityChanged()
 {
     dbgKrita << m_d->node->isEditable();
-    setLocked( !m_d->node->isEditable() );
+    setGeometryProtected(!m_d->node->isEditable());
     KoCanvasController* canvas = KoToolManager::instance()->activeCanvasController();
-    if( canvas )
-    {
-        recursiveFindActiveLayerInChildren( canvas->canvas()->shapeManager()->selection(), this );
+    if (canvas) {
+        recursiveFindActiveLayerInChildren(canvas->canvas()->shapeManager()->selection(), this);
     }
 }
 
-KisImageSP KisNodeShape::getImage() const
+KisImageWSP KisNodeShape::getImage() const
 {
 
-    if ( m_d->node->inherits( "KisLayer" ) ) {
-        return dynamic_cast<KisLayer*>( m_d->node.data() )->image();
-    }
-    else if ( m_d->node->inherits( "KisMask" ) ) {
-        return dynamic_cast<KisLayer*>( m_d->node->parent().data() )->image();
+    if (m_d->node->inherits("KisLayer")) {
+        return dynamic_cast<KisLayer*>(m_d->node.data())->image();
+    } else if (m_d->node->inherits("KisMask")) {
+        return dynamic_cast<KisLayer*>(m_d->node->parent().data())->image();
     }
 
     return 0;
