@@ -20,6 +20,7 @@
 #include "kis_popup_palette.h"
 #include "kis_paintop_box.h"
 #include "ko_favorite_resource_manager.h"
+#include "KoColorSpaceRegistry.h"
 #include <kis_types.h>
 #include <QtGui>
 #include <QDebug>
@@ -39,21 +40,27 @@ public:
         QMouseEvent* mouseEvent = 0;
         switch(event->type())
         {
-            case QEvent::TabletPress: mouseEvent = new QMouseEvent(QEvent::MouseButtonPress, event->pos(),
-                                                                Qt::LeftButton, event->modifiers());
-                                      m_dragging = true;
-                                      mousePressEvent(mouseEvent);
-                                      break;
-            case QEvent::TabletMove: mouseEvent = new QMouseEvent(QEvent::MouseMove, event->pos(),
-                                                                  (m_dragging) ? Qt::LeftButton : Qt::NoButton, event->modifiers());
-                                     mouseMoveEvent(mouseEvent);
-                                     break;
-            case QEvent::TabletRelease: mouseEvent = new QMouseEvent(QEvent::MouseButtonRelease, event->pos(),
-                                                                    Qt::LeftButton, event->modifiers());
-                                        m_dragging = false;
-                                        mouseReleaseEvent(mouseEvent);
-                                        break;
-            default: break;
+        case QEvent::TabletPress:
+            mouseEvent = new QMouseEvent(QEvent::MouseButtonPress, event->pos(),
+                                         Qt::LeftButton, Qt::LeftButton, event->modifiers());
+            m_dragging = true;
+            mousePressEvent(mouseEvent);
+            break;
+        case QEvent::TabletMove:
+            mouseEvent = new QMouseEvent(QEvent::MouseMove, event->pos(),
+                                         (m_dragging) ? Qt::LeftButton : Qt::NoButton,
+                                         (m_dragging) ? Qt::LeftButton : Qt::NoButton, event->modifiers());
+            mouseMoveEvent(mouseEvent);
+            break;
+        case QEvent::TabletRelease:
+            mouseEvent = new QMouseEvent(QEvent::MouseButtonRelease, event->pos(),
+                                         Qt::LeftButton,
+                                         Qt::LeftButton,
+                                         event->modifiers());
+            m_dragging = false;
+            mouseReleaseEvent(mouseEvent);
+            break;
+        default: break;
         }
         delete mouseEvent;
     }
@@ -79,7 +86,7 @@ KisPopupPalette::KisPopupPalette(KoFavoriteResourceManager* manager, QWidget *pa
     connect(m_triangleColorSelector, SIGNAL(colorChanged(const QColor& )), SLOT(slotChangefGColor(QColor)));
     connect(this, SIGNAL(sigChangeActivePaintop(int)), m_resourceManager, SLOT(slotChangeActivePaintop(int)));
     connect(this, SIGNAL(sigUpdateRecentColor(int)), m_resourceManager, SLOT(slotUpdateRecentColor(int)));
-    connect(this, SIGNAL(sigChangefGColor(const KoColor&)), m_resourceManager, SIGNAL(sigSetFGColor(KoColor)));
+    connect(this, SIGNAL(sigChangefGColor(const KoColor&)), m_resourceManager, SIGNAL(sigSetFGColor(const KoColor&)));
     connect(m_resourceManager, SIGNAL(sigChangeFGColorSelector(const QColor&)), m_triangleColorSelector, SLOT(setQColor(const QColor&)));
 
     // This is used to handle a bug:
@@ -93,6 +100,11 @@ KisPopupPalette::KisPopupPalette(KoFavoriteResourceManager* manager, QWidget *pa
     connect(m_timer, SIGNAL(timeout()), this, SLOT(slotEnableChangeFGColor()));
     connect(this, SIGNAL(sigEnableChangeFGColor(bool)), m_resourceManager, SIGNAL(sigEnableChangeColor(bool)));
 
+    m_colorChangeTimer = new QTimer(this);
+    m_colorChangeTimer->setInterval(50);
+    m_colorChangeTimer->setSingleShot(true);
+    connect(m_colorChangeTimer, SIGNAL(timeout()), this, SLOT(slotColorChangeTimeout()));
+
     setMouseTracking(true);
     setHoveredBrush(-1);
     setHoveredColor(-1);
@@ -113,11 +125,14 @@ void KisPopupPalette::setHoveredColor(int x) { m_hoveredColor = x; }
 int KisPopupPalette::selectedColor() const { return m_selectedColor; }
 void KisPopupPalette::setSelectedColor(int x) { m_selectedColor = x; }
 
-void KisPopupPalette::slotChangefGColor(const QColor& newColor)
+void KisPopupPalette::slotChangefGColor(const QColor& /*newColor*/)
 {
-    KoColor color;
-    color.fromQColor(newColor);
+    m_colorChangeTimer->start();
+}
 
+void KisPopupPalette::slotColorChangeTimeout()
+{
+    KoColor color ( m_triangleColorSelector->color(), KoColorSpaceRegistry::instance()->rgb16(0));
     emit sigChangefGColor(color);
 }
 
@@ -159,18 +174,20 @@ void KisPopupPalette::showPopupPalette (const QPoint &p)
 void KisPopupPalette::showPopupPalette(bool show)
 {
     if (show)
+    {
         emit sigEnableChangeFGColor(!show);
-    else
-        emit sigTriggerTimer();
-
-    if(show)
         QApplication::setOverrideCursor(Qt::ArrowCursor);
+    }
     else
+    {
+        emit sigTriggerTimer();
         QApplication::restoreOverrideCursor();
+    }
 
     setVisible(show);
 }
 
+//redefinition of setVariable function to change the scope to private
 void KisPopupPalette::setVisible(bool b)
 {
     QWidget::setVisible(b);
@@ -230,21 +247,21 @@ void KisPopupPalette::paintEvent(QPaintEvent* e)
         painter.drawPixmap(QPoint(pointTemp.x()-pixmapOffset.x(), pointTemp.y()-pixmapOffset.y()), pixmap);
     }
 
+    QColor currColor;
+    //painting currently used color as color selector background
+    if (selectedColor() > -1)
+    {
+        m_resourceManager->recentColorAt(selectedColor()).toQColor(&currColor);
+        painter.setPen(Qt::NoPen);
+        QPainterPath path;
+        path.addEllipse(QPoint(0,0), colorInnerRadius-5, colorInnerRadius-5);
+        painter.fillPath(path, currColor);
+        painter.drawPath(path);
+    }
+
     //painting recent colors : bevel (raised)
-    float radius = 51.0;
-    //outer radius
-    drawArcRisen
-            (painter, QColor(Qt::white), radius, 45, cos(M_PI/4)*radius, -1*sin(M_PI/4)*radius, 2*radius+0.5, 2*radius+0.5);
-    drawArcRisen
-            (painter, QColor(Qt::black), radius, 180+45, -1*cos(M_PI/4)*radius, sin(M_PI/4)*radius, 2*radius+0.5, 2*radius+0.5);
-
-    //inner radius
-    radius = 33.5;
-    drawArcRisen
-            (painter, QColor(Qt::black), radius, 45, cos(M_PI/4)*radius, -1*sin(M_PI/4)*radius, 2*radius+0.5, 2*radius+0.5);
-    drawArcRisen
-            (painter, QColor(Qt::white), radius, 180+45, -1*cos(M_PI/4)*radius, sin(M_PI/4)*radius, 2*radius, 2*radius);
-
+    drawArcRisen (painter, Qt::darkGray, Qt::white, 51);
+    drawArcRisen (painter, Qt::white, Qt::darkGray, 34);
 
     //painting recent colors
     painter.setPen(Qt::NoPen);
@@ -255,7 +272,7 @@ void KisPopupPalette::paintEvent(QPaintEvent* e)
 
     for (int pos = 0; pos < m_resourceManager->recentColorsTotal(); pos++)
     {
-        QPainterPath path(drawDonutPathAngle(colorInnerRadius , colorOuterRadius, m_resourceManager->recentColorsTotal()));
+        QPainterPath path(drawDonutPathAngle(colorInnerRadius, colorOuterRadius, m_resourceManager->recentColorsTotal()));
 
         //accessing recent color of index pos
         kcolor = m_resourceManager->recentColorAt(pos);
@@ -288,7 +305,7 @@ void KisPopupPalette::paintEvent(QPaintEvent* e)
         else
         {
             painter.rotate((m_resourceManager->recentColorsTotal() + hoveredColor()) *rotationAngle);
-            QPainterPath path(drawDonutPathAngle(colorInnerRadius,colorOuterRadius, m_resourceManager->recentColorsTotal()));
+            QPainterPath path(drawDonutPathAngle(colorInnerRadius, colorOuterRadius, m_resourceManager->recentColorsTotal()));
             painter.drawPath(path);
             painter.rotate(hoveredColor() *-1 *rotationAngle);
         }
@@ -314,14 +331,14 @@ void KisPopupPalette::paintEvent(QPaintEvent* e)
     }
 }
 
-void KisPopupPalette::drawArcRisen
-        (QPainter& painter, QColor color, int radius, int startAngle, float x, float y, float w, float h)
+void KisPopupPalette::drawArcRisen (QPainter& painter, const QColor& color1, const QColor& color2, int radius)
 {
-    painter.setPen(QPen(color ,2, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
-    QPainterPath path;
-    path.moveTo(x,y);
-    path.arcTo(-1*radius, -1*radius, w, h, startAngle, 180);
-    painter.drawPath(path);
+    QRadialGradient gradient;
+    gradient.setColorAt(1, color1);
+    gradient.setColorAt(0, color2);
+    gradient.setFocalPoint(sin(M_PI/4)*radius, cos(M_PI/4)*radius);
+    painter.setPen(QPen(QBrush(gradient), 3, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
+    painter.drawEllipse(-1*radius+0.5, -1*radius+0.5, 2*radius+0.5, 2*radius+0.5);
 }
 
 QPainterPath KisPopupPalette::drawDonutPathFull(int x, int y, int inner_radius, int outer_radius)
@@ -337,9 +354,8 @@ QPainterPath KisPopupPalette::drawDonutPathFull(int x, int y, int inner_radius,
 QPainterPath KisPopupPalette::drawDonutPathAngle(int inner_radius, int outer_radius, int limit)
 {
     QPainterPath path;
-    path.moveTo(-1*outer_radius * sin(M_PI/limit),
-                outer_radius * cos(M_PI/limit));
-    path.arcTo((-1*outer_radius), -1*outer_radius, 2*outer_radius,2*outer_radius,-90.0 - 180.0/limit,
+    path.moveTo(-0.999 * outer_radius * sin(M_PI/limit), 0.999 * outer_radius * cos(M_PI/limit));
+    path.arcTo(-1*outer_radius, -1*outer_radius, 2*outer_radius,2*outer_radius,-90.0 - 180.0/limit,
                360.0/limit);
     path.arcTo(-1*inner_radius, -1*inner_radius, 2*inner_radius,2*inner_radius,-90.0 + 180.0/limit,
                - 360.0/limit);
@@ -364,8 +380,8 @@ void KisPopupPalette::mouseMoveEvent (QMouseEvent* event)
         int pos = calculateIndex(point, m_resourceManager->favoriteBrushesTotal());
 
         if (pos >= 0 && pos < m_resourceManager->favoriteBrushesTotal()
-            && isPointInPixmap(point, pos))
-            {
+                && isPointInPixmap(point, pos))
+        {
             setHoveredBrush(pos);
         }
     }
@@ -389,14 +405,13 @@ void KisPopupPalette::mousePressEvent(QMouseEvent* event)
     if (event->button() == Qt::LeftButton)
     {
         QPainterPath pathBrush(drawDonutPathFull(width()/2, height()/2, brushInnerRadius, brushOuterRadius));
-        //        QPainterPath pathColor(drawDonutPathFull(width()/2, height()/2, colorInnerRadius, colorOuterRadius));
 
         if (pathBrush.contains(point))
         { //in favorite brushes area
             int pos = calculateIndex(point, m_resourceManager->favoriteBrushesTotal());
             if (pos >= 0 && pos < m_resourceManager->favoriteBrushesTotal()
-                && isPointInPixmap(point, pos))
-                {
+                    && isPointInPixmap(point, pos))
+            {
                 setSelectedBrush(pos);
                 update();
             }
@@ -413,8 +428,6 @@ void KisPopupPalette::mouseReleaseEvent ( QMouseEvent * event )
     {
         QPainterPath pathBrush(drawDonutPathFull(width()/2, height()/2, brushInnerRadius, brushOuterRadius));
         QPainterPath pathColor(drawDonutPathFull(width()/2, height()/2, colorInnerRadius, colorOuterRadius));
-        QPainterPath pathSelCol;
-        pathSelCol.addEllipse(QPoint(width()/2, height()/2), 30,30);
 
         if (pathBrush.contains(point))
         { //in favorite brushes area
@@ -434,7 +447,7 @@ void KisPopupPalette::mouseReleaseEvent ( QMouseEvent * event )
             }
         }
     }
-    else if (event->button() == Qt::MidButton)
+    else if (event->button() == Qt::RightButton)
     {
         showPopupPalette(false);
     }
@@ -446,19 +459,23 @@ void KisPopupPalette::tabletEvent(QTabletEvent* event)
     QMouseEvent* mouseEvent = 0;
     switch(event->type())
     {
-        case QEvent::TabletPress: mouseEvent = new QMouseEvent(QEvent::MouseButtonPress, event->pos(),
-                                                               Qt::LeftButton, event->modifiers());
-                                  mousePressEvent(mouseEvent);
-                                  break;
-        case QEvent::TabletMove: mouseEvent = new QMouseEvent(QEvent::MouseMove, event->pos(),
-                                                              Qt::NoButton, event->modifiers());
-                                 mouseMoveEvent(mouseEvent);
-                                 break;
-        case QEvent::TabletRelease: mouseEvent = new QMouseEvent(QEvent::MouseButtonRelease, event->pos(),
-                                                                 Qt::LeftButton, event->modifiers());
-                                    mouseReleaseEvent(mouseEvent);
-                                    break;
-        default: break;
+    case QEvent::TabletPress:
+
+        mouseEvent = new QMouseEvent(QEvent::MouseButtonPress, event->pos(),
+                                     Qt::LeftButton, Qt::LeftButton, event->modifiers());
+        mousePressEvent(mouseEvent);
+        break;
+    case QEvent::TabletMove:
+        mouseEvent = new QMouseEvent(QEvent::MouseMove, event->pos(),
+                                     Qt::NoButton, Qt::NoButton, event->modifiers());
+        mouseMoveEvent(mouseEvent);
+        break;
+    case QEvent::TabletRelease:
+        mouseEvent = new QMouseEvent(QEvent::MouseButtonRelease, event->pos(),
+                                     Qt::LeftButton, Qt::LeftButton, event->modifiers());
+        mouseReleaseEvent(mouseEvent);
+        break;
+    default: break;
     }
     delete mouseEvent;
 }
@@ -493,8 +510,7 @@ bool KisPopupPalette::isPointInPixmap(QPointF& point, int pos)
                  brushRadius*cos(angle)-pixmap.height()/2+height()/2,
                  pixmap.width(), pixmap.height());
 
-    if(path.contains(point) || pixmap.isNull()) return true;
-    else return false;
+    return path.contains(point) || pixmap.isNull();
 }
 
 KisPopupPalette::~KisPopupPalette()
