@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2006 Boudewijn Rempt <boud@valdyas.org>
+ *  Copyright (c) 2006, 2010 Boudewijn Rempt <boud@valdyas.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include <QLabel>
 #include <QWidget>
 #include <QPolygonF>
-#include <QMatrix>
+#include <QTransform>
 
 #include <klocale.h>
 #include <kaction.h>
@@ -63,6 +63,7 @@
 
 #include "kis_canvas_resource_provider.h"
 #include "canvas/kis_canvas2.h"
+#include "kis_coordinates_converter.h"
 #include "filter/kis_filter_configuration.h"
 #include "kis_config.h"
 #include "kis_config_notifier.h"
@@ -70,11 +71,13 @@
 #include <recorder/kis_recorded_paint_action.h>
 
 struct KisTool::Private {
-    Private() : currentPattern(0),
-            currentGradient(0),
-            currentPaintOpPreset(0),
-            currentGenerator(0),
-            optionWidget(0) { }
+    Private()
+        : currentPattern(0),
+          currentGradient(0),
+          currentPaintOpPreset(0),
+          currentGenerator(0),
+          optionWidget(0),
+          spacePressed(0) { }
     QCursor cursor; // the cursor that should be shown on tool activation.
 
     // From the canvas resources
@@ -92,6 +95,10 @@ struct KisTool::Private {
     KAction* toggleFgBg;
     KAction* resetFgBg;
 
+    bool spacePressed;
+    QPointF lastDocumentPoint;
+    QPointF initialGestureDocPoint;
+    QPoint initialGestureGlobalPoint;
 };
 
 KisTool::KisTool(KoCanvasBase * canvas, const QCursor & cursor)
@@ -113,6 +120,7 @@ KisTool::KisTool(KoCanvasBase * canvas, const QCursor & cursor)
     connect(d->resetFgBg, SIGNAL(triggered()), this, SLOT(slotResetFgBg()));
     addAction("reset_fg_bg", d->resetFgBg);
 
+    setMode(HOVER_MODE);
 }
 
 KisTool::~KisTool()
@@ -183,6 +191,23 @@ void KisTool::resourceChanged(int key, const QVariant & v)
     };
 }
 
+QPointF KisTool::widgetCenterInWidgetPixels()
+{
+    KisCanvas2 *kritaCanvas = dynamic_cast<KisCanvas2*>(canvas());
+    Q_ASSERT(kritaCanvas);
+
+    const KisCoordinatesConverter *converter = kritaCanvas->coordinatesConverter();
+    return converter->flakeToWidget(converter->flakeCenterPoint());
+}
+
+QPointF KisTool::convertDocumentToWidget(const QPointF& pt)
+{
+    KisCanvas2 *kritaCanvas = dynamic_cast<KisCanvas2*>(canvas());
+    Q_ASSERT(kritaCanvas);
+
+    return kritaCanvas->coordinatesConverter()->documentToWidget(pt);
+}
+
 QPointF KisTool::convertToPixelCoord(KoPointerEvent *e)
 {
     if (!image())
@@ -253,7 +278,7 @@ QRectF KisTool::pixelToView(const QRectF &pixelRect) const
 
 QPainterPath KisTool::pixelToView(const QPainterPath &pixelPolygon) const
 {
-    QMatrix matrix;
+    QTransform matrix;
     qreal zoomX, zoomY;
     canvas()->viewConverter()->zoom(&zoomX, &zoomY);
     matrix.scale(zoomX/image()->xRes(), zoomY/ image()->yRes());
@@ -262,7 +287,7 @@ QPainterPath KisTool::pixelToView(const QPainterPath &pixelPolygon) const
 
 QPolygonF KisTool::pixelToView(const QPolygonF &pixelPath) const
 {
-    QMatrix matrix;
+    QTransform matrix;
     qreal zoomX, zoomY;
     canvas()->viewConverter()->zoom(&zoomX, &zoomY);
     matrix.scale(zoomX/image()->xRes(), zoomY/ image()->yRes());
@@ -355,21 +380,183 @@ KisFilterConfiguration * KisTool::currentGenerator()
     return d->currentGenerator;
 }
 
+bool KisTool::specialModifierActive()
+{
+    KisCanvas2 *canvas2 = dynamic_cast<KisCanvas2 *>(canvas());
+    bool popupPalletActive = canvas2->handlePopupPaletteIsVisible();
+
+    KisConfig cfg;
+    return popupPalletActive || (d->spacePressed && !cfg.clicklessSpacePan());
+}
+
+void KisTool::setMode(ToolMode mode) {
+    m_mode = mode;
+}
+
+KisTool::ToolMode KisTool::mode() {
+    return m_mode;
+}
+
 void KisTool::mousePressEvent(KoPointerEvent *event)
 {
-    event->ignore();
+    KisConfig cfg;
+
+    if (mode() == KisTool::HOVER_MODE &&
+        ((event->button() == Qt::MidButton &&
+          event->modifiers() == Qt::NoModifier) ||
+         (d->spacePressed && !cfg.clicklessSpacePan()))) {
+
+        initPan(event->point);
+        event->accept();
+    }
+    else if (mode() == KisTool::HOVER_MODE &&
+             (event->button() == Qt::LeftButton &&
+              event->modifiers() == Qt::ShiftModifier)) {
+
+        initGesture(event->point);
+        event->accept();
+    }
+
+
+    d->lastDocumentPoint = event->point;
 }
 
 void KisTool::mouseMoveEvent(KoPointerEvent *event)
 {
-    event->ignore();
+    if(mode() == PAN_MODE) {
+        pan(event->point);
+        event->accept();
+    }
+    else if (mode() == GESTURE_MODE) {
+        processGesture(event->point);
+        event->accept();
+    }
+
+    d->lastDocumentPoint = event->point;
 }
 
 void KisTool::mouseReleaseEvent(KoPointerEvent *event)
 {
+    KisConfig cfg;
+
+    if(mode() == PAN_MODE) {
+        if (event->button() == Qt::MidButton ||
+            (event->button() == Qt::LeftButton && !cfg.clicklessSpacePan())) {
+
+            endPan();
+            event->accept();
+        }
+    }
+    else if (mode() == GESTURE_MODE) {
+        if (event->button() == Qt::LeftButton) {
+            endGesture();
+            event->accept();
+        }
+    }
+
+    d->lastDocumentPoint = event->point;
+}
+
+void KisTool::keyPressEvent(QKeyEvent *event)
+{
+    if (event->key() == Qt::Key_Space) {
+
+        KisConfig cfg;
+        if(cfg.clicklessSpacePan()) {
+            initPan(d->lastDocumentPoint);
+        }
+        else {
+            d->spacePressed = true;
+        }
+
+        event->accept();
+        return;
+    }
+
     event->ignore();
 }
 
+void KisTool::keyReleaseEvent(QKeyEvent* event)
+{
+    if (event->key() == Qt::Key_Space) {
+
+        KisConfig cfg;
+        if(cfg.clicklessSpacePan()) {
+            endPan();
+        }
+        else {
+            d->spacePressed = false;
+        }
+
+        event->accept();
+        return;
+    }
+
+    event->ignore();
+}
+
+void KisTool::initPan(const QPointF &docPoint)
+{
+    setMode(PAN_MODE);
+    m_lastPosition = convertDocumentToWidget(docPoint);
+    useCursor(QCursor(Qt::ClosedHandCursor));
+}
+
+void KisTool::pan(const QPointF &docPoint)
+{
+    Q_ASSERT(canvas());
+    Q_ASSERT(canvas()->canvasController());
+
+    QPointF actualPosition = convertDocumentToWidget(docPoint);
+    QPointF distance(m_lastPosition - actualPosition);
+    canvas()->canvasController()->pan(distance.toPoint());
+
+    m_lastPosition = actualPosition;
+}
+
+void KisTool::endPan()
+{
+    setMode(HOVER_MODE);
+    resetCursorStyle();
+}
+
+void KisTool::initGesture(const QPointF &docPoint)
+{
+    setMode(GESTURE_MODE);
+    d->initialGestureDocPoint = docPoint;
+    d->initialGestureGlobalPoint = QCursor::pos();
+    useCursor(KisCursor::blankCursor());
+}
+
+void KisTool::processGesture(const QPointF &docPoint)
+{
+    QPointF lastWidgetPosition = convertDocumentToWidget(d->lastDocumentPoint);
+    QPointF actualWidgetPosition = convertDocumentToWidget(docPoint);
+
+    QPointF offset = actualWidgetPosition - lastWidgetPosition;
+
+    /**
+     * view pixels != widget pixels, but we do this anyway, we only
+     * need to scale the gesture down, not rotate or anything
+     */
+    QPointF scaledOffset = canvas()->viewConverter()->viewToDocument(offset);
+    gesture(scaledOffset, d->initialGestureDocPoint);
+}
+
+void KisTool::endGesture()
+{
+    gesture(QPointF(), d->initialGestureDocPoint);
+    setMode(HOVER_MODE);
+    resetCursorStyle();
+    QCursor::setPos(d->initialGestureGlobalPoint);
+}
+
+void KisTool::gesture(const QPointF &offsetInDocPixels, const QPointF &initialDocPoint)
+{
+    Q_UNUSED(offsetInDocPixels);
+    Q_UNUSED(initialDocPoint);
+}
+
 void KisTool::setupPainter(KisPainter* painter)
 {
     Q_ASSERT(currentImage());
@@ -414,8 +601,11 @@ QWidget* KisTool::optionWidget()
 void KisTool::paintToolOutline(QPainter* painter, const QPainterPath &path)
 {
     //KisToolSelectMagnetic uses custom painting, so don't forget to update that as well
+    KisConfig cfg;
+    bool useWorkaround = cfg.useOpenGLToolOutlineWorkaround();
 #if defined(HAVE_OPENGL)
-    if (m_outlinePaintMode==XOR_MODE && isCanvasOpenGL()) {
+
+    if (m_outlinePaintMode==XOR_MODE && isCanvasOpenGL() && !useWorkaround) {
         beginOpenGL();
 
         glEnable(GL_LINE_SMOOTH);
@@ -443,7 +633,7 @@ void KisTool::paintToolOutline(QPainter* painter, const QPainterPath &path)
     else
 #endif
 #ifdef INDEPENDENT_CANVAS
-    if (m_outlinePaintMode==XOR_MODE) {
+    if (m_outlinePaintMode==XOR_MODE && !(isCanvasOpenGL() && useWorkaround)) {
         painter->setCompositionMode(QPainter::RasterOp_SourceXorDestination);
         painter->setPen(QColor(128, 255, 128));
         painter->drawPath(path);
@@ -481,7 +671,11 @@ void KisTool::resetCursorStyle()
         break;
 #if defined(HAVE_OPENGL)
     case CURSOR_STYLE_3D_MODEL:
-        useCursor(d->cursor);
+        if(isCanvasOpenGL()) {
+            useCursor(d->cursor);
+        } else {
+            useCursor(KisCursor::upArrowCursor());
+        }
         break;
 #endif
     case CURSOR_STYLE_OUTLINE:
@@ -509,7 +703,7 @@ void KisTool::slotResetFgBg()
 
 bool KisTool::isCanvasOpenGL() const
 {
-    return canvas()->canvasController()->isCanvasOpenGL();
+    return canvas()->canvasIsOpenGL();
 }
 
 void KisTool::beginOpenGL()
@@ -520,6 +714,7 @@ void KisTool::beginOpenGL()
 
     if (canvasWidget) {
         canvasWidget->beginOpenGL();
+        canvasWidget->setupFlakeToWidgetTransformation();
     }
 #endif
 }
@@ -536,5 +731,12 @@ void KisTool::endOpenGL()
 #endif
 }
 
+void KisTool::setCurrentNodeLocked(bool locked)
+{
+    if (currentNode()) {
+        currentNode()->setSystemLocked(locked, false);
+    }
+}
+
 #include "kis_tool.moc"
 
