@@ -28,6 +28,28 @@
 #include <krita_export.h>
 #include <kis_types.h>
 
+#define PRESS_CONDITION(_event, _mode, _button, _modifier)              \
+    (mode() == (_mode) && (_event)->button() == (_button) &&            \
+     (_event)->modifiers() == (_modifier) && !specialModifierActive())
+
+#define PRESS_CONDITION_WB(_event, _mode, _button, _modifier)            \
+    (mode() == (_mode) && (_event)->button() & (_button) &&            \
+     (_event)->modifiers() == (_modifier) && !specialModifierActive())
+
+#define PRESS_CONDITION_OM(_event, _mode, _button, _modifier)           \
+    (mode() == (_mode) && (_event)->button() == (_button) &&            \
+     ((_event)->modifiers() & (_modifier) ||                            \
+      (_event)->modifiers() == Qt::NoModifier) &&                       \
+     !specialModifierActive())
+
+#define RELEASE_CONDITION(_event, _mode, _button)               \
+    (mode() == (_mode) && (_event)->button() == (_button))
+
+#define RELEASE_CONDITION_WB(_event, _mode, _button)               \
+    (mode() == (_mode) && (_event)->button() & (_button))
+
+#define MOVE_CONDITION(_event, _mode) (mode() == (_mode))
+
 class KoCanvasBase;
 class KisPattern;
 class KoAbstractGradient;
@@ -74,20 +96,21 @@ public slots:
 
     virtual void resourceChanged(int key, const QVariant & res);
 
-public:
+protected:
 
-    /// reimplemented from superclass
     virtual void mousePressEvent(KoPointerEvent *event);
-
-    /// reimplemented from superclass
     virtual void mouseMoveEvent(KoPointerEvent *event);
-
-    /// reimplemented from superclass
     virtual void mouseReleaseEvent(KoPointerEvent *event);
 
+    virtual void keyPressEvent(QKeyEvent *event);
+    virtual void keyReleaseEvent(QKeyEvent* event);
+
     /// reimplemented from superclass
     virtual void mouseDoubleClickEvent(KoPointerEvent *) {}  // when a krita tool is enabled, don't push double click on
 
+    QPointF widgetCenterInWidgetPixels();
+    QPointF convertDocumentToWidget(const QPointF& pt);
+
     /// Convert from native (postscript points) to image pixel
     /// coordinates.
     QPointF convertToPixelCoord(KoPointerEvent *e);
@@ -132,6 +155,10 @@ public:
 
 protected:
 
+    bool specialModifierActive();
+    virtual void gesture(const QPointF &offsetInDocPixels,
+                         const QPointF &initialDocPoint);
+
     KisImageWSP image() const;
     QCursor cursor() const;
 
@@ -169,6 +196,23 @@ protected:
     /// Call after finishing use of native OpenGL commands when painting this tool's decorations.
     /// This is a convenience method that calls endOpenGL() on the OpenGL canvas object.
     void endOpenGL();
+    
+    /// Sets the systemLocked for the current node, this will not deactivate the tool buttons
+    void setCurrentNodeLocked(bool locked);
+
+protected:
+    enum ToolMode {
+        HOVER_MODE,
+        PAINT_MODE,
+        SECONDARY_PAINT_MODE,
+        GESTURE_MODE,
+        PAN_MODE,
+        OTHER // not used now
+    };
+
+    virtual void setMode(ToolMode mode);
+    virtual ToolMode mode();
+
 
 protected slots:
     /**
@@ -181,8 +225,19 @@ private slots:
     void slotToggleFgBg();
     void slotResetFgBg();
 
+private:
+    void initPan(const QPointF &docPoint);
+    void pan(const QPointF &docPoint);
+    void endPan();
+
+    void initGesture(const QPointF &docPoint);
+    void processGesture(const QPointF &docPoint);
+    void endGesture();
+
 private:
     PaintMode m_outlinePaintMode;
+    ToolMode m_mode;
+    QPointF m_lastPosition;
 
     struct Private;
     Private* const d;
