@@ -1,4 +1,5 @@
 /* This file is part of the KDE project
+   Copyright 2010 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
    Copyright 2006-2007 Stefan Nikolaus <stefan.nikolaus@kdemail.net>
    Copyright 2005 Raphael Langerhorst <raphael.langerhorst@kdemail.net>
    Copyright 2004-2005 Tomas Mecir <mecirt@gmail.com>
@@ -59,7 +60,6 @@
 #include "OdfLoadingContext.h"
 #include "OdfSavingContext.h"
 #include "RowColumnFormat.h"
-#include "Selection.h"
 #include "ShapeApplicationData.h"
 #include "Sheet.h"
 #include "Style.h"
@@ -90,7 +90,6 @@
 #include <KoParagraphStyle.h>
 
 #include <kdebug.h>
-#include <KNotification>
 
 #include <QTimer>
 #include <QTextDocument>
@@ -104,8 +103,8 @@ public:
     Private() : sheet(0), column(0), row(0) {}
 
     Sheet*  sheet;
-uint    column  : 16; // KS_colMax
-uint    row     : 16; // KS_rowMax
+    uint    column  : 17; // KS_colMax
+    uint    row     : 17; // KS_rowMax
 };
 
 
@@ -180,6 +179,27 @@ bool Cell::isDefault() const
     return true;
 }
 
+// Return true if this is the default cell (apart from maybe a custom style).
+bool Cell::hasDefaultContent() const
+{
+    // check each stored attribute
+    if (value() != Value())
+        return false;
+    if (formula() != Formula::empty())
+        return false;
+    if (!link().isEmpty())
+        return false;
+    if (doesMergeCells() == true)
+        return false;
+    if (!comment().isEmpty())
+        return false;
+    if (!conditions().isEmpty())
+        return false;
+    if (!validity().isEmpty())
+        return false;
+    return true;
+}
+
 bool Cell::isEmpty() const
 {
     // empty = no value or formula
@@ -331,8 +351,10 @@ Style Cell::effectiveStyle() const
 {
     Style style = sheet()->cellStorage()->style(d->column, d->row);
     // use conditional formatting attributes
-    if (Style* conditialStyle = conditions().testConditions(*this, sheet()->map()->styleManager()))
-        style.merge(*conditialStyle);
+    const Style conditionalStyle = conditions().testConditions(*this);
+    if (!conditionalStyle.isEmpty()) {
+        style.merge(conditionalStyle);
+    }
     return style;
 }
 
@@ -369,6 +391,8 @@ QString Cell::userInput() const
 
 void Cell::setUserInput(const QString& string)
 {
+    QString old = userInput();
+
     if (!string.isEmpty() && string[0] == '=') {
         // set the formula
         Formula formula(sheet(), *this);
@@ -378,13 +402,15 @@ void Cell::setUserInput(const QString& string)
         sheet()->cellStorage()->setUserInput(d->column, d->row, QString());
     } else {
         // remove an existing formula
-        setFormula(Formula());
+        setFormula(Formula::empty());
         // set the value
         sheet()->cellStorage()->setUserInput(d->column, d->row, string);
     }
 
-    // remove any existing richtext
-    setRichText(QSharedPointer<QTextDocument>());
+    if (old != string) {
+        // remove any existing richtext
+        setRichText(QSharedPointer<QTextDocument>());
+    }
 }
 
 
@@ -700,11 +726,7 @@ bool Cell::makeFormula()
 
     // parse the formula and check for errors
     if (!formula().isValid()) {
-        KNotification *notify = new KNotification("ParsingError");
-        notify->setText(i18n("Parsing of formula in cell <i>%1</i> failed.", fullName()));
-        notify->addContext("cell", fullName());
-        notify->addContext("expression", userInput());
-        QTimer::singleShot(0, notify, SLOT(sendEvent()));
+        sheet()->showStatusMessage(i18n("Parsing of formula in cell %1 failed.", fullName()));
         setValue(Value::errorPARSE());
         return false;
     }
@@ -758,7 +780,7 @@ void Cell::parseUserInput(const QString& text)
     if (text.isEmpty()) {
         setValue(Value::empty());
         setUserInput(text);
-        setFormula(Formula());
+        setFormula(Formula::empty());
         return;
     }
 
@@ -770,11 +792,7 @@ void Cell::parseUserInput(const QString& text)
 
         // parse the formula and check for errors
         if (!formula.isValid()) {
-            KNotification *notify = new KNotification("ParsingError");
-            notify->setText(i18n("Parsing of formula in cell <i>%1</i> failed.", fullName()));
-            notify->addContext("cell", fullName());
-            notify->addContext("expression", userInput());
-            QTimer::singleShot(0, notify, SLOT(sendEvent()));
+            sheet()->showStatusMessage(i18n("Parsing of formula in cell %1 failed.", fullName()));
             setValue(Value::errorPARSE());
             return;
         }
@@ -931,14 +949,14 @@ QDomElement Cell::save(QDomDocument& doc, int xOffset, int yOffset, bool era)
 
     Conditions conditions = this->conditions();
     if (!conditions.isEmpty()) {
-        QDomElement conditionElement = conditions.saveConditions(doc);
+        QDomElement conditionElement = conditions.saveConditions(doc, sheet()->map()->converter());
         if (!conditionElement.isNull())
             cell.appendChild(conditionElement);
     }
 
     Validity validity = this->validity();
     if (!validity.isEmpty()) {
-        QDomElement validityElement = validity.saveXML(doc);
+        QDomElement validityElement = validity.saveXML(doc, sheet()->map()->converter());
         if (!validityElement.isNull())
             cell.appendChild(validityElement);
     }
@@ -1056,7 +1074,7 @@ QString Cell::saveOdfCellStyle(KoGenStyle &currentCellStyle, KoGenStyles &mainSt
     if (!conditions.isEmpty()) {
         // this has to be an automatic style
         currentCellStyle = KoGenStyle(KoGenStyle::TableCellAutoStyle, "table-cell");
-        conditions.saveOdfConditions(currentCellStyle);
+        conditions.saveOdfConditions(currentCellStyle, sheet()->map()->converter());
     }
     return style().saveOdf(currentCellStyle, mainStyles, d->sheet->map()->styleManager());
 }
@@ -1087,19 +1105,22 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     if (link().isEmpty())
         saveOdfValue(xmlwriter);
 
+    const Style cellStyle = style();
+
     // Either there's no column and row default and the style's not the default style,
     // or the style is different to one of them. The row default takes precedence.
     if ((!tableContext.rowDefaultStyles.contains(row) &&
             !tableContext.columnDefaultStyles.contains(column) &&
-            !(style().isDefault() && conditions().isEmpty())) ||
-            (tableContext.rowDefaultStyles.contains(row) && tableContext.rowDefaultStyles[row] != style()) ||
-            (tableContext.columnDefaultStyles.contains(column) && tableContext.columnDefaultStyles[column] != style())) {
+            !(cellStyle.isDefault() && conditions().isEmpty())) ||
+            (tableContext.rowDefaultStyles.contains(row) && tableContext.rowDefaultStyles[row] != cellStyle) ||
+            (tableContext.columnDefaultStyles.contains(column) && tableContext.columnDefaultStyles[column] != cellStyle)) {
         KoGenStyle currentCellStyle; // the type determined in saveOdfCellStyle
         saveOdfCellStyle(currentCellStyle, mainStyles);
         // skip 'table:style-name' attribute for the default style
         if (!currentCellStyle.isDefaultStyle()) {
-            if (mainStyles.styles().contains(currentCellStyle))
-                xmlwriter.addAttribute("table:style-name", mainStyles.styles().find(currentCellStyle).value());
+            QString styleName = mainStyles.styles().value(currentCellStyle, QString());
+            if (!styleName.isEmpty())
+                xmlwriter.addAttribute("table:style-name", styleName);
         }
     }
 
@@ -1129,15 +1150,15 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
 
             if (nextCell.isPartOfMerged() || nextCell.doesMergeCells() ||
                     !nextCell.comment().isEmpty() || tableContext.cellHasAnchoredShapes(sheet(), row, column) ||
-                    !(nextCell.style() == style() && nextCell.conditions() == conditions())) {
+                    !(nextCell.style() == cellStyle && nextCell.conditions() == conditions())) {
                 break;
             }
             ++repeated;
             // get the next cell and set the index to the adjacent cell
             nextCell = sheet()->cellStorage()->nextInRow(j++, row);
         }
-        kDebug(36003) << "Cell::saveOdf: empty cell in column" << column
-        << "repeated" << repeated << "time(s)" << endl;
+        //kDebug(36003) << "Cell::saveOdf: empty cell in column" << column
+        //<< "repeated" << repeated << "time(s)" << endl;
 
         if (repeated > 1)
             xmlwriter.addAttribute("table:number-columns-repeated", QString::number(repeated));
@@ -1145,7 +1166,7 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
 
     Validity validity = Cell(sheet(), column, row).validity();
     if (!validity.isEmpty()) {
-        GenValidationStyle styleVal(&validity);
+        GenValidationStyle styleVal(&validity, sheet()->map()->converter());
         xmlwriter.addAttribute("table:validation-name", tableContext.valStyle.insert(styleVal));
     }
     if (isFormula()) {
@@ -1181,6 +1202,9 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     if (!isEmpty() && link().isEmpty()) {
         QSharedPointer<QTextDocument> doc = richText();
         if (doc) {
+            QTextCharFormat format = style().asCharFormat();
+            sheet()->map()->textStyleManager()->defaultParagraphStyle()->characterStyle()->copyProperties(format);
+
             KoEmbeddedDocumentSaver saver;
             KoShapeSavingContext shapeContext(xmlwriter, mainStyles, saver);
             KoTextWriter writer(shapeContext);
@@ -1273,7 +1297,7 @@ void Cell::saveOdfValue(KoXmlWriter &xmlWriter)
         if (isTime()) {
             xmlWriter.addAttribute("office:value-type", "time");
             xmlWriter.addAttribute("office:time-value",
-                                   value().asTime(sheet()->map()->calculationSettings()).toString("PThhHmmMssS"));
+                                   value().asTime(sheet()->map()->calculationSettings()).toString("'PT'hh'H'mm'M'ss'S'"));
         } else {
             xmlWriter.addAttribute("office:value-type", "date");
             xmlWriter.addAttribute("office:date-value",
@@ -1284,7 +1308,7 @@ void Cell::saveOdfValue(KoXmlWriter &xmlWriter)
     case Value::fmt_Time: {
         xmlWriter.addAttribute("office:value-type", "time");
         xmlWriter.addAttribute("office:time-value",
-                               value().asTime(sheet()->map()->calculationSettings()).toString("PThhHmmMssS"));
+                               value().asTime(sheet()->map()->calculationSettings()).toString("'PT'hh'H'mm'M'ss'S'"));
         break;
     }
     case Value::fmt_String: {
@@ -1295,24 +1319,47 @@ void Cell::saveOdfValue(KoXmlWriter &xmlWriter)
     };
 }
 
-bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
+bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext, const Styles& autoStyles, const QString& cellStyleName)
 {
+    static const QString sFormula           = QString::fromLatin1("formula");
+    static const QString sValidationName    = QString::fromLatin1("validation-name");
+    static const QString sValueType         = QString::fromLatin1("value-type");
+    static const QString sBoolean           = QString::fromLatin1("boolean");
+    static const QString sBooleanValue      = QString::fromLatin1("boolean-value");
+    static const QString sTrue              = QString::fromLatin1("true");
+    static const QString sFalse             = QString::fromLatin1("false");
+    static const QString sFloat             = QString::fromLatin1("float");
+    static const QString sValue             = QString::fromLatin1("value");
+    static const QString sCurrency          = QString::fromLatin1("currency");
+    static const QString sPercentage        = QString::fromLatin1("percentage");
+    static const QString sDate              = QString::fromLatin1("date");
+    static const QString sDateValue         = QString::fromLatin1("date-value");
+    static const QString sTime              = QString::fromLatin1("time");
+    static const QString sTimeValue         = QString::fromLatin1("time-value");
+    static const QString sString            = QString::fromLatin1("string");
+    static const QString sStringValue       = QString::fromLatin1("string-value");
+    static const QString sNumberColumnsSpanned = QString::fromLatin1("number-columns-spanned");
+    static const QString sNumberRowsSpanned = QString::fromLatin1("number-rows-spanned");
+    static const QString sAnnotation        = QString::fromLatin1("annotation");
+    static const QString sP                 = QString::fromLatin1("p");
+
+    static const QStringList formulaNSPrefixes = QStringList() << "oooc:" << "kspr:" << "of:" << "msoxl:";
+
     //Search and load each paragraph of text. Each paragraph is separated by a line break.
-    loadOdfCellText(element, tableContext);
+    loadOdfCellText(element, tableContext, autoStyles, cellStyleName);
 
     //
     // formula
     //
     bool isFormula = false;
-    if (element.hasAttributeNS(KoXmlNS::table, "formula")) {
-        kDebug(36003) << "cell:" << name() << "formula :" << element.attributeNS(KoXmlNS::table, "formula", QString());
+    if (element.hasAttributeNS(KoXmlNS::table, sFormula)) {
         isFormula = true;
-        QString oasisFormula(element.attributeNS(KoXmlNS::table, "formula", QString()));
+        QString oasisFormula(element.attributeNS(KoXmlNS::table, sFormula, QString()));
+        // kDebug(36003) << "cell:" << name() << "formula :" << oasisFormula;
         // each spreadsheet application likes to safe formulas with a different namespace
         // prefix, so remove all of them
-        QStringList prefixes = QStringList() << "oooc:" << "kspr:" << "of:" << "msoxl:";
         QString namespacePrefix;
-        foreach(const QString &prefix, prefixes) {
+        foreach(const QString &prefix, formulaNSPrefixes) {
             if (oasisFormula.startsWith(prefix)) {
                 oasisFormula = oasisFormula.mid(prefix.length());
                 namespacePrefix = prefix;
@@ -1327,9 +1374,9 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
     //
     // validation
     //
-    if (element.hasAttributeNS(KoXmlNS::table, "validation-name")) {
-        const QString validationName = element.attributeNS(KoXmlNS::table, "validation-name", QString());
-        kDebug(36003) << "cell:" << name() << "validation-name:" << validationName;
+    if (element.hasAttributeNS(KoXmlNS::table, sValidationName)) {
+        const QString validationName = element.attributeNS(KoXmlNS::table, sValidationName, QString());
+        kDebug(36003) << "cell:" << name() << sValidationName << validationName;
         Validity validity;
         validity.loadOdfValidation(this, validationName, tableContext);
         if (!validity.isEmpty())
@@ -1339,19 +1386,19 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
     //
     // value type
     //
-    if (element.hasAttributeNS(KoXmlNS::office, "value-type")) {
-        const QString valuetype = element.attributeNS(KoXmlNS::office, "value-type", QString());
-        kDebug(36003) << "cell:" << name() << "value-type:" << valuetype;
-        if (valuetype == "boolean") {
-            const QString val = element.attributeNS(KoXmlNS::office, "boolean-value", QString()).toLower();
-            if ((val == "true") || (val == "false"))
-                setValue(Value(val == "true"));
+    if (element.hasAttributeNS(KoXmlNS::office, sValueType)) {
+        const QString valuetype = element.attributeNS(KoXmlNS::office, sValueType, QString());
+        // kDebug(36003) << "cell:" << name() << "value-type:" << valuetype;
+        if (valuetype == sBoolean) {
+            const QString val = element.attributeNS(KoXmlNS::office, sBooleanValue, QString()).toLower();
+            if ((val == sTrue) || (val == sFalse))
+                setValue(Value(val == sTrue));
         }
 
         // integer and floating-point value
-        else if (valuetype == "float") {
+        else if (valuetype == sFloat) {
             bool ok = false;
-            Value value(element.attributeNS(KoXmlNS::office, "value", QString()).toDouble(&ok));
+            Value value(element.attributeNS(KoXmlNS::office, sValue, QString()).toDouble(&ok));
             if (ok) {
                 value.setFormat(Value::fmt_Number);
                 setValue(value);
@@ -1367,27 +1414,27 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
         }
 
         // currency value
-        else if (valuetype == "currency") {
+        else if (valuetype == sCurrency) {
             bool ok = false;
-            Value value(element.attributeNS(KoXmlNS::office, "value", QString()).toDouble(&ok));
+            Value value(element.attributeNS(KoXmlNS::office, sValue, QString()).toDouble(&ok));
             if (ok) {
                 value.setFormat(Value::fmt_Money);
                 setValue(value);
 
                 Currency currency;
-                if (element.hasAttributeNS(KoXmlNS::office, "currency")) {
-                    currency = Currency(element.attributeNS(KoXmlNS::office, "currency", QString()));
+                if (element.hasAttributeNS(KoXmlNS::office, sCurrency)) {
+                    currency = Currency(element.attributeNS(KoXmlNS::office, sCurrency, QString()));
                 }
-
-                if( style().isEmpty() ) {
+                /* TODO: somehow make this work again, all setStyle calls here will be overwritten by cell styles later
+                if( style.isEmpty() ) {
                     Style style;
                     style.setCurrency(currency);
                     setStyle(style);
-                }
+                } */
             }
-        } else if (valuetype == "percentage") {
+        } else if (valuetype == sPercentage) {
             bool ok = false;
-            Value value(element.attributeNS(KoXmlNS::office, "value", QString()).toDouble(&ok));
+            Value value(element.attributeNS(KoXmlNS::office, sValue, QString()).toDouble(&ok));
             if (ok) {
                 value.setFormat(Value::fmt_Percent);
                 setValue(value);
@@ -1400,8 +1447,8 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                 setStyle(style);
 #endif
             }
-        } else if (valuetype == "date") {
-            QString value = element.attributeNS(KoXmlNS::office, "date-value", QString());
+        } else if (valuetype == sDate) {
+            QString value = element.attributeNS(KoXmlNS::office, sDateValue, QString());
 
             // "1980-10-15" or "2001-01-01T19:27:41"
             int year = 0, month = 0, day = 0, hours = 0, minutes = 0, seconds = 0;
@@ -1449,10 +1496,10 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                 s.setFormatType(Format::ShortDate);
                 setStyle(s);
 #endif
-                kDebug(36003) << "cell:" << name() << "Type: date, value:" << value << "Date:" << year << " -" << month << " -" << day;
+                // kDebug(36003) << "cell:" << name() << "Type: date, value:" << value << "Date:" << year << " -" << month << " -" << day;
             }
-        } else if (valuetype == "time") {
-            QString value = element.attributeNS(KoXmlNS::office, "time-value", QString());
+        } else if (valuetype == sTime) {
+            QString value = element.attributeNS(KoXmlNS::office, sTimeValue, QString());
 
             // "PT15H10M12S"
             int hours = 0, minutes = 0, seconds = 0;
@@ -1487,11 +1534,11 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                 style.setFormatType(Format::Time);
                 setStyle(style);
 #endif
-                kDebug(36003) << "cell:" << name() << "Type: time:" << value << "Hours:" << hours << "," << minutes << "," << seconds;
+                // kDebug(36003) << "cell:" << name() << "Type: time:" << value << "Hours:" << hours << "," << minutes << "," << seconds;
             }
-        } else if (valuetype == "string") {
-            if (element.hasAttributeNS(KoXmlNS::office, "string-value")) {
-                QString value = element.attributeNS(KoXmlNS::office, "string-value", QString());
+        } else if (valuetype == sString) {
+            if (element.hasAttributeNS(KoXmlNS::office, sStringValue)) {
+                QString value = element.attributeNS(KoXmlNS::office, sStringValue, QString());
                 setValue(Value(value));
             } else {
                 // use the paragraph(s) read in before
@@ -1504,12 +1551,12 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
             setStyle(style);
 #endif
         } else {
-            kDebug(36003) << "cell:" << name() << "  Unknown type. Parsing user input.";
+            // kDebug(36003) << "cell:" << name() << "  Unknown type. Parsing user input.";
             // Set the value by parsing the user input.
             parseUserInput(userInput());
         }
     } else { // no value-type attribute
-        kDebug(36003) << "cell:" << name() << "  No value type specified. Parsing user input.";
+        // kDebug(36003) << "cell:" << name() << "  No value type specified. Parsing user input.";
         // Set the value by parsing the user input.
         parseUserInput(userInput());
     }
@@ -1519,14 +1566,14 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
     //
     int colSpan = 1;
     int rowSpan = 1;
-    if (element.hasAttributeNS(KoXmlNS::table, "number-columns-spanned")) {
+    if (element.hasAttributeNS(KoXmlNS::table, sNumberColumnsSpanned)) {
         bool ok = false;
-        int span = element.attributeNS(KoXmlNS::table, "number-columns-spanned", QString()).toInt(&ok);
+        int span = element.attributeNS(KoXmlNS::table, sNumberColumnsSpanned, QString()).toInt(&ok);
         if (ok) colSpan = span;
     }
-    if (element.hasAttributeNS(KoXmlNS::table, "number-rows-spanned")) {
+    if (element.hasAttributeNS(KoXmlNS::table, sNumberRowsSpanned)) {
         bool ok = false;
-        int span = element.attributeNS(KoXmlNS::table, "number-rows-spanned", QString()).toInt(&ok);
+        int span = element.attributeNS(KoXmlNS::table, sNumberRowsSpanned, QString()).toInt(&ok);
         if (ok) rowSpan = span;
     }
     if (colSpan > 1 || rowSpan > 1)
@@ -1535,14 +1582,14 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
     //
     // cell comment/annotation
     //
-    KoXmlElement annotationElement = KoXml::namedItemNS(element, KoXmlNS::office, "annotation");
+    KoXmlElement annotationElement = KoXml::namedItemNS(element, KoXmlNS::office, sAnnotation);
     if (!annotationElement.isNull()) {
         QString comment;
         KoXmlNode node = annotationElement.firstChild();
         while (!node.isNull()) {
             KoXmlElement commentElement = node.toElement();
             if (!commentElement.isNull())
-                if (commentElement.localName() == "p" && commentElement.namespaceURI() == KoXmlNS::text) {
+                if (commentElement.localName() == sP && commentElement.namespaceURI() == KoXmlNS::text) {
                     if (!comment.isEmpty()) comment.append('\n');
                     comment.append(commentElement.text());
                 }
@@ -1553,13 +1600,13 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
             setComment(comment);
     }
 
-    loadOdfObjects(element, tableContext.odfContext);
+    loadOdfObjects(element, tableContext);
 
     return true;
 }
 
 // Similar to KoXml::namedItemNS except that children of span tags will be evaluated too.
-KoXmlElement namedItemNSWithSpan(const KoXmlNode& node, const char* nsURI, const char* localName)
+KoXmlElement namedItemNSWithSpan(const KoXmlNode& node, const QString &nsURI, const QString &localName)
 {
     KoXmlNode n = node.firstChild();
     for (; !n.isNull(); n = n.nextSibling()) {
@@ -1569,7 +1616,7 @@ KoXmlElement namedItemNSWithSpan(const KoXmlNode& node, const char* nsURI, const
             }
             if (n.localName() == "span" && n.namespaceURI() == nsURI) {
                 KoXmlElement e = KoXml::namedItemNS(n, nsURI, localName); // not recursive
-                if(!e.isNull()) {
+                if (!e.isNull()) {
                     return e;
                 }
             }
@@ -1592,20 +1639,20 @@ static bool findDrawElements(const KoXmlElement& parent)
     return false;
 }
 
-void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableContext)
+void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableContext, const Styles& autoStyles, const QString& cellStyleName)
 {
     //Search and load each paragraph of text. Each paragraph is separated by a line break
     KoXmlElement textParagraphElement;
     QString cellText;
 
     bool multipleTextParagraphsFound = false;
-    const bool hasRichText = KoTextLoader::containsRichText(parent);
+    bool hasRichText = KoTextLoader::containsRichText(parent);
 
     forEachElement(textParagraphElement , parent) {
         if (textParagraphElement.localName() == "p" &&
                 textParagraphElement.namespaceURI() == KoXmlNS::text) {
 
-            // our text, could contain formating for value or result of formul
+            // our text, could contain formating for value or result of formula
             if (cellText.isEmpty()) {
                 cellText = textParagraphElement.text();
             } else {
@@ -1620,6 +1667,7 @@ void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableC
                     QString link = textA.attributeNS(KoXmlNS::xlink, "href", QString());
                     cellText = textA.text();
                     setUserInput(cellText);
+                    hasRichText = false;
                     // The value will be set later in loadOdf().
                     if ((!link.isEmpty()) && (link[0] == '#'))
                         link = link.remove(0, 1);
@@ -1637,6 +1685,21 @@ void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableC
             // for now we don't support richtext and embedded shapes in the same cell;
             // this is because they would currently be loaded twice, once by the KoTextLoader
             // and later properly by the cell itself
+
+            Style style; style.setDefault();
+            if (!cellStyleName.isEmpty()) {
+                if (autoStyles.contains(cellStyleName))
+                    style.merge(autoStyles[cellStyleName]);
+                else {
+                    const CustomStyle* namedStyle = sheet()->map()->styleManager()->style(cellStyleName);
+                    if (namedStyle)
+                        style.merge(*namedStyle);
+                }
+            }
+
+            QTextCharFormat format = style.asCharFormat();
+            sheet()->map()->textStyleManager()->defaultParagraphStyle()->characterStyle()->copyProperties(format);
+
             KoTextLoader loader(*tableContext.shapeContext);
             QSharedPointer<QTextDocument> doc(new QTextDocument);
             KoTextDocument(doc.data()).setStyleManager(sheet()->map()->textStyleManager());
@@ -1651,13 +1714,13 @@ void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableC
 
     //Enable word wrapping if multiple lines of text have been found.
     if (multipleTextParagraphsFound) {
-        Style style;
-        style.setWrapText(true);
-        setStyle(style);
+        Style newStyle;
+        newStyle.setWrapText(true);
+        setStyle(newStyle);
     }
 }
 
-void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfContext)
+void Cell::loadOdfObjects(const KoXmlElement &parent, OdfLoadingContext& tableContext)
 {
     // Register additional attributes, that identify shapes anchored in cells.
     // Their dimensions need adjustment after all rows are loaded,
@@ -1672,71 +1735,74 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
                 KoXmlNS::table, "end-y",
                 "table:end-y"));
 
-    KoShapeLoadingContext shapeContext(odfContext, d->sheet->resourceManager());
     KoXmlElement element;
     forEachElement(element, parent) {
         if (element.namespaceURI() != KoXmlNS::draw)
             continue;
 
-        KoShape* shape = KoShapeRegistry::instance()->createShapeFromOdf(element, shapeContext);
-        if (!shape) {
-            kDebug(36003) << "Unable to load shape.";
-            continue;
-        }
-
-        d->sheet->addShape(shape);
-
-        // All three attributes are necessary for cell anchored shapes.
-        // Otherwise, they are anchored in the sheet.
-        if (!shape->hasAdditionalAttribute("table:end-cell-address") ||
-                !shape->hasAdditionalAttribute("table:end-x") ||
-                !shape->hasAdditionalAttribute("table:end-y")) {
-            kDebug(36003) << "Not all attributes found, that are necessary for cell anchoring.";
-            continue;
-        }
-
-        Region endCell(Region::loadOdf(shape->additionalAttribute("table:end-cell-address")),
-                       d->sheet->map(), d->sheet);
-        if (!endCell.isValid() || !endCell.isSingular())
-            continue;
-
-        QString string = shape->additionalAttribute("table:end-x");
-        if (string.isNull())
-            continue;
-        double endX = KoUnit::parseValue(string);
-
-        string = shape->additionalAttribute("table:end-y");
-        if (string.isNull())
-            continue;
-        double endY = KoUnit::parseValue(string);
-
-        // The position is relative to the upper left sheet corner until now. Move it.
-        QPointF position = shape->position();
-        // Remember how far we're off from the top-left corner of this cell
-        double offsetX = position.x();
-        double offsetY = position.y();
-        for (int col = 1; col < column(); ++col)
-            position += QPointF(d->sheet->columnFormat(col)->width(), 0.0);
-        for (int row = 1; row < this->row(); ++row)
-            position += QPointF(0.0, d->sheet->rowFormat(row)->height());
-        shape->setPosition(position);
-
-        // The column dimensions are already the final ones, but not the row dimensions.
-        // The default height is used for the not yet loaded rows.
-        // TODO Stefan: Honor non-default row heights later!
-        // subtract offset because the accumulated width and height we calculate below starts
-        // at the top-left corner of this cell, but the shape can have an offset to that corner
-        QSizeF size = QSizeF(endX - offsetX, endY - offsetY);
-        for (int col = column(); col < endCell.firstRange().left(); ++col)
-            size += QSizeF(d->sheet->columnFormat(col)->width(), 0.0);
-        for (int row = this->row(); row < endCell.firstRange().top(); ++row)
-            size += QSizeF(0.0, d->sheet->rowFormat(row)->height());
-        shape->setSize(size);
-
-        dynamic_cast<ShapeApplicationData*>(shape->applicationData())->setAnchoredToCell(true);
+        loadOdfObject(element, *tableContext.shapeContext);
     }
 }
 
+void Cell::loadOdfObject(const KoXmlElement &element, KoShapeLoadingContext &shapeContext)
+{
+    KoShape* shape = KoShapeRegistry::instance()->createShapeFromOdf(element, shapeContext);
+    if (!shape) {
+        kDebug(36003) << "Unable to load shape.";
+        return;
+    }
+
+    d->sheet->addShape(shape);
+
+    // The position is relative to the upper left sheet corner until now. Move it.
+    QPointF position = shape->position();
+    // Remember how far we're off from the top-left corner of this cell
+    double offsetX = position.x();
+    double offsetY = position.y();
+    for (int col = 1; col < column(); ++col)
+        position += QPointF(d->sheet->columnFormat(col)->width(), 0.0);
+    for (int row = 1; row < this->row(); ++row)
+        position += QPointF(0.0, d->sheet->rowFormat(row)->height());
+    shape->setPosition(position);
+
+    dynamic_cast<ShapeApplicationData*>(shape->applicationData())->setAnchoredToCell(true);
+
+    // All three attributes are necessary for cell anchored shapes.
+    // Otherwise, they are anchored in the sheet.
+    if (!shape->hasAdditionalAttribute("table:end-cell-address") ||
+            !shape->hasAdditionalAttribute("table:end-x") ||
+            !shape->hasAdditionalAttribute("table:end-y")) {
+        kDebug(36003) << "Not all attributes found, that are necessary for cell anchoring.";
+        return;
+    }
+
+    Region endCell(Region::loadOdf(shape->additionalAttribute("table:end-cell-address")),
+                   d->sheet->map(), d->sheet);
+    if (!endCell.isValid() || !endCell.isSingular())
+        return;
+
+    QString string = shape->additionalAttribute("table:end-x");
+    if (string.isNull())
+        return;
+    double endX = KoUnit::parseValue(string);
+
+    string = shape->additionalAttribute("table:end-y");
+    if (string.isNull())
+        return;
+    double endY = KoUnit::parseValue(string);
+
+    // The column dimensions are already the final ones, but not the row dimensions.
+    // The default height is used for the not yet loaded rows.
+    // TODO Stefan: Honor non-default row heights later!
+    // subtract offset because the accumulated width and height we calculate below starts
+    // at the top-left corner of this cell, but the shape can have an offset to that corner
+    QSizeF size = QSizeF(endX - offsetX, endY - offsetY);
+    for (int col = column(); col < endCell.firstRange().left(); ++col)
+        size += QSizeF(d->sheet->columnFormat(col)->width(), 0.0);
+    for (int row = this->row(); row < endCell.firstRange().top(); ++row)
+        size += QSizeF(0.0, d->sheet->rowFormat(row)->height());
+    shape->setSize(size);
+}
 
 bool Cell::load(const KoXmlElement & cell, int _xshift, int _yshift,
                 Paste::Mode mode, Paste::Operation op, bool paste)
@@ -1809,7 +1875,9 @@ bool Cell::load(const KoXmlElement & cell, int _xshift, int _yshift,
     KoXmlElement conditionsElement = cell.namedItem("condition").toElement();
     if (!conditionsElement.isNull()) {
         Conditions conditions;
-        conditions.loadConditions(sheet()->map()->styleManager(), conditionsElement);
+        Map *const map = sheet()->map();
+        ValueParser *const valueParser = map->parser();
+        conditions.loadConditions(conditionsElement, valueParser);
         if (!conditions.isEmpty())
             setConditions(conditions);
     } else if (paste && (mode == Paste::Normal || mode == Paste::NoBorder)) {
