@@ -21,14 +21,16 @@
 #define KSPREAD_FORMULA_STORAGE
 
 #include "Formula.h"
-#include "Global.h"
+#include "kspread_limits.h"
 #include "PointStorage.h"
 
 namespace KSpread
 {
 
 /**
- * FormulaStorage
+ * \ingroup Storage
+ * \ingroup Value
+ * Stores formulas.
  */
 class FormulaStorage : public PointStorage<Formula>
 {
