@@ -24,19 +24,21 @@
 #include <klocale.h>
 #include <KoXmlReader.h>
 
+#include "kspread_export.h"
+
 class QDomElement;
 class QDomDocument;
 
 namespace KSpread
 {
 
-class Localization : public KLocale
+class KSPREAD_EXPORT Localization : public KLocale
 {
 public:
     Localization();
 
-    void load( const KoXmlElement& element );
-    QDomElement save( QDomDocument& doc ) const;
+    void load(const KoXmlElement& element);
+    QDomElement save(QDomDocument& doc) const;
     void defaultSystemConfig();
 };
 
