@@ -1,4 +1,5 @@
 /* This file is part of the KDE project
+   Copyright 2010 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
 
    This library is free software; you can redistribute it and/or
@@ -23,8 +24,10 @@
 #include <stdlib.h>
 #include <time.h>
 
+#include <QTimer>
 
 #include <kcodecs.h>
+#include <kcompletion.h>
 #include <ktemporaryfile.h>
 
 #include <KoGenStyles.h>
@@ -40,14 +43,15 @@
 #include <KoShapeLoadingContext.h>
 #include <KoTextSharedLoadingData.h>
 #include <KoParagraphStyle.h>
+#include <KoShapeRegistry.h>
 
 #include "ApplicationSettings.h"
 #include "BindingManager.h"
 #include "CalculationSettings.h"
-#include "part/Canvas.h" // FIXME detach from part
+#include "CellStorage.h"
 #include "Damages.h"
 #include "DependencyManager.h"
-#include "part/Doc.h" // FIXME detach from part
+#include "DocBase.h"
 #include "LoadingInfo.h"
 #include "Localization.h"
 #include "NamedAreaManager.h"
@@ -55,7 +59,6 @@
 #include "OdfSavingContext.h"
 #include "RecalcManager.h"
 #include "RowColumnFormat.h"
-#include "Selection.h"
 #include "Sheet.h"
 #include "StyleManager.h"
 #include "Validity.h"
@@ -63,21 +66,16 @@
 #include "ValueConverter.h"
 #include "ValueFormatter.h"
 #include "ValueParser.h"
-#include "part/View.h" // FIXME detach from part
 
 // database
 #include "database/DatabaseManager.h"
 
-// D-Bus
-#include "interfaces/MapAdaptor.h"
-#include <QtDBus/QtDBus>
-
 using namespace KSpread;
 
 class Map::Private
 {
 public:
-    Doc* doc;
+    DocBase* doc;
 
     /**
      * List of all sheets in this map.
@@ -85,11 +83,6 @@ public:
     QList<Sheet*> lstSheets;
     QList<Sheet*> lstDeletedSheets;
 
-    /**
-     * Password to protect the map from being changed.
-     */
-    QByteArray strPassword;
-
     // used to give every Sheet a unique default name.
     int tableId;
 
@@ -121,14 +114,18 @@ public:
 
     QList<Damage*> damages;
     bool isLoading;
+
+    int syntaxVersion;
+
+    KCompletion listCompletion;
 };
 
 
-Map::Map(Doc* doc, const char* name)
+Map::Map(DocBase* doc, int syntaxVersion)
         : QObject(doc),
         d(new Private)
 {
-    setObjectName(name);   // necessary for D-Bus
+    setObjectName("Map"); // necessary for D-Bus
     d->doc = doc;
     d->tableId = 1;
     d->overallRowCount = 0;
@@ -146,9 +143,6 @@ Map::Map(Doc* doc, const char* name)
 
     d->applicationSettings = new ApplicationSettings();
     d->calculationSettings = new CalculationSettings();
-    if (doc) {
-        d->calculationSettings->setFileName(doc->url().prettyUrl()); // for FILENAME function ;)
-    }
 
     d->parser = new ValueParser(d->calculationSettings);
     d->converter = new ValueConverter(d->parser);
@@ -159,16 +153,26 @@ Map::Map(Doc* doc, const char* name)
     d->defaultRowFormat = new RowFormat();
 
     QFont font(KoGlobal::defaultFont());
-    d->defaultRowFormat->setHeight(font.pointSizeF() + 3);
-    d->defaultColumnFormat->setWidth((font.pointSizeF() + 3) * 5);
+    d->defaultRowFormat->setHeight(font.pointSizeF() + 4);
+    d->defaultColumnFormat->setWidth((font.pointSizeF() + 4) * 5);
 
     d->isLoading = false;
 
-    new MapAdaptor(this);
-    if (doc) {
-        QDBusConnection::sessionBus().registerObject('/' + doc->objectName() + '/' + objectName(), this);
-    }
+    // default document properties
+    d->syntaxVersion = syntaxVersion;
 
+    connect(this, SIGNAL(sheetAdded(Sheet*)),
+            d->dependencyManager, SLOT(addSheet(Sheet*)));
+    connect(this, SIGNAL(sheetAdded(Sheet*)),
+            d->recalcManager, SLOT(addSheet(Sheet*)));
+    connect(this, SIGNAL(sheetRemoved(Sheet*)),
+            d->dependencyManager, SLOT(removeSheet(Sheet*)));
+    connect(this, SIGNAL(sheetRemoved(Sheet*)),
+            d->recalcManager, SLOT(removeSheet(Sheet*)));
+    connect(this, SIGNAL(sheetRevived(Sheet*)),
+            d->dependencyManager, SLOT(addSheet(Sheet*)));
+    connect(this, SIGNAL(sheetRevived(Sheet*)),
+            d->recalcManager, SLOT(addSheet(Sheet*)));
     connect(d->namedAreaManager, SIGNAL(namedAreaModified(const QString&)),
             d->dependencyManager, SLOT(namedAreaModified(const QString&)));
     connect(this, SIGNAL(damagesFlushed(const QList<Damage*>&)),
@@ -177,6 +181,17 @@ Map::Map(Doc* doc, const char* name)
 
 Map::~Map()
 {
+    // Because some of the shapes might be using a sheet in this map, delete
+    // all shapes in each sheet before all sheets are deleted together.
+    foreach(Sheet *sheet, d->lstSheets)
+        sheet->deleteShapes();
+    // we have to explicitly delete the Sheets, not let QObject take care of that
+    // as the sheet in its destructor expects the Map to still exist
+    qDeleteAll(d->lstSheets);
+    d->lstSheets.clear();
+
+    deleteLoadingInfo();
+
     delete d->bindingManager;
     delete d->databaseManager;
     delete d->dependencyManager;
@@ -193,10 +208,11 @@ Map::~Map()
 
     delete d->defaultColumnFormat;
     delete d->defaultRowFormat;
+
     delete d;
 }
 
-Doc* Map::doc() const
+DocBase* Map::doc() const
 {
     return d->doc;
 }
@@ -214,8 +230,11 @@ bool Map::isReadWrite() const
 bool Map::completeLoading(KoStore *store)
 {
     Q_UNUSED(store);
-    // update all dependencies and recalc all cells
-    addDamage(new WorkbookDamage(this, WorkbookDamage::Formula | WorkbookDamage::Value));
+    // Initial build of all cell dependencies.
+    d->dependencyManager->updateAllDependencies(this);
+    // Recalc the whole workbook now, since there may be formulas other spreadsheets support,
+    // but KSpread does not.
+    d->recalcManager->recalcMap();
     return true;
 }
 
@@ -312,15 +331,14 @@ CalculationSettings* Map::calculationSettings() const
     return d->calculationSettings;
 }
 
-void Map::setProtected(QByteArray const & passwd)
+Sheet* Map::createSheet(const QString& name)
 {
-    d->strPassword = passwd;
-}
-
-Sheet* Map::createSheet()
-{
-    QString name(i18n("Sheet%1", d->tableId++));
-    Sheet* sheet = new Sheet(this, name);
+    QString sheetName(i18n("Sheet%1", d->tableId++));
+    if ( !name.isEmpty() )
+        sheetName = name;
+    Sheet* sheet = new Sheet(this, sheetName);
+    connect(sheet, SIGNAL(statusMessage(const QString &, int)),
+            this, SIGNAL(statusMessage(const QString &, int)));
     return sheet;
 }
 
@@ -330,9 +348,9 @@ void Map::addSheet(Sheet *_sheet)
     emit sheetAdded(_sheet);
 }
 
-Sheet *Map::addNewSheet()
+Sheet *Map::addNewSheet(const QString& name)
 {
-    Sheet *t = createSheet();
+    Sheet *t = createSheet(name);
     addSheet(t);
     return t;
 }
@@ -382,41 +400,6 @@ void Map::loadOdfSettings(KoOasisSettings &settings)
     }
 }
 
-void Map::saveOdfSettings(KoXmlWriter &settingsWriter)
-{
-    settingsWriter.addConfigItem("ViewId", QString::fromLatin1("View1"));
-    // Save visual info for the first view, such as active sheet and active cell
-    // It looks like a hack, but reopening a document creates only one view anyway (David)
-    View * view = d->doc->views().isEmpty() ? 0 : static_cast<View*>(d->doc->views().first());
-    if (view) { // no view if embedded document
-        // save current sheet selection before to save marker, otherwise current pos is not saved
-        view->saveCurrentSheetSelection();
-        //<config:config-item config:name="ActiveTable" config:type="string">Feuille1</config:config-item>
-        if(Sheet* sheet = view->activeSheet())
-            settingsWriter.addConfigItem("ActiveTable",  sheet->sheetName());
-    }
-
-    //<config:config-item-map-named config:name="Tables">
-    settingsWriter.startElement("config:config-item-map-named");
-    settingsWriter.addAttribute("config:name", "Tables");
-    foreach(Sheet* sheet, d->lstSheets) {
-        settingsWriter.startElement("config:config-item-map-entry");
-        settingsWriter.addAttribute("config:name", sheet->sheetName());
-        if (view) {
-            QPoint marker = view->markerFromSheet(sheet);
-            QPointF offset = view->offsetFromSheet(sheet);
-            settingsWriter.addConfigItem("CursorPositionX", marker.x() - 1);
-            settingsWriter.addConfigItem("CursorPositionY", marker.y() - 1);
-            settingsWriter.addConfigItem("xOffset", offset.x());
-            settingsWriter.addConfigItem("yOffset", offset.y());
-        }
-        sheet->saveOdfSettings(settingsWriter);
-        settingsWriter.endElement();
-    }
-    settingsWriter.endElement();
-}
-
-
 bool Map::saveOdf(KoXmlWriter & xmlWriter, KoShapeSavingContext & savingContext)
 {
     // Saving the custom cell styles including the default cell style.
@@ -434,11 +417,13 @@ bool Map::saveOdf(KoXmlWriter & xmlWriter, KoShapeSavingContext & savingContext)
     defaultRowStyle.setDefaultStyle(true);
     savingContext.mainStyles().insert(defaultRowStyle, "Default", KoGenStyles::DontAddNumberToName);
 
-    if (!d->strPassword.isNull()) {
+    QByteArray password;
+    this->password(password);
+    if (!password.isNull()) {
         xmlWriter.addAttribute("table:structure-protected", "true");
-        QByteArray str = KCodecs::base64Encode(d->strPassword);
+        QByteArray str = KCodecs::base64Encode(password);
         // FIXME Stefan: see OpenDocument spec, ch. 17.3 Encryption
-        xmlWriter.addAttribute("table:protection-key", QString(str));
+        xmlWriter.addAttribute("table:protection-key", QString(str.data()));
     }
 
     OdfSavingContext tableContext(savingContext);
@@ -473,21 +458,16 @@ QDomElement Map::save(QDomDocument& doc)
     spread.appendChild(s);
 
     QDomElement mymap = doc.createElement("map");
-    // Save visual info for the first view, such as active sheet and active cell
-    // It looks like a hack, but reopening a document creates only one view anyway (David)
-    View * view = d->doc->views().isEmpty() ? 0 : static_cast<View*>(d->doc->views().first());
-    if (view) { // no view if embedded document
-        Canvas * canvas = view->canvasWidget();
-        mymap.setAttribute("activeTable",  canvas->activeSheet()->sheetName());
-        mymap.setAttribute("markerColumn", view->selection()->marker().x());
-        mymap.setAttribute("markerRow",    view->selection()->marker().y());
-        mymap.setAttribute("xOffset",      canvas->xOffset());
-        mymap.setAttribute("yOffset",      canvas->yOffset());
-    }
 
-    if (!d->strPassword.isNull()) {
-        QByteArray str = KCodecs::base64Encode(d->strPassword);
-        mymap.setAttribute("protected", QString(str.data()));
+    QByteArray password;
+    this->password(password);
+    if (!password.isNull()) {
+        if (password.size() > 0) {
+            QByteArray str = KCodecs::base64Encode(password);
+            mymap.setAttribute("protected", QString(str.data()));
+        } else {
+            mymap.setAttribute("protected", "");
+        }
     }
 
     foreach(Sheet* sheet, d->lstSheets) {
@@ -499,6 +479,33 @@ QDomElement Map::save(QDomDocument& doc)
     return mymap;
 }
 
+static void fixupStyle(KoCharacterStyle* style)
+{
+    style->removeHardCodedDefaults();
+
+    QTextCharFormat format;
+    style->applyStyle(format);
+    switch (style->underlineStyle()) {
+        case KoCharacterStyle::NoLineStyle:
+            format.setUnderlineStyle(QTextCharFormat::NoUnderline); break;
+        case KoCharacterStyle::SolidLine:
+            format.setUnderlineStyle(QTextCharFormat::SingleUnderline); break;
+        case KoCharacterStyle::DottedLine:
+            format.setUnderlineStyle(QTextCharFormat::DotLine); break;
+        case KoCharacterStyle::DashLine:
+            format.setUnderlineStyle(QTextCharFormat::DashUnderline); break;
+        case KoCharacterStyle::DotDashLine:
+            format.setUnderlineStyle(QTextCharFormat::DashDotLine); break;
+        case KoCharacterStyle::DotDotDashLine:
+            format.setUnderlineStyle(QTextCharFormat::DashDotDotLine); break;
+        case KoCharacterStyle::LongDashLine:
+            format.setUnderlineStyle(QTextCharFormat::DashUnderline); break;
+        case KoCharacterStyle::WaveLine:
+            format.setUnderlineStyle(QTextCharFormat::WaveUnderline); break;
+    }
+    style->copyProperties(format);
+}
+
 bool Map::loadOdf(const KoXmlElement& body, KoOdfLoadingContext& odfContext)
 {
     d->isLoading = true;
@@ -510,14 +517,26 @@ bool Map::loadOdf(const KoXmlElement& body, KoOdfLoadingContext& odfContext)
     OdfLoadingContext tableContext(odfContext);
     tableContext.validities = Validity::preloadValidities(body); // table:content-validations
 
-    // load text styles for rich-text content
-    KoShapeLoadingContext shapeContext(odfContext, 0); // TODO find a proper documentResourceManager somewhere.
+    // load text styles for rich-text content and TOS
+    KoShapeLoadingContext shapeContext(tableContext.odfContext, resourceManager());
     tableContext.shapeContext = &shapeContext;
     KoTextSharedLoadingData * sharedData = new KoTextSharedLoadingData();
     sharedData->loadOdfStyles(shapeContext, textStyleManager());
-    textStyleManager()->defaultParagraphStyle()->characterStyle()->removeHardCodedDefaults();
+
+    fixupStyle(textStyleManager()->defaultParagraphStyle()->characterStyle());
+    foreach (KoCharacterStyle* style, sharedData->characterStyles(true)) {
+        fixupStyle(style);
+    }
+    foreach (KoCharacterStyle* style, sharedData->characterStyles(false)) {
+        fixupStyle(style);
+    }
     shapeContext.addSharedData(KOTEXT_SHARED_LOADING_ID, sharedData);
 
+    QVariant variant;
+    variant.setValue(textStyleManager());
+    resourceManager()->setResource(KoText::StyleManager, variant);
+
+
     // load default column style
     const KoXmlElement* defaultColumnStyle = odfContext.stylesReader().defaultStyle("table-column");
     if (defaultColumnStyle) {
@@ -552,13 +571,7 @@ bool Map::loadOdf(const KoXmlElement& body, KoOdfLoadingContext& odfContext)
 
     d->calculationSettings->loadOdf(body); // table::calculation-settings
     if (body.hasAttributeNS(KoXmlNS::table, "structure-protected")) {
-        if (body.hasAttributeNS(KoXmlNS::table, "protection-key")) {
-            QString p = body.attributeNS(KoXmlNS::table, "protection-key", QString());
-            if(!p.isNull()) {
-                QByteArray str(p.toUtf8());
-                d->strPassword = KCodecs::base64Decode(str);
-            }
-        }
+        loadOdfProtection(body);
     }
 
     KoXmlNode sheetNode = KoXml::namedItemNS(body, KoXmlNS::table, "table");
@@ -582,8 +595,9 @@ bool Map::loadOdf(const KoXmlElement& body, KoOdfLoadingContext& odfContext)
 
             if (sheetElement.nodeName() == "table:table") {
                 if (!sheetElement.attributeNS(KoXmlNS::table, "name", QString()).isEmpty()) {
-                    Sheet* sheet = addNewSheet();
-                    sheet->setSheetName(sheetElement.attributeNS(KoXmlNS::table, "name", QString()), true);
+                    const QString sheetName = sheetElement.attributeNS(KoXmlNS::table, "name", QString());
+                    Sheet* sheet = addNewSheet(sheetName);
+                    sheet->setSheetName(sheetName, true);
                     d->overallRowCount += KoXml::childNodesCount(sheetElement);
                 }
             }
@@ -596,7 +610,8 @@ bool Map::loadOdf(const KoXmlElement& body, KoOdfLoadingContext& odfContext)
 
     //pre-load auto styles
     QHash<QString, Conditions> conditionalStyles;
-    Styles autoStyles = d->styleManager->loadOdfAutoStyles(odfContext.stylesReader(), conditionalStyles);
+    Styles autoStyles = d->styleManager->loadOdfAutoStyles(odfContext.stylesReader(),
+                        conditionalStyles, parser());
 
     // load the sheet
     sheetNode = body.firstChild();
@@ -623,6 +638,11 @@ bool Map::loadOdf(const KoXmlElement& body, KoOdfLoadingContext& odfContext)
         sheetNode = sheetNode.nextSibling();
     }
 
+    // make sure always at least one sheet exists
+    if (count() == 0) {
+        addNewSheet();
+    }
+
     //delete any styles which were not used
     d->styleManager->releaseUnusedAutoStyles(autoStyles);
 
@@ -664,11 +684,7 @@ bool Map::loadXML(const KoXmlElement& mymap)
         n = n.nextSibling();
     }
 
-    if (mymap.hasAttribute("protected")) {
-        QString passwd = mymap.attribute("protected");
-        QByteArray str(passwd.toUtf8());
-        d->strPassword = KCodecs::base64Decode(str);
-    }
+    loadXmlProtection(mymap);
 
     if (!activeSheet.isEmpty()) {
         // Used by View's constructor
@@ -770,21 +786,6 @@ QStringList Map::hiddenSheets() const
     return result;
 }
 
-void Map::password(QByteArray & passwd) const
-{
-    passwd = d->strPassword;
-}
-
-bool Map::isProtected() const
-{
-    return !d->strPassword.isNull();
-}
-
-bool Map::checkPassword(QByteArray const & passwd) const
-{
-    return (passwd == d->strPassword);
-}
-
 Sheet* Map::sheet(int index) const
 {
     return d->lstSheets.value(index);
@@ -805,17 +806,29 @@ int Map::count() const
     return d->lstSheets.count();
 }
 
-void Map::increaseLoadedRowsCounter(int number)
+int Map::increaseLoadedRowsCounter(int number)
 {
     d->loadedRowsCounter += number;
     if (d->overallRowCount) {
-        d->doc->emitProgress(100 * d->loadedRowsCounter / d->overallRowCount);
+        return 100 * d->loadedRowsCounter / d->overallRowCount;
     }
+    return -1;
 }
 
 bool Map::isLoading() const
 {
-    return d->isLoading;
+    // The KoDocument state is necessary to avoid damages while importing a file (through a filter).
+    return d->isLoading || (d->doc && d->doc->isLoading());
+}
+
+int Map::syntaxVersion() const
+{
+    return d->syntaxVersion;
+}
+
+void Map::setSyntaxVersion(int version)
+{
+    d->syntaxVersion = version;
 }
 
 LoadingInfo* Map::loadingInfo() const
@@ -832,11 +845,24 @@ void Map::deleteLoadingInfo()
     d->loadingInfo = 0;
 }
 
+KCompletion& Map::stringCompletion()
+{
+    return d->listCompletion;
+}
+
+void Map::addStringCompletion(const QString &stringCompletion)
+{
+    if (d->listCompletion.items().contains(stringCompletion) == 0) {
+        d->listCompletion.addItem(stringCompletion);
+    }
+}
+
 void Map::addDamage(Damage* damage)
 {
     // Do not create a new Damage, if we are in loading process. Check for it before
     // calling this function. This prevents unnecessary memory allocations (new).
-    Q_ASSERT(!isLoading());
+    // see FIXME in Sheet::setSheetName().
+//     Q_ASSERT(!isLoading());
     Q_CHECK_PTR(damage);
 
 #ifndef NDEBUG
@@ -878,14 +904,19 @@ void Map::handleDamages(const QList<Damage*>& damages)
     QList<Damage*>::ConstIterator end(damages.end());
     for (QList<Damage*>::ConstIterator it = damages.begin(); it != end; ++it) {
         Damage* damage = *it;
-        if (!damage) continue;
 
         if (damage->type() == Damage::Cell) {
             CellDamage* cellDamage = static_cast<CellDamage*>(damage);
             kDebug(36007) << "Processing\t" << *cellDamage;
             Sheet* const damagedSheet = cellDamage->sheet();
-            const Region region = cellDamage->region();
+            const Region& region = cellDamage->region();
+            const CellDamage::Changes changes = cellDamage->changes();
 
+            // TODO Stefan: Detach the style cache from the CellView cache.
+            if ((changes.testFlag(CellDamage::Appearance))) {
+                // Rebuild the style storage cache.
+                damagedSheet->cellStorage()->invalidateStyleCache(); // FIXME more fine-grained
+            }
             if ((cellDamage->changes() & CellDamage::Binding) &&
                     !workbookChanges.testFlag(WorkbookDamage::Value)) {
                 bindingChangedRegion.add(region, damagedSheet);
@@ -957,4 +988,15 @@ void Map::handleDamages(const QList<Damage*>& damages)
     }
 }
 
+void Map::addCommand(QUndoCommand *command)
+{
+    emit commandAdded(command);
+}
+
+KoResourceManager* Map::resourceManager() const
+{
+    if (!doc()) return 0;
+    return doc()->resourceManager();
+}
+
 #include "Map.moc"
