@@ -21,14 +21,16 @@
 #define KSPREAD_SHAPE_APPLICATION_DATA
 
 #include <KoShapeApplicationData.h>
+#include "kspread_export.h"
 
 namespace KSpread
 {
 
 /**
+ * \ingroup Embedding
  * KSpread specific data attached to each shape.
  */
-class ShapeApplicationData : public KoShapeApplicationData
+class KSPREAD_EXPORT ShapeApplicationData : public KoShapeApplicationData
 {
 public:
     /**
