@@ -1,4 +1,5 @@
 /* This file is part of the KDE project
+   Copyright 2010 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
    Copyright 2007 Stefan Nikolaus <stefan.nikolaus@kdemail.net>
    Copyright 1998,1999 Torben Weis <weis@kde.org>
    Copyright 1999-2007 The KSpread Team <koffice-devel@kde.org>
@@ -25,7 +26,6 @@
 #include <QClipboard>
 #include <QHash>
 #include <QList>
-#include <QPixmap>
 #include <QRect>
 //#include <QWidget>
 
@@ -38,9 +38,11 @@
 #include "Cell.h"
 #include "Style.h"
 #include "Global.h"
+#include "ProtectableObject.h"
 
 #include "kspread_export.h"
 
+class QAbstractItemModel;
 class QDomElement;
 class QUndoCommand;
 class QWidget;
@@ -63,17 +65,16 @@ class ColumnFormat;
 class CommentStorage;
 class ConditionsStorage;
 class FormulaStorage;
-class Doc;
+class DocBase;
 class FusionStorage;
 class LinkStorage;
+class HeaderFooter;
 class Map;
 class OdfLoadingContext;
 class OdfSavingContext;
-class PrintManager;
 class PrintSettings;
 class Region;
 class RowFormat;
-class Selection;
 class Sheet;
 class SheetPrint;
 class Style;
@@ -82,11 +83,13 @@ class Validity;
 class ValidityStorage;
 class ValueStorage;
 class View;
+template<typename T> class IntervalMap;
 
 /**
  * A sheet contains several cells.
  */
-class KSPREAD_EXPORT Sheet : public KoShapeUserData, public KoShapeControllerBase
+class KSPREAD_EXPORT Sheet : public KoShapeUserData, public KoShapeControllerBase,
+        public ProtectableObject
 {
     Q_OBJECT
     Q_PROPERTY(QString sheetName READ sheetName)
@@ -113,6 +116,11 @@ public:
      */
     ~Sheet();
 
+    /**
+     * \return a model for this sheet
+     */
+    QAbstractItemModel *model() const;
+
     /**
      * \return the map this sheet belongs to
      */
@@ -121,11 +129,17 @@ public:
     /**
      * \return the document this sheet belongs to
      */
-    Doc* doc() const;
+    DocBase* doc() const;
 
     // KoShapeControllerBase interface
     virtual void addShape(KoShape* shape);
     virtual void removeShape(KoShape* shape);
+    virtual KoResourceManager* resourceManager() const;
+
+    /**
+     * Deletes all shapes without emitting shapeRemoved()
+     */
+    void deleteShapes();
 
     /**
      * \ingroup Embedding
@@ -134,13 +148,6 @@ public:
      */
     QList<KoShape*> shapes() const;
 
-    /**
-     *  \param size of the thumbnail
-     *  Creates thumbnail of the sheet
-     *  \return the thumbnail of this sheet
-     */
-    QPixmap generateThumbnail(const QSize& size);
-
     //////////////////////////////////////////////////////////////////////////
     //
     //BEGIN Methods related to sheet properties
@@ -270,6 +277,54 @@ public:
      */
     void setShowPageBorders(bool _b);
 
+    struct BackgroundImageProperties {
+        BackgroundImageProperties()
+        : repeat(Repeat)
+        , opacity(1.0)
+        , horizontalPosition(HorizontalCenter)
+        , verticalPosition(VerticalCenter)
+        {}
+
+        enum Repetition {
+            NoRepeat,
+            Repeat,
+            Stretch
+        };
+        Repetition repeat;
+
+        float opacity;
+
+        enum HorizontalPosition {
+            Left,
+            HorizontalCenter,
+            Right
+        };
+        HorizontalPosition horizontalPosition;
+
+        enum VerticalPosition {
+            Top,
+            VerticalCenter,
+            Bottom
+        };
+        VerticalPosition verticalPosition;
+
+        //TODO filterName
+    };
+
+    /**
+     * Set background image for this sheet
+     */
+    void setBackgroundImage( const QImage& image );
+
+    /**
+     * @return The QImage used as the background picture for this sheet
+     */
+    QImage backgroundImage() const;
+
+    void setBackgroundImageProperties( const BackgroundImageProperties& properties );
+
+    BackgroundImageProperties backgroundImageProperties() const;
+
     //
     //END Methods related to sheet properties
     //
@@ -328,6 +383,11 @@ public:
      */
     void saveOdfHeaderFooter(KoXmlWriter &xmlWriter) const;
 
+    /**
+     * \ingroup OpenDocument
+     */
+    void saveOdfBackgroundImage(KoXmlWriter& xmlWriter) const;
+
     /**
      * \ingroup OpenDocument
      */
@@ -338,54 +398,24 @@ public:
      */
     void saveOdfSettings(KoXmlWriter &settingsWriter) const;
 
-    /**
-     * \ingroup OpenDocument
-     */
-    void saveOdfPrintStyleLayout(KoGenStyle &style) const;
-
+    void loadOdfObject(const KoXmlElement& element, KoShapeLoadingContext& shapeContext);
     //
     //END Methods related to the OpenDocument file format
     //
     //////////////////////////////////////////////////////////////////////////
     //
-    //BEGIN Methods related to protection
-    //
-
-    /**
-     * \ingroup Protection
-     */
-    void password(QByteArray & passwd) const ;
-
-    /**
-     * \ingroup Protection
-     */
-    bool isProtected() const;
-
-    /**
-     * \ingroup Protection
-     */
-    void setProtected(QByteArray const & passwd);
-
-    /**
-     * \ingroup Protection
-     */
-    bool checkPassword(QByteArray const & passwd) const;
-
-    //
-    //END Methods related to protection
-    //
-    //////////////////////////////////////////////////////////////////////////
-    //
     //BEGIN Methods related to row formats
     //
 
     /**
+     * \ingroup ColumnRowFormat
      * \return the row format of row \p _row . The default row format,
      * if no special one exists.
      */
     const RowFormat* rowFormat(int _row) const;
 
     /**
+     * \ingroup ColumnRowFormat
      * If no special RowFormat exists for this row, then a new one is created.
      *
      * @return a non default RowFormat for this row.
@@ -393,10 +423,14 @@ public:
     RowFormat* nonDefaultRowFormat(int _row, bool force_creation = true);
 
     /**
+     * \ingroup ColumnRowFormat
      * \return the first non-default row format
      */
     RowFormat* firstRow() const;
 
+    /**
+     * \ingroup ColumnRowFormat
+     */
     void setDefaultHeight(double height);
 
     //
@@ -408,12 +442,14 @@ public:
     //
 
     /**
+     * \ingroup ColumnRowFormat
      * \return the column format of column \p _column . The default column format,
      * if no special one exists.
      */
     const ColumnFormat* columnFormat(int _column) const;
 
     /**
+     * \ingroup ColumnRowFormat
      * If no special ColumnFormat exists for this column, then a new one is created.
      *
      * @return a non default ColumnFormat for this column.
@@ -421,10 +457,14 @@ public:
     ColumnFormat* nonDefaultColumnFormat(int _column, bool force_creation = true);
 
     /**
+     * \ingroup ColumnRowFormat
      * \return the first non-default row format
      */
     ColumnFormat* firstCol() const;
 
+    /**
+     * \ingroup ColumnRowFormat
+     */
     void setDefaultWidth(double width);
 
     //
@@ -436,6 +476,7 @@ public:
     //
 
     /**
+     * \ingroup Storage
      * \return the cell storage
      */
     CellStorage* cellStorage() const;
@@ -445,16 +486,18 @@ public:
     const FormulaStorage* formulaStorage() const;
     const FusionStorage* fusionStorage() const;
     const LinkStorage* linkStorage() const;
-    StyleStorage* styleStorage() const;
+    const StyleStorage* styleStorage() const;
     const ValidityStorage* validityStorage() const;
     const ValueStorage* valueStorage() const;
 
     /**
+     * \ingroup Coordinates
+     * \ingroup Storage
      * Determines the used area, i.e. the area spanning from A1 to the maximum
      * occupied column and row.
      * \return the used area
      */
-    QRect usedArea() const;
+    QRect usedArea(bool onlyContent = false) const;
 
     //
     //END Methods for Storage access
@@ -465,6 +508,7 @@ public:
     //
 
     /**
+     * \ingroup Coordinates
      * Determines the row for a given position \p _ypos . If the position is
      * on the border between two cells, the upper row is returned. Also, the offset
      * between the coordinate system root and the upper row border is determined.
@@ -477,6 +521,7 @@ public:
     int topRow(double _ypos, double &_top) const;
 
     /**
+     * \ingroup Coordinates
      * Determines the row for a given position \p _ypos . If the position is
      * on the border between two cells, the lower row is returned.
      *
@@ -487,6 +532,7 @@ public:
     int bottomRow(double _ypos) const;
 
     /**
+     * \ingroup Coordinates
      * Determines the column for a given position \p _xpos . If the position is
      * on the border between two cells, the left column is returned. Also, the offset
      * between the coordinate system root and the left column border is determined.
@@ -499,6 +545,7 @@ public:
     int leftColumn(double _xpos, double &_left) const;
 
     /**
+     * \ingroup Coordinates
      * Determines the column for a given position \p _xpos . If the position is
      * on the border between two cells, the right column is returned.
      *
@@ -509,6 +556,7 @@ public:
     int rightColumn(double _xpos) const;
 
     /**
+     * \ingroup Coordinates
      * Calculates the region in document coordinates occupied by a range of cells.
      * \param cellRange the range of cells
      * \return the document area covered by the cells
@@ -516,6 +564,7 @@ public:
     QRectF cellCoordinatesToDocument(const QRect& cellRange) const;
 
     /**
+     * \ingroup Coordinates
      * Calculates the cell range covering a document area.
      * \param area the document area
      * \return the cell range covering the area
@@ -523,6 +572,7 @@ public:
     QRect documentToCellCoordinates(const QRectF& area) const;
 
     /**
+     * \ingroup Coordinates
      * @return the left corner of the column as double.
      * Use this method, when you later calculate other positions depending on this one
      * to avoid rounding problems
@@ -531,6 +581,7 @@ public:
     double columnPosition(int col) const;
 
     /**
+     * \ingroup Coordinates
      * @return the top corner of the row as double.
      * Use this method, when you later calculate other positions depending on this one
      * to avoid rounding problems
@@ -539,41 +590,25 @@ public:
     double rowPosition(int _row) const;
 
     /**
+     * \ingroup Coordinates
      * \return the document size
      */
     QSizeF documentSize() const;
 
     /**
+     * \ingroup Coordinates
      * Adjusts the internal reference of the sum of the widths of all columns.
      * Used in resizing of columns.
      */
     void adjustDocumentWidth(double deltaWidth);
 
     /**
+     * \ingroup Coordinates
      * Adjusts the internal reference of the sum of the heights of all rows.
      * Used in resizing of rows.
      */
     void adjustDocumentHeight(double deltaHeight);
 
-    /**
-     * Attempts to guess the title (or 'header') of a column, within a given area of the sheet
-     * This is used, for example, by the Data Sort dialog, to guess the names of columns
-     * within the selected area.  An empty string may be returned if guessColumnTitle does not think
-     * that column @p col has a title.
-     * @param area The area within the sheet to guess from
-     * @param col The column to find the title (or 'header') for.
-     */
-    QString guessColumnTitle(QRect& area, int col);
-
-    /**
-     * Attempts to guess the title (or 'header') of a row, within a given area of the sheet
-     * This is used, for example, by the Data Sort dialog, to guess the names of rows within the selected area.
-     * An empty string may be returned if guessRowTitle does not think that row @p row has a title.
-     * @param area The area within the sheet to guess from
-     * @param row The row to find the title (or 'header') for.
-     */
-    QString guessRowTitle(QRect& area, int row);
-
     //
     //END UNSORTED METHODS
     //
@@ -583,98 +618,48 @@ public:
     //
 
     /**
-     * @param selection the selection of cells to work on
+     * \ingroup Commands
      */
-    void copySelection(Selection* selection);
-    /**
-     * @param selection the selection of cells to work on
-     */
-    void cutSelection(Selection* selection);
-
-    /**
-     * @return @c true if there are text value in cell
-     * so you can create list selection
-     * @param selection the selection of cells to work on
-     */
-    bool testListChoose(Selection* selection);
-
-    /**
-     * returns the text to be copied to the clipboard
-     * @param selection the selection of cells to work on
-     */
-    QString copyAsText(Selection* selection);
-
     bool areaIsEmpty(const Region& area, TestType _type = Text) ;
 
-    /**
-     * Deletes all cells in the given rectangle.
-     * The display is NOT updated by this function.
-     * This function can be used to clear an area before you paste something from the clipboard
-     * in this area.
-     *
-     * @param region The region that contains the cells that should be deleted
-     */
-    void deleteCells(const Region& region);
-
     //
     //END Methods related to manipulations of selected cells
     //
     //////////////////////////////////////////////////////////////////////////
     //
-    //BEGIN Methods related to cut & paste
-    //
-
-    /**
-     * A convenience function which retrieves the data to be pasted
-     * from the clipboard.
-     */
-    void paste(const QRect & pasteArea, bool makeUndo = true,
-               Paste::Mode = Paste::Normal, Paste::Operation = Paste::OverWrite,
-               bool insert = false, int insertTo = 0, bool pasteFC = false,
-               QClipboard::Mode clipboardMode = QClipboard::Clipboard);
-
-    void paste(const QByteArray & data, const QRect & pasteArea,
-               bool makeUndo = false, Paste::Mode = Paste::Normal, Paste::Operation = Paste::OverWrite,
-               bool insert = false, int insertTo = 0, bool pasteFC = false);
-
-    /**
-     * A function which allows to paste a text plain from the clipboard
-     */
-    void pasteTextPlain(const QString& _text, const QRect& pasteArea);
-
-    //
-    //END Methods related to cut & paste
-    //
-    //////////////////////////////////////////////////////////////////////////
-    //
     //BEGIN Methods related to column/row operations
     //
 
     /**
+     * \ingroup Commands
      * Helper method.
      * \see ShiftManipulator
      */
     void insertShiftRight(const QRect& rect);
 
     /**
+     * \ingroup Commands
      * Helper method.
      * \see ShiftManipulator
      */
     void insertShiftDown(const QRect& rect);
 
     /**
+     * \ingroup Commands
      * Helper method.
      * \see ShiftManipulator
      */
     void removeShiftUp(const QRect& rect);
 
     /**
+     * \ingroup Commands
      * Helper method.
      * \see ShiftManipulator
      */
     void removeShiftLeft(const QRect& rect);
 
     /**
+     * \ingroup ColumnRowFormat
      * Helper method.
      * \see InsertDeleteColumnManipulator
      * Moves all columns which are >= \p col \p number positions to the right
@@ -691,6 +676,7 @@ public:
     void insertRows(int row, int numbers);
 
     /**
+     * \ingroup ColumnRowFormat
      * Helper method.
      * \see InsertDeleteColumnManipulator
      * Deletes \p number columns beginning at \p col .
@@ -698,21 +684,12 @@ public:
     void removeColumns(int row, int numbers);
 
     /**
+     * \ingroup ColumnRowFormat
      * Helper method.
      * \see InsertDeleteRowManipulator
      * Deletes \p number rows beginning at \p row .
      */
-    void removeRows(int row, int numbers);
-
-    /**
-     * Updates vertical border and view.
-     */
-    void emitHideRow();
-
-    /**
-     * Updates horizontal border and view.
-     */
-    void emitHideColumn();
+    void removeRows(int row, int number);
 
     //
     //END Methods related column/row operations
@@ -725,6 +702,7 @@ public:
     void hideSheet(bool _hide);
 
     /**
+     * \ingroup Value
      * Change name of reference when the user inserts or removes a column,
      * a row or a cell (= insertion of a row [or column] on a single column [or row]).
      * For example the formula =Sheet1!A1 is changed into =Sheet1!B1 if a Column
@@ -742,104 +720,69 @@ public:
                            const QString& sheetName, int number);
 
     /**
+     * \ingroup ColumnRowFormat
      * Insert the non-default column format \p columnFormat.
      */
     void insertColumnFormat(ColumnFormat* columnFormat);
 
     /**
+     * \ingroup ColumnRowFormat
      * Inserts the non-default row format \p rowFormat.
      */
     void insertRowFormat(RowFormat* rowFormat);
 
     /**
+     * \ingroup ColumnRowFormat
      * Deletes the column format at \p column.
      */
     void deleteColumnFormat(int column);
 
     /**
+     * \ingroup ColumnRowFormat
      * Deletes the row format at \p row.
      */
     void deleteRowFormat(int row);
 
-    /**
-     * @param era set this to true if you want to encode relative references
-     *            absolutely (they will be switched back to relative
-     *            references during decoding) - used for cut to clipboard
-     */
-    QDomDocument saveCellRegion(const Region&, bool era = false);
-
-    /**
-     * insertTo defined if you insert to the bottom or right
-     * insert to bottom if insertTo==1
-     * insert to right if insertTo ==-1
-     * insertTo used just for insert/paste an area
-     * @see paste
-     */
-    bool loadSelection(const KoXmlDocument& doc, const QRect &pasteArea,
-                       int _xshift, int _yshift, bool makeUndo,
-                       Paste::Mode = Paste::Normal, Paste::Operation = Paste::OverWrite,
-                       bool insert = false, int insertTo = 0, bool paste = false);
-
-    void loadSelectionUndo(const KoXmlDocument & doc, const QRect &loadArea,
-                           int _xshift, int _yshift, bool insert, int insertTo);
-
-    /**
-     * Used when you insert and paste cell
-     * return true if it's a area
-     * false if it's a column/row
-     * it's used to select if you want to insert at the bottom or right
-     * @see paste
-     */
-    bool testAreaPasteInsert()const;
-
     //
     //END UNSORTED METHODS
     //
     //////////////////////////////////////////////////////////////////////////
     //
-    //BEGIN Methods related to painting
-    //
-
-    /**
-     * \ingroup Painting
-     * set a region of the spreadsheet to be 'paint dirty' meaning it
-     * needs repainted.  This is not a flag on the cell itself since quite
-     * often this needs set on a default cell
-     */
-    void setRegionPaintDirty(const Region & region);
-
-    void setRegionPaintDirty(const QRect & rect);
-
-    /**
-     * \ingroup Painting
-     * repaints all visible cells
-     */
-    void updateView();
-
-    /**
-     * \ingroup Painting
-     * repaints all visible cells in \p region
-     */
-    void updateView(const Region& region);
-
-    //
-    //END Methods related to painting
-    //
-    //////////////////////////////////////////////////////////////////////////
-    //
     //BEGIN UNSORTED METHODS !!!
     //
 
-    void emit_updateRow(RowFormat* rowFormat, int _row, bool repaint = true);
-    void emit_updateColumn(ColumnFormat* columnFormat, int _column);
+    /**
+     * Shows a status \p message in the status bar for \p timeout msecs.
+     */
+    void showStatusMessage(const QString &message, int timeout = 3000);
 
     void updateLocale();
 
 
-    SheetPrint * print() const;
+    /**
+     * \ingroup Page
+     * The page layout manager.
+     */
+    SheetPrint *print() const;
+
+    /**
+     * \ingroup Page
+     * Print settings.
+     */
     PrintSettings* printSettings() const;
+
+    /**
+     * \ingroup Page
+     * Sets the print settings.
+     */
     void setPrintSettings(const PrintSettings& settings);
 
+    /**
+     * \ingroup Page
+     * \return the header & footer object
+     */
+    HeaderFooter *headerFooter() const;
+
 #ifndef NDEBUG
     void printDebug();
 #endif
@@ -850,26 +793,43 @@ public:
     //////////////////////////////////////////////////////////////////////////
 
 signals:
-    void sig_refreshView();
-    void sig_updateView(Sheet *_sheet);
-    void sig_updateView(Sheet *_sheet, const Region&);
-    void sig_updateColumnHeader(Sheet *_sheet);
-    void sig_updateRowHeader(Sheet *_sheet);
     /**
-     * @see setSheetName
+     * Emitted, if the document size changed.
+     * E.g. if some columns were inserted.
+     * \param size new size
      */
-    void sig_nameChanged(Sheet* sheet, const QString& old_name);
+    void documentSizeChanged(const QSizeF &size);
 
-    void sig_SheetHidden(Sheet* sheet);
-    void sig_SheetShown(Sheet* sheet);
-    void sig_SheetRemoved(Sheet* sheet);
-    void sig_SheetActivated(Sheet*);
-    void sig_RefreshView(Sheet*);
-    void documentSizeChanged(const QSizeF&);
+    /**
+     * Emitted, if the visible size changed.
+     * E.g. if the document size changed or the user selected an area,
+     * which was not visible before.
+     */
     void visibleSizeChanged();
 
+    /**
+     * Emitted, if a status \p message should be shown in the status bar
+     * for \p timeout msecs.
+     */
+    void statusMessage(const QString& message, int timeout);
+
+    /**
+     * \ingroup Embedding
+     * Emitted, if a \p shape was added.
+     * \param sheet this sheet (for the View to determine, if it's the active one)
+     */
+    void shapeAdded(Sheet *sheet, KoShape *shape);
+
+    /**
+     * \ingroup Embedding
+     * Emitted, if a \p shape was removed.
+     * \param sheet this sheet (for the View to determine, if it's the active one)
+     */
+    void shapeRemoved(Sheet *sheet, KoShape *shape);
+
 protected:
     /**
+     * \ingroup Value
      * Change the name of a sheet in all formulas.
      * When you change name sheet Sheet1 -> Price
      * for all cell which refere to Sheet1, this function changes the name.
@@ -882,13 +842,26 @@ protected:
     //BEGIN Methods related to the OpenDocument file format
     //
 
+    void loadColumnNodes(const KoXmlElement& parent, int& indexCol,
+                            int& maxColumn, KoOdfLoadingContext& odfContext,
+                            QHash<QString, QRegion>& columnStyleRegions,
+                            IntervalMap<QString>& columnStyles);
+    void loadRowNodes(const KoXmlElement& parent, int& rowIndex,
+                            int& maxColumn, OdfLoadingContext& tableContext,
+                            QHash<QString, QRegion>& rowStyleRegions,
+                            QHash<QString, QRegion>& cellStyleRegions,
+                            const IntervalMap<QString>& columnStyles,
+                            const Styles& autoStyles);
+
     /**
      * \ingroup OpenDocument
      */
     int loadRowFormat(const KoXmlElement& row, int &rowIndex,
                        OdfLoadingContext& odfContext,
                        QHash<QString, QRegion>& rowStyleRegions,
-                       QHash<QString, QRegion>& cellStyleRegions);
+                       QHash<QString, QRegion>& cellStyleRegions,
+                       const IntervalMap<QString>& columnStyles,
+                       const Styles& autoStyles);
 
     /**
      * \ingroup OpenDocument
@@ -897,7 +870,8 @@ protected:
      */
     bool loadColumnFormat(const KoXmlElement& row,
                           const KoOdfStylesReader& stylesReader, int & indexCol,
-                          QHash<QString, QRegion>& columnStyleRegions);
+                          QHash<QString, QRegion>& columnStyleRegions,
+                          IntervalMap<QString>& columnStyles);
 
     /**
      * \ingroup OpenDocument
@@ -909,7 +883,9 @@ protected:
     void loadOdfInsertStyles(const Styles& autoStyles,
                              const QHash<QString, QRegion>& styleRegions,
                              const QHash<QString, Conditions>& conditionalStyles,
-                             const QRect& usedArea);
+                             const QRect& usedArea,
+                             QList<QPair<QRegion, Style> >& outStyleRegions,
+                             QList<QPair<QRegion, Conditions> >& outConditionalStyles);
 
     /**
      * \ingroup OpenDocument
@@ -969,17 +945,20 @@ protected:
     //////////////////////////////////////////////////////////////////////////
     //
 
-    // helper function for areaIsEmpty
+    /**
+     * \ingroup Commands
+     * \see areaIsEmpty()
+     */
     bool cellIsEmpty(const Cell& cell, TestType _type);
 
+    /**
+     * \ingroup Value
+     * \see changeNameCellRef()
+     */
     QString changeNameCellRefHelper(const QPoint& pos, bool fullRowOrColumn, ChangeRef ref,
                                     int NbCol, const QPoint& point, bool isColumnFixed,
                                     bool isRowFixed);
 
-    static Sheet* find(int _id);
-    static int s_id;
-    static QHash<int, Sheet*>* s_mapSheets;
-
 private:
     /**
      * \ingroup NativeFormat
