@@ -1,4 +1,5 @@
 /* This file is part of the KDE project
+   Copyright 2010 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
    Copyright 2006,2007 Stefan Nikolaus <stefan.nikolaus@kdemail.net>
 
    This library is free software; you can redistribute it and/or
@@ -87,10 +88,12 @@ Style StyleStorage::contains(const QPoint& point) const
     }
     // not found, lookup in the tree
     QList<SharedSubStyle> subStyles = d->tree.contains(point);
+
     if (subStyles.isEmpty())
         return *styleManager()->defaultStyle();
     Style* style = new Style();
     (*style) = composeStyle(subStyles);
+
     // insert style into the cache
     d->cache.insert(point, style);
     d->cachedArea += QRect(point, point);
@@ -120,7 +123,8 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::undoData(const Region& regi
             // trim the rects
             pairs[i].first = pairs[i].first.intersected(rect);
         }
-        result << pairs;
+        // Always a default subStyle first, even if there are no pairs.
+        result << qMakePair(QRectF(rect), SharedSubStyle()) << pairs;
     }
     return result;
 }
@@ -211,7 +215,7 @@ void StyleStorage::insert(const QRect& rect, const SharedSubStyle& subStyle)
 //     kDebug(36006) <<"StyleStorage: inserting" << SubStyle::name(subStyle->type()) <<" into" << rect;
     // keep track of the used area
     const bool isDefault = subStyle->type() == Style::DefaultStyleKey;
-    if (rect.top() == 1 && rect.bottom() == KS_colMax) {
+    if (rect.top() == 1 && rect.bottom() >= KS_rowMax) {
         for (int i = rect.left(); i <= rect.right(); ++i) {
             if (isDefault)
                 d->usedColumns.remove(i);
@@ -220,7 +224,7 @@ void StyleStorage::insert(const QRect& rect, const SharedSubStyle& subStyle)
         }
         if (isDefault)
             d->usedArea -= rect;
-    } else if (rect.left() == 1 && rect.right() == KS_rowMax) {
+    } else if (rect.left() == 1 && rect.right() >= KS_colMax) {
         for (int i = rect.top(); i <= rect.bottom(); ++i) {
             if (isDefault)
                 d->usedRows.remove(i);
@@ -268,6 +272,64 @@ void StyleStorage::insert(const Region& region, const Style& style)
     }
 }
 
+void StyleStorage::load(const QList<QPair<QRegion, Style> >& styles)
+{
+    QList<QPair<QRegion, SharedSubStyle> > subStyles;
+
+    d->usedArea = QRegion();
+    d->usedColumns.clear();
+    d->usedRows.clear();
+    d->cachedArea = QRegion();
+    d->cache.clear();
+    typedef QPair<QRegion, Style> StyleRegion;
+    foreach (const StyleRegion& styleArea, styles) {
+        const QRegion& reg = styleArea.first;
+        const Style& style = styleArea.second;
+        if (style.isEmpty()) continue;
+
+        // update used areas
+        QRect bound = reg.boundingRect();
+        if ((bound.top() == 1 && bound.bottom() >= KS_rowMax) || (bound.left() == 1 && bound.right() >= KS_colMax)) {
+            foreach (const QRect& rect, reg.rects()) {
+                if (rect.top() == 1 && rect.bottom() >= KS_rowMax) {
+                    for (int i = rect.left(); i <= rect.right(); ++i) {
+                        d->usedColumns.insert(i, true);
+                    }
+                } else if (rect.left() == 1 && rect.right() >= KS_colMax) {
+                    for (int i = rect.top(); i <= rect.bottom(); ++i) {
+                        d->usedRows.insert(i, true);
+                    }
+                } else {
+                    d->usedArea += rect;
+                }
+            }
+        } else {
+            d->usedArea += reg;
+        }
+
+        // find substyles
+        foreach(const SharedSubStyle& subStyle, style.subStyles()) {
+            bool foundShared = false;
+            typedef const QList< SharedSubStyle> StoredSubStyleList;
+            StoredSubStyleList& storedSubStyles(d->subStyles.value(subStyle->type()));
+            StoredSubStyleList::ConstIterator end(storedSubStyles.end());
+            for (StoredSubStyleList::ConstIterator it(storedSubStyles.begin()); it != end; ++it) {
+                if (Style::compare(subStyle.data(), (*it).data())) {
+        //             kDebug(36006) <<"[REUSING EXISTING SUBSTYLE]";
+                    subStyles.append(qMakePair(reg, *it));
+                    foundShared = true;
+                    break;
+                }
+            }
+            if (!foundShared) {
+                // insert substyle and add to the used substyle list
+                subStyles.append(qMakePair(reg, subStyle));
+            }
+        }
+    }
+    d->tree.load(subStyles);
+}
+
 QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertRows(int position, int number)
 {
     const QRect invalidRect(1, position, KS_colMax, KS_rowMax);
@@ -292,7 +354,9 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertRows(int position, in
         d->usedRows.remove(it.key());
     d->usedRows.unite(map);
     // process the tree
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.insertRows(position, number);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(1, KS_rowMax - number + 1, KS_colMax, number), SharedSubStyle());
+    undoData << d->tree.insertRows(position, number);
     return undoData;
 }
 
@@ -320,7 +384,9 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertColumns(int position,
         d->usedColumns.remove(it.key());
     d->usedColumns.unite(map);
     // process the tree
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.insertColumns(position, number);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(KS_colMax - number + 1, 1, number, KS_rowMax), SharedSubStyle());
+    undoData << d->tree.insertColumns(position, number);
     return undoData;
 }
 
@@ -345,7 +411,9 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::removeRows(int position, in
         d->usedRows.remove(it.key());
     d->usedRows.unite(map);
     // process the tree
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.removeRows(position, number);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(1, position, KS_colMax, number), SharedSubStyle());
+    undoData << d->tree.removeRows(position, number);
     return undoData;
 }
 
@@ -370,14 +438,18 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::removeColumns(int position,
         d->usedColumns.remove(it.key());
     d->usedColumns.unite(map);
     // process the tree
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.removeColumns(position, number);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(position, 1, number, KS_rowMax), SharedSubStyle());
+    undoData << d->tree.removeColumns(position, number);
     return undoData;
 }
 
 QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertShiftRight(const QRect& rect)
 {
     const QRect invalidRect(rect.topLeft(), QPoint(KS_colMax, rect.bottom()));
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.insertShiftRight(rect);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(rect), SharedSubStyle());
+    undoData << d->tree.insertShiftRight(rect);
     regionChanged(invalidRect);
     // update the used area
     const QRegion usedArea = d->usedArea & invalidRect;
@@ -401,7 +473,9 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertShiftRight(const QRec
 QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertShiftDown(const QRect& rect)
 {
     const QRect invalidRect(rect.topLeft(), QPoint(rect.right(), KS_rowMax));
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.insertShiftDown(rect);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(rect), SharedSubStyle());
+    undoData << d->tree.insertShiftDown(rect);
     regionChanged(invalidRect);
     // update the used area
     const QRegion usedArea = d->usedArea & invalidRect;
@@ -425,7 +499,9 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::insertShiftDown(const QRect
 QList< QPair<QRectF, SharedSubStyle> > StyleStorage::removeShiftLeft(const QRect& rect)
 {
     const QRect invalidRect(rect.topLeft(), QPoint(KS_colMax, rect.bottom()));
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.removeShiftLeft(rect);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(rect), SharedSubStyle());
+    undoData << d->tree.removeShiftLeft(rect);
     regionChanged(invalidRect);
     // update the used area
     const QRegion usedArea = d->usedArea & QRect(rect.right() + 1, rect.top(), KS_colMax, rect.height());
@@ -444,7 +520,9 @@ QList< QPair<QRectF, SharedSubStyle> > StyleStorage::removeShiftLeft(const QRect
 QList< QPair<QRectF, SharedSubStyle> > StyleStorage::removeShiftUp(const QRect& rect)
 {
     const QRect invalidRect(rect.topLeft(), QPoint(rect.right(), KS_rowMax));
-    QList< QPair<QRectF, SharedSubStyle> > undoData = d->tree.removeShiftUp(rect);
+    QList< QPair<QRectF, SharedSubStyle> > undoData;
+    undoData << qMakePair(QRectF(rect), SharedSubStyle());
+    undoData << d->tree.removeShiftUp(rect);
     regionChanged(invalidRect);
     // update the used area
     const QRegion usedArea = d->usedArea & QRect(rect.left(), rect.bottom() + 1, rect.width(), KS_rowMax);
@@ -577,7 +655,7 @@ void StyleStorage::garbageCollection()
             kDebug(36006) << "removing" << currentPair.second->debugData()
             << "at" << Region(currentPair.first.toRect()).name()
             << "used" << currentPair.second->ref << "times" << endl;
-            d->tree.remove(currentPair.first.toRect(), currentPair.second);
+            d->tree.remove(currentPair.first.toRect(), currentPair.second, currentZIndex);
 #if 0
             kDebug(36006) << "StyleStorage: usage of" << currentPair.second->debugData() << " is" << currentPair.second->ref;
             // FIXME Stefan: The usage of substyles used once should be
