@@ -29,11 +29,11 @@
 #include <klocale.h>
 
 #include "CellStorage.h"
-#include "CellStorage_p.h"
 #include "Damages.h"
 #include "Map.h"
 #include "RowColumnFormat.h"
 #include "Sheet.h"
+#include "Value.h"
 
 using namespace KSpread;
 
@@ -60,6 +60,10 @@ bool ResizeColumnManipulator::process(Element* element)
             m_oldSizes[col] = format->width();
         format->setWidth(qMax(2.0, m_reverse ? m_oldSizes[col] : m_newSize));
     }
+    // Just repaint everything visible; no need to invalidate the visual cache.
+    m_sheet->map()->addDamage(new SheetDamage(m_sheet, SheetDamage::ContentChanged));
+    // TODO: only invalidate the cells that are actually effected by this resize (so everythin in this column, and everything that covers something in this column)
+    m_sheet->map()->addDamage(new CellDamage(m_sheet, Region(1, 1, KS_colMax, KS_rowMax, m_sheet), CellDamage::Appearance));
     return true;
 }
 
@@ -88,6 +92,10 @@ bool ResizeRowManipulator::process(Element* element)
             m_oldSizes[row] = rl->height();
         rl->setHeight(qMax(2.0, m_reverse ? m_oldSizes[row] : m_newSize));
     }
+    // Just repaint everything visible; no need to invalidate the visual cache.
+    m_sheet->map()->addDamage(new SheetDamage(m_sheet, SheetDamage::ContentChanged));
+    // TODO: only invalidate the cells that are actually effected by this resize (so everythin in this row, and everything that covers something in this row)
+    m_sheet->map()->addDamage(new CellDamage(m_sheet, Region(1, 1, KS_colMax, KS_rowMax, m_sheet), CellDamage::Appearance));
     return true;
 }
 
@@ -189,13 +197,8 @@ bool HideShowManipulator::preProcessing()
 
 bool HideShowManipulator::postProcessing()
 {
-    if (m_manipulateColumns) {
-        m_sheet->emitHideColumn();
-    }
-    if (m_manipulateRows) {
-        m_sheet->emitHideRow();
-    }
-
+    // Just repaint everything visible; no need to invalidate the visual cache.
+    m_sheet->map()->addDamage(new SheetDamage(m_sheet, SheetDamage::ContentChanged));
     return true;
 }
 
@@ -308,6 +311,12 @@ bool AdjustColumnRowManipulator::process(Element* element)
             }
         }
     }
+    // The cell width(s) or height(s) changed, which are cached: rebuild them.
+    const Region region(m_adjustRow ? 1 : range.left(),
+                        m_adjustColumn ? 1 : range.top(),
+                        m_adjustRow ? KS_colMax : range.width(),
+                        m_adjustColumn ? KS_rowMax : range.height());
+    m_sheet->map()->addDamage(new CellDamage(m_sheet, region, CellDamage::Appearance));
     return true;
 }
 
@@ -418,6 +427,23 @@ bool AdjustColumnRowManipulator::preProcessing()
     return AbstractRegionCommand::preProcessing();
 }
 
+bool AdjustColumnRowManipulator::postProcessing()
+{
+    if (!m_adjustColumn && !m_adjustRow) {
+        return false;
+    }
+    // Update the column/row header, if necessary.
+    SheetDamage::Changes changes = SheetDamage::None;
+    if (m_adjustColumn) {
+        changes |= SheetDamage::ColumnsChanged;
+    }
+    if (m_adjustRow) {
+        changes |= SheetDamage::RowsChanged;
+    }
+    m_sheet->map()->addDamage(new SheetDamage(m_sheet, changes));
+    return AbstractRegionCommand::postProcessing();
+}
+
 class DummyWidget : public QWidget
 {
     int metric(PaintDeviceMetric metric) const {
@@ -451,7 +477,7 @@ QSizeF AdjustColumnRowManipulator::textSize(const QString& text, const Style& st
             offsetFont = fontMetrics.underlinePos() + 1;
 
         size.setHeight((fontMetrics.ascent() + fontMetrics.descent() + offsetFont)
-                       * (text.count('\n') + 1));
+                       *(text.count('\n') + 1));
     } else if (style.angle() != 0) {
         // Rotated text.
 
@@ -532,16 +558,23 @@ QString AdjustColumnRowManipulator::name() const
   class InsertDeleteColumnManipulator
 ****************************************************************************/
 
-InsertDeleteColumnManipulator::InsertDeleteColumnManipulator()
-        : AbstractRegionCommand()
+InsertDeleteColumnManipulator::InsertDeleteColumnManipulator(QUndoCommand *parent)
+        : AbstractRegionCommand(parent)
         , m_mode(Insert)
+        , m_template(0)
 {
     setText(i18n("Insert Columns"));
 }
 
 InsertDeleteColumnManipulator::~InsertDeleteColumnManipulator()
 {
-    delete m_undoData;
+    delete m_template;
+}
+
+void InsertDeleteColumnManipulator::setTemplate(const ColumnFormat &columnFormat)
+{
+    delete m_template;
+    m_template = new ColumnFormat(columnFormat);
 }
 
 void InsertDeleteColumnManipulator::setReverse(bool reverse)
@@ -556,37 +589,89 @@ void InsertDeleteColumnManipulator::setReverse(bool reverse)
 
 bool InsertDeleteColumnManipulator::process(Element* element)
 {
-    if (m_firstrun)
-        m_sheet->cellStorage()->startUndoRecording(); // FIXME Stefan: Must be global!
-
     const QRect range = element->rect();
     const int pos = range.left();
     const int num = range.width();
     if (!m_reverse) { // insertion
         // insert rows
         m_sheet->insertColumns(pos, num);
+        if (m_template) {
+            m_template->setSheet(m_sheet);
+            const int end = pos + num - 1;
+            for (int col = pos; col <= end; ++col) {
+                m_template->setColumn(col);
+                m_sheet->insertColumnFormat(m_template);
+            }
+        }
         m_sheet->cellStorage()->insertColumns(pos, num);
 
         // undo deletion
-        if (m_mode == Delete)
-            m_sheet->cellStorage()->undo(m_undoData);
-    } else { // deletion
+        if (m_mode == Delete) {
+            QUndoCommand::undo(); // process child commands (from CellStorage)
+        }
+    } else {
         // delete rows
         m_sheet->removeColumns(pos, num);
         m_sheet->cellStorage()->removeColumns(pos, num);
 
         // undo insertion
-        if (m_mode == Insert)
-            m_sheet->cellStorage()->undo(m_undoData);
+        if (m_mode == Insert) {
+            QUndoCommand::undo(); // process child commands (from CellStorage)
+        }
     }
-
-    if (m_firstrun)
-        m_undoData = m_sheet->cellStorage()->stopUndoRecording();
     return true;
 }
 
+bool elementLeftColumnLessThan(const KSpread::Region::Element *e1, const KSpread::Region::Element *e2)
+{
+    return e1->rect().left() < e2->rect().left();
+}
+
+bool InsertDeleteColumnManipulator::preProcessing()
+{
+    if (m_firstrun) {
+        // If we have an NCS, create a child command for each element.
+        if (cells().count() > 1) { // non-contiguous selection
+            // Sort the elements by their top row.
+            qStableSort(cells().begin(), cells().end(), elementLeftColumnLessThan);
+            // Create sub-commands.
+            const Region::ConstIterator end(constEnd());
+            for (Region::ConstIterator it = constBegin(); it != end; ++it) {
+                InsertDeleteColumnManipulator *const command = new InsertDeleteColumnManipulator(this);
+                command->setSheet(m_sheet);
+                command->add(Region((*it)->rect(), (*it)->sheet()));
+                if (m_mode == Delete) {
+                    command->setReverse(true);
+                }
+            }
+        } else { // contiguous selection
+            m_sheet->cellStorage()->startUndoRecording();
+        }
+    }
+    return AbstractRegionCommand::preProcessing();
+}
+
+bool InsertDeleteColumnManipulator::mainProcessing()
+{
+    if (cells().count() > 1) { // non-contiguous selection
+        if ((m_reverse && m_mode == Insert) || (!m_reverse && m_mode == Delete)) {
+            QUndoCommand::undo(); // process all sub-commands
+        } else {
+            QUndoCommand::redo(); // process all sub-commands
+        }
+        return true;
+    }
+    return AbstractRegionCommand::mainProcessing(); // calls process(Element*)
+}
+
 bool InsertDeleteColumnManipulator::postProcessing()
 {
+    if (cells().count() > 1) { // non-contiguous selection
+        return true;
+    }
+    if (m_firstrun) {
+        m_sheet->cellStorage()->stopUndoRecording(this);
+    }
     const QRect rect(QPoint(boundingRect().left(), 1), QPoint(KS_colMax, KS_rowMax));
     m_sheet->map()->addDamage(new CellDamage(m_sheet, Region(rect, m_sheet), CellDamage::Appearance));
     return true;
@@ -596,16 +681,23 @@ bool InsertDeleteColumnManipulator::postProcessing()
   class InsertDeleteRowManipulator
 ****************************************************************************/
 
-InsertDeleteRowManipulator::InsertDeleteRowManipulator()
-        : AbstractRegionCommand()
+InsertDeleteRowManipulator::InsertDeleteRowManipulator(QUndoCommand *parent)
+        : AbstractRegionCommand(parent)
         , m_mode(Insert)
+        , m_template(0)
 {
     setText(i18n("Insert Rows"));
 }
 
 InsertDeleteRowManipulator::~InsertDeleteRowManipulator()
 {
-    delete m_undoData;
+    delete m_template;
+}
+
+void InsertDeleteRowManipulator::setTemplate(const RowFormat &rowFormat)
+{
+    delete m_template;
+    m_template = new RowFormat(rowFormat);
 }
 
 void InsertDeleteRowManipulator::setReverse(bool reverse)
@@ -620,37 +712,89 @@ void InsertDeleteRowManipulator::setReverse(bool reverse)
 
 bool InsertDeleteRowManipulator::process(Element* element)
 {
-    if (m_firstrun)
-        m_sheet->cellStorage()->startUndoRecording(); // FIXME Stefan: Must be global!
-
     const QRect range = element->rect();
     const int pos = range.top();
     const int num = range.height();
     if (!m_reverse) { // insertion
         // insert rows
         m_sheet->insertRows(pos, num);
+        if (m_template) {
+            m_template->setSheet(m_sheet);
+            const int end = pos + num - 1;
+            for (int row = pos; row <= end; ++row) {
+                m_template->setRow(row);
+                m_sheet->insertRowFormat(m_template);
+            }
+        }
         m_sheet->cellStorage()->insertRows(pos, num);
 
         // undo deletion
-        if (m_mode == Delete)
-            m_sheet->cellStorage()->undo(m_undoData);
-    } else { // deletion
+        if (m_mode == Delete) {
+            QUndoCommand::undo(); // process child commands (from CellStorage)
+        }
+    } else {
         // delete rows
         m_sheet->removeRows(pos, num);
         m_sheet->cellStorage()->removeRows(pos, num);
 
         // undo insertion
-        if (m_mode == Insert)
-            m_sheet->cellStorage()->undo(m_undoData);
+        if (m_mode == Insert) {
+            QUndoCommand::undo(); // process child commands (from CellStorage)
+        }
     }
-
-    if (m_firstrun)
-        m_undoData = m_sheet->cellStorage()->stopUndoRecording();
     return true;
 }
 
+bool elementTopRowLessThan(const KSpread::Region::Element *e1, const KSpread::Region::Element *e2)
+{
+    return e1->rect().top() < e2->rect().top();
+}
+
+bool InsertDeleteRowManipulator::preProcessing()
+{
+    if (m_firstrun) {
+        // If we have an NCS, create a child command for each element.
+        if (cells().count() > 1) { // non-contiguous selection
+            // Sort the elements by their top row.
+            qStableSort(cells().begin(), cells().end(), elementTopRowLessThan);
+            // Create sub-commands.
+            const Region::ConstIterator end(constEnd());
+            for (Region::ConstIterator it = constBegin(); it != end; ++it) {
+                InsertDeleteRowManipulator *const command = new InsertDeleteRowManipulator(this);
+                command->setSheet(m_sheet);
+                command->add(Region((*it)->rect(), (*it)->sheet()));
+                if (m_mode == Delete) {
+                    command->setReverse(true);
+                }
+            }
+        } else { // contiguous selection
+            m_sheet->cellStorage()->startUndoRecording();
+        }
+    }
+    return AbstractRegionCommand::preProcessing();
+}
+
+bool InsertDeleteRowManipulator::mainProcessing()
+{
+    if (cells().count() > 1) { // non-contiguous selection
+        if ((m_reverse && m_mode == Insert) || (!m_reverse && m_mode == Delete)) {
+            QUndoCommand::undo(); // process all sub-commands
+        } else {
+            QUndoCommand::redo(); // process all sub-commands
+        }
+        return true;
+    }
+    return AbstractRegionCommand::mainProcessing(); // calls process(Element*)
+}
+
 bool InsertDeleteRowManipulator::postProcessing()
 {
+    if (cells().count() > 1) { // non-contiguous selection
+        return true;
+    }
+    if (m_firstrun) {
+        m_sheet->cellStorage()->stopUndoRecording(this);
+    }
     const QRect rect(QPoint(1, boundingRect().top()), QPoint(KS_colMax, KS_rowMax));
     m_sheet->map()->addDamage(new CellDamage(m_sheet, Region(rect, m_sheet), CellDamage::Appearance));
     return true;
