@@ -31,8 +31,10 @@
 #include <klocale.h>
 #include <knuminput.h>
 
+#include <KoCanvasBase.h>
+
 #include "Cell.h"
-#include "Selection.h"
+#include "ui/Selection.h"
 #include "Sheet.h"
 
 #include "commands/StyleCommand.h"
