@@ -4,6 +4,7 @@
  * Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Pierre Ducroquet <pinaraf@pinaraf.info>
  * Copyright (C) 2008 Sebastian Sauer <mail@dipe.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +28,7 @@
 #include "KWFactory.h"
 #include "KWView.h"
 #include "KWCanvas.h"
+#include "KWCanvasItem.h"
 #include "KWPageManager.h"
 #include "KWPage.h"
 #include "KWPageStyle.h"
@@ -68,6 +70,9 @@
 #include <rdf/KoDocumentRdf.h>
 #endif
 
+#include <KoProgressUpdater.h>
+#include <KoUpdater.h>
+
 // KDE + Qt includes
 #include <klocale.h>
 #include <kstandardaction.h>
@@ -171,6 +176,7 @@ KWDocument::KWDocument(QWidget *parentWidget, QObject *parent, bool singleViewMo
 KWDocument::~KWDocument()
 {
     delete m_magicCurtain;
+    m_config.setUnit(unit());
     saveConfig();
     qDeleteAll(m_frameSets);
 }
@@ -191,8 +197,8 @@ void KWDocument::addShape(KoShape *shape)
     addFrameSet(frame->frameSet());
 
     foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
-        canvas->shapeManager()->add(shape);
+        KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
+        canvas->shapeManager()->addShape(shape);
     }
 }
 
@@ -208,7 +214,7 @@ void KWDocument::removeShape(KoShape *shape)
             fs->removeFrame(frame);
     } else { // not a frame, but we still have to remove it from views.
         foreach (KoView *view, views()) {
-            KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
+            KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
             canvas->shapeManager()->remove(shape);
         }
     }
@@ -216,26 +222,30 @@ void KWDocument::removeShape(KoShape *shape)
 
 void KWDocument::paintContent(QPainter&, const QRect &rect)
 {
-    // TODO
+    Q_UNUSED(rect);
+#ifdef __GNUC__
+    #warning TODO: implement KWDocument::paintContent
+#endif
+
 }
 
 KoView *KWDocument::createViewInstance(QWidget *parent)
 {
     KWView *view = new KWView(m_viewMode, this, parent);
     if (m_magicCurtain)
-        view->kwcanvas()->shapeManager()->add(m_magicCurtain, KoShapeManager::AddWithoutRepaint);
+        view->canvasBase()->shapeManager()->addShape(m_magicCurtain, KoShapeManager::AddWithoutRepaint);
 
     bool switchToolCalled = false;
     foreach (KWFrameSet *fs, m_frameSets) {
         if (fs->frameCount() == 0)
             continue;
         foreach (KWFrame *frame, fs->frames())
-            view->kwcanvas()->shapeManager()->add(frame->shape(), KoShapeManager::AddWithoutRepaint);
+            view->canvasBase()->shapeManager()->addShape(frame->shape(), KoShapeManager::AddWithoutRepaint);
         if (switchToolCalled)
             continue;
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         if (tfs && tfs->textFrameSetType() == KWord::MainTextFrameSet) {
-            KoSelection *selection = view->kwcanvas()->shapeManager()->selection();
+            KoSelection *selection = view->canvasBase()->shapeManager()->selection();
             selection->select(fs->frames().first()->shape());
 
             KoToolManager::instance()->switchToolRequested(
@@ -249,6 +259,25 @@ KoView *KWDocument::createViewInstance(QWidget *parent)
     return view;
 }
 
+QGraphicsItem *KWDocument::createCanvasItem()
+{
+    // caller owns the canvas item
+    KWCanvasItem *item = new KWCanvasItem(m_viewMode, this);
+
+    if (m_magicCurtain) {
+        item->shapeManager()->addShape(m_magicCurtain, KoShapeManager::AddWithoutRepaint);
+    }
+    foreach (KWFrameSet *fs, m_frameSets) {
+        if (fs->frameCount() == 0) {
+            continue;
+        }
+        foreach (KWFrame *frame, fs->frames()) {
+            item->shapeManager()->addShape(frame->shape(), KoShapeManager::AddWithoutRepaint);
+        }
+    }
+    return item;
+}
+
 KWPage KWDocument::insertPage(int afterPageNum, const QString &masterPageName)
 {
     KWPageInsertCommand *cmd = new KWPageInsertCommand(this, afterPageNum, masterPageName);
@@ -273,7 +302,7 @@ void KWDocument::removePage(int pageNumber)
 
     KWPage page = m_pageManager.page(pageNumber);
     if (! page.isValid()) {
-        kWarning(32001) << "remove page requested for a non exiting page!";
+        kWarning(32001) << "remove page requested for a non exiting page!" << pageNumber;
         return;
     }
     addCommand(new KWPageRemoveCommand(this, page));
@@ -293,7 +322,7 @@ void KWDocument::removeFrameSet(KWFrameSet *fs)
     foreach (KWFrame *frame, fs->frames())
         removeFrame(frame);
     foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
+        KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
         canvas->resourceManager()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
     }
 }
@@ -317,7 +346,7 @@ void KWDocument::relayout()
             }
 
             foreach (KoView *view, views()) {
-                KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
+                KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
                 canvas->shapeManager()->remove(frame->shape());
             }
             tfs->removeFrame(frame);
@@ -359,11 +388,11 @@ void KWDocument::addFrameSet(KWFrameSet *fs)
 void KWDocument::addFrame(KWFrame *frame)
 {
     foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
+        KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
         if (frame->outlineShape())
-            canvas->shapeManager()->add(frame->outlineShape()->parent());
+            canvas->shapeManager()->addShape(frame->outlineShape()->parent());
         else
-            canvas->shapeManager()->add(frame->shape());
+            canvas->shapeManager()->addShape(frame->shape());
         canvas->resourceManager()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
     }
     if (frame->loadingPageNumber() > 0) {
@@ -371,7 +400,7 @@ void KWDocument::addFrame(KWFrame *frame)
             m_magicCurtain = new MagicCurtain();
             m_magicCurtain->setVisible(false);
             foreach (KoView *view, views())
-                static_cast<KWView*>(view)->kwcanvas()->shapeManager()->add(m_magicCurtain);
+                static_cast<KWView*>(view)->canvasBase()->shapeManager()->addShape(m_magicCurtain);
         }
         m_magicCurtain->addFrame(frame);
     }
@@ -562,6 +591,12 @@ bool KWDocument::loadXML(const KoXmlDocument &doc, KoStore *store)
 
 void KWDocument::endOfLoading() // called by both oasis and oldxml
 {
+    QPointer<KoUpdater> updater;
+    if (progressUpdater()) {
+        updater = progressUpdater()->startSubtask(1, "KWDocument::endOfLoading");
+        updater->setProgress(0);
+    }
+
     // Get the master page name of the first page.
     QString firstPageMasterName;
     if (mainFrameSet()) {
@@ -608,6 +643,8 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
         }
     }
 
+    if (updater) updater->setProgress(50);
+
 #if 0
     // do some sanity checking on document.
     for (int i = frameSetCount() - 1; i > -1; i--) {
@@ -693,7 +730,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
                     m_magicCurtain = new MagicCurtain();
                     m_magicCurtain->setVisible(false);
                     foreach (KoView *view, views())
-                        static_cast<KWView*>(view)->kwcanvas()->shapeManager()->add(m_magicCurtain);
+                        static_cast<KWView*>(view)->canvasBase()->shapeManager()->addShape(m_magicCurtain);
                 }
                 m_magicCurtain->addShape(anchor->shape());
             }
@@ -701,7 +738,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
         tfs->setAllowLayout(true);
     }
 
-    emit sigProgress(-1);
+    if (updater) updater->setProgress(100);
 
     kDebug(32001) << "KWDocument::endOfLoading done";
 
@@ -807,7 +844,7 @@ void KWDocument::removeFrameFromViews(KWFrame *frame)
 {
     Q_ASSERT(frame);
     foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
+        KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
         canvas->shapeManager()->remove(frame->shape());
     }
 }
