@@ -182,6 +182,11 @@ bool KWOdfSharedLoadingData::fillFrameProperties(KWFrame *frame, const KoXmlElem
         frame->setTextRunAround(KWord::NoRunAround);
     } else if (wrap == "run-through") {
         frame->setTextRunAround(KWord::RunThrough);
+        /*QString runTrought = properties.attributeNS(KoXmlNS::style, "run-through", "background");
+        if (runTrought == "background") {
+            // TODO handle this case
+        }
+        */
     } else {
         frame->setTextRunAround(KWord::RunAround);
         if (wrap == "biggest")
