@@ -51,6 +51,8 @@ void KWPageStylePrivate::clear()
     mainFrame = true;
     headerDistance = 10; // ~3mm
     footerDistance = 10;
+    headerMinimumHeight = 0;
+    footerMinimumHeight = 0;
     footNoteDistance = 10;
     endNoteDistance = 10;
     headers = KWord::HFTypeNone;
@@ -63,6 +65,7 @@ void KWPageStylePrivate::clear()
         delete fullPageBackground;
     }
     fullPageBackground = 0;
+    nextStyleName.clear();
 }
 
 ///////////
@@ -108,7 +111,7 @@ void KWPageStyle::setHeaderPolicy(KWord::HeaderFooterType p)
     d->headers = p;
 }
 
-const KoPageLayout KWPageStyle::pageLayout() const
+KoPageLayout KWPageStyle::pageLayout() const
 {
     return d->pageLayout;
 }
@@ -158,6 +161,26 @@ void KWPageStyle::setHeaderDistance(qreal distance)
     d->headerDistance = distance;
 }
 
+qreal KWPageStyle::headerMinimumHeight() const
+{
+    return d->headerMinimumHeight;
+}
+
+void KWPageStyle::setHeaderMinimumHeight(qreal height)
+{
+    d->headerMinimumHeight = height;
+}
+
+qreal KWPageStyle::footerMinimumHeight() const
+{
+    return d->footerMinimumHeight;
+}
+
+void KWPageStyle::setFooterMinimumHeight(qreal height)
+{
+    d->footerMinimumHeight = height;
+}
+
 qreal KWPageStyle::footerDistance() const
 {
     return d->footerDistance;
@@ -314,7 +337,7 @@ KoGenStyle KWPageStyle::saveOdf() const
     return pageLayout;
 }
 
-void KWPageStyle::loadOdf(KoOdfLoadingContext &context, const KoXmlElement &style, KoResourceManager *documentResources)
+void KWPageStyle::loadOdf(KoOdfLoadingContext &context, const KoXmlElement &masterNode, const KoXmlElement &style, KoResourceManager *documentResources)
 {
     d->pageLayout.loadOdf(style);
     KoXmlElement props = KoXml::namedItemNS(style, KoXmlNS::style, "page-layout-properties");
@@ -339,6 +362,7 @@ void KWPageStyle::loadOdf(KoOdfLoadingContext &context, const KoXmlElement &styl
         KoXmlElement hfprops = KoXml::namedItemNS(header, KoXmlNS::style, "header-footer-properties");
         if (! hfprops.isNull())
             d->headerDistance = KoUnit::parseValue(hfprops.attributeNS(KoXmlNS::fo, "margin-bottom"));
+            d->headerMinimumHeight = KoUnit::parseValue(hfprops.attributeNS(KoXmlNS::fo, "min-height"));
         // TODO there are quite some more properties we want to at least preserve between load and save
     }
 
@@ -347,18 +371,19 @@ void KWPageStyle::loadOdf(KoOdfLoadingContext &context, const KoXmlElement &styl
         KoXmlElement hfprops = KoXml::namedItemNS(footer, KoXmlNS::style, "header-footer-properties");
         if (! hfprops.isNull())
             d->footerDistance = KoUnit::parseValue(hfprops.attributeNS(KoXmlNS::fo, "margin-top"));
+            d->footerMinimumHeight = KoUnit::parseValue(hfprops.attributeNS(KoXmlNS::fo, "min-height"));
         // TODO there are quite some more properties we want to at least preserve between load and save
     }
 
     // Load background picture
     KoXmlElement propBackgroundImage = KoXml::namedItemNS(props, KoXmlNS::style, "background-image");
     if (!propBackgroundImage.isNull()) {
-        KoPatternBackground *background = new KoPatternBackground(documentResources->imageCollection());
-        d->fullPageBackground = background;
-        d->fullPageBackground->ref();
-
         const QString href = propBackgroundImage.attributeNS(KoXmlNS::xlink, "href", QString());
         if (!href.isEmpty()) {
+            KoPatternBackground *background = new KoPatternBackground(documentResources->imageCollection());
+            d->fullPageBackground = background;
+            d->fullPageBackground->ref();
+
             KoImageCollection *imageCollection = documentResources->imageCollection();
             if (imageCollection != 0) {
                 KoImageData *imageData = imageCollection->createImageData(href,context.store());
@@ -380,6 +405,19 @@ void KWPageStyle::loadOdf(KoOdfLoadingContext &context, const KoXmlElement &styl
             d->fullPageBackground->ref();
         }
     }
+
+    // Load next master-page style name
+    d->nextStyleName = masterNode.attributeNS(KoXmlNS::style, "next-style-name", QString());
+}
+
+QString KWPageStyle::nextStyleName() const
+{
+    return d->nextStyleName;
+}
+
+void KWPageStyle::setNextStyleName(const QString &nextStyleName)
+{
+    d->nextStyleName = nextStyleName;
 }
 
 KoText::Direction KWPageStyle::direction() const
@@ -397,6 +435,11 @@ bool KWPageStyle::operator==(const KWPageStyle &other) const
     return d == other.d;
 }
 
+KWPageStylePrivate *KWPageStyle::priv()
+{
+    return d.data();
+}
+
 const KWPageStylePrivate *KWPageStyle::priv() const
 {
     return d.data();
@@ -407,8 +450,20 @@ uint KWPageStyle::hash() const
     return ((uint) d) + 1;
 }
 
+bool KWPageStyle::isPageSpread() const
+{
+    return d->pageLayout.leftMargin < 0;
+}
+
 uint qHash(const KWPageStyle &style)
 {
     return style.hash();
 }
 
+void KWPageStyle::detach(const QString &newName)
+{
+    if (d->fullPageBackground)
+        d->fullPageBackground->ref();
+    d.detach();
+    d->name = newName;
+}
