@@ -21,24 +21,21 @@
 #ifndef KO_TOOL_MANAGER
 #define KO_TOOL_MANAGER
 
-#include <KoInputDevice.h>
+#include "KoInputDevice.h"
 #include "flake_export.h"
 
 #include <QObject>
 #include <QCursor>
 #include <QList>
 
-class ToolHelper;
 class KoCanvasController;
 class KoCanvasBase;
-class KoTool;
+class KoToolBase;
 class KoCreateShapesTool;
 class KActionCollection;
 class KoShape;
-class KoToolProxy;
 class QToolButton;
-class KoDeviceEvent;
-class KoGuidesTool;
+class KoInputDeviceHandlerEvent;
 class KoShapeLayer;
 
 /**
@@ -146,7 +143,7 @@ public:
      *    who's tool you want.
      * @see addController()
      */
-    KoGuidesTool * guidesTool(KoCanvasBase * canvas) const;
+    KoToolBase *toolById(KoCanvasBase *canvas, const QString &id) const;
 
     /// @return the currently active pointing device
     KoInputDevice currentInputDevice() const;
@@ -162,7 +159,7 @@ public:
     struct Button {
         QToolButton *button;///< a newly created button.
         QString section;        ///< The section the button wants to be in.
-        int priority;           ///< Higher priority means coming first in the section.
+        int priority;           ///< Lower number (higher priority) means coming first in the section.
         int buttonGroupId;      ///< An unique ID for this button as passed by changedTool()
         QString visibilityCode; ///< This button should become visible when we emit this string in toolCodesSelected()
     };
@@ -175,10 +172,24 @@ public:
     QList<Button> createToolList(KoCanvasBase *canvas) const;
 
     /// Request tool activation for the given canvas controller
-    void requestToolActivation(KoCanvasController * controller);
+    void requestToolActivation(KoCanvasController *controller);
 
     /// Injects an input event from a plugin based input device
-    void injectDeviceEvent(KoDeviceEvent * event);
+    void injectDeviceEvent(KoInputDeviceHandlerEvent *event);
+
+    /// Returns the toolId of the currently active tool
+    QString activeToolId() const;
+
+    void updateReadWrite(KoCanvasController *cc, bool readWrite);
+
+    class Private;
+    /**
+     * \internal return the private object for the toolmanager.
+     */
+    KoToolManager::Private *priv();
+
+    /// reimplemented from QObject
+    virtual bool eventFilter(QObject *object, QEvent *event);
 
 public slots:
     /**
@@ -193,7 +204,7 @@ signals:
      * @param canvas the currently active canvas.
      * @param uniqueToolId a random but unique code for the new tool.
      */
-    void changedTool(const KoCanvasController *canvas, int uniqueToolId);
+    void changedTool(KoCanvasController *canvas, int uniqueToolId);
 
     /**
      * Emitted after the selection changed to state which unique shape-types are now
@@ -201,14 +212,14 @@ signals:
      * @param canvas the currently active canvas.
      * @param types a list of string that are the shape types of the selected objects.
      */
-    void toolCodesSelected(const KoCanvasController *canvas, QList<QString> types);
+    void toolCodesSelected(const KoCanvasController *canvas, const QList<QString> &types);
 
     /**
      * Emitted after the current layer changed either its properties or to a new layer.
      * @param canvas the currently active canvas.
      * @param layer the layer that is selected.
      */
-    void currentLayerChanged(const KoCanvasController *canvas, const KoShapeLayer* layer);
+    void currentLayerChanged(const KoCanvasController *canvas, const KoShapeLayer *layer);
 
     /**
      * Every time a new input device gets used by a tool, this event is emitted.
@@ -220,59 +231,29 @@ signals:
      * Emitted whenever the active canvas changed.
      * @param canvas the new activated canvas (might be 0)
      */
-    void changedCanvas(const KoCanvasBase * canvas);
+    void changedCanvas(const KoCanvasBase *canvas);
 
     /**
      * Emitted whenever the active tool changes the status text.
      * @param statusText the new status text
      */
     void changedStatusText(const QString &statusText);
-protected:
-    friend class KoToolProxy;
-    /**
-     * Request a switch from to the param input device.
-     * This will cause the tool for that device to be selected.
-     */
-    void switchInputDevice(const KoInputDevice &device);
-
-    /**
-     * Whenever a new tool proxy class is instantiated, it will use this method to register itself
-     * so the toolManager can update it to the latest active tool.
-     * @param proxy the proxy to register.
-     * @param canvas which canvas the proxy is associated with; whenever a new tool is selected for that canvas,
-     *        the proxy gets an update.
-     */
-    void registerToolProxy(KoToolProxy *proxy, KoCanvasBase *canvas);
-
-    // make it a friend so we can temporary switch tool from there
-    friend class KoCanvasController;
-    void switchToolByShortcut(QKeyEvent *event);
-
-protected slots:
-    void switchToolTemporaryRequested(const QString &id);
 
 private:
     KoToolManager();
     KoToolManager(const KoToolManager&);
     KoToolManager operator=(const KoToolManager&);
-    void setup();
-    void switchTool(KoTool *tool, bool temporary);
-    void switchTool(const QString &id, bool temporary);
-    void postSwitchTool(bool temporary);
-    bool eventFilter(QObject *object, QEvent *event);
 
-private slots:
-    void toolActivated(ToolHelper *tool);
-    void detachCanvas(KoCanvasController *controller);
-    void attachCanvas(KoCanvasController *controller);
-    void movedFocus(QWidget *from, QWidget *to);
-    void updateCursor(QCursor cursor);
-    void switchBackRequested();
-    void selectionChanged(QList<KoShape*> shapes);
-    void currentLayerChanged(const KoShapeLayer *layer);
+    Q_PRIVATE_SLOT(d, void toolActivated(ToolHelper *tool))
+    Q_PRIVATE_SLOT(d, void detachCanvas(KoCanvasController *controller))
+    Q_PRIVATE_SLOT(d, void attachCanvas(KoCanvasController *controller))
+    Q_PRIVATE_SLOT(d, void movedFocus(QWidget *from, QWidget *to))
+    Q_PRIVATE_SLOT(d, void updateCursor(const QCursor &cursor))
+    Q_PRIVATE_SLOT(d, void switchBackRequested())
+    Q_PRIVATE_SLOT(d, void selectionChanged(QList<KoShape*> shapes))
+    Q_PRIVATE_SLOT(d, void currentLayerChanged(const KoShapeLayer *layer))
+    Q_PRIVATE_SLOT(d, void switchToolTemporaryRequested(const QString &id))
 
-private:
-    class Private;
     Private *const d;
 };
 
