@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
  *
- * Copyright (c) 2006 Boudewijn Rempt <boud@valdyas.org>
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (c) 2006, 2010 Boudewijn Rempt <boud@valdyas.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,16 +37,19 @@ class KoToolSelection;
 class KoToolBase;
 class KoCanvasBase;
 class KoCanvasController;
+class KoToolProxyPrivate;
 class QInputMethodEvent;
+class KoPointerEvent;
 
 /**
- * Simple proxy interface that provides a point d'appui for canvas
- * implementations to pass events to the current tool. For the paint
- * event it is possible that there more than one tool is asked to
- * paint their decorations because tools can be stacked.
+ * Tool proxy object which allows an application to address the current tool.
  *
- * The implementator of KoToolProxy should be solely responsible
- * for knowing which tool is currently in the user's hands.
+ * Applications typically have a canvas and a canvas requires a tool for
+ * the user to do anything.  Since the flake system is responsible for handling
+ * tools and also to change the active tool when needed we provide one class
+ * that can be used by an application canvas to route all the native events too
+ * which will transparantly be routed to the active tool.  Without the application
+ * having to bother about which tool is active.
  */
 class FLAKE_EXPORT KoToolProxy : public QObject
 {
@@ -68,18 +71,23 @@ public:
     void tabletEvent(QTabletEvent *event, const QPointF &point);
     /// Forwarded to the current KoToolBase
     void mousePressEvent(QMouseEvent *event, const QPointF &point);
+    void mousePressEvent(KoPointerEvent *event);
     /// Forwarded to the current KoToolBase
     void mouseDoubleClickEvent(QMouseEvent *event, const QPointF &point);
+    void mouseDoubleClickEvent(KoPointerEvent *event);
     /// Forwarded to the current KoToolBase
     void mouseMoveEvent(QMouseEvent *event, const QPointF &point);
+    void mouseMoveEvent(KoPointerEvent *event);
     /// Forwarded to the current KoToolBase
     void mouseReleaseEvent(QMouseEvent *event, const QPointF &point);
+    void mouseReleaseEvent(KoPointerEvent *event);
     /// Forwarded to the current KoToolBase
     void keyPressEvent(QKeyEvent *event);
     /// Forwarded to the current KoToolBase
     void keyReleaseEvent(QKeyEvent *event);
     /// Forwarded to the current KoToolBase
     void wheelEvent(QWheelEvent * event, const QPointF &point);
+    void wheelEvent(KoPointerEvent *event);
     /// Forwarded to the current KoToolBase
     QVariant inputMethodQuery(Qt::InputMethodQuery query, const KoViewConverter &converter) const;
     /// Forwarded to the current KoToolBase
@@ -89,6 +97,10 @@ public:
     /// Forwarded to the current KoToolBase
     void deleteSelection();
 
+    /// This method gives the proxy a chance to do things. for example it is need to have working singlekey
+    /// shortcuts. call it from the canvas' event function and forward it to QWidget::event() later.
+    void processEvent(QEvent *) const;
+
     /**
      * Retrieves the entire collection of actions for the active tool
      * or an empty hash if there is no active tool yet.
@@ -111,6 +123,9 @@ public:
     /// Set the new active tool.
     void setActiveTool(KoToolBase *tool);
 
+    /// \internal
+    KoToolProxyPrivate *priv();
+
 signals:
     /**
      * A tool can have a selection that is copy-able, this signal is emitted when that status changes.
@@ -125,19 +140,12 @@ signals:
      */
     void toolChanged(const QString &toolId);
 
-protected:
-    friend class KoToolManager;
-    friend class KoToolManager::Private;
-    /// the toolManager tells us which KoCanvasController this toolProxy is working for.
-    void setCanvasController(KoCanvasController *controller);
-
 private:
     Q_PRIVATE_SLOT(d, void timeout())
     Q_PRIVATE_SLOT(d, void selectionChanged(bool))
 
-    class Private;
-    Private * const d;
+    friend class KoToolProxyPrivate;
+    KoToolProxyPrivate * const d;
 };
 
-
 #endif // _KO_TOOL_PROXY_H_
