@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007, 2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2007, 2009-2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Johannes Simon <johannes.simon@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -98,7 +99,13 @@ public:
     Private(KoTextDocumentLayout *parent_)
             : inlineTextObjectManager(0),
             scheduled(false),
-            parent(parent_) {
+            parent(parent_),
+            resizeMethod(KoTextDocument::NoResize) {
+    }
+
+    ~Private()
+    {
+        delete paintDevice;
     }
 
     void relayoutPrivate() {
@@ -106,13 +113,62 @@ public:
         parent->relayout();
     }
 
+    void postLayoutHook() {
+        Q_ASSERT(parent);
+        Q_ASSERT(parent->m_state);
+        KoShape *shape = parent->m_state->shape;
+        if (shape == 0)
+            return;
+        KoTextShapeData *data = qobject_cast<KoTextShapeData*>(shape->userData());
+        qreal offset = 0;
+        if (data->verticalAlignment() == Qt::AlignVCenter) {
+            offset = (shape->size().height() - (parent->m_state->y() - data->documentOffset())) / 2.;
+        } else if (data->verticalAlignment() == Qt::AlignBottom) {
+            offset = shape->size().height() - (parent->m_state->y() - data->documentOffset());
+        }
+        if (offset != 0) {
+            data->setDocumentOffset(data->documentOffset() - offset);
+        }
+    }
+
+    void adjustSize();
+
     QList<KoShape *> shapes;
     KoInlineTextObjectManager *inlineTextObjectManager;
     bool scheduled;
     KoTextDocumentLayout *parent;
+    KoTextDocument::ResizeMethod resizeMethod;
     KoPostscriptPaintDevice *paintDevice;
 };
 
+void KoTextDocumentLayout::Private::adjustSize()
+{
+    if (parent->resizeMethod() == KoTextDocument::NoResize)
+        return;
+
+    if (parent->shapes().isEmpty())
+        return;
+    // Limit auto-resizing to the first shape only (there won't be more
+    // with auto-resizing turned on, unless specifically set)
+    KoShape *shape = parent->shapes().first();
+
+    // Determine the maximum width of all text lines
+    qreal width = 0;
+    for (QTextBlock block = parent->document()->begin(); block.isValid(); block = block.next()) {
+        // The block layout's wrap mode must be QTextOption::NoWrap, thus the line count
+        // of a valid block must be 1 (otherwise this resizing scheme wouldn't work)
+        Q_ASSERT(block.layout()->lineCount() == 1);
+        QTextLine line = block.layout()->lineAt(0);
+        width = qMax(width, line.naturalTextWidth());
+    }
+
+    // Use position and height of last text line to calculate height
+    QTextLine line = parent->document()->lastBlock().layout()->lineAt(0);
+    qreal height = line.position().y() + line.height();
+
+    shape->setSize(QSizeF(width, height));
+}
+
 // ------------------- KoTextDocumentLayout --------------------
 KoTextDocumentLayout::KoTextDocumentLayout(QTextDocument *doc, KoTextDocumentLayout::LayoutState *layout)
         : QAbstractTextDocumentLayout(doc),
@@ -123,11 +179,13 @@ KoTextDocumentLayout::KoTextDocumentLayout(QTextDocument *doc, KoTextDocumentLay
     setPaintDevice(d->paintDevice);
     if (m_state == 0)
         m_state = new LayoutStateDummy();
+
+    connect (this, SIGNAL(finishedLayout()), this, SLOT(postLayoutHook()));
+    connect(this, SIGNAL(finishedLayout()), SLOT(adjustSize()));
 }
 
 KoTextDocumentLayout::~KoTextDocumentLayout()
 {
-    delete d->paintDevice;
     delete d;
     delete m_state;
     m_state = 0;
@@ -136,6 +194,7 @@ KoTextDocumentLayout::~KoTextDocumentLayout()
 void KoTextDocumentLayout::setLayout(LayoutState *layout)
 {
     Q_ASSERT(layout);
+    delete m_state;
     m_state = layout;
     scheduleLayout();
 }
@@ -182,7 +241,7 @@ QRectF KoTextDocumentLayout::blockBoundingRect(const QTextBlock &block) const
 QSizeF KoTextDocumentLayout::documentSize() const
 {
     // nobody calls this code and I have no way of implementing it anyway...
-    kWarning() << "KoTextDocumentLayout::documentSize is not implemented";
+    //kWarning() << "KoTextDocumentLayout::documentSize is not implemented";
     return QSizeF(10, 10);
 }
 
@@ -202,7 +261,7 @@ QRectF KoTextDocumentLayout::frameBoundingRect(QTextFrame *frame) const
 {
     Q_UNUSED(frame);
     // nobody calls this code and I have no way of implementing it anyway...
-    kWarning() << "KoTextDocumentLayout::frameBoundingRect is not implemented";
+    //kWarning() << "KoTextDocumentLayout::frameBoundingRect is not implemented";
     return QRectF(0, 0, 10, 10);
 }
 
@@ -453,4 +512,17 @@ KoShape* KoTextDocumentLayout::shapeForPosition(int position) const
     return 0;
 }
 
+void KoTextDocumentLayout::setResizeMethod(KoTextDocument::ResizeMethod method)
+{
+    if (d->resizeMethod == method)
+        return;
+    d->resizeMethod = method;
+    scheduleLayout();
+}
+
+KoTextDocument::ResizeMethod KoTextDocumentLayout::resizeMethod() const
+{
+    return d->resizeMethod;
+}
+
 #include <KoTextDocumentLayout.moc>
