@@ -26,7 +26,7 @@
 // TODO: find a better place or way to define those stuff
 const quint8 OPACITY_TRANSPARENT_U8 = 0;
 const quint8 OPACITY_OPAQUE_U8 = UCHAR_MAX;
-const quint8 OPACITY_TRANSPARENT_F = 0.0;
-const quint8 OPACITY_OPAQUE_F = 1.0;
+const qreal OPACITY_TRANSPARENT_F = 0.0;
+const qreal OPACITY_OPAQUE_F = 1.0;
 
 #endif
