@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
-   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
    Copyright (C) 2008-2009 Jan Hambrecht <jaham@gmx.net>
    Copyright (C) 2008 Casper Boemann <cbr@boemann.dk>
 
@@ -137,13 +137,13 @@ DefaultTool::DefaultTool(KoCanvasBase *canvas)
     rotatePixmap.load(KStandardDirs::locate("data", "koffice/icons/rotate.png"));
     shearPixmap.load(KStandardDirs::locate("data", "koffice/icons/shear.png"));
 
-    m_rotateCursors[0] = QCursor(rotatePixmap.transformed(QMatrix().rotate(45)));
-    m_rotateCursors[1] = QCursor(rotatePixmap.transformed(QMatrix().rotate(90)));
-    m_rotateCursors[2] = QCursor(rotatePixmap.transformed(QMatrix().rotate(135)));
-    m_rotateCursors[3] = QCursor(rotatePixmap.transformed(QMatrix().rotate(180)));
-    m_rotateCursors[4] = QCursor(rotatePixmap.transformed(QMatrix().rotate(225)));
-    m_rotateCursors[5] = QCursor(rotatePixmap.transformed(QMatrix().rotate(270)));
-    m_rotateCursors[6] = QCursor(rotatePixmap.transformed(QMatrix().rotate(315)));
+    m_rotateCursors[0] = QCursor(rotatePixmap.transformed(QTransform().rotate(45)));
+    m_rotateCursors[1] = QCursor(rotatePixmap.transformed(QTransform().rotate(90)));
+    m_rotateCursors[2] = QCursor(rotatePixmap.transformed(QTransform().rotate(135)));
+    m_rotateCursors[3] = QCursor(rotatePixmap.transformed(QTransform().rotate(180)));
+    m_rotateCursors[4] = QCursor(rotatePixmap.transformed(QTransform().rotate(225)));
+    m_rotateCursors[5] = QCursor(rotatePixmap.transformed(QTransform().rotate(270)));
+    m_rotateCursors[6] = QCursor(rotatePixmap.transformed(QTransform().rotate(315)));
     m_rotateCursors[7] = QCursor(rotatePixmap);
 /*
     m_rotateCursors[0] = QCursor(Qt::RotateNCursor);
@@ -156,13 +156,13 @@ DefaultTool::DefaultTool(KoCanvasBase *canvas)
     m_rotateCursors[7] = QCursor(Qt::RotateNWCursor);
 */
     m_shearCursors[0] = QCursor(shearPixmap);
-    m_shearCursors[1] = QCursor(shearPixmap.transformed(QMatrix().rotate(45)));
-    m_shearCursors[2] = QCursor(shearPixmap.transformed(QMatrix().rotate(90)));
-    m_shearCursors[3] = QCursor(shearPixmap.transformed(QMatrix().rotate(135)));
-    m_shearCursors[4] = QCursor(shearPixmap.transformed(QMatrix().rotate(180)));
-    m_shearCursors[5] = QCursor(shearPixmap.transformed(QMatrix().rotate(225)));
-    m_shearCursors[6] = QCursor(shearPixmap.transformed(QMatrix().rotate(270)));
-    m_shearCursors[7] = QCursor(shearPixmap.transformed(QMatrix().rotate(315)));
+    m_shearCursors[1] = QCursor(shearPixmap.transformed(QTransform().rotate(45)));
+    m_shearCursors[2] = QCursor(shearPixmap.transformed(QTransform().rotate(90)));
+    m_shearCursors[3] = QCursor(shearPixmap.transformed(QTransform().rotate(135)));
+    m_shearCursors[4] = QCursor(shearPixmap.transformed(QTransform().rotate(180)));
+    m_shearCursors[5] = QCursor(shearPixmap.transformed(QTransform().rotate(225)));
+    m_shearCursors[6] = QCursor(shearPixmap.transformed(QTransform().rotate(270)));
+    m_shearCursors[7] = QCursor(shearPixmap.transformed(QTransform().rotate(315)));
     m_sizeCursors[0] = Qt::SizeVerCursor;
     m_sizeCursors[1] = Qt::SizeBDiagCursor;
     m_sizeCursors[2] = Qt::SizeHorCursor;
@@ -626,8 +626,21 @@ void DefaultTool::mouseDoubleClickEvent(KoPointerEvent *event)
         }
     }
 
+    QList<KoShape*> shapes2;
+    foreach (KoShape *shape, shapes) {
+        QSet<KoShape*> delegates = shape->toolDelegates();
+        if (delegates.isEmpty()) {
+            shapes2.append(shape);
+        } else {
+            foreach (KoShape *delegatedShape, delegates) {
+                shapes2.append(delegatedShape);
+            }
+        }
+    }
+
+
     KoToolManager::instance()->switchToolRequested(
-            KoToolManager::instance()->preferredToolForSelection(shapes));
+            KoToolManager::instance()->preferredToolForSelection(shapes2));
 }
 
 bool DefaultTool::moveSelection(int direction, Qt::KeyboardModifiers modifiers)
@@ -872,11 +885,11 @@ void DefaultTool::recalcSelectionBox()
         return;
 
     if (koSelection()->count()>1) {
-        QMatrix matrix = koSelection()->absoluteTransformation(0);
+        QTransform matrix = koSelection()->absoluteTransformation(0);
         m_selectionOutline = matrix.map(QPolygonF(QRectF(QPointF(0, 0), koSelection()->size())));
         m_angle = 0.0; //koSelection()->rotation();
     } else {
-        QMatrix matrix = koSelection()->firstSelectedShape()->absoluteTransformation(0);
+        QTransform matrix = koSelection()->firstSelectedShape()->absoluteTransformation(0);
         m_selectionOutline = matrix.map(QPolygonF(QRectF(QPointF(0, 0), koSelection()->firstSelectedShape()->size())));
         m_angle = 0.0; //koSelection()->firstSelectedShape()->rotation();
     }
@@ -994,7 +1007,7 @@ void DefaultTool::selectionUngroup()
         KoShapeGroup *group = dynamic_cast<KoShapeGroup*>(shape);
         if (group) {
             cmd = cmd ? cmd : new QUndoCommand(i18n("Ungroup shapes"));
-            new KoShapeUngroupCommand(group, group->childShapes(),
+            new KoShapeUngroupCommand(group, group->shapes(),
                                       group->parent()? QList<KoShape*>(): canvas()->shapeManager()->topLevelShapes(),
                                       cmd);
             canvas()->shapeController()->removeShape(group, cmd);
@@ -1169,7 +1182,7 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
     if ((event->buttons() & Qt::LeftButton) == 0)
         return 0;  // Nothing to do for middle/right mouse button
 
-    KoShape *shape(shapeManager->shapeAt(event->point, selectNextInStack ? KoFlake::NextUnselected : KoFlake::ShapeOnTop));
+    KoShape *shape = shapeManager->shapeAt(event->point, selectNextInStack ? KoFlake::NextUnselected : KoFlake::ShapeOnTop);
 
     if (!shape && handle == KoFlake::NoHandle) {
         // check if we have hit a guide
