@@ -32,7 +32,7 @@ public:
     explicit RectangleShapeFactory(QObject *parent);
     ~RectangleShapeFactory() {}
     virtual KoShape *createDefaultShape(KoResourceManager *documentResources = 0) const;
-    virtual bool supports(const KoXmlElement &e) const;
+    virtual bool supports(const KoXmlElement &e, KoShapeLoadingContext &context) const;
     virtual QList<KoShapeConfigWidgetBase*> createShapeOptionPanels();
 };
 
