@@ -2,6 +2,7 @@
  This file is part of the KDE project
  * Copyright (C) 2009 Ganesh Paramasivam <ganesh@crystalfab.com>
  * Copyright (C) 2009 Pierre Stirnweiss <pstirnweiss@googlemail.com>
+ * Copyright (C) 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -46,6 +47,7 @@
 #include <KDebug>
 //#include <iostream>
 #include <QDebug>
+#include <QWeakPointer>
 
 //A convenience function to get a ListIdType from a format
 
@@ -79,7 +81,7 @@ void ChangeTrackedDeleteCommand::undo()
     TextCommandBase::undo();
     UndoRedoFinalizer finalizer(this);
 
-    QTextDocument *document = m_tool->m_textEditor->document();
+    QTextDocument *document = m_tool->m_textEditor.data()->document();
     KoTextDocument(document).changeTracker()->elementById(m_addedChangeElement)->setValid(false);
     foreach (int changeId, m_removedElements) {
       KoTextDocument(document).changeTracker()->elementById(changeId)->setValid(true);
@@ -94,25 +96,25 @@ void ChangeTrackedDeleteCommand::redo()
     if (!m_first) {
         TextCommandBase::redo();
         UndoRedoFinalizer finalizer(this);
-        QTextDocument *document = m_tool->m_textEditor->document();
+        QTextDocument *document = m_tool->m_textEditor.data()->document();
         KoTextDocument(document).changeTracker()->elementById(m_addedChangeElement)->setValid(true);
         foreach (int changeId, m_removedElements) {
           KoTextDocument(document).changeTracker()->elementById(changeId)->setValid(false);
         }
     } else {
         m_first = false;
-        m_tool->m_textEditor->beginEditBlock();
+        m_tool->m_textEditor.data()->beginEditBlock();
         if(m_mode == PreviousChar)
             deletePreviousChar();
         else
             deleteChar();
-        m_tool->m_textEditor->endEditBlock();
+        m_tool->m_textEditor.data()->endEditBlock();
     }
 }
 
 void ChangeTrackedDeleteCommand::deleteChar()
 {
-    QTextCursor *caret = m_tool->m_textEditor->cursor();
+    QTextCursor *caret = m_tool->m_textEditor.data()->cursor();
 
     if (caret->atEnd() && !caret->hasSelection())
         return;
@@ -125,7 +127,7 @@ void ChangeTrackedDeleteCommand::deleteChar()
 
 void ChangeTrackedDeleteCommand::deletePreviousChar()
 {
-    QTextCursor *caret = m_tool->m_textEditor->cursor();
+    QTextCursor *caret = m_tool->m_textEditor.data()->cursor();
 
     if (caret->atStart() && !caret->hasSelection())
         return;
@@ -177,7 +179,7 @@ void ChangeTrackedDeleteCommand::handleListItemDelete(QTextCursor &selection)
     selection.insertBlock();
     // Mark it as inserted content
     QTextCharFormat format = selection.charFormat();
-    m_tool->m_textEditor->registerTrackedChange(selection, KoGenChange::insertChange, i18n("Key Press"), format, format, false); 
+    m_tool->m_textEditor.data()->registerTrackedChange(selection, KoGenChange::InsertChange, i18n("Key Press"), format, format, false);
     //Paste the selected text
     TextPasteCommand *pasteCommand = new TextPasteCommand(QClipboard::Clipboard, m_tool, this);
     pasteCommand->redo();
@@ -197,7 +199,7 @@ void ChangeTrackedDeleteCommand::handleListItemDelete(QTextCursor &selection)
 
 void ChangeTrackedDeleteCommand::deleteSelection(QTextCursor &selection)
 {
-    QTextDocument *document = m_tool->m_textEditor->document();
+    QTextDocument *document = m_tool->m_textEditor.data()->document();
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(document->documentLayout());
     Q_ASSERT(layout);
     Q_ASSERT(layout->inlineTextObjectManager());
@@ -239,7 +241,7 @@ void ChangeTrackedDeleteCommand::deleteSelection(QTextCursor &selection)
     }
 
     checker.setPosition(selectionBegin);
-    
+
     if (!KoTextDocument(document).changeTracker()->displayChanges()) {
         QChar charAtPos = document->characterAt(checker.position() - 1);
         if (layout->inlineTextObjectManager()->inlineTextObject(checker) && charAtPos == QChar::ObjectReplacementCharacter) {
@@ -255,7 +257,7 @@ void ChangeTrackedDeleteCommand::deleteSelection(QTextCursor &selection)
 
     if (KoTextDocument(document).changeTracker()->containsInlineChanges(checker.charFormat())) {
         int changeId = checker.charFormat().property(KoCharacterStyle::ChangeTrackerId).toInt();
-        if (KoTextDocument(document).changeTracker()->elementById(changeId)->getChangeType() == KoGenChange::deleteChange) {
+        if (KoTextDocument(document).changeTracker()->elementById(changeId)->getChangeType() == KoGenChange::DeleteChange) {
             QTextDocumentFragment prefix =  KoTextDocument(document).changeTracker()->elementById(changeId)->getDeleteData();
             selectionBegin -= (fragmentLength(prefix) + 1 );
             KoTextDocument(document).changeTracker()->elementById(changeId)->setValid(false);
@@ -359,8 +361,27 @@ void ChangeTrackedDeleteCommand::insertDeleteFragment(QTextCursor &cursor, KoDel
                 } 
                 currentList->add(cursor.block(), deletedListItemLevel);
             }
+        } else if (tempCursor.currentTable()) {
+            QTextTable *deletedTable = tempCursor.currentTable();
+            int numRows = deletedTable->rows();
+            int numColumns = deletedTable->columns();
+            QTextTable *insertedTable = cursor.insertTable(numRows, numColumns, deletedTable->format());
+            for (int i=0; i<numRows; i++) {
+                for (int j=0; j<numColumns; j++) {
+                    tempCursor.setPosition(deletedTable->cellAt(i,j).firstCursorPosition().position());
+                    tempCursor.setPosition(deletedTable->cellAt(i,j).lastCursorPosition().position(), QTextCursor::KeepAnchor);
+                    insertedTable->cellAt(i,j).setFormat(deletedTable->cellAt(i,j).format().toTableCellFormat());
+                    cursor.setPosition(insertedTable->cellAt(i,j).firstCursorPosition().position());
+                    cursor.insertFragment(tempCursor.selection());
+                }
+            }
+            tempCursor.setPosition(deletedTable->cellAt(numRows-1,numColumns-1).lastCursorPosition().position());
+            currentBlock = tempCursor.block();
+            //Move the cursor outside of table
+            cursor.setPosition(cursor.position() + 1);
+            continue;
         } else {
-            // This block does not contain a list. So no special work here. 
+            // This block does not contain a list. So no special work here.
             if (currentBlock != tempDoc.begin())
                 cursor.insertBlock(currentBlock.blockFormat(), currentBlock.charFormat());
         }
@@ -404,6 +425,20 @@ int ChangeTrackedDeleteCommand::fragmentLength(QTextDocumentFragment &fragment)
             deletedListItem = currentBlock.blockFormat().property(KoDeleteChangeMarker::DeletedListItem).toBool();
             if (currentBlock != tempDoc.begin() && deletedListItem)
                 length += 1; //For the Block separator
+        } else if (tempCursor.currentTable()) {
+            QTextTable *deletedTable = tempCursor.currentTable();
+            int numRows = deletedTable->rows();
+            int numColumns = deletedTable->columns();
+            for (int i=0; i<numRows; i++) {
+                for (int j=0; j<numColumns; j++) {
+                    length += 1;
+                    length += (deletedTable->cellAt(i,j).lastCursorPosition().position() - deletedTable->cellAt(i,j).firstCursorPosition().position());
+                }
+            }
+            tempCursor.setPosition(deletedTable->cellAt(numRows-1,numColumns-1).lastCursorPosition().position());
+            currentBlock = tempCursor.block();
+            length += 1;
+            continue;
         } else {
             if (currentBlock != tempDoc.begin())
                 length += 1; //For the Block Separator
@@ -437,18 +472,18 @@ bool ChangeTrackedDeleteCommand::mergeWith( const QUndoCommand *command)
         {}
 
         void undo() {
-            if (m_document.isNull())
-                return;
-            m_document->undo(KoTextDocument(m_document).textEditor()->cursor());
+            QTextDocument *doc = m_document.data();
+            if (doc)
+                doc->undo(KoTextDocument(doc).textEditor()->cursor());
         }
 
         void redo() {
-            if (m_document.isNull())
-                return;
-            m_document->redo(KoTextDocument(m_document).textEditor()->cursor());
+            QTextDocument *doc = m_document.data();
+            if (doc)
+                doc->redo(KoTextDocument(doc).textEditor()->cursor());
         }
 
-        QPointer<QTextDocument> m_document;
+        QWeakPointer<QTextDocument> m_document;
     };
 
     if (command->id() != id())
@@ -473,7 +508,7 @@ bool ChangeTrackedDeleteCommand::mergeWith( const QUndoCommand *command)
         m_length = other->m_length;
 
         for(int i=0; i < command->childCount(); i++)
-            new UndoTextCommand(m_tool->m_textEditor->document(), this);
+            new UndoTextCommand(m_tool->m_textEditor.data()->document(), this);
 
         return true;
     }
@@ -483,7 +518,7 @@ bool ChangeTrackedDeleteCommand::mergeWith( const QUndoCommand *command)
 void ChangeTrackedDeleteCommand::updateListIds(QTextCursor &cursor)
 {
     m_newListIds.clear();
-    QTextDocument *document = m_tool->m_textEditor->document();
+    QTextDocument *document = m_tool->m_textEditor.data()->document();
     QTextCursor tempCursor(document);
     QTextBlock startBlock = document->findBlock(cursor.anchor());
     QTextBlock endBlock = document->findBlock(cursor.position());
@@ -500,7 +535,7 @@ void ChangeTrackedDeleteCommand::updateListIds(QTextCursor &cursor)
 }
 void ChangeTrackedDeleteCommand::updateListChanges()
 {
-    QTextDocument *document = m_tool->m_textEditor->document();
+    QTextDocument *document = m_tool->m_textEditor.data()->document();
     QTextCursor tempCursor(document);
     QTextBlock startBlock = document->findBlock(m_position);
     QTextBlock endBlock = document->findBlock(m_position + m_length);
@@ -535,7 +570,7 @@ ChangeTrackedDeleteCommand::~ChangeTrackedDeleteCommand()
 
 void ChangeTrackedDeleteCommand::removeChangeElement(int changeId)
 {
-    QTextDocument *document = m_tool->m_textEditor->document();
+    QTextDocument *document = m_tool->m_textEditor.data()->document();
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(document->documentLayout());
     KoChangeTrackerElement *element = KoTextDocument(document).changeTracker()->elementById(changeId);
     KoDeleteChangeMarker *marker = element->getDeleteChangeMarker();
