@@ -153,6 +153,8 @@ void SimpleStyleWidget::setCurrentBlock(const QTextBlock &block)
         switch (layout->textOption().textDirection()) {
         case Qt::LeftToRight: updateDirection(LTR); break;
         case Qt::RightToLeft: updateDirection(RTL); break;
+        default:
+            break;
         }
     }
 
@@ -238,9 +240,7 @@ void SimpleStyleWidget::directionChangeRequested()
             QTextBlockFormat bf = cursor.blockFormat();
             bf.clearProperty(KoParagraphStyle::TextProgressionDirection);
             cursor.setBlockFormat(bf);
-            int oldPos = cursor.position();
-            cursor.movePosition(QTextCursor::NextBlock);
-            if (cursor.position() == oldPos) // end of document
+            if (!cursor.movePosition(QTextCursor::NextBlock))
                 break;
         }
         emit doneWithFocus();
